
/* ###--------------------------------------------------------------### */
/* file		: dlx_dlx.lex						*/
/* date		: Dec  6 1999						*/
/* version	: v0.7							*/
/* author	: Pirouz BAZARGAN SABET					*/
/* description	: DLX assembler - lex rules				*/
/* ###--------------------------------------------------------------### */

%{
#include <stdio.h>
#include MUT_H
#include "dlx_type.h"
#include "y.tab.h"
#include "dlx_defs.h"
#include "dlx_lex.h"
%}

register		[rR\$]([0-9]|[12][0-9]|3[01])
digit			[0-9]
hexadecimal_valu	0x[0-9a-fA-F]+
octal_valu		0o[0-7]+
binary_valu		0b[0-1]+
decimal_valu		{digit}+
letter			[a-zA-Z]
letter_or_digit		{letter}|{digit}
ident			(\.|_|{letter})(\.|_|{letter_or_digit})*

%%
\n				{
/*printf ("CarriageReturn : %d\n", DLX_LINNUM);*/
			DLX_LINNUM++;
			return (CarriageReturn);
				}
\:				{
/*printf ("Colon\n");*/
			return (Colon);
				}
[ \t]				{
/*printf ("space\n");*/
				}
#				{
/*printf ("Sharp\n");*/
			return (Sharp);
				}
\&				{
/*printf ("Ampersand\n");*/
			return (Ampersand);
				}
\|				{
/*printf ("Bar\n");*/
			return (Bar);
				}
\^				{
/*printf ("Circomflex\n");*/
			return (Circomflex);
				}
\+				{
/*printf ("Plus\n");*/
			return (Plus);
				}
\/				{
/*printf ("Slash\n");*/
			return (Slash);
				}
\*				{
/*printf ("Star\n");*/
			return (Star);
				}
\<\<				{
/*printf ("DoubleLess\n");*/
			return (DoubleLess);
				}
\>\>				{
/*printf ("DoubleGreat\n");*/
			return (DoubleGreat);
				}
\-				{
/*printf ("Minus\n");*/
			return (Minus);
				}
\(				{
/*printf ("LeftParen\n");*/
			return (LeftParen);
				}
\)				{
/*printf ("RightParen\n");*/
			return (RightParen);
				}
\,				{
/*printf ("Comma\n");*/
			return (Comma);
				}
{register}			{
/*printf ("IntegerRegister : %s\n", yytext);*/
			sscanf (&yytext[1], "%lu", &yylval.valu);
			return (IntegerRegister);
				}
{hexadecimal_valu}		{
/*printf ("Literal (X) : %s\n", yytext);*/
 			yylval.valu = hextoint (yytext);
			return (Litteral);
				}
{octal_valu}			{
/*printf ("Literal (O) : %s\n", yytext);*/
 			yylval.valu = octtoint (yytext);
			return (Litteral);
				}
{binary_valu}			{
/*printf ("Literal (B) : %s\n", yytext);*/
 			yylval.valu = bintoint (yytext);
			return (Litteral);
				}
{decimal_valu}			{
/*printf ("Literal (D) : %s\n", yytext);*/
 			if (sscanf (yytext, "%ld", &yylval.valu) == 0)
			  yylval.valu = 0;
			return (Litteral);
				}

\".*\"				{
			extern char *dlx_allocstr ();

/*printf ("String : %s\n", yytext);*/
			yylval.text = dlx_allocstr (yytext);
			return (String);
				}
{ident}				{
			int          type           ;
			extern char *dlx_allocstr ();

			yylval.text = dlx_allocstr (yytext);

			type = dlx_searchtype (yylval.text);
			if (type == -1)
			  {
/*printf ("Identifier : %s\n", yytext);*/
			  return (Identifier);
			  }
			else
			  {
/*printf ("Key Word : %s\n", yytext);*/
			  yylval.valu = dlx_searchvalu (yylval.text);
			  return (type);
			  }
			}
\;.*$				{
/*printf ("Comment : %s\n", yytext);*/
				}
%%

static int dlx_searchtype (str)

char *str;
  {
  extern char *dlx_allocstr ()   ;
  static ht   *type_hashpt = NULL;

  if (type_hashpt == NULL)
    {
    type_hashpt = addht (128);

    addhtitem (type_hashpt, dlx_allocstr ("iar"    ), SpecialRegister);
    addhtitem (type_hashpt, dlx_allocstr ("sr"     ), SpecialRegister);
    addhtitem (type_hashpt, dlx_allocstr ("svs"    ), SpecialRegister);
    addhtitem (type_hashpt, dlx_allocstr ("tnr"    ), SpecialRegister);

    addhtitem (type_hashpt, dlx_allocstr (".align" ), _ALIGN         );
    addhtitem (type_hashpt, dlx_allocstr (".ascii" ), _ASCII         );
    addhtitem (type_hashpt, dlx_allocstr (".asciiz"), _ASCIIZ        );
    addhtitem (type_hashpt, dlx_allocstr (".byte"  ), _BYTE          );
    addhtitem (type_hashpt, dlx_allocstr (".bytez" ), _BYTEZ         );
    addhtitem (type_hashpt, dlx_allocstr (".data"  ), _DATA          );
    addhtitem (type_hashpt, dlx_allocstr (".end"   ), _END           );
    addhtitem (type_hashpt, dlx_allocstr (".equ"   ), _EQU           );
    addhtitem (type_hashpt, dlx_allocstr (".global"), _GLOBAL        );
    addhtitem (type_hashpt, dlx_allocstr (".org"   ), _ORG           );
    addhtitem (type_hashpt, dlx_allocstr (".skip"  ), _SKIP          );
    addhtitem (type_hashpt, dlx_allocstr (".space" ), _SPACE         );
    addhtitem (type_hashpt, dlx_allocstr (".start" ), _START         );
    addhtitem (type_hashpt, dlx_allocstr (".text"  ), _TEXT          );
    addhtitem (type_hashpt, dlx_allocstr (".word"  ), _WORD          );

    addhtitem (type_hashpt, dlx_allocstr ("nop"    ), Macro_n        );
    addhtitem (type_hashpt, dlx_allocstr ("loadi"  ), Macro_rd_longi );

    if (DLX_ARCHIT == DLX_MIPSDFN)
      {
      addhtitem (type_hashpt, dlx_allocstr ("add"    ), Codop_rd_rs_rt  );
      addhtitem (type_hashpt, dlx_allocstr ("addi"   ), Codop_rd_rs_imd );
      addhtitem (type_hashpt, dlx_allocstr ("addu"   ), Codop_rd_rs_rt  );
      addhtitem (type_hashpt, dlx_allocstr ("addiu"  ), Codop_rd_rs_imd );
      addhtitem (type_hashpt, dlx_allocstr ("and"    ), Codop_rd_rs_rt  );
      addhtitem (type_hashpt, dlx_allocstr ("andi"   ), Codop_rd_rs_imd );
      addhtitem (type_hashpt, dlx_allocstr ("beq"    ), Codop_rs_rt_labl);
      addhtitem (type_hashpt, dlx_allocstr ("bgez"   ), Codop_rs_labl   );
      addhtitem (type_hashpt, dlx_allocstr ("bgezal" ), Codop_rs_labl   );
      addhtitem (type_hashpt, dlx_allocstr ("bgtz"   ), Codop_rs_labl   );
      addhtitem (type_hashpt, dlx_allocstr ("blez"   ), Codop_rs_labl   );
      addhtitem (type_hashpt, dlx_allocstr ("bltz"   ), Codop_rs_labl   );
      addhtitem (type_hashpt, dlx_allocstr ("bltzal" ), Codop_rs_labl   );
      addhtitem (type_hashpt, dlx_allocstr ("bne"    ), Codop_rs_rt_labl);
      addhtitem (type_hashpt, dlx_allocstr ("break"  ), Codop_imd       );
      addhtitem (type_hashpt, dlx_allocstr ("j"      ), Codop_labl      );
      addhtitem (type_hashpt, dlx_allocstr ("jal"    ), Codop_labl      );
      addhtitem (type_hashpt, dlx_allocstr ("jalr"   ), Codop_od_rs     );
      addhtitem (type_hashpt, dlx_allocstr ("jr"     ), Codop_rs        );
      addhtitem (type_hashpt, dlx_allocstr ("lb"     ), Codop_rd_mem    );
      addhtitem (type_hashpt, dlx_allocstr ("lbu"    ), Codop_rd_mem    );
      addhtitem (type_hashpt, dlx_allocstr ("lh"     ), Codop_rd_mem    );
      addhtitem (type_hashpt, dlx_allocstr ("lhu"    ), Codop_rd_mem    );
      addhtitem (type_hashpt, dlx_allocstr ("lui"    ), Codop_rd_imd    );
      addhtitem (type_hashpt, dlx_allocstr ("lw"     ), Codop_rd_mem    );
      addhtitem (type_hashpt, dlx_allocstr ("movi2s" ), Codop_dspe_rt   );
      addhtitem (type_hashpt, dlx_allocstr ("movs2i" ), Codop_rd_sspe   );
      addhtitem (type_hashpt, dlx_allocstr ("nor"    ), Codop_rd_rs_rt  );
      addhtitem (type_hashpt, dlx_allocstr ("or"     ), Codop_rd_rs_rt  );
      addhtitem (type_hashpt, dlx_allocstr ("ori"    ), Codop_rd_rs_imd );
      addhtitem (type_hashpt, dlx_allocstr ("rfe"    ), Codop_n         );
      addhtitem (type_hashpt, dlx_allocstr ("sb"     ), Codop_rt_mem    );
      addhtitem (type_hashpt, dlx_allocstr ("sh"     ), Codop_rt_mem    );
      addhtitem (type_hashpt, dlx_allocstr ("sleep"  ), Codop_n         );
      addhtitem (type_hashpt, dlx_allocstr ("sll"    ), Codop_rd_rt_sham);
      addhtitem (type_hashpt, dlx_allocstr ("sllv"   ), Codop_rd_rt_rs  );
      addhtitem (type_hashpt, dlx_allocstr ("slt"    ), Codop_rd_rs_rt  );
      addhtitem (type_hashpt, dlx_allocstr ("slti"   ), Codop_rd_rs_imd );
      addhtitem (type_hashpt, dlx_allocstr ("sltiu"  ), Codop_rd_rs_imd );
      addhtitem (type_hashpt, dlx_allocstr ("sltu"   ), Codop_rd_rs_rt  );
      addhtitem (type_hashpt, dlx_allocstr ("sra"    ), Codop_rd_rt_sham);
      addhtitem (type_hashpt, dlx_allocstr ("srav"   ), Codop_rd_rt_rs  );
      addhtitem (type_hashpt, dlx_allocstr ("srl"    ), Codop_rd_rt_sham);
      addhtitem (type_hashpt, dlx_allocstr ("srlv"   ), Codop_rd_rt_rs  );
      addhtitem (type_hashpt, dlx_allocstr ("sub"    ), Codop_rd_rs_rt  );
      addhtitem (type_hashpt, dlx_allocstr ("subu"   ), Codop_rd_rs_rt  );
      addhtitem (type_hashpt, dlx_allocstr ("sw"     ), Codop_rt_mem    );
      addhtitem (type_hashpt, dlx_allocstr ("swap"   ), Codop_rt_mem    );
      addhtitem (type_hashpt, dlx_allocstr ("syscall"), Codop_n         );
      addhtitem (type_hashpt, dlx_allocstr ("xor"    ), Codop_rd_rs_rt  );
      addhtitem (type_hashpt, dlx_allocstr ("xori"   ), Codop_rd_rs_imd );
      }

    if (DLX_ARCHIT == DLX_DLXPDFN)
      {
      addhtitem (type_hashpt, dlx_allocstr ("add"    ), Codop_rd_rs_rt  );
      addhtitem (type_hashpt, dlx_allocstr ("addi"   ), Codop_rd_rs_imd );
      addhtitem (type_hashpt, dlx_allocstr ("addu"   ), Codop_rd_rs_rt  );
      addhtitem (type_hashpt, dlx_allocstr ("addiu"  ), Codop_rd_rs_imd );
      addhtitem (type_hashpt, dlx_allocstr ("and"    ), Codop_rd_rs_rt  );
      addhtitem (type_hashpt, dlx_allocstr ("andi"   ), Codop_rd_rs_imd );
      addhtitem (type_hashpt, dlx_allocstr ("beqz"   ), Codop_rs_labl   );
      addhtitem (type_hashpt, dlx_allocstr ("bnez"   ), Codop_rs_labl   );
      addhtitem (type_hashpt, dlx_allocstr ("j"      ), Codop_labl      );
      addhtitem (type_hashpt, dlx_allocstr ("jal"    ), Codop_labl      );
      addhtitem (type_hashpt, dlx_allocstr ("jalr"   ), Codop_rs        );
      addhtitem (type_hashpt, dlx_allocstr ("jr"     ), Codop_rs        );
      addhtitem (type_hashpt, dlx_allocstr ("lb"     ), Codop_rd_mem    );
      addhtitem (type_hashpt, dlx_allocstr ("lbu"    ), Codop_rd_mem    );
      addhtitem (type_hashpt, dlx_allocstr ("lh"     ), Codop_rd_mem    );
      addhtitem (type_hashpt, dlx_allocstr ("lhi"    ), Codop_rd_imd    );
      addhtitem (type_hashpt, dlx_allocstr ("lhu"    ), Codop_rd_mem    );
      addhtitem (type_hashpt, dlx_allocstr ("lw"     ), Codop_rd_mem    );
      addhtitem (type_hashpt, dlx_allocstr ("movi2s" ), Codop_dspe_rt   );
      addhtitem (type_hashpt, dlx_allocstr ("movs2i" ), Codop_rd_sspe   );
      addhtitem (type_hashpt, dlx_allocstr ("or"     ), Codop_rd_rs_rt  );
      addhtitem (type_hashpt, dlx_allocstr ("ori"    ), Codop_rd_rs_imd );
      addhtitem (type_hashpt, dlx_allocstr ("rfe"    ), Codop_n         );
      addhtitem (type_hashpt, dlx_allocstr ("sb"     ), Codop_rt_mem    );
      addhtitem (type_hashpt, dlx_allocstr ("seq"    ), Codop_rd_rs_rt  );
      addhtitem (type_hashpt, dlx_allocstr ("seqi"   ), Codop_rd_rs_imd );
      addhtitem (type_hashpt, dlx_allocstr ("sne"    ), Codop_rd_rs_rt  );
      addhtitem (type_hashpt, dlx_allocstr ("snei"   ), Codop_rd_rs_imd );
      addhtitem (type_hashpt, dlx_allocstr ("sge"    ), Codop_rd_rs_rt  );
      addhtitem (type_hashpt, dlx_allocstr ("sgei"   ), Codop_rd_rs_imd );
      addhtitem (type_hashpt, dlx_allocstr ("sgeu"   ), Codop_rd_rs_rt  );
      addhtitem (type_hashpt, dlx_allocstr ("sgeiu"  ), Codop_rd_rs_imd );
      addhtitem (type_hashpt, dlx_allocstr ("sgt"    ), Codop_rd_rs_rt  );
      addhtitem (type_hashpt, dlx_allocstr ("sgti"   ), Codop_rd_rs_imd );
      addhtitem (type_hashpt, dlx_allocstr ("sgtu"   ), Codop_rd_rs_rt  );
      addhtitem (type_hashpt, dlx_allocstr ("sgtiu"  ), Codop_rd_rs_imd );
      addhtitem (type_hashpt, dlx_allocstr ("sh"     ), Codop_rt_mem    );
      addhtitem (type_hashpt, dlx_allocstr ("sle"    ), Codop_rd_rs_rt  );
      addhtitem (type_hashpt, dlx_allocstr ("sleep"  ), Codop_n         );
      addhtitem (type_hashpt, dlx_allocstr ("slei"   ), Codop_rd_rs_imd );
      addhtitem (type_hashpt, dlx_allocstr ("sleu"   ), Codop_rd_rs_rt  );
      addhtitem (type_hashpt, dlx_allocstr ("sleiu"  ), Codop_rd_rs_imd );
      addhtitem (type_hashpt, dlx_allocstr ("sll"    ), Codop_rd_rt_rs  );
      addhtitem (type_hashpt, dlx_allocstr ("slli"   ), Codop_rd_rt_sham);
      addhtitem (type_hashpt, dlx_allocstr ("slt"    ), Codop_rd_rs_rt  );
      addhtitem (type_hashpt, dlx_allocstr ("slti"   ), Codop_rd_rs_imd );
      addhtitem (type_hashpt, dlx_allocstr ("sltu"   ), Codop_rd_rs_rt  );
      addhtitem (type_hashpt, dlx_allocstr ("sltiu"  ), Codop_rd_rs_imd );
      addhtitem (type_hashpt, dlx_allocstr ("sra"    ), Codop_rd_rt_rs  );
      addhtitem (type_hashpt, dlx_allocstr ("srai"   ), Codop_rd_rt_sham);
      addhtitem (type_hashpt, dlx_allocstr ("srl"    ), Codop_rd_rt_rs  );
      addhtitem (type_hashpt, dlx_allocstr ("srli"   ), Codop_rd_rt_sham);
      addhtitem (type_hashpt, dlx_allocstr ("sub"    ), Codop_rd_rs_rt  );
      addhtitem (type_hashpt, dlx_allocstr ("subi"   ), Codop_rd_rs_imd );
      addhtitem (type_hashpt, dlx_allocstr ("subu"   ), Codop_rd_rs_rt  );
      addhtitem (type_hashpt, dlx_allocstr ("subiu"  ), Codop_rd_rs_imd );
      addhtitem (type_hashpt, dlx_allocstr ("sw"     ), Codop_rt_mem    );
      addhtitem (type_hashpt, dlx_allocstr ("swap"   ), Codop_rt_mem    );
      addhtitem (type_hashpt, dlx_allocstr ("trap"   ), Codop_imd       );
      addhtitem (type_hashpt, dlx_allocstr ("xor"    ), Codop_rd_rs_rt  );
      addhtitem (type_hashpt, dlx_allocstr ("xori"   ), Codop_rd_rs_imd );
      }

    if (DLX_ARCHIT == DLX_DLXMDFN)
      {
      addhtitem (type_hashpt, dlx_allocstr ("add"    ), Codop_rd_rs_rt  );
      addhtitem (type_hashpt, dlx_allocstr ("addi"   ), Codop_rd_rs_imd );
      addhtitem (type_hashpt, dlx_allocstr ("and"    ), Codop_rd_rs_rt  );
      addhtitem (type_hashpt, dlx_allocstr ("andi"   ), Codop_rd_rs_imd );
      addhtitem (type_hashpt, dlx_allocstr ("beqz"   ), Codop_rs_labl   );
      addhtitem (type_hashpt, dlx_allocstr ("bnez"   ), Codop_rs_labl   );
      addhtitem (type_hashpt, dlx_allocstr ("j"      ), Codop_labl      );
      addhtitem (type_hashpt, dlx_allocstr ("jal"    ), Codop_labl      );
      addhtitem (type_hashpt, dlx_allocstr ("jalr"   ), Codop_rs        );
      addhtitem (type_hashpt, dlx_allocstr ("jr"     ), Codop_rs        );
      addhtitem (type_hashpt, dlx_allocstr ("lbu"    ), Codop_rd_mem    );
      addhtitem (type_hashpt, dlx_allocstr ("lhi"    ), Codop_rd_imd    );
      addhtitem (type_hashpt, dlx_allocstr ("lw"     ), Codop_rd_mem    );
      addhtitem (type_hashpt, dlx_allocstr ("movi2s" ), Codop_dspe_rt   );
      addhtitem (type_hashpt, dlx_allocstr ("movs2i" ), Codop_rd_sspe   );
      addhtitem (type_hashpt, dlx_allocstr ("or"     ), Codop_rd_rs_rt  );
      addhtitem (type_hashpt, dlx_allocstr ("ori"    ), Codop_rd_rs_imd );
      addhtitem (type_hashpt, dlx_allocstr ("rfe"    ), Codop_n         );
      addhtitem (type_hashpt, dlx_allocstr ("sb"     ), Codop_rt_mem    );
      addhtitem (type_hashpt, dlx_allocstr ("seq"    ), Codop_rd_rs_rt  );
      addhtitem (type_hashpt, dlx_allocstr ("seqi"   ), Codop_rd_rs_imd );
      addhtitem (type_hashpt, dlx_allocstr ("sne"    ), Codop_rd_rs_rt  );
      addhtitem (type_hashpt, dlx_allocstr ("snei"   ), Codop_rd_rs_imd );
      addhtitem (type_hashpt, dlx_allocstr ("sge"    ), Codop_rd_rs_rt  );
      addhtitem (type_hashpt, dlx_allocstr ("sgei"   ), Codop_rd_rs_imd );
      addhtitem (type_hashpt, dlx_allocstr ("sgt"    ), Codop_rd_rs_rt  );
      addhtitem (type_hashpt, dlx_allocstr ("sgti"   ), Codop_rd_rs_imd );
      addhtitem (type_hashpt, dlx_allocstr ("sle"    ), Codop_rd_rs_rt  );
      addhtitem (type_hashpt, dlx_allocstr ("slei"   ), Codop_rd_rs_imd );
      addhtitem (type_hashpt, dlx_allocstr ("sll"    ), Codop_rd_rt_rs  );
      addhtitem (type_hashpt, dlx_allocstr ("slli"   ), Codop_rd_rt_sham);
      addhtitem (type_hashpt, dlx_allocstr ("slt"    ), Codop_rd_rs_rt  );
      addhtitem (type_hashpt, dlx_allocstr ("slti"   ), Codop_rd_rs_imd );
      addhtitem (type_hashpt, dlx_allocstr ("sra"    ), Codop_rd_rt_rs  );
      addhtitem (type_hashpt, dlx_allocstr ("srai"   ), Codop_rd_rt_sham);
      addhtitem (type_hashpt, dlx_allocstr ("srl"    ), Codop_rd_rt_rs  );
      addhtitem (type_hashpt, dlx_allocstr ("srli"   ), Codop_rd_rt_sham);
      addhtitem (type_hashpt, dlx_allocstr ("sub"    ), Codop_rd_rs_rt  );
      addhtitem (type_hashpt, dlx_allocstr ("subi"   ), Codop_rd_rs_imd );
      addhtitem (type_hashpt, dlx_allocstr ("sw"     ), Codop_rt_mem    );
      addhtitem (type_hashpt, dlx_allocstr ("trap"   ), Codop_imd       );
      addhtitem (type_hashpt, dlx_allocstr ("xor"    ), Codop_rd_rs_rt  );
      addhtitem (type_hashpt, dlx_allocstr ("xori"   ), Codop_rd_rs_imd );
      }

    }
  return (gethtitem (type_hashpt, str));
  }

static int dlx_searchvalu (str)

char *str;
  {
  extern char *dlx_allocstr ()   ;
  static ht   *valu_hashpt = NULL;

  if (valu_hashpt == NULL)
    {
    valu_hashpt = addht (128);

    addhtitem (valu_hashpt, dlx_allocstr ("iar"    ), _IAR    );
    addhtitem (valu_hashpt, dlx_allocstr ("sr"     ), _SR     );
    addhtitem (valu_hashpt, dlx_allocstr ("svs"    ), _SVS    );
    addhtitem (valu_hashpt, dlx_allocstr ("tnr"    ), _TNR    );

    addhtitem (valu_hashpt, dlx_allocstr (".align" ), _ALIGN  );
    addhtitem (valu_hashpt, dlx_allocstr (".ascii" ), _ASCII  );
    addhtitem (valu_hashpt, dlx_allocstr (".asciiz"), _ASCIIZ );
    addhtitem (valu_hashpt, dlx_allocstr (".byte"  ), _BYTE   );
    addhtitem (valu_hashpt, dlx_allocstr (".bytez" ), _BYTEZ  );
    addhtitem (valu_hashpt, dlx_allocstr (".data"  ), _DATA   );
    addhtitem (valu_hashpt, dlx_allocstr (".end"   ), _END    );
    addhtitem (valu_hashpt, dlx_allocstr (".equ"   ), _EQU    );
    addhtitem (valu_hashpt, dlx_allocstr (".global"), _GLOBAL );
    addhtitem (valu_hashpt, dlx_allocstr (".org"   ), _ORG    );
    addhtitem (valu_hashpt, dlx_allocstr (".skip"  ), _SKIP   );
    addhtitem (valu_hashpt, dlx_allocstr (".space" ), _SPACE  );
    addhtitem (valu_hashpt, dlx_allocstr (".start" ), _START  );
    addhtitem (valu_hashpt, dlx_allocstr (".text"  ), _TEXT   );
    addhtitem (valu_hashpt, dlx_allocstr (".word"  ), _WORD   );

    addhtitem (valu_hashpt, dlx_allocstr ("nop"    ), _NOP    );
    addhtitem (valu_hashpt, dlx_allocstr ("loadi"  ), _LOADI  );

    if (DLX_ARCHIT == DLX_MIPSDFN)
      {
      addhtitem (valu_hashpt, dlx_allocstr ("add"    ), _ADD    );
      addhtitem (valu_hashpt, dlx_allocstr ("addi"   ), _ADDI   );
      addhtitem (valu_hashpt, dlx_allocstr ("addu"   ), _ADDU   );
      addhtitem (valu_hashpt, dlx_allocstr ("addiu"  ), _ADDIU  );
      addhtitem (valu_hashpt, dlx_allocstr ("and"    ), _AND    );
      addhtitem (valu_hashpt, dlx_allocstr ("andi"   ), _ANDI   );
      addhtitem (valu_hashpt, dlx_allocstr ("beq"    ), _BEQ    );
      addhtitem (valu_hashpt, dlx_allocstr ("bgez"   ), _BGEZ   );
      addhtitem (valu_hashpt, dlx_allocstr ("bgezal" ), _BGEZAL );
      addhtitem (valu_hashpt, dlx_allocstr ("bgtz"   ), _BGTZ   );
      addhtitem (valu_hashpt, dlx_allocstr ("blez"   ), _BLEZ   );
      addhtitem (valu_hashpt, dlx_allocstr ("bltz"   ), _BLTZ   );
      addhtitem (valu_hashpt, dlx_allocstr ("bltzal" ), _BLTZAL );
      addhtitem (valu_hashpt, dlx_allocstr ("bne"    ), _BNE    );
      addhtitem (valu_hashpt, dlx_allocstr ("break"  ), _BREAK  );
      addhtitem (valu_hashpt, dlx_allocstr ("j"      ), _J      );
      addhtitem (valu_hashpt, dlx_allocstr ("jal"    ), _JAL    );
      addhtitem (valu_hashpt, dlx_allocstr ("jalr"   ), _JALR   );
      addhtitem (valu_hashpt, dlx_allocstr ("jr"     ), _JR     );
      addhtitem (valu_hashpt, dlx_allocstr ("lb"     ), _LB     );
      addhtitem (valu_hashpt, dlx_allocstr ("lbu"    ), _LBU    );
      addhtitem (valu_hashpt, dlx_allocstr ("lh"     ), _LH     );
      addhtitem (valu_hashpt, dlx_allocstr ("lhu"    ), _LHU    );
      addhtitem (valu_hashpt, dlx_allocstr ("lui"    ), _LUI    );
      addhtitem (valu_hashpt, dlx_allocstr ("lw"     ), _LW     );
      addhtitem (valu_hashpt, dlx_allocstr ("movi2s" ), _MOVI2S );
      addhtitem (valu_hashpt, dlx_allocstr ("movs2i" ), _MOVS2I );
      addhtitem (valu_hashpt, dlx_allocstr ("nor"    ), _NOR    );
      addhtitem (valu_hashpt, dlx_allocstr ("or"     ), _OR     );
      addhtitem (valu_hashpt, dlx_allocstr ("ori"    ), _ORI    );
      addhtitem (valu_hashpt, dlx_allocstr ("rfe"    ), _RSTFE  );
      addhtitem (valu_hashpt, dlx_allocstr ("sb"     ), _SB     );
      addhtitem (valu_hashpt, dlx_allocstr ("sh"     ), _SH     );
      addhtitem (valu_hashpt, dlx_allocstr ("sleep"  ), _SLEEP  );
      addhtitem (valu_hashpt, dlx_allocstr ("sll"    ), _SLL    );
      addhtitem (valu_hashpt, dlx_allocstr ("sllv"   ), _SLLV   );
      addhtitem (valu_hashpt, dlx_allocstr ("slt"    ), _SLT    );
      addhtitem (valu_hashpt, dlx_allocstr ("slti"   ), _SLTI   );
      addhtitem (valu_hashpt, dlx_allocstr ("sltiu"  ), _SLTIU  );
      addhtitem (valu_hashpt, dlx_allocstr ("sltu"   ), _SLTU   );
      addhtitem (valu_hashpt, dlx_allocstr ("sra"    ), _SRA    );
      addhtitem (valu_hashpt, dlx_allocstr ("srav"   ), _SRAV   );
      addhtitem (valu_hashpt, dlx_allocstr ("srl"    ), _SRL    );
      addhtitem (valu_hashpt, dlx_allocstr ("srlv"   ), _SRLV   );
      addhtitem (valu_hashpt, dlx_allocstr ("sub"    ), _SUB    );
      addhtitem (valu_hashpt, dlx_allocstr ("subu"   ), _SUBU   );
      addhtitem (valu_hashpt, dlx_allocstr ("sw"     ), _SW     );
      addhtitem (valu_hashpt, dlx_allocstr ("swap"   ), _SWAP   );
      addhtitem (valu_hashpt, dlx_allocstr ("syscall"), _SYSCALL);
      addhtitem (valu_hashpt, dlx_allocstr ("xor"    ), _XOR    );
      addhtitem (valu_hashpt, dlx_allocstr ("xori"   ), _XORI   );
      }

    if (DLX_ARCHIT == DLX_DLXPDFN)
      {
      addhtitem (valu_hashpt, dlx_allocstr ("add"    ), _ADD    );
      addhtitem (valu_hashpt, dlx_allocstr ("addi"   ), _ADDI   );
      addhtitem (valu_hashpt, dlx_allocstr ("addu"   ), _ADDU   );
      addhtitem (valu_hashpt, dlx_allocstr ("addiu"  ), _ADDIU  );
      addhtitem (valu_hashpt, dlx_allocstr ("and"    ), _AND    );
      addhtitem (valu_hashpt, dlx_allocstr ("andi"   ), _ANDI   );
      addhtitem (valu_hashpt, dlx_allocstr ("beqz"   ), _BEQZ   );
      addhtitem (valu_hashpt, dlx_allocstr ("bnez"   ), _BNEZ   );
      addhtitem (valu_hashpt, dlx_allocstr ("j"      ), _J      );
      addhtitem (valu_hashpt, dlx_allocstr ("jal"    ), _JAL    );
      addhtitem (valu_hashpt, dlx_allocstr ("jalr"   ), _JALR31 );
      addhtitem (valu_hashpt, dlx_allocstr ("jr"     ), _JR     );
      addhtitem (valu_hashpt, dlx_allocstr ("lb"     ), _LB     );
      addhtitem (valu_hashpt, dlx_allocstr ("lbu"    ), _LBU    );
      addhtitem (valu_hashpt, dlx_allocstr ("lh"     ), _LH     );
      addhtitem (valu_hashpt, dlx_allocstr ("lhi"    ), _LUI    );
      addhtitem (valu_hashpt, dlx_allocstr ("lhu"    ), _LHU    );
      addhtitem (valu_hashpt, dlx_allocstr ("lw"     ), _LW     );
      addhtitem (valu_hashpt, dlx_allocstr ("movi2s" ), _MOVI2S );
      addhtitem (valu_hashpt, dlx_allocstr ("movs2i" ), _MOVS2I );
      addhtitem (valu_hashpt, dlx_allocstr ("or"     ), _OR     );
      addhtitem (valu_hashpt, dlx_allocstr ("ori"    ), _ORI    );
      addhtitem (valu_hashpt, dlx_allocstr ("rfe"    ), _RETFE  );
      addhtitem (valu_hashpt, dlx_allocstr ("sb"     ), _SB     );
      addhtitem (valu_hashpt, dlx_allocstr ("seq"    ), _SEQ    );
      addhtitem (valu_hashpt, dlx_allocstr ("seqi"   ), _SEQI   );
      addhtitem (valu_hashpt, dlx_allocstr ("sne"    ), _SNE    );
      addhtitem (valu_hashpt, dlx_allocstr ("snei"   ), _SNEI   );
      addhtitem (valu_hashpt, dlx_allocstr ("sge"    ), _SGE    );
      addhtitem (valu_hashpt, dlx_allocstr ("sgei"   ), _SGEI   );
      addhtitem (valu_hashpt, dlx_allocstr ("sgeu"   ), _SGEU   );
      addhtitem (valu_hashpt, dlx_allocstr ("sgeiu"  ), _SGEIU  );
      addhtitem (valu_hashpt, dlx_allocstr ("sgt"    ), _SGT    );
      addhtitem (valu_hashpt, dlx_allocstr ("sgti"   ), _SGTI   );
      addhtitem (valu_hashpt, dlx_allocstr ("sgtu"   ), _SGTU   );
      addhtitem (valu_hashpt, dlx_allocstr ("sgtiu"  ), _SGTIU  );
      addhtitem (valu_hashpt, dlx_allocstr ("sh"     ), _SH     );
      addhtitem (valu_hashpt, dlx_allocstr ("sle"    ), _SLE    );
      addhtitem (valu_hashpt, dlx_allocstr ("sleep"  ), _SLEEP  );
      addhtitem (valu_hashpt, dlx_allocstr ("slei"   ), _SLEI   );
      addhtitem (valu_hashpt, dlx_allocstr ("sleu"   ), _SLEU   );
      addhtitem (valu_hashpt, dlx_allocstr ("sleiu"  ), _SLEIU  );
      addhtitem (valu_hashpt, dlx_allocstr ("sll"    ), _SLLV   );
      addhtitem (valu_hashpt, dlx_allocstr ("slli"   ), _SLL    );
      addhtitem (valu_hashpt, dlx_allocstr ("slt"    ), _SLT    );
      addhtitem (valu_hashpt, dlx_allocstr ("slti"   ), _SLTI   );
      addhtitem (valu_hashpt, dlx_allocstr ("sltu"   ), _SLTU   );
      addhtitem (valu_hashpt, dlx_allocstr ("sltiu"  ), _SLTIU  );
      addhtitem (valu_hashpt, dlx_allocstr ("sra"    ), _SRAV   );
      addhtitem (valu_hashpt, dlx_allocstr ("srai"   ), _SRA    );
      addhtitem (valu_hashpt, dlx_allocstr ("srl"    ), _SRLV   );
      addhtitem (valu_hashpt, dlx_allocstr ("srli"   ), _SRL    );
      addhtitem (valu_hashpt, dlx_allocstr ("sub"    ), _SUB    );
      addhtitem (valu_hashpt, dlx_allocstr ("subi"   ), _SUBI   );
      addhtitem (valu_hashpt, dlx_allocstr ("subu"   ), _SUBU   );
      addhtitem (valu_hashpt, dlx_allocstr ("subiu"  ), _SUBIU  );
      addhtitem (valu_hashpt, dlx_allocstr ("sw"     ), _SW     );
      addhtitem (valu_hashpt, dlx_allocstr ("swap"   ), _SWAP   );
      addhtitem (valu_hashpt, dlx_allocstr ("trap"   ), _TRAP   );
      addhtitem (valu_hashpt, dlx_allocstr ("xor"    ), _XOR    );
      addhtitem (valu_hashpt, dlx_allocstr ("xori"   ), _XORI   );
      }

    if (DLX_ARCHIT == DLX_DLXMDFN)
      {
      addhtitem (valu_hashpt, dlx_allocstr ("add"    ), _ADD    );
      addhtitem (valu_hashpt, dlx_allocstr ("addi"   ), _ADDI   );
      addhtitem (valu_hashpt, dlx_allocstr ("and"    ), _AND    );
      addhtitem (valu_hashpt, dlx_allocstr ("andi"   ), _ANDI   );
      addhtitem (valu_hashpt, dlx_allocstr ("beqz"   ), _BEQZ   );
      addhtitem (valu_hashpt, dlx_allocstr ("bnez"   ), _BNEZ   );
      addhtitem (valu_hashpt, dlx_allocstr ("j"      ), _J      );
      addhtitem (valu_hashpt, dlx_allocstr ("jal"    ), _JAL    );
      addhtitem (valu_hashpt, dlx_allocstr ("jalr"   ), _JALR31 );
      addhtitem (valu_hashpt, dlx_allocstr ("jr"     ), _JR     );
      addhtitem (valu_hashpt, dlx_allocstr ("lbu"    ), _LBU    );
      addhtitem (valu_hashpt, dlx_allocstr ("lhi"    ), _LUI    );
      addhtitem (valu_hashpt, dlx_allocstr ("lw"     ), _LW     );
      addhtitem (valu_hashpt, dlx_allocstr ("movi2s" ), _MOVI2S );
      addhtitem (valu_hashpt, dlx_allocstr ("movs2i" ), _MOVS2I );
      addhtitem (valu_hashpt, dlx_allocstr ("or"     ), _OR     );
      addhtitem (valu_hashpt, dlx_allocstr ("ori"    ), _ORI    );
      addhtitem (valu_hashpt, dlx_allocstr ("rfe"    ), _RETFE  );
      addhtitem (valu_hashpt, dlx_allocstr ("sb"     ), _SB     );
      addhtitem (valu_hashpt, dlx_allocstr ("seq"    ), _SEQ    );
      addhtitem (valu_hashpt, dlx_allocstr ("seqi"   ), _SEQI   );
      addhtitem (valu_hashpt, dlx_allocstr ("sne"    ), _SNE    );
      addhtitem (valu_hashpt, dlx_allocstr ("snei"   ), _SNEI   );
      addhtitem (valu_hashpt, dlx_allocstr ("sge"    ), _SGE    );
      addhtitem (valu_hashpt, dlx_allocstr ("sgei"   ), _SGEI   );
      addhtitem (valu_hashpt, dlx_allocstr ("sgt"    ), _SGT    );
      addhtitem (valu_hashpt, dlx_allocstr ("sgti"   ), _SGTI   );
      addhtitem (valu_hashpt, dlx_allocstr ("sle"    ), _SLE    );
      addhtitem (valu_hashpt, dlx_allocstr ("slei"   ), _SLEI   );
      addhtitem (valu_hashpt, dlx_allocstr ("sll"    ), _SLLV   );
      addhtitem (valu_hashpt, dlx_allocstr ("slli"   ), _SLL    );
      addhtitem (valu_hashpt, dlx_allocstr ("slt"    ), _SLT    );
      addhtitem (valu_hashpt, dlx_allocstr ("slti"   ), _SLTI   );
      addhtitem (valu_hashpt, dlx_allocstr ("sra"    ), _SRAV   );
      addhtitem (valu_hashpt, dlx_allocstr ("srai"   ), _SRA    );
      addhtitem (valu_hashpt, dlx_allocstr ("srl"    ), _SRLV   );
      addhtitem (valu_hashpt, dlx_allocstr ("srli"   ), _SRL    );
      addhtitem (valu_hashpt, dlx_allocstr ("sub"    ), _SUB    );
      addhtitem (valu_hashpt, dlx_allocstr ("subi"   ), _SUBI   );
      addhtitem (valu_hashpt, dlx_allocstr ("sw"     ), _SW     );
      addhtitem (valu_hashpt, dlx_allocstr ("trap"   ), _TRAP   );
      addhtitem (valu_hashpt, dlx_allocstr ("xor"    ), _XOR    );
      addhtitem (valu_hashpt, dlx_allocstr ("xori"   ), _XORI   );
      }

    }
  return (gethtitem (valu_hashpt, str));
  }

static unsigned long hextoint (str)

char *str;

  {
  unsigned long valu = 0;
  char          c    = 0;

  while ((c = *str) != '\0')
    {
    if ((c >= '0') && (c <= '9'))
      valu = (valu << 4) + c - '0';
    else
      if ((c >= 'a') && (c <= 'f'))
        valu = (valu << 4) + c - 'a' + 10;
      else
        if ((c >= 'A') && (c <= 'F'))
          valu = (valu << 4) + c - 'A' + 10;
    str++;
    }

  return (valu);
  }

static unsigned long octtoint (str)

char *str;

  {
  unsigned long valu = 0;
  char          c    = 0;

  while ((c = *str) != '\0')
    {
    if ((c >= '0') && (c <= '7'))
      valu = (valu << 3) + c - '0';

    str++;
    }

  return (valu);
  }

static unsigned long bintoint (str)

char *str;

  {
  unsigned long valu = 0;
  char          c    = 0;

  while ((c = *str) != '\0')
    {
    if ((c == '0') || (c == '1'))
      valu = (valu << 1) + c - '0';

    str++;
    }

  return (valu);
  }

int yywrap ()
  {
  return (1);
  }
