/*------------------------------------------------------------\
|                                                             |
| This file is part of the Alliance CAD System Copyright      |
| (C) Laboratoire LIP6 - Dpartement ASIM Universite P&M Curie|
|                                                             |
| Home page      : http://www-asim.lip6.fr/alliance/          |
| E-mail support : mailto:alliance-support@asim.lip6.fr       |
|                                                             |
| This progam is  free software; you can redistribute it      |
| and/or modify it under the  terms of the GNU Library General|
| Public License as published by the Free Software Foundation |
| either version 2 of the License, or (at your option) any    |
| later version.                                              |
|                                                             |
| Alliance VLSI  CAD System  is distributed  in the hope that |
| it  will be useful, but WITHOUT  ANY WARRANTY;              |
| without even the  implied warranty of MERCHANTABILITY or    |
| FITNESS FOR A PARTICULAR PURPOSE. See the GNU General       |
| Public License for more details.                            |
|                                                             |
| You should have received a copy  of the GNU General Public  |
| License along with the GNU C Library; see the file COPYING. |
| If not, write to the Free Software Foundation, Inc.,        |
| 675 Mass Ave, Cambridge, MA 02139, USA.                     |
|                                                             |
\------------------------------------------------------------*/
/*------------------------------------------------------\
|                                                       |
|  Title   :             C4_layout											|
|                                                       |
|  Date    :             30.10.01                       |
|                                                       |
|  Authors :           Greiner Alain                    |
|                                                       |
\------------------------------------------------------*/

/****************************************************************/
/*	fonction C4_layout()              			*/
/*	                                	 		*/
/*	construit la phfig de la cellule au gabarit SXLIB	*/
/*	hauteur de la cellule 50 lambdas         		*/
/*	les pistes 5 et 45 sont utilisees pour les prises	*/
/*	d'alimentation						*/
/*	Les pistes 10/15/20 sont utilisees pour les segments	*/
/*	horizontaux de la zone N				*/
/*	Les pistes 30/35/40 sont utilisees pour les segments	*/
/*	horizontaux de la zone P				*/
/*	La piste 25 est utilisee pour les contacts POLY		*/
/*	Le parametre "type" definit le buffer:			*/
/*		type = 0 	buf_x2				*/
/*		type = 1 	buf_x4				*/
/*		type = 2 	inv_x2				*/
/*		type = 3 	inv_x4				*/
/*	Le parametre "pr" est un pointeur sur une structure 	*/
/*	C4_fig_route contenant une liste d'instances a creer	*/
/*	et la liste des segments horizontaux			*/
/****************************************************************/

#include "C4_layout.h"
#include "genlib.h"

phfig_list *C4_layout(type,pr)
int		type;
C4_fig_route	*pr;

{
C4_instance_list *pi;
C4_segment_list *ps;
int	reftab[9] = {45,40,35,30,25,20,15,10,5};
int	x1,x2,y1,y2,w;
int	i,min,max;

GENLIB_DEF_PHFIG(pr->NOM);

//////////////////////////////////////////////////////////////
//           placement des instances
//////////////////////////////////////////////////////////////

switch(type) {
	case 0 :
	GENLIB_PLACE("c4_buf_x2","ins0",NOSYM,0,0);
	break;
        case 1 :
	GENLIB_PLACE("c4_buf_x4","ins0",NOSYM,0,0);
	break;
	case 2 :
	GENLIB_PLACE("c4_inv_x2","ins0",NOSYM,0,0);
	break;
	case 3 :
	GENLIB_PLACE("c4_inv_x4","ins0",NOSYM,0,0);
	break;
	default : {
	printf("The buffer type is not correct\n"); 
	printf("in the C4_layout() function\n");
	exit(1); }
	}
x1 = 0;
for(pi=pr->INS->NEXT ; pi ;pi=pi->NEXT) {
	if(strcmp(pi->NOM,C4_FICTIVE))
	GENLIB_PLACE_RIGHT("c4_in",GENLIB_NAME("ins%d",pi->INDEX),NOSYM);
	else
	GENLIB_PLACE_RIGHT("c4_tr",GENLIB_NAME("ins%d",pi->INDEX),NOSYM);
	x1 = GENLIB_GET_REF_X(GENLIB_NAME("ins%d",pi->INDEX),"in_25");
	}
x2 = x1 % 5;
if(x2 != 0) x2 = 5 - x2;
GENLIB_DEF_AB(0,0,x2,0);


////////////////////////////////////////////////////////////
//    placement des contacts DIF ainsi que des extensions
////////////////////////////////////////////////////////////

for(pi=pr->INS->NEXT ; pi ;pi=pi->NEXT) {
	/* contacts DIFP */
	for(i=0 ; i<=3 ; i++) {
	if(pi->VIA[i] != 0) {
	x1 = GENLIB_GET_REF_X(GENLIB_NAME("ins%d",pi->INDEX),GENLIB_NAME("grid_%d",reftab[i]));
	y1 = GENLIB_GET_REF_Y(GENLIB_NAME("ins%d",pi->INDEX),GENLIB_NAME("grid_%d",reftab[i]));
	GENLIB_PHVIA(CONT_DIF_P,x1,y1);}
        }
	/* contacts DIFN */
	for(i=5 ; i<=8 ; i++) {
	if(pi->VIA[i] != 0) {
	x1 = GENLIB_GET_REF_X(GENLIB_NAME("ins%d",pi->INDEX),GENLIB_NAME("grid_%d",reftab[i]));
	y1 = GENLIB_GET_REF_Y(GENLIB_NAME("ins%d",pi->INDEX),GENLIB_NAME("grid_%d",reftab[i]));
	GENLIB_PHVIA(CONT_DIF_N,x1,y1);
	if((y1 == 15) || (y1 == 20)) GENLIB_PHSEG(NDIF,3,"",x1,10,x1,y1);}
	}}


////////////////////////////////////////////////////////////
//    placement des segments ALU1 horizontaux
////////////////////////////////////////////////////////////

for(ps=pr->SEG ; ps ; ps=ps->NEXT) {
	x1 = GENLIB_GET_REF_X(GENLIB_NAME("ins%d",ps->REF1->INDEX),
			GENLIB_NAME("grid_%d",reftab[ps->INDEX1]));
	y1 = GENLIB_GET_REF_Y(GENLIB_NAME("ins%d",ps->REF1->INDEX),
			GENLIB_NAME("grid_%d",reftab[ps->INDEX1]));
	x2 = GENLIB_GET_REF_X(GENLIB_NAME("ins%d",ps->REF2->INDEX),
			GENLIB_NAME("grid_%d",reftab[ps->INDEX2]));
	y2 = GENLIB_GET_REF_Y(GENLIB_NAME("ins%d",ps->REF2->INDEX),
			GENLIB_NAME("grid_%d",reftab[ps->INDEX2]));
	if((y1 != y2) || (y1 == 5) || (y1 == 25) || (y1 == 45)) {
	printf("Error in the segment list in function C4_layout() :\n");
	printf("all segments must be horizontal and located on\n");
	printf("tracks 10 /15 / 20 / 30 / 35 / 40\n");
	exit(1);}
	GENLIB_PHSEG(ALU1,1,ps->SIG->NAMECHAIN->DATA,x1,y1,x2,y2);
        }

////////////////////////////////////////////////////////////
//    placement des contacts POLY ainsi que des 
//    connecteurs ALU1 verticaux et references d'entree
////////////////////////////////////////////////////////////

for(pi=pr->INS->NEXT ; pi ; pi = pi->NEXT) {
        if(strcmp(pi->NOM,C4_FICTIVE)) {
	if(pi->NEXT == NULL) {
	printf("Error in the instance list in function C4_layout() :\n");
	printf("the last instance should be a cut\n");
	exit(1);}
	/* calcul ordonnees min et max du segment vertical */
        min = 4;
        max = 4;
	for(i=4 ; i>0 ; i--) {
		if((pi->SIG[i] == NULL) && (pi->NEXT->SIG[i] == NULL))
		min =i;else i=0;}
	for(i=4 ; i<8 ; i++) {
		if((pi->SIG[i] == NULL) && (pi->NEXT->SIG[i] == NULL)) 
		max =i;else i=8;}
	/* calcul abcisse x2 contact POLY et segmemt vertical */
	x1 = GENLIB_GET_REF_X(GENLIB_NAME("ins%d",pi->INDEX),"in_25");
	y2 = GENLIB_GET_REF_Y(GENLIB_NAME("ins%d",pi->INDEX),"in_25");
	switch(x1%5) {
		case 0 :
		x2 = x1;
		break;
		case 1 :
		x2 = x1-1;
		break;
		case 2 :
		x2 = x1-2;
		GENLIB_PHSEG(POLY,3,"",x1,y2,x2,y2);
		break;
		case 3 :
		x2 = x1+2;
		GENLIB_PHSEG(POLY,3,"",x1,y2,x2,y2);
		break;
		case 4 :
		x2 = x1+1;
		break;
		}
	GENLIB_PHVIA(CONT_POLY,x2,y2);
	if(min != max) GENLIB_PHSEG(CALU1,1,pi->NOM,x2,reftab[min],x2,reftab[max]);
	for(i=min ; i<=max ; i++) {
	GENLIB_PHREF("REF_REF",GENLIB_NAME("%s_%d",pi->NOM,
			reftab[i]),x2,reftab[i]);}
	}}


////////////////////////////////////////////////////////////
//    placement du connecteur et des references de sortie
////////////////////////////////////////////////////////////


// MAJ VINCENT
//GENLIB_COPY_UP_SEG("out","ins0","out");
GENLIB_COPY_UP_SEG("out","ins0",pr->SORTIE);

GENLIB_COPY_UP_REF("out_10","ins0",GENLIB_NAME("%s_10",pr->SORTIE));
GENLIB_COPY_UP_REF("out_15","ins0",GENLIB_NAME("%s_15",pr->SORTIE));
GENLIB_COPY_UP_REF("out_20","ins0",GENLIB_NAME("%s_20",pr->SORTIE));
GENLIB_COPY_UP_REF("out_25","ins0",GENLIB_NAME("%s_25",pr->SORTIE));
GENLIB_COPY_UP_REF("out_30","ins0",GENLIB_NAME("%s_30",pr->SORTIE));
GENLIB_COPY_UP_REF("out_35","ins0",GENLIB_NAME("%s_35",pr->SORTIE));
GENLIB_COPY_UP_REF("out_40","ins0",GENLIB_NAME("%s_40",pr->SORTIE));


////////////////////////////////////////////////////////////
//    placement des alimentations et du caisson
////////////////////////////////////////////////////////////

w = GENLIB_WIDTH(pr->NOM);
GENLIB_PHSEG(NWELL,24,"",0,39,w,39);
GENLIB_PHSEG(CALU1,6,"vss",0,3,w,3);
GENLIB_PHSEG(CALU1,6,"vdd",0,47,w,47);

return(getphfig(pr->NOM,'A'));
}
