/*------------------------------------------------------------\
|                                                             |
| This file is part of the Alliance CAD System Copyright      |
| (C) Laboratoire LIP6 - Dpartement ASIM Universite P&M Curie|
|                                                             |
| Home page      : http://www-asim.lip6.fr/alliance/          |
| E-mail support : mailto:alliance-support@asim.lip6.fr       |
|                                                             |
| This progam is  free software; you can redistribute it      |
| and/or modify it under the  terms of the GNU Library General|
| Public License as published by the Free Software Foundation |
| either version 2 of the License, or (at your option) any    |
| later version.                                              |
|                                                             |
| Alliance VLSI  CAD System  is distributed  in the hope that |
| it  will be useful, but WITHOUT  ANY WARRANTY;              |
| without even the  implied warranty of MERCHANTABILITY or    |
| FITNESS FOR A PARTICULAR PURPOSE. See the GNU General       |
| Public License for more details.                            |
|                                                             |
| You should have received a copy  of the GNU General Public  |
| License along with the GNU C Library; see the file COPYING. |
| If not, write to the Free Software Foundation, Inc.,        |
| 675 Mass Ave, Cambridge, MA 02139, USA.                     |
|                                                             |
\------------------------------------------------------------*/
/*------------------------------------------------------\
|                                                       |
|  Title   :             C4_main												|
|                                                       |
|  Date    :             30.10.01                       |
|                                                       |
|  Authors :  Greiner Alain, Beaudenon Vincent          |
|                                                       |
\------------------------------------------------------*/

#include "C4_main.h"

/*------------------------------------------------------------\
|                                                             |
|       La fonction C4_main est la fonction principale        |
|                                                             |
|	Elle prend en entree un pointeur de befig et          |
|	le nom souhaite pour le(s) fichier(s) de sortie       |
|	et des flags definissant le traitement demamde        |
|	- FlagNetlist : on veut la netlist                    |
|	- Flaglayout  : on veut le layout                     |
|	- FlagNetlist : on veut un VHDL caraterise            |
|	- FlagPower   : on veut un buffer x4                  |
|	Elle renvoie 0 en cas de succes.                      |
|	Elle renvoie -1 si le modele VHDL fourni en entree    |
|	ne respecte pas les contraintes C4.                   |
|	Elle renvoie -2 si l'expression booleenne est trop    |
|	complexe pour etre synthetisee.                       |
\------------------------------------------------------------*/

int C4_main(befig_list *BEFIG,
	char *OutputFileName,
	int FlagNetlist,
	int FlagLayout,
	int FlagCarac,
	int FlagPower,
	int FlagVerbose)
{
  phfig_list	*PHFIG;
  lofig_list	*LOFIG;
  C4_fig_route	*C4FIG;

  int		buffer;

  mbkenv ();
  autenv ();
  ablenv ();
  bddenv ();

// verification de la befig 
// si buffer = 0 la cellule ne respecte pas les conditions de synthese
// si buffer = 1 la cellule est inverseuse
// si buffer = 2 elle est non inverseuse

  buffer = C4_check(BEFIG);
  if(buffer == 0) return(-1);

  if (FlagVerbose)
    {
      fprintf (stdout, "befig %s successfully checked\n", BEFIG->NAME);
    }

// construction de la lofig et ecriture sur disque eventuelle

  LOFIG = C4_lofig(BEFIG,buffer);
/*  if(FlagNetlist) 
	{
	LOFIG->NAME = namealloc (OutputFileName);
	savelofig(LOFIG);
	}
 //Voir plus bas.
 */

  if (FlagVerbose)
    {
      fprintf (stdout, "lofig %s successfully build\n", LOFIG->NAME);
    }

// construction de la c4fig 
// si la figure n'est pas routable, le pointeur C4FIG est nul

  C4FIG = C4_topologie(LOFIG, BEFIG);
//	C4FIG->INS->INDEX=0;
  if(C4FIG == (C4_fig_route *) 0) return (-2);

  if (FlagVerbose)
    {
      fprintf (stdout, "c4fig %s successfully build\n", C4FIG->NOM);
      C4_aff_fig(C4FIG);
    }
	
//La sauvegarde de la netlist doit avoir lieu APRES la construction
//de la C4_fig, car cette tape remet en cause la netlist
//(Permutation des alimentations avec la sortie).
	if(FlagNetlist) 
	{
	LOFIG->NAME = namealloc (OutputFileName);
	savelofig(LOFIG);
	}

// construction phfig et ecriture sur disque eventuelle

  if (FlagLayout)
	{
	if ((buffer==1) && (FlagPower == 0)) PHFIG = C4_layout(0,C4FIG);
	if ((buffer==1) && (FlagPower != 0)) PHFIG = C4_layout(1,C4FIG);
	if ((buffer==2) && (FlagPower == 0)) PHFIG = C4_layout(2,C4FIG);
	if ((buffer==2) && (FlagPower != 0)) PHFIG = C4_layout(3,C4FIG);
	PHFIG->NAME = namealloc (OutputFileName);
	savephfig(PHFIG);
	}

// caracterisation befig et ecriture sur disque

  if (FlagCarac)
    {

/*
      C4_carac(BEFIG,LOFIG);
*/	
      BEFIG->NAME = namealloc (OutputFileName);
      vhdlsavebefig (BEFIG, 0);
      if (FlagVerbose)
	{
	  fprintf (stdout, "characterized befig %s saved\n", BEFIG->NAME);
	}
    }

  return (0);
}

