/*------------------------------------------------------------\
|                                                             |
| Tool    :                     c4                            |
|                                                             |
| File    :                    main.c                         |
|                                                             |
| Date    :                   11.09.99                        |
|                                                             |
| Author  :               Alain Greiner                       |
|                                                             |
\------------------------------------------------------------*/


/*------------------------------------------------------------\
|                                                             |
|                         Include Files                       |
|                                                             |
\------------------------------------------------------------*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>


#include "C4_topologie.h"
#include "C4_check.h"
#include "C4_lofig.h"
#include "C4_layout.h"
#include "C4_main.h"

/*------------------------------------------------------------\
|                                                             |
|                          Functions                          |
|                                                             |
\------------------------------------------------------------*/

void 
C4Usage ()
{
  fprintf (stderr, "\t\tC4 [Options] Input_name [Output_name]\n\n");
  fprintf (stdout, "\t\tOptions : -V Verbose mode on\n");
  fprintf (stdout, "\t\tOptions : -P Output buffer X4  \n");
  fprintf (stdout, "\t\tOptions : -C Saves caracterised befig  \n");
  fprintf (stdout, "\t\tOptions : -N Saves logig    \n");
  fprintf (stdout, "\t\tOptions : -L Saves phgig    \n");
  fprintf (stdout, "\n");

  exit (1);
}

/*--------------------------------------------------------------\
|                                                             	|
|                             main                            	|
|                                                             	|
|	La fonction main() analyse les parametres de la		|
|	ligne de commande, charge la befig,                     |
|       et affiche les messages d'erreurs			|
|                                                             	|
\--------------------------------------------------------------*/

int 
main (argc, argv)

     int argc;
     char *argv[];

{
  char *InputFileName;
  char *OutputFileName;
  int Number;
  int Index;
  int Status;
  char Option;
  befig_list *BEFIG;
  int FlagVerbose = 0;
  int FlagPower =0;
  int FlagCarac = 0;
  int FlagNetlist = 0;
  int FlagLayout = 0;

  InputFileName = (char *) 0;
  OutputFileName = (char *) 0;

  if (argc < 2)
    C4Usage ();

  for (Number = 1; Number < argc; Number++)
    {
      if (argv[Number][0] == '-')
	{
	  for (Index = 1; argv[Number][Index] != '\0'; Index++)
	    {
	      Option = argv[Number][Index];

	      switch (Option)
		{
		case 'C':
                    FlagCarac = 1;
                    break;
                case 'V':
                    FlagVerbose =1;
		  break;
                case 'P':
                    FlagPower =1;     
                    break;
                case 'N':
                    FlagNetlist =1;     
                    break;
                case 'L':
                    FlagLayout =1;     
                    break;
		default:
		    C4Usage ();
		}
	    }
	}
      else if (InputFileName == (char *) 0)
	InputFileName = argv[Number];
      else if (OutputFileName == (char *) 0)
	OutputFileName = argv[Number];
      else
	C4Usage ();
    }

  if (InputFileName == (char *) 0)
    C4Usage ();
  if (OutputFileName == (char *) 0)
    OutputFileName = InputFileName;

// chargement de la befig et appel de C4_main

  BEFIG = vhdlloadbefig ((befig_list *) 0, InputFileName, 3);
  if (FlagVerbose)
	fprintf (stdout, "befig %s successfully loaded\n", BEFIG->NAME);
  Status = C4_main(BEFIG, OutputFileName,
		FlagNetlist, FlagLayout, FlagCarac,
		FlagPower, FlagVerbose);
  if(Status == -1) { 
  	fprintf(stdout,
	"C4 error : the VHDL model of %s is not synthesisable by C4\n",
		InputFileName);
  	fprintf(stdout,
	"The model must respect the following constraints :\n");
  	fprintf(stdout,
	"- no internal signal\n");
  	fprintf(stdout,
	"- no register\n");
  	fprintf(stdout,
	"- no bus\n");
  	fprintf(stdout,
	"- only one output signal\n");
  	fprintf(stdout,
	"- only OR and AND operators\n");
  	fprintf(stdout,
	"- only one optional NOT to define the output polarity\n");
  	fprintf(stdout,
	"- no multiple instanciation of input variables\n");
	}
  if(Status == -2) { 
  	fprintf(stdout,
	"C4 error : the boolean expression of %s\n", InputFileName);
  	fprintf(stdout,
	"is to complex to be routed by C4 in the SXLIB format\n");
	}
}

