/*------------------------------------------------------------\
|                                                             |
| This file is part of the Alliance CAD System Copyright      |
| (C) Laboratoire LIP6 - Dpartement ASIM Universite P&M Curie|
|                                                             |
| Home page      : http://www-asim.lip6.fr/alliance/          |
| E-mail support : mailto:alliance-support@asim.lip6.fr       |
|                                                             |
| This progam is  free software; you can redistribute it      |
| and/or modify it under the  terms of the GNU Library General|
| Public License as published by the Free Software Foundation |
| either version 2 of the License, or (at your option) any    |
| later version.                                              |
|                                                             |
| Alliance VLSI  CAD System  is distributed  in the hope that |
| it  will be useful, but WITHOUT  ANY WARRANTY;              |
| without even the  implied warranty of MERCHANTABILITY or    |
| FITNESS FOR A PARTICULAR PURPOSE. See the GNU General       |
| Public License for more details.                            |
|                                                             |
| You should have received a copy  of the GNU General Public  |
| License along with the GNU C Library; see the file COPYING. |
| If not, write to the Free Software Foundation, Inc.,        |
| 675 Mass Ave, Cambridge, MA 02139, USA.                     |
|                                                             |
\------------------------------------------------------------*/
/*------------------------------------------------------\
|                                                       |
|  Title   :            Node (c4map) 	  								|
|                                                       |
|  Date    :             30.10.01                       |
|                                                       |
|  Authors :  Greiner Alain, Beaudenon Vincent          |
|                                                       |
\------------------------------------------------------*/
#include "node.h"

//*************** SHOW *********************

char *intToTopo(i)
				int i;
{
				if (i==IN_NODE) return namealloc("IN");
				if (i==OUT_NODE) return namealloc("OUT");
				if (i==AUX_NODE) return namealloc("AUX");
return namealloc ("UNKNOWN");
}

char *intToFonc(i)
				int i;
{
				if (i==C4_NODE) return namealloc("C4");
				if (i==BUS_NODE) return namealloc("BUS");
				if (i==REG_NODE) return namealloc("REG");
				if (i==XOR_NODE) return namealloc("XOR");
				if (i==NXOR_NODE) return namealloc("NXOR");
				if (i==INV_NODE) return namealloc("INV");
				
return namealloc ("UNKNOWN");
}


void
showNodeSp (NODE, S)
     c_node_list *NODE;
     int S;
{
  int i;
  c_link_list *ptr;
  if (NODE == NULL)
    return;
  for (i = 0; i < S; i++)
    printf (" ");
  printf ("Node : %s %s %s I : %d D : %f U : %f\n", NODE->SIGNAME, intToTopo(NODE->TYPE_TOPO), intToFonc(NODE->TYPE_FONC),
	  NODE->INV, NODE->D, NODE->U);
  ptr = NODE->SUCC;
  while ((ptr != NULL) && (ptr->SOURCE == NODE))
    {
      printf ("|");
      showLinkSp (ptr, 0);
      ptr = ptr->SUCC;
    }
  showTreeSp (NODE->TREE, S + 1);
  ptr = NODE->PRED;
  while ((ptr != NULL))// && (ptr->SOURCE == NODE))
    {
      printf ("->");
      showLinkSp (ptr, 0);
      ptr = ptr->PRED;
    }
//  showLinkSp (NODE->PRED, S + 1);
  showNodeSp (NODE->NEXT, S);
}

void
showNode (NODE)
     c_node_list *NODE;
{
  showNodeSp (NODE, 0);
}

//************* SEARCH NODE ***************

c_node_list *
searchNode (NAME, NODELIST)
     char *NAME;
     c_node_list *NODELIST;
{
  if (NODELIST == NULL)
    return NULL;
  if (NAME == NULL)
    return NULL;
  if ((namealloc (NODELIST->SIGNAME) == namealloc (NAME))
      && (!((NODELIST->TYPE_FONC == REG_NODE) && (NODELIST->TYPE_TOPO==OUT_NODE))))
    return NODELIST;
  return searchNode (NAME, NODELIST->NEXT);
}

//**************** DIC ********************
void
add_dictionary (T, Name)
     authtable *T;
     char *Name;
{
  addauthelem (T, namealloc (Name), 0);
}

int
search_dictionary (T, Name)
     authtable *T;
     char *Name;
{
  authelem *HashElement;
  HashElement = searchauthelem (T, namealloc (Name));
  return (HashElement != (authelem *) 0);
}

void
del_dictionary (T, Name)
     authtable *T;
     char *Name;
{
  delauthelem (T, namealloc (Name));
}

//***************** GEN FROM NAME *************

c_node_list *genNodeFromName(NAME)
				char *NAME;
{
  c_node_list *new;
if (NAME==NULL) return NULL;
  new = malloc (sizeof *new);
  new->SIGNAME = namealloc (NAME);
  new->TYPE_TOPO = 0;
  new->TYPE_FONC = 0;
	new->INV = 0;
  new->TREE = NULL;
  new->D = 0;
  new->U = 0;
  new->PRED = NULL;
  new->SUCC = NULL;
  new->NEXT = NULL;
  new->LOFIG = NULL;
	new->OUTSIG = NULL;
return new;
}

//**************** OUT ********************

c_node_list *
genNodeFromOut (O, T)
     beout_list *O;
     authtable *T;
{
  c_node_list *new;
  if (O == NULL)
    return NULL;
  if (search_dictionary (T, O->NAME))
    {printf("Erreur : le noeud %s apparait deux fois dans la befig",O->NAME); exit(-1);}
  add_dictionary (T, O->NAME);
  new = genNodeFromName(O->NAME);
  new->TYPE_TOPO = OUT_NODE;
  new->TYPE_FONC = C4_NODE;
  O->ABL = reportInv (O->ABL);
	O->ABL = simplif10Expr(O->ABL);
  flatablexpr (O->ABL);
  new->TREE = genTreeFromABL (O->ABL);
  new->TREE->NODE = new;
  return new;
}

c_node_list *
addNodeFromOut (LIST, O, T)
     c_node_list *LIST;
     beout_list *O;
     authtable *T;
{
  c_node_list *new;
  if (O == NULL)
    return LIST;
  new = genNodeFromOut (O, T);
  if (new != NULL)
    new->NEXT = LIST;
  else
    new = LIST;
  return addNodeFromOut (new, O->NEXT, T);
}

//***************** AUX *******************

c_node_list *
genNodeFromAux (AUX, T)
     beaux_list *AUX;
     authtable *T;
{
  c_node_list *new;
  if (AUX == NULL)
    return NULL;
  if (search_dictionary (T, AUX->NAME))
    {printf("Erreur : le noeud %s apparait deux fois dans la befig",AUX->NAME); exit(-1);}
  add_dictionary (T, AUX->NAME);
  new = genNodeFromName(AUX->NAME);
  new->TYPE_TOPO = AUX_NODE;
  new->TYPE_FONC = C4_NODE;
  AUX->ABL = reportInv (AUX->ABL);
	AUX->ABL = simplif10Expr(AUX->ABL);
  flatablexpr (AUX->ABL);
  new->TREE = genTreeFromABL (AUX->ABL);
  new->TREE->NODE = new;
  return new;
}

c_node_list *
addNodeFromAux (LIST, AUX, T)
     c_node_list *LIST;
     beout_list *AUX;
     authtable *T;
{
  c_node_list *new;
  if (AUX == NULL)
    return LIST;
  new = genNodeFromAux (AUX, T);
  if (new != NULL)
    new->NEXT = LIST;
  else
    new = LIST;
  return addNodeFromAux (new, AUX->NEXT, T);
}


//***************** IN  *******************
c_node_list *
genNodeFromIn (I, T)
     berin_list *I;
     authtable *T;
{
  c_node_list *new;
  if (I == NULL)
    return NULL;
  if (search_dictionary (T, I->NAME))
    return NULL;
  add_dictionary (T, I->NAME);
  new =genNodeFromName(I->NAME);
  new->TYPE_TOPO = IN_NODE;
  new->TYPE_FONC = C4_NODE;
  return new;
}

c_node_list *
addNodeFromIn (LIST, I, T)
     c_node_list *LIST;
     berin_list *I;
     authtable *T;
{
  c_node_list *new;
  if (I == NULL)
    return LIST;
  new = genNodeFromIn (I, T);
  if (new != NULL)
    new->NEXT = LIST;
  else
    new = LIST;
  return addNodeFromIn (new, I->NEXT, T);
}

//****************CND ********************

/*char *getRegCnd(ABL)
				ablexpr *ABL;
{
ablexpr *temp;
//viewablexpr(ABL,0);printf("\n");

if (ABL==NULL) return NULL;
if (ABL_ATOM(ABL)) return NULL;
if (ABL_OPER(ABL)!=ABL_AND) return NULL;
if (getablexprlength(ABL)!=2) return NULL;
if (isabloperinexpr(ABL_CADR(ABL),ABL_STABLE)) temp = ABL_CAR(ABL_CDR(ABL_CDR(ABL)));
else temp = ABL_CADR(ABL);
if (ABL_OPER(temp) !=ABL_NOT) return NULL;
temp = ABL_CADR(temp);
if (ABL_OPER(temp)!=ABL_XOR) return NULL;
if (getablexprlength(temp)!=2) return NULL;
if (namealloc(ABL_ATOM_VALUE(ABL_CADR(temp)))==namealloc("1")) return ABL_ATOM_VALUE(ABL_CADR(ABL_CDR(temp)));
return ABL_ATOM_VALUE(ABL_CADR(temp));
}
*/

char *getRegCnd(ABL)
				ablexpr *ABL;
{
ablexpr *ptr;
char *cnd;
if (ABL_ATOM(ABL)) {
								if (ABL_ATOM_VALUE(ABL)==namealloc("1")) return NULL;
								else if (ABL_ATOM_VALUE(ABL)==namealloc("0")) return NULL;
								else return ABL_ATOM_VALUE(ABL);
}
else {
				ptr = ABL->NEXT;
				while (ptr!=NULL) {
								cnd = getRegCnd(ptr->DATA);
								if (cnd!=NULL) return cnd;
								ptr = ptr->NEXT;
				}
				return NULL;
}
}

//***************** REG  *******************
c_node_list *
genNodeFromReg (R, T)
     bereg_list *R;
     authtable *T;
{
  c_node_list *regin, *regout, *aux1, *aux2;
	c_link_list *link1, *link2;
	int size; char *name;
  if (R == NULL)
    return NULL;
  if (search_dictionary (T, R->NAME))
    {printf("Erreur : le noeud %s apparait deux fois dans la befig",R->NAME); exit(-1);}
	if (R->BIABL->NEXT!=NULL) {printf ("Registre Bidirectionnel, Bonne journe\n"); exit (-1);}
	size = strlen (R->NAME);
	name = malloc ((size + 13) * sizeof(char));
	add_dictionary (T, R->NAME);
  regin =genNodeFromName (R->NAME);
  regin->TYPE_TOPO = IN_NODE;
  regin->TYPE_FONC = REG_NODE;
  regout = genNodeFromName(R->NAME);
	regout->TYPE_TOPO = OUT_NODE;
  regout->TYPE_FONC = REG_NODE;
  regout->NEXT = regin;
	sprintf(name,"%s%s_data",C_PREFIXE,R->NAME);
  aux1=genNodeFromName(name);
  aux1->TYPE_TOPO = AUX_NODE;
  aux1->TYPE_FONC = C4_NODE;
  aux1->NEXT = regout;
	R->BIABL->VALABL = reportInv(R->BIABL->VALABL);
	R->BIABL->VALABL = simplif10Expr(R->BIABL->VALABL);
	flatablexpr(R->BIABL->VALABL);
	aux1->TREE = genTreeFromABL (R->BIABL->VALABL);
	aux1->TREE->NODE = aux1;
/*  aux2 = malloc (sizeof *aux2);
	sprintf(name,"c4map_%s_en",R->NAME);
  aux2->SIGNAME = namealloc (name);
  aux2->TYPE = AUX_NODE;
  aux2->INV = 0;
  aux2->D = 0;
  aux2->U = 0;
  aux2->PRED = NULL;
  aux2->SUCC = NULL;
  aux2->NEXT = aux1;
  aux2->LOFIG = NULL;
	getCnd(R->BIABL->CNDABL);*/
//	aux2->TREE = genTreeFromABL (R->BIABL->CNDABL);
name = getRegCnd(R->BIABL->CNDABL);
if (name==NULL) {printf("Mauvaise Condition sur le registre %s\n",R->NAME); viewablexpr(R->BIABL->CNDABL,0); exit (-1);}
	link1 = genLinkFromName (C_REG_DATA);
	link2 = genLinkFromName (name);
	link1->PRED = link2;
	link1->DEST = regout;
	link2->DEST = regout;
	link1->SOURCE = aux1;
//	link2->SOURCE = aux2;
	regout->PRED = link1;
	return aux1;
}

c_node_list *
addNodeFromReg (LIST, R, T)
     c_node_list *LIST;
     bereg_list *R;
     authtable *T;
{
  c_node_list *new;
  if (R == NULL)
    return LIST;
  new = genNodeFromReg (R, T);
  if (new != NULL)
    new->NEXT->NEXT->NEXT = LIST;
  else
    new = LIST;
  return addNodeFromReg (new, R->NEXT, T);
}

//***************** BIABL->NODELIST ************

c_node_list *genNodeListFromBIABL (BIABL,NODE,INDEX)
				biabl_list *BIABL;
				c_node_list *NODE;
				int INDEX;
{
c_node_list *d, *e;
c_link_list *ld, *le;
char *nd, *ne;
if (BIABL==NULL) return NULL;
//if (INDEX>=1) printf("OK\n");
nd = malloc ((strlen(NODE->SIGNAME)+getSize(INDEX)+6)*sizeof(char));
ne = malloc ((strlen(NODE->SIGNAME)+getSize(INDEX)+4)*sizeof(char));
sprintf(nd,"%s_data%x",NODE->SIGNAME,INDEX);
sprintf(ne,"%s_en%x",NODE->SIGNAME,INDEX);
d = genNodeFromName(nd);
e = genNodeFromName(ne);
ld = genLinkFromName(C_BUS_DATA);
le = genLinkFromName (C_BUS_CMD);
ld->SOURCE = d;
le->SOURCE = e;
ld->DEST = NODE;
le->DEST = NODE;
ld->PRED = le;
le->PRED = NODE->PRED;
NODE->PRED = ld;
d->TYPE_TOPO = AUX_NODE;
d->TYPE_FONC = C4_NODE;
e->TYPE_TOPO = AUX_NODE;
e->TYPE_FONC = C4_NODE;
BIABL->VALABL = reportInv(BIABL->VALABL);
BIABL->CNDABL = reportInv(BIABL->CNDABL);
flatablexpr(BIABL->VALABL);
flatablexpr(BIABL->CNDABL);
BIABL->VALABL = simplif10Expr(BIABL->VALABL);
BIABL->CNDABL = simplif10Expr(BIABL->CNDABL);
d->TREE = genTreeFromABL(BIABL->VALABL);
e->TREE = genTreeFromABL(BIABL->CNDABL);
d->TREE->NODE = d;
e->TREE->NODE = e;
d->NEXT = e;
e->NEXT = genNodeListFromBIABL (BIABL->NEXT,NODE,INDEX+1);
return d;
}

//****************** BUS ***********************

c_node_list *genNodeFromBus (B,T)
				bebus_list *B;
				authtable *T;
			//	int Type;
{
  c_node_list *new;
  if (B == NULL)
    return NULL;
  if (search_dictionary (T, B->NAME))
    {printf("Erreur : le noeud %s apparait deux fois dans la befig",B->NAME); exit(-1);}
	add_dictionary (T, B->NAME);
  new = malloc (sizeof *new);
  new->SIGNAME = namealloc (B->NAME);
  new->TYPE_TOPO = OUT_NODE;
	new->TYPE_FONC = BUS_NODE;
  new->INV = 0;
  new->TREE = NULL;
  new->D = 0;
  new->U = 0;
  new->PRED = NULL;
  new->SUCC = NULL;
  new->NEXT = genNodeListFromBIABL(B->BIABL,new,0);
  new->LOFIG = NULL;
  return new;

}

c_node_list * addNodeFromBus(LIST, B, T, Type)
				c_node_list *LIST;
				bebus_list *B;
				authtable *T;
	//			int Type;
{
	c_node_list *new,*tmp;
	if (B==NULL)
					return LIST;
	new = genNodeFromBus (B,T);
	if (new!=NULL) {
					tmp = new;
					while (tmp->NEXT != NULL) tmp = tmp->NEXT;
					tmp->NEXT = LIST;
	}
	else new = LIST;
	return addNodeFromBus (new, B->NEXT, T);
}


//****************** BUX ***********************

c_node_list *genNodeFromBux (B,T)
				bebux_list *B;
				authtable *T;
			//	int Type;
{
  c_node_list *new;
  if (B == NULL)
    return NULL;
  if (search_dictionary (T, B->NAME))
    {printf("Erreur : le noeud %s apparait deux fois dans la befig",B->NAME); exit(-1);}
//printf("BUX %s\n",B->NAME);
	add_dictionary (T, B->NAME);
  new = malloc (sizeof *new);
  new->SIGNAME = namealloc (B->NAME);
  new->TYPE_TOPO = AUX_NODE;
	new->TYPE_FONC = BUS_NODE;
  new->INV = 0;
  new->TREE = NULL;
  new->D = 0;
  new->U = 0;
  new->PRED = NULL;
  new->SUCC = NULL;
  new->NEXT = genNodeListFromBIABL(B->BIABL,new,0);
  new->LOFIG = NULL;
  return new;

}

c_node_list * addNodeFromBux(LIST, B, T, Type)
				c_node_list *LIST;
				bebux_list *B;
				authtable *T;
	//			int Type;
{
	c_node_list *new,*tmp;
	if (B==NULL)
					return LIST;
	new = genNodeFromBux (B,T);
	if (new!=NULL) {
					tmp = new;
					while (tmp->NEXT != NULL) tmp = tmp->NEXT;
					tmp->NEXT = LIST;
	}
	else new = LIST;
	return addNodeFromBux (new, B->NEXT, T);
}


//***************** DLY ***********************

c_node_list *
genNodeFromDly (D, T)
     berin_list *D;
     authtable *T;
{
  c_node_list *new;
  if (D == NULL)
    return NULL;
  if (search_dictionary (T, D->NAME)) 
    {printf("Erreur : le noeud %s apparait deux fois dans la befig",D->NAME); exit(-1);}
  add_dictionary (T, D->NAME);
	new = NULL;
	return new;
}

c_node_list *
addNodeFromDly (LIST, D, T)
     c_node_list *LIST;
     berin_list *D;
     authtable *T;
{
  c_node_list *new;
  if (D == NULL)
    return LIST;
  new = genNodeFromDly (D, T);
  if (new != NULL)
    new->NEXT = LIST;
  else
    new = LIST;
  return addNodeFromDly (new, D->NEXT, T);
}

//****************** VERIF ********************

bepor_list *searchPor(NAME,POR)
				char *NAME;
				bepor_list *POR;
{
bepor_list *ptr;
if (POR==NULL) return NULL;
ptr = POR;
while(ptr!=NULL) {
//				printf("searchPor %s %s\n",ptr->NAME,NAME);
				if (namealloc(ptr->NAME)==namealloc(NAME)) return ptr;
				ptr = ptr->NEXT;
}
return NULL;
}

void inOutInPor (NODE,POR)
				c_node_list *NODE;
				bepor_list *POR;
{
c_node_list *ptr;
ptr = NODE;
while (ptr!=NULL) {
				if (((ptr->TYPE_TOPO==IN_NODE)||(ptr->TYPE_TOPO==OUT_NODE))&&(ptr->TYPE_FONC!=REG_NODE))
								if (searchPor(ptr->SIGNAME,POR)==NULL) {
																printf("Le noeud %s devrait tre rfrenc comme port dans la Befig\n",NODE->SIGNAME);
																exit(-1);
								}
				ptr = ptr->NEXT;
}
}

void porInOutIn (NODE,POR)
				c_node_list *NODE;
				bepor_list *POR;
{
bepor_list *ptr;
c_node_list *n;
ptr = POR;
while (ptr!=NULL) {
				n = searchNode(ptr->NAME,NODE);
				if (n==NULL) {
																printf("Le port %s n'existe pas en tant que noeud\n",ptr->NAME);
																exit(-1);
								}

				if ((n->TYPE_TOPO!=OUT_NODE)&&(n->TYPE_TOPO!=IN_NODE)) {
								printf("Le port %s donne un Noeud dont le type Topologique est erron\n",ptr->NAME);
								exit (-1);
				}
				ptr = ptr->NEXT;
}
}

void verifInOutEqPor (NODE, POR)
				c_node_list *NODE;
				bepor_list *POR;
{
inOutInPor(NODE,POR);
porInOutIn(NODE,POR);
}

//***************** GEN NODE *******************

c_node_list *
genNodeFromBefig (BEFIG)
     befig_list *BEFIG;
{
  c_node_list *new, *n;
  authtable *T;
  if (BEFIG == NULL)
    return NULL;
  new = NULL;
  T = createauthtable (100);
  new = addNodeFromOut (new, BEFIG->BEOUT, T);
	new = addNodeFromBus (new, BEFIG->BEBUS, T);
  new = addNodeFromAux (new, BEFIG->BEAUX, T);
	new = addNodeFromBux (new, BEFIG->BEBUX, T);
  new = addNodeFromReg (new, BEFIG->BEREG, T);
	new = addNodeFromDly (new, BEFIG->BEDLY, T);
  new = addNodeFromIn (new, BEFIG->BERIN, T);
	verifInOutEqPor(new,BEFIG->BEPOR);
	destroyauthtable (T);
  return new;
}

//***************** GEN FIG **********************

void addLinkFromNode (c_node_list *, c_node_list *, authtable *);

void
addLinkFromTree (TREE, NODE, NODELIST, T)
     c_tree_list *TREE;
     c_node_list *NODE;
     c_node_list *NODELIST;
//     int inv;
     authtable *T;
{
  c_tree_list *ptr;
  c_link_list *new;
  if (TREE == NULL)
    return;
//  if ((TREE->OPER > ABL_MAX_OPERATOR)||(TREE->OPER== ABL_NOT))
  if (isLeaf (TREE))
    {
      //    printf ("LEAF %s\n", TREE->SIGNAME);
      new = genLinkFromName (TREE->SIGNAME);
      new->SOURCE = searchNode (TREE->SIGNAME, NODELIST);
//if (new->SOURCE==NULL) printf("YOUSTONE\n");
      new->DEST = NODE;
      if (TREE->OPER == ABL_NOT)
	{
	  new->INV = 1;

	}
      new->PRED = NULL;
      new->SUCC = NULL;
//      TREE->NODE->PRED = new;
      TREE->VAR = new;
      addLinkFromNode (searchNode (TREE->SIGNAME, NODELIST), NODELIST, T);
    }
  else
    {
/*      if (TREE->OPER == ABL_NOT)
	inv = 1;
      else
	inv = 0;*/
      ptr = TREE->SUP;
      while (ptr != NULL)
	{
	  addLinkFromTree (ptr, NODE, NODELIST, T);
	  ptr = ptr->NEXT;
	}
    }
}

void setRegLink(NODE, NODELIST)
				c_node_list *NODE;
				c_node_list *NODELIST;
{
c_node_list *n;
if (NODE->PRED->PRED->SOURCE!=NULL) return;
n = searchNode (NODE->PRED->PRED->SIGNAME,NODELIST);
if (n==NULL) {printf("Noeud %s introuvable\n",NODE->PRED->PRED->SIGNAME); exit(-1);}
NODE->PRED->PRED->SOURCE=n;
NODE->PRED->PRED->SIGNAME = namealloc(C_REG_CK);
}

/*
void
addLinkFromNode (NODE, NODELIST, T)
     c_node_list *NODE;
     c_node_list *NODELIST;
     authtable *T;
{
				c_link_list *ptr;
  if (NODE == NULL)
    return;
	if (NODE->INV == 1) return;
	NODE->INV = 1;
  if (search_dictionary (T, NODE->SIGNAME))
    {
      printf ("PAS UN DAG\n");
      exit (-1);
    }
  if (NODE->TYPE_TOPO == IN_NODE)
    {
      return;//    resetauthtable (T);
    }
  if ((NODE->PRED != NULL) &&(NODE->TYPE_FONC!=REG_NODE))
    {
      //    resetauthtable (T);
    }
  else
    {
//                      printf("ADDLINK\n");
//                      showNodeSp(NODE,3);
      if (NODE->TYPE_FONC != REG_NODE)
	add_dictionary (T, NODE->SIGNAME);
			else setRegLink(NODE,NODELIST);
      addLinkFromTree (NODE->TREE, NODE, NODELIST, T);
      if (NODE->TYPE_FONC != REG_NODE)
	del_dictionary (T, NODE->SIGNAME);
    }
	if ((NODE->TYPE_FONC ==BUS_NODE) || ((NODE->TYPE_FONC == REG_NODE) && (NODE->TYPE_TOPO!=IN_NODE))) {
					ptr = NODE->PRED;
					while (ptr!=NULL) {
									printf("AddFromNode %s\n",ptr->SOURCE->SIGNAME);
									addLinkFromNode(ptr->SOURCE,NODELIST,T);
									ptr = ptr->PRED;
					}
	}
					
//  addLinkFromNode (NODE->NEXT, NODELIST, T);
}*/


void
addLinkFromNode (NODE, NODELIST, T)
     c_node_list *NODE;
     c_node_list *NODELIST;
     authtable *T;
{
				c_link_list *ptr;
  if (NODE == NULL)
    return;
//		printf("AddFromNode %s\n",NODE->SIGNAME);
	if (NODE->INV == 1) return;
	NODE->INV = 1;
  if (search_dictionary (T, NODE->SIGNAME))
    {
      printf ("PAS UN DAG\n");
      exit (-1);
    }
  if (NODE->TYPE_TOPO == IN_NODE)
    {
      return;//    resetauthtable (T);
    }
  if (NODE->TYPE_FONC==REG_NODE) {
					setRegLink(NODE,NODELIST);
	}
	if ((NODE->TYPE_FONC==BUS_NODE) || (NODE->TYPE_FONC==REG_NODE)) {
					ptr = NODE->PRED;
					while (ptr!=NULL) {
//									printf("AddFromNode %s\n",ptr->SOURCE->SIGNAME);
									addLinkFromNode(ptr->SOURCE,NODELIST,T);
									ptr = ptr->PRED;
					}
	}
	if (NODE->TYPE_FONC==C4_NODE) {
	add_dictionary (T, NODE->SIGNAME);
  addLinkFromTree (NODE->TREE, NODE, NODELIST, T);
	del_dictionary (T, NODE->SIGNAME);
	}
}
									
void
genDAGFromNode (NODE)
     c_node_list *NODE;
{
  authtable *T;
  c_node_list *tmp;
  T = createauthtable (1000);
	tmp = NODE;
//  printf("GEN DAG..........\n");
  while (tmp != NULL)
    {
//      if ((tmp->TYPE_TOPO == OUT_NODE) || (tmp->TYPE_FONC == REG_NODE))
//	{
	  addLinkFromNode (tmp, NODE, T);
	  resetauthtable (T);
//	}
      tmp = tmp->NEXT;
    }
	tmp = NODE;
	while(tmp!=NULL) {tmp->INV = 0; tmp = tmp->NEXT;}
  destroyauthtable (T);
}

// ****************** UNARY *********************

void removeUnaryLink (LINK)
				c_link_list *LINK;
{
c_node_list *source;
if (LINK==NULL) return;
//printf("RM LINK %s ...\n",LINK->SIGNAME);
source = LINK->SOURCE;
if (source ==NULL) {printf("Arc %s mal dfini, Bonne journe\n",LINK->SIGNAME); exit (-1);}
if ((source->TYPE_TOPO==AUX_NODE) && (source->TYPE_FONC==C4_NODE)) {
				if (source->TREE==NULL) {printf("Noeud %s mal dfini, Bonne journe\n",source->SIGNAME); exit (-1);}
				if (source->TREE->VAR!=NULL) {
//printf("RM LINK %s -> %s\n",LINK->SIGNAME,LINK->SOURCE->TREE->VAR->SIGNAME);
								
								LINK ->SOURCE = source->TREE->VAR->SOURCE;
								LINK-> INV = LINK->INV ^ source->TREE->VAR->INV;
				}
}
}

void removeUnaryPred(LINK)
				c_link_list *LINK;
{
c_link_list *ptr;
if (LINK==NULL) return;
//printf("RM PRED\n");
ptr = LINK;
while (ptr!=NULL) {
				removeUnaryLink(ptr);
				ptr = ptr->PRED;
}
//printf ("END PRED\n");
}

void removeUnaryVars(TREE)
				c_tree_list *TREE;
{
c_tree_list *ptr;
//c_node_list *source;
if (TREE==NULL) return;
if (TREE->VAR!=NULL) {
		//		source = TREE->VAR->SOURCE;
//printf("RM VAR\n");
				removeUnaryLink (TREE->VAR);
				return;
ptr = TREE->SUP;
while (ptr !=NULL) {
				removeUnaryVars(ptr);
				ptr = ptr->NEXT;
}
}
				}
c_node_list *nodeListWithoutUN(NODE)
				c_node_list *NODE;
{
c_node_list *next;
if (NODE==NULL) return NULL;
if ((NODE->TYPE_FONC==C4_NODE) &&(NODE->TYPE_TOPO==AUX_NODE)) {
				if (NODE->TREE->VAR!=NULL) {
								next = NODE->NEXT;
								free(NODE->TREE->VAR);
								free(NODE->TREE);
								free(NODE);
								return nodeListWithoutUN(next);
				}
}
NODE->NEXT = nodeListWithoutUN(NODE->NEXT);
return NODE;
}

c_node_list *removeUnaryNodes (NODE)
				c_node_list *NODE;
{
c_node_list /**next,*/ *ptr;
//c_link_list *pred, *succ;
if (NODE==NULL) return NULL;
ptr = NODE;
while (ptr!=NULL) {
				if ((ptr->TYPE_FONC==C4_NODE) && (ptr->TYPE_TOPO!=IN_NODE))
				removeUnaryVars(ptr->TREE);
				else if (ptr->PRED!=NULL) removeUnaryPred(ptr->PRED);
ptr = ptr->NEXT;}
//return NODE;
return nodeListWithoutUN(NODE);
}

// ****************** BEFIG **********************

befig_list *
genBefigFromNode (NODE)
     c_node_list *NODE;
{
  befig_list *new;
  c_link_list *ptr;
  ablexpr *abl;
  if (NODE == NULL)
    return NULL;
  if (NODE->TREE == NULL)
    return NULL;
  new = beh_addbefig (NULL, NODE->TREE->GNAME);
  abl = genABLFromTree (NODE->TREE);
//  viewablexpr (abl, 0);
//  printf ("\n");
  new->BEOUT = beh_addbeout (NULL, C_C4_OUT, abl, NULL);
  ptr = NODE->PRED;
  while (ptr != NULL)
    {
      new->BERIN = beh_addberin (new->BERIN, ptr->SIGNAME);
			new->BEPOR = beh_addbepor (new->BEPOR, ptr->SIGNAME,'I','B');
      ptr = ptr->PRED;
    }
	new->BERIN= beh_addberin (new->BERIN,"vdd");
	new->BERIN= beh_addberin (new->BERIN,"vss");
	new->BEPOR = beh_addbepor (new->BEPOR, "vdd",'I','B');
	new->BEPOR = beh_addbepor (new->BEPOR, "vss",'I','B');
	new->BEPOR = beh_addbepor (new->BEPOR, C_C4_OUT,'O','B');
vhdlsavebefig (new,NODE->TREE->GNAME);
	//printf("FIN GEN BEFIG");
  return new;
}
