/*------------------------------------------------------------\
|                                                             |
| This file is part of the Alliance CAD System Copyright      |
| (C) Laboratoire LIP6 - Dpartement ASIM Universite P&M Curie|
|                                                             |
| Home page      : http://www-asim.lip6.fr/alliance/          |
| E-mail support : mailto:alliance-support@asim.lip6.fr       |
|                                                             |
| This progam is  free software; you can redistribute it      |
| and/or modify it under the  terms of the GNU Library General|
| Public License as published by the Free Software Foundation |
| either version 2 of the License, or (at your option) any    |
| later version.                                              |
|                                                             |
| Alliance VLSI  CAD System  is distributed  in the hope that |
| it  will be useful, but WITHOUT  ANY WARRANTY;              |
| without even the  implied warranty of MERCHANTABILITY or    |
| FITNESS FOR A PARTICULAR PURPOSE. See the GNU General       |
| Public License for more details.                            |
|                                                             |
| You should have received a copy  of the GNU General Public  |
| License along with the GNU C Library; see the file COPYING. |
| If not, write to the Free Software Foundation, Inc.,        |
| 675 Mass Ave, Cambridge, MA 02139, USA.                     |
|                                                             |
\------------------------------------------------------------*/
/*------------------------------------------------------\
|                                                       |
|  Title   :            tree (c4map)  									|
|                                                       |
|  Date    :             30.10.01                       |
|                                                       |
|  Authors :  Greiner Alain, Beaudenon Vincent          |
|                                                       |
\------------------------------------------------------*/
#include "tree.h"

char
oToC (op)
     long op;
{
  char *t;
  t = namealloc ("OAXNUUYU");
  if (op < 7)
    return t[op];
  return '.';
}

void
showVarSp (TREE, S)
     c_tree_list *TREE;
     int S;
{
  if (TREE == NULL)
    return;
  if (TREE->VAR == NULL)
    return;
  showLinkSp (TREE->VAR, S + 2);
}

void
showTreeSp (TREE, S)
     c_tree_list *TREE;
     int S;
{
  int i;
  if (TREE == NULL)
    return;
  for (i = 0; i < S; i++)
    printf (" ");
  printf ("%s %s Root :%d Oper:%c Arity:%d N:%d P:%d U:%f D:%f",
	  TREE->SIGNAME, TREE->GNAME, TREE->ROOT, oToC (TREE->OPER),
	  TREE->ARITY, TREE->N, TREE->P, TREE->U, TREE->D);
  if (TREE->NODE != NULL)
    printf (" ROOT");
  printf ("\n");
  showVarSp (TREE, S);
  showTreeSp (TREE->SUP, S + 1);
  showTreeSp (TREE->NEXT, S);
}

void
showTree (TREE)
     c_tree_list *TREE;
{
  showTreeSp (TREE, 0);
}

//************************ TESTS *****************

int
isLeaf (TREE)
     c_tree_list *TREE;
{
  if (TREE == NULL)
    return 0;
  if (TREE->ROOT == 1)
    return 1;
  if (TREE->OPER == ABL_NOT)
    return 1;
  if (TREE->OPER > ABL_MAX_OPERATOR)
    return 1;
  return 0;
}

//********************** COPY ***********************

c_tree_list *
copyTree (TREE)
     c_tree_list *TREE;
{
  c_tree_list *new;
  if (TREE == NULL)
    return NULL;
  new = malloc (sizeof *new);
  new->SIGNAME = namealloc (TREE->SIGNAME);
  new->GNAME = namealloc ("gn");
  new->NEXT = NULL;
  new->OPER = TREE->OPER;
  new->ARITY = TREE->ARITY;
  new->SUP = TREE->SUP;
  new->N = TREE->N;
  new->P = TREE->P;
  new->U = TREE->U;
  new->D = TREE->D;
  new->NODE = NULL;
  new->VAR = TREE->VAR;
  new->ROOT = TREE->ROOT;
  return new;
}

//*********************** GEN **********************
c_tree_list *
genTreeFromABL (ABL)
     ablexpr *ABL;
//     struct c_node *NODE;
{
  c_tree_list *new;
  ablexpr *ptr;
  c_tree_list *tmp, *next;
  if (ABL == NULL)
    return NULL;
  new = malloc (sizeof *new);
//********************** INIT ************
  new->NEXT = NULL;
  new->SIGNAME = namealloc ("sn");
//  new->SIGNAME = namealloc (giveName());
  new->GNAME = namealloc ("gn");
  new->ARITY = 0;
  new->N = 0;
  new->P = 0;
  new->U = 0;
  new->D = 0;
  new->SUP = NULL;
  new->NODE = NULL;
  new->ROOT = 0;
  new->VAR = NULL;
//****************************************
  if (ABL_ATOM (ABL))
    {
      new->OPER = ABL_MAX_OPERATOR + 1;
      new->N = 1;
      new->P = 1;
      new->SIGNAME = namealloc (ABL_ATOM_VALUE (ABL));
    }
  else
    {
      new->OPER = ABL_OPER (ABL);
      ptr = ABL;
      next = NULL;
      if (new->OPER == ABL_NOT)
	{
	  new->N = 1;
	  new->P = 1;
	  new->SIGNAME = namealloc (ABL_ATOM_VALUE (ABL_CAR (ABL_CDR (ABL))));
	}
      else
	{
	  while ((ptr = ABL_CDR (ptr)) != NULL)
	    {
	      new->ARITY++;
	      tmp = genTreeFromABL (ABL_CAR (ptr));
	      tmp->NEXT = next;
	      next = tmp;
	    }
	  new->SUP = tmp;
	}
    }
  return new;
}

c_tree_list *
genTreeFromOper (OPER)
     long OPER;
{
  c_tree_list *new;
  if (OPER < 0)
    return NULL;
  if (OPER > ABL_MAX_OPERATOR)
    return NULL;
  new = malloc (sizeof *new);
  new->SIGNAME = namealloc ("sn");
  new->GNAME = namealloc ("gn");
  new->OPER = OPER;
  new->ARITY = 0;
  new->SUP = NULL;
  new->N = 0;
  new->P = 0;
  new->D = 0;
  new->U = 0;
  new->NODE = NULL;
  new->VAR = NULL;
  new->ROOT = 0;
  new->NEXT = NULL;
  return new;
}

//************************** ORDER  **************************

c_tree_list *
insertInTreeList (ELEM, LIST)
     c_tree_list *ELEM;
     c_tree_list *LIST;
{
  if (LIST == NULL)
    return ELEM;
  if (ELEM == NULL)
    return LIST;
  if (strcmp (ELEM->GNAME, LIST->GNAME) <= 0)
    {
      ELEM->NEXT = LIST;
      return ELEM;
    }
  LIST->NEXT = insertInTreeList (ELEM, LIST->NEXT);
  return LIST;
}

void
orderTree (TREE)
     c_tree_list *TREE;
{
  c_tree_list *list, *ptr, *tmp;
  list = NULL;
  ptr = TREE->SUP;
  tmp = ptr;
  while (ptr != NULL)
    {
      ptr = ptr->NEXT;
      tmp->NEXT = NULL;
      list = insertInTreeList (tmp, list);
      tmp = ptr;
    }
  TREE->SUP = list;
}

//**************************** GNAME *****************************

void
extractGName (TREE)
     c_tree_list *TREE;
{
  int size;
  c_tree_list *ptr;
  char *name;
  if (TREE == NULL)
    return;
  if (TREE->VAR != NULL)
    {
      TREE->GNAME = namealloc ("");
      return;
    }
  ptr = TREE->SUP;
  while (ptr != NULL)
    {
      extractGName (ptr);
      ptr = ptr->NEXT;
    }
  orderTree (TREE);
  ptr = TREE->SUP;
  size = 3;
  while (ptr != NULL)
    {
      size = size + strlen (ptr->GNAME);
      ptr = ptr->NEXT;
    }
  name = malloc (size * (sizeof (char)));
  sprintf (name, "%c", oToC (TREE->OPER));
  ptr = TREE->SUP;
  while (ptr != NULL)
    {
      sprintf (name, "%s%s", name, ptr->GNAME);
      ptr = ptr->NEXT;
    }
  sprintf (name, "%s%d", name, TREE->ARITY);
  TREE->GNAME = namealloc (name);
  free (name);
}

//**************************** MODEL ***************************

int
modelTree (TREE, INDEX)
     c_tree_list *TREE;
     int INDEX;
{
  char *name;
  c_tree_list *ptr;
  if (TREE == NULL)
    return 0;
  if (TREE->VAR != NULL)
    {
      name = malloc (6 * sizeof (char));
      sprintf (name, "%s%x", C_C4_IN,INDEX);
      TREE->SIGNAME = namealloc (name);
      TREE->VAR->SIGNAME = namealloc (name);
      return INDEX + 1;
    }
  ptr = TREE->SUP;
  while (ptr != NULL)
    {
      INDEX = modelTree (ptr, INDEX);
      ptr = ptr->NEXT;
    }
  return INDEX;
}

//************************** NORMALIZE **************************

void
normalizeTree (TREE)
     c_tree_list *TREE;
{
  if (TREE == NULL)
    return;
  extractGName (TREE);
  modelTree (TREE, 0);
}

//************************** GEN ABL FROM TREE ******************

ablexpr *
genABLFromTree (TREE)
     c_tree_list *TREE;
{
  c_tree_list *ptr;
  ablexpr *abl, *new;
  if (TREE == NULL)
    return NULL;
  if (TREE->VAR != NULL)
    return createablatom (TREE->SIGNAME);
  ptr = TREE->SUP;
  abl = NULL;
  while (ptr != NULL)
    {
      abl = addchain (abl, genABLFromTree (ptr));
      ptr = ptr->NEXT;
    }
  new = createabloper (TREE->OPER);
  ABL_CDR (new) = abl;
  return new;
}

//******************* GENROOT **************************

void
genRoot (TREE)
     c_tree_list *TREE;
{
  if (TREE == NULL)
    return;
//  if (isLeaf (TREE))
//    return;
  if (TREE->ROOT == 1)
    return;
//  if (TREE->NODE != NULL)
//    return;
//  getDelay (TREE);
	if (TREE->VAR!=NULL) return;
	TREE->ROOT = 1;
  TREE->N = 1;
  TREE->P = 1;
//      printf("ROOT : %s %d %d\n",TREE->SIGNAME, TREE->N, TREE->P);
  //  TREE->SIGNAME = namealloc (giveName ());
}


//*********************** DECOMPOSITION *****************

void
removeSon (TREE, SON)
     c_tree_list *TREE;
     c_tree_list *SON;
{
  c_tree_list *last;
  c_tree_list *ptr;
  if (TREE == NULL)
    return;
  if (SON == NULL)
    return;
  if (TREE->SUP == NULL)
    return;
  if (TREE->SUP == SON)
    TREE->SUP = TREE->SUP->NEXT;
  else
    {
      last = TREE->SUP;
      ptr = last->NEXT;
      while (ptr != NULL)
	{
	  if (ptr == SON)
	    {
	      last->NEXT = ptr->NEXT;
	      break;
	    }
	  ptr = ptr->NEXT;
	  last = last->NEXT;
	}
    }
  SON->NEXT = NULL;
  TREE->ARITY--;
}

void
insertSon (TREE, SON)
     c_tree_list *TREE;
     c_tree_list *SON;
{
  if (TREE == NULL)
    return;
  if (SON == NULL)
    return;
  SON->NEXT = TREE->SUP;
  TREE->SUP = SON;
  TREE->ARITY++;
}

//***************** DELAIS *****************
/*
void
getDelayAO (TREE)
     c_tree_list *TREE;
{
  c_tree_list *ptr;
  double maxu, maxd;
  if (TREE == NULL)
    return;
  if (isLeaf (TREE))
    {
      TREE->U = C_TN + TREE->U;
      TREE->D = C_TP + TREE->D;
      return;
    }
//if (TREE->ROOT==1) return;
// OPERATEUR ET/OU
  ptr = TREE->SUP;
  maxu = 0;
  maxd = 0;
  while (ptr != NULL)
    {
      getDelayAO (ptr);
      if (ptr->D > maxd)
	maxd = ptr->D;
      if (ptr->U > maxu)
	maxu = ptr->U;
      if (TREE->OPER == ABL_AND)
	{
	  TREE->U = maxu + ((TREE->ARITY) - 1) * C_TN;
	  TREE->D = maxd;
	}
      else
	{
	  TREE->D = maxd + ((TREE->ARITY) - 1) * C_TP;
	  TREE->U = maxu;
	}
      ptr = ptr->NEXT;
    }
}
*/
void
getDelay (TREE)
     c_tree_list *TREE;
{

  if (TREE == NULL)
    return;
  if (isLeaf (TREE))
    return;
  if (TREE->ROOT == 1)
    return;
  if (TREE->OPER == ABL_XOR)
    return;
  if (TREE->OPER == ABL_NXOR)
    return;
//  getDelayAO (TREE);
  TREE->U = TREE->U + C_TUU;
  TREE->D = TREE->D + C_TDD;
}

void
tegDelay (TREE)
     c_tree_list *TREE;
{

  if (TREE == NULL)
    return;
  if (isLeaf (TREE))
    return;
  if (TREE->ROOT == 1)
    return;
  if (TREE->OPER == ABL_XOR)
    return;
  if (TREE->OPER == ABL_NXOR)
    return;
//  getDelayAO (TREE);
  TREE->U = TREE->U - C_TUU;
  TREE->D = TREE->D - C_TDD;
}


c_tree_list *
getMaxU (TREE)
     c_tree_list *TREE;
{
  c_tree_list *ptr;
  c_tree_list *tree;
  if (TREE == NULL)
    return NULL;
//if (isLeaf(TREE)) return TREE;
  tree = TREE;
  ptr = TREE;
  while (ptr != NULL)
    {
      if (ptr->U > tree->U)
	tree = ptr;
      ptr = ptr->NEXT;
    }
  return tree;
}

c_tree_list *
getMaxD (TREE)
     c_tree_list *TREE;
{
  c_tree_list *ptr;
  c_tree_list *tree;
  if (TREE == NULL)
    return NULL;
//if (isLeaf(TREE)) return TREE;
  tree = TREE;
  ptr = TREE;
  while (ptr != NULL)
    {
      if (ptr->D > tree->D)
	tree = ptr;
      ptr = ptr->NEXT;
    }
  return tree;
}

c_tree_list *
getMinU (TREE /*,UNUSED */ )
     c_tree_list *TREE;
//               int UNUSED;
{
  c_tree_list *ptr;
  c_tree_list *tree;
  if (TREE == NULL)
    return NULL;
//                              if (isLeaf(TREE)) return TREE;
  tree = TREE;
  ptr = TREE;
//      if ((UNUSED==1) && (TREE->SIGNAME!=namealloc("sn"))) return getMinU(TREE->NEXT,UNUSED);
  while (ptr != NULL)
    {
      if ((ptr->U <
	   tree->U) /* && ((UNUSED==0)||(ptr->SIGNAME!=namealloc("sn"))) */ )
	tree = ptr;
      ptr = ptr->NEXT;
    }
  return tree;
}

c_tree_list *
getMinD (TREE /*,UNUSED */ )
     c_tree_list *TREE;
//               int UNUSED;
{
  c_tree_list *ptr;
  c_tree_list *tree;
  if (TREE == NULL)
    return NULL;
//                              if (isLeaf(TREE)) return TREE;
  tree = TREE;
  ptr = TREE;
//      if ((UNUSED==1) && (TREE->SIGNAME!=namealloc("sn"))) return getMinD(TREE->NEXT,UNUSED);
  while (ptr != NULL)
    {
      if ((ptr->D <
	   tree->D) /* &&((UNUSED==0)||(ptr->SIGNAME!=namealloc("sn"))) */ )
	tree = ptr;
      ptr = ptr->NEXT;
    }
  return tree;
}

c_tree_list *
getMinUD (TREE)
     c_tree_list *TREE;
{
  c_tree_list *ptr;
  c_tree_list *tree;
  if (TREE == NULL)
    return NULL;
//                              if (isLeaf(TREE)) return TREE;
  tree = TREE;
  ptr = TREE;
  while (ptr != NULL)
    {
      if ((ptr->D + ptr->U) < (tree->D + tree->U))
	tree = ptr;
      ptr = ptr->NEXT;
    }
  return tree;
}

int
getMaxN (TREE)
     c_tree_list *TREE;
{
  c_tree_list *ptr;
  int max = 0;
  if (TREE == NULL)
    return 0;
//                              if (isLeaf(TREE)) return TREE;
  ptr = TREE;
  while (ptr != NULL)
    {
      if (ptr->N > max)
	max = ptr->N;
      ptr = ptr->NEXT;
    }
  return max;
}

int
getMaxP (TREE)
     c_tree_list *TREE;
{
  c_tree_list *ptr;
  int max = 0;
  if (TREE == NULL)
    return 0;
//                              if (isLeaf(TREE)) return TREE;
  ptr = TREE;
  while (ptr != NULL)
    {
      if (ptr->P > max)
	max = ptr->P;
      ptr = ptr->NEXT;
    }
  return max;
}

int
getSumN (TREE)
     c_tree_list *TREE;
{
  c_tree_list *ptr;
  int sum = 0;
  if (TREE == NULL)
    return 0;
//                              if (isLeaf(TREE)) return TREE;
  ptr = TREE;
  while (ptr != NULL)
    {
      sum = sum + ptr->N;
      ptr = ptr->NEXT;
    }
  return sum;
}

int
getSumP (TREE)
     c_tree_list *TREE;
{
  c_tree_list *ptr;
  int sum = 0;
  if (TREE == NULL)
    return 0;
//                              if (isLeaf(TREE)) return TREE;
  ptr = TREE;
  while (ptr != NULL)
    {
      sum = sum + ptr->P;
      ptr = ptr->NEXT;
    }
  return sum;
}
