/*
 * This file is part of the Alliance CAD System
 * Copyright (C) Laboratoire LIP6 - Dpartement ASIM
 * Universite Pierre et Marie Curie
 * 
 * Home page          : http://www-asim.lip6.fr/alliance/
 * E-mail support     : mailto:alliance-support@asim.lip6.fr
 * 
 * This progam is  free software; you can redistribute it  and/or modify it
 * under the  terms of the GNU  General Public License as  published by the
 * Free Software Foundation;  either version 2 of the License,  or (at your
 * option) any later version.
 * 
 * Alliance VLSI  CAD System  is distributed  in the hope  that it  will be
 * useful, but WITHOUT  ANY WARRANTY; without even the  implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 * 
 * You should have received a copy  of the GNU General Public License along
 * with the GNU C Library; see the  file COPYING. If not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*********************************************************************\

    File        :    placer_structure.c

    Date        :   26-09-94
    Version     :   1

    Auteur      :   BEAURIN oliver

    Description :   structure working for dp placer.

\*********************************************************************/

/*---------------------------------------------------------\
                     includes
\---------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

#include MUT_H
#include MLO_H
#include MPH_H

#include "dpr.h"
#include "placer_structure.h"

/*---------------------------------------------------------\
                     display_placement
\---------------------------------------------------------*/
void display_placement( ptplacement )
placement *ptplacement;
{
    column  *ptcolumn;
    slice   *ptslice;
    long     colindex;
    long     slicei;
    con_NS  *ptconNS;
    con_EW  *ptconEW;
    long     nbslices;

    nbslices = ptplacement->NB_SLICES;

    fprintf( stdout,"Slices array\n" );
    if( ( ptplacement->YSLICE == NULL ) || ( ptplacement->SLICESHEIGHTS == NULL ) )
    {
        fprintf( stdout, "_ Empty\n" );
    }
    else
    {
        for( slicei = 0; slicei <= nbslices; slicei ++ )
        {
            fprintf( stdout,
                     "_ slice %3ld : %ld %ld\n",
                     slicei,
                     ptplacement->YSLICE[ slicei ],
                     ptplacement->SLICESHEIGHTS[ slicei ] );
        }
    }

    fprintf( stdout,"\nSymbolic grid\n" );
    if( !ptplacement->COLUMNS )
    {
        fprintf( stdout, "_ Empty\n" );
    }

    for( colindex = 1, ptcolumn = ptplacement->COLUMNS;
         ptcolumn;
         colindex++, ptcolumn = ptcolumn->NEXT )
    {
        fprintf( stdout, "_ column %ld (%ld) : %s %s\n",
                         colindex,
                         ptcolumn->INDEX,
                         ptcolumn->MODEL_NAME,
                         ptcolumn->INSTANCE_NAME );

        if( !ptcolumn->SLICES )
        {
            fprintf( stdout, " |_ Empty\n" );
        }

        for( ptslice = ptcolumn->SLICES;
             ptslice;
             ptslice = ptslice->NEXT )
        {
            fprintf( stdout,
                     " |_ slice %ld : %s %s %ld %s %s\n",
                     ptslice->INDEX,
                     ptslice->INSTANCE_NAME,
                     ptslice->MODEL_NAME,
                     ptslice->HEIGHT,
                     ptslice->TYPE_DOWN == TYPE_VDD ? "VDD" : "VSS",
                     ptslice->TYPE_UP   == TYPE_VDD ? "VDD" : "VSS" );
        }

        fprintf( stdout, " |__ N/S CONNECTORS\n" );

        if( ptcolumn->CON_NS == NULL )
        {
            fprintf( stdout, "   |_ Empty\n");
        }
        for( ptconNS = ptcolumn->CON_NS; ptconNS; ptconNS = ptconNS->NEXT )
        {
            fprintf( stdout,
                     "   |_ %s %c %d %d\n",
                     ptconNS->NAME,
                     ptconNS->TYPE,
                     (int) ptconNS->POSX,
                     (int) ptconNS->WIDTH );
        }

        fprintf( stdout, " |__ E/W CONNECTORS\n" );

        if( ptcolumn->CON_EW == NULL )
        {
            fprintf( stdout, "   |_ Empty\n");
        }
        for( ptconEW = ptcolumn->CON_EW; ptconEW; ptconEW=ptconEW->NEXT )
        {
            fprintf( stdout,
                     "   |_ %s %d\n",
                     ptconEW->NAME,
                     (int)ptconEW->YPOS );
        }
    }
} /* fin de display_placement */

/*---------------------------------------------------------\
                     add_con_NS
\---------------------------------------------------------*/
con_NS *add_con_NS( ptnext, name, type, posx, width)
con_NS *ptnext;
char   *name;
char    type;
long    posx;
long    width;
{
    con_NS *ptcon;

    for( ptcon = ptnext; ptcon; ptcon=ptcon->NEXT )
    {
        if( ( ptcon->NAME ==  name )
          &&( ptcon->TYPE ==  type )
          &&( ptcon->POSX ==  posx )
          &&( ptcon->WIDTH == width ) )
        {
            return ptnext;
        }
    }

    ptcon = (con_NS *)mbkalloc( sizeof( con_NS ) );

    ptcon->NEXT  = ptnext;
    ptcon->NAME  = name;
    ptcon->TYPE  = type;
    ptcon->POSX  = posx;
    ptcon->WIDTH = width;

    return ptcon;

} /* fin de add_con_NS */

/*---------------------------------------------------------\
                     free_con_NS_list
\---------------------------------------------------------*/
void free_con_NS_list( ptcon )
con_NS *ptcon;
{
    con_NS *ptconaux;

    for( ; ptcon; ptcon = ptconaux )
    {
        ptconaux = ptcon->NEXT;
        free( (char *)ptcon );
    }
} /* fin de free_con_NS_list */

/*---------------------------------------------------------\
                     add_con_EW
\---------------------------------------------------------*/
con_EW *add_con_EW( ptnext, name, ypos )
con_EW *ptnext;
char   *name;
long    ypos;
{
    con_EW *ptcon;

    for( ptcon = ptnext; ptcon; ptcon=ptcon->NEXT )
    {
        if( ptcon->NAME == name )
        {
            return ptnext;
        }
    }

    ptcon = (con_EW *)mbkalloc( sizeof( con_EW ) );

    ptcon->NEXT    = ptnext;
    ptcon->NAME    = name;
    ptcon->YPOS    = ypos;

    return ptcon;

} /* fin de add_con_EW */

/*---------------------------------------------------------\
                     free_con_EW_list
\---------------------------------------------------------*/
void free_con_EW_list( ptcon )
con_EW *ptcon;
{
    con_EW *ptconaux;

    for( ; ptcon; ptcon = ptconaux )
    {
        ptconaux = ptcon->NEXT;
        free( (char *)ptcon );
    }
} /* fin de free_con_EW_list */

/*---------------------------------------------------------\
                     getcon_NS
\---------------------------------------------------------*/
con_NS *getcon_NS( ptcon, name )
con_NS *ptcon;
char   *name;
{
    for( ; (ptcon) && (ptcon->NAME != name); ptcon=ptcon->NEXT );
    return ptcon;
} /* fin de getcon_NS */

/*---------------------------------------------------------\
                     getcon_EW
\---------------------------------------------------------*/
con_EW *getcon_EW( ptcon, name )
con_EW *ptcon;
char   *name;
{
    for( ; (ptcon) && (ptcon->NAME != name); ptcon=ptcon->NEXT );
    return ptcon;
} /* fin de getcon_EW */

/*---------------------------------------------------------\
                     addinstance
\---------------------------------------------------------*/
instance *addinstance( ptnextinstance, modelname, insname, index )
instance *ptnextinstance;
char     *modelname;
char     *insname;
long      index;
{
    instance *ptinstance;

    ptinstance = (instance *)mbkalloc( sizeof( instance ) );

    ptinstance->NEXT      = ptnextinstance;
    ptinstance->MODELNAME = modelname;
    ptinstance->INSNAME   = insname;
    ptinstance->INDEX     = index;

    return( ptinstance );

} /* fin de addinstance */

/*---------------------------------------------------------\
                     freeinstance
\---------------------------------------------------------*/
void freeinstance( ptinstance )
instance *ptinstance;
{
    instance *ptinstanceaux;

    for( ; ptinstance; ptinstance = ptinstanceaux )
    {
        ptinstanceaux = ptinstance->NEXT;
        mbkfree( (void *)ptinstance );
    }
} /* fin de freeinstance */

/*---------------------------------------------------------\
                     addalim
\---------------------------------------------------------*/
alim *addalim( ptnextalim, type, y )
alim *ptnextalim;
char  type;
long  y;
{
    alim *ptalim;

    ptalim = (alim *)mbkalloc( sizeof( alim ) );

    ptalim->NEXT = ptnextalim;
    ptalim->TYPE = type;
    ptalim->Y    = y;

    return( ptalim );

} /* fin de addalim */

/*---------------------------------------------------------\
                     freealim
\---------------------------------------------------------*/
void freealim( ptalim )
alim *ptalim;
{
    alim *ptalimaux;

    for( ; ptalim; ptalim = ptalimaux )
    {
        ptalimaux = ptalim->NEXT;
        mbkfree( (void *)ptalim );
    }
} /* fin de freealim */

/*---------------------------------------------------------\
                     addslice
\---------------------------------------------------------*/
slice *addslice( ptnextslice, modelname, insname, index, width, height, type_down, type_up )
slice *ptnextslice;
char  *modelname;
char  *insname;
long   index;
long   width;
long   height;
char   type_down;
char   type_up;
{
    slice *ptslice;

    ptslice = (slice *)mbkalloc( sizeof( slice ) );

    ptslice->NEXT          = ptnextslice;
    ptslice->INSTANCE_NAME = insname;
    ptslice->MODEL_NAME    = modelname;
    ptslice->INDEX         = index;
    ptslice->WIDTH         = width;
    ptslice->HEIGHT        = height;
    ptslice->TYPE_DOWN     = type_down;
    ptslice->TYPE_UP       = type_up;

    return( ptslice );

} /* fin de addslice */

/*---------------------------------------------------------\
                        freeslicelist
\---------------------------------------------------------*/
void freeslicelist ( ptslicehead )
slice *ptslicehead;
{
    slice *ptslice;

    for( ptslice = ptslicehead; ptslice; ptslice = ptslicehead )
    {
        ptslicehead = ptslice->NEXT;
        free( (char *)ptslice );
    }
} /* end of freeslicelist */

/*---------------------------------------------------------\
                     addcolumn
\---------------------------------------------------------*/
column *addcolumn( ptnextcolumn, ptslice, modelname, insname, type )
column *ptnextcolumn;
slice  *ptslice;
char   *modelname;
char   *insname;
char    type;
{
    column *ptcolumn;

    ptcolumn = (column *)mbkalloc( sizeof( column ) );

    ptcolumn->NEXT          = ptnextcolumn;
    ptcolumn->SLICES        = ptslice;
    ptcolumn->MODEL_NAME    = modelname;
    ptcolumn->INSTANCE_NAME = insname;
    ptcolumn->WIDTH         = 0L;
    ptcolumn->X             = 0L;
    ptcolumn->INDEX         = 0L;
    ptcolumn->CON_NS        = NULL;
    ptcolumn->CON_EW        = NULL;
    ptcolumn->TYPE          = type;
    ptcolumn->LOINS         = NULL;

    return( ptcolumn );

} /* fin de addcolumn */

/*---------------------------------------------------------\
                   freecolumnlist
\---------------------------------------------------------*/
void freecolumnlist ( ptcolumnhead )
column *ptcolumnhead;
{
    column *ptcolumn;

    for( ptcolumn = ptcolumnhead; ptcolumn; ptcolumn = ptcolumnhead )
    {
        ptcolumnhead = ptcolumn->NEXT;

        free_con_NS_list( ptcolumn->CON_NS );
        free_con_EW_list( ptcolumn->CON_EW );
        freeslicelist   ( ptcolumn->SLICES );
        free            ( (char *)ptcolumn );
    }
} /* end of freecolumnlist */

/*---------------------------------------------------------\
                     addplacement
\---------------------------------------------------------*/
placement *addplacement( model_name )
char *model_name;
{
    placement *ptplacement;

    ptplacement = (placement *)mbkalloc( sizeof( placement ) );

    ptplacement->COLUMNS       = NULL;
    ptplacement->NB_SLICES     = 0L;
    ptplacement->SLICESHEIGHTS = NULL;
    ptplacement->YSLICE        = NULL;
    ptplacement->TYPES         = NULL;
    ptplacement->MODEL_NAME    = model_name;

    return( ptplacement );

} /* fin de addplacement */

/*---------------------------------------------------------\
                      freeplacement
\---------------------------------------------------------*/
void freeplacement ( ptplacement )
placement *ptplacement;
{
    freecolumnlist( ptplacement->COLUMNS );
    free( (char *)ptplacement->SLICESHEIGHTS );
    free( (char *)ptplacement->YSLICE );
    free( (char *)ptplacement );
} /* end of freeplacement */

