/*
 * This file is part of the Alliance CAD System
 * Copyright (C) Laboratoire LIP6 - Dpartement ASIM
 * Universite Pierre et Marie Curie
 * 
 * Home page          : http://www-asim.lip6.fr/alliance/
 * E-mail support     : mailto:alliance-support@asim.lip6.fr
 * 
 * This progam is  free software; you can redistribute it  and/or modify it
 * under the  terms of the GNU  General Public License as  published by the
 * Free Software Foundation;  either version 2 of the License,  or (at your
 * option) any later version.
 * 
 * Alliance VLSI  CAD System  is distributed  in the hope  that it  will be
 * useful, but WITHOUT  ANY WARRANTY; without even the  implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 * 
 * You should have received a copy  of the GNU General Public License along
 * with the GNU C Library; see the  file COPYING. If not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*********************************************************************\

    File        :   router_init_router.c

    Date        :   02 04 94
    Version     :   1

    Auteur      :   BEAURIN oliver

    Description :   Initialisation de la grille symbolique
                    pour le routage global.

\*********************************************************************/

/*---------------------------------------------------------\
                     include
\---------------------------------------------------------*/
#include MUT_H
#include MLO_H
#include MPH_H

#include "router_dpr.h"
#include "router_acces.h"
#include "router_load.h"

/*---------------------------------------------------------\
                     insertacces

   Recherche s'il faut inserer une nouvelle
   possibilite d'acces dans une case de la grille
   symbolique
\---------------------------------------------------------*/
void            insertacces (noeud, ptcon, ptpos, orient)
NODE           *noeud;
CONNECTOR      *ptcon;
POSSIBILITY    *ptpos;
char            orient;

{
    ACCESS         *acc1 = NULL, *acc2 = NULL, *acc3 = NULL;
    chain_list     *ptchain;

    for (ptchain = unused_signals; ptchain; ptchain = ptchain->NEXT)
        if (((SIGNAL *) ptcon->SIG->DATA)->INDEX == (long) ptchain->DATA)
            return;

    if (!noeud->DPR_RIGHT)
    {
        acc3 = mkacces ((ACCESS *) NULL, (ACCESS *) NULL, ptpos, orient, (SIGNAL *) ptcon->SIG->DATA, FALSE);
        noeud->DPR_LEFT = acc3;
        noeud->DPR_RIGHT = acc3;
    }
    else
    {
        for (acc1 = noeud->DPR_LEFT, acc2 = acc1; acc1; acc2 = acc1, acc1 = acc1->NEXT)
        {
            if (ptpos->X < acc1->POS->X)        /* position d'insertion
                                                 * detectee */
                break;
        }
        /* Insertions */
        if (acc2 == acc1)
        {
            /* Insertion en debut de liste */
            acc3 = mkacces (acc1, (ACCESS *) NULL, ptpos, orient, (SIGNAL *) ptcon->SIG->DATA, FALSE);
            acc1->PREV = acc3;
            noeud->DPR_LEFT = acc3;
        }
        else
        {
            if (acc1 == NULL)
            {
                /* Insertion en fin de liste */
                acc3 = mkacces ((ACCESS *) NULL, acc2, ptpos, orient, (SIGNAL *) ptcon->SIG->DATA, FALSE);
                acc2->NEXT = acc3;
                noeud->DPR_RIGHT = acc3;
            }
            else
            {
                /* Insertion en milieu de liste */
                acc3 = mkacces (acc1, acc2, ptpos, orient, (SIGNAL *) ptcon->SIG->DATA, FALSE);
                acc1->PREV = acc3;
                acc2->NEXT = acc3;
            }
        }
    }
}

/*---------------------------------------------------------\
                     put_in_grid

  Traduit pour une abscisse X donnee, toutes
  possibilites (eventuelles) d'un connecteur
  en structures ACCESS de la grille symbolique
  en vue du routage global
  Insere la possibilite minimum et la possibilite
  maximum (selon y) pour le X recherche
\---------------------------------------------------------*/
void            put_in_grid (noeud, y, ptcon)
NODE           *noeud;
long            y;
CONNECTOR      *ptcon;

{
    POSSIBILITY    *ptpos = NULL;
    POSSIBILITY    *ptpos_min = NULL;
    POSSIBILITY    *ptpos_max = NULL;
    chain_list     *ptcpos = NULL;

    for (ptcpos = ptcon->POS; ptcpos; ptcpos = ptcpos->NEXT)
    {
        ptpos = (POSSIBILITY *) ptcpos->DATA;
        if (ptpos->Y == y)
        {
            if ((ptpos->ACCES == WEST) || (ptpos->ACCES == EAST_WEST))
            {
                if (ptcon->TYPE == EXTERNAL)
                {
                    if ((ptpos_max == NULL) || (ptpos->X > ptpos_max->X))
                        ptpos_max = ptpos;
                }
                else
                {
                    if ((ptpos_min == NULL) || (ptpos->X < ptpos_min->X))
                        ptpos_min = ptpos;
                }
            }
            if ((ptpos->ACCES == EAST) || (ptpos->ACCES == EAST_WEST))
            {
                if (ptcon->TYPE == EXTERNAL)
                {
                    if ((ptpos_min == NULL) || (ptpos->X < ptpos_min->X))
                        ptpos_min = ptpos;
                }
                else
                {
                    if ((ptpos_max == NULL) || (ptpos->X > ptpos_max->X))
                        ptpos_max = ptpos;
                }
            }
        }
    }
    if (ptpos_min)
        insertacces (noeud, ptcon, ptpos_min, WEST);
    if (ptpos_max)
        insertacces (noeud, ptcon, ptpos_max, EAST);
}

/*---------------------------------------------------------\
                     getccon
\---------------------------------------------------------*/
chain_list     *getccon (ptcheadcon, ptccol, ptclin)
chain_list     *ptcheadcon;
chain_list     *ptccol;
chain_list     *ptclin;
{
    CONNECTOR      *ptcon;

    for (; ptcheadcon; ptcheadcon = ptcheadcon->NEXT)
    {
        ptcon = (CONNECTOR *) ptcheadcon->DATA;
        if ((ptcon->COL == ptccol) && (ptcon->LIN == ptclin))
            return ptcheadcon;
    }
    return NULL;
}

/*---------------------------------------------------------\
                     addnoeud
\---------------------------------------------------------*/
NODE           *addnoeud (pthead, nbr)
NODE           *pthead;
long            nbr;

{
    NODE           *ptnoeud;
    NODE           *ptnoeud_aux;

    ptnoeud = (NODE *) mbkalloc (sizeof (NODE));
    ptnoeud->NEXT = NULL;
    ptnoeud->PREV = NULL;
    ptnoeud->DPR_UP = NULL;
    ptnoeud->DPR_DWN = NULL;
    ptnoeud->DPR_RIGHT = NULL;
    ptnoeud->DPR_LEFT = NULL;

    ptnoeud_aux = ptnoeud;

    for (nbr = nbr - 1; nbr != 0; nbr--)
    {
        ptnoeud_aux->DPR_UP = (NODE *) mbkalloc (sizeof (NODE));
        ptnoeud_aux->DPR_UP->DPR_DWN = ptnoeud_aux;
        ptnoeud_aux = ptnoeud_aux->DPR_UP;
        ptnoeud_aux->DPR_UP = NULL;
        ptnoeud_aux->NEXT = NULL;
        ptnoeud_aux->PREV = NULL;
        ptnoeud_aux->DPR_RIGHT = NULL;
        ptnoeud_aux->DPR_LEFT = NULL;
    }

    for (ptnoeud_aux = ptnoeud; pthead; pthead = pthead->DPR_UP, ptnoeud_aux = ptnoeud_aux->DPR_UP)
    {
        pthead->NEXT = ptnoeud_aux;
        ptnoeud_aux->PREV = pthead;
    }

    return ptnoeud;
}

/*---------------------------------------------------------\
                     grille
\---------------------------------------------------------*/
NODE           *grille (ptccol, ptclin)
chain_list     *ptccol;
chain_list     *ptclin;

{
    chain_list     *ptccon;
    LINE           *ptlin = NULL;
    GABARIT        *ptgab = NULL;
    NODE           *pthead = NULL;
    NODE           *ptfilnoeud = NULL;
    NODE           *ptnoeud = NULL;
    TRACK          *pttrack;

    ptlin = (LINE *) ptclin->DATA;
    ptgab = ptlin->GAB;

    pthead = addnoeud (pthead, ptgab->NBTRACK - 1);
    ptfilnoeud = pthead;
    ptccon = getccon (ptlin->CON, ptccol, ptclin);
    for (; ptccon; ptccon = getccon (ptccon->NEXT, ptccol, ptclin))
    {
        pttrack = ptgab->TRACK->NEXT;
        for (ptnoeud = pthead; ptnoeud; ptnoeud = ptnoeud->DPR_UP, pttrack = pttrack->NEXT)
            put_in_grid (ptnoeud, pttrack->Y, (CONNECTOR *) ptccon->DATA);
    }

    for (ptccol = ptccol->NEXT->NEXT; ptccol->NEXT; ptccol = ptccol->NEXT->NEXT)
    {
        pthead = addnoeud (pthead, ptgab->NBTRACK - 1);
        ptccon = getccon (ptlin->CON, ptccol, ptclin);
        for (; ptccon; ptccon = getccon (ptccon->NEXT, ptccol, ptclin))
        {
            pttrack = ptgab->TRACK->NEXT;
            for (ptnoeud = pthead; ptnoeud; ptnoeud = ptnoeud->DPR_UP, pttrack = pttrack->NEXT)
                put_in_grid (ptnoeud, pttrack->Y, (CONNECTOR *) ptccon->DATA);
        }
    }

    /*
     * creation de la structure NODE pour les connecteur de la ligne RIGHT
     */
    pthead = addnoeud (pthead, ptgab->NBTRACK - 1);
    ptccon = getccon (ptlin->CON, ptccol, ptclin);
    for (; ptccon; ptccon = getccon (ptccon->NEXT, ptccol, ptclin))
    {
        pttrack = ptgab->TRACK->NEXT;
        for (ptnoeud = pthead; ptnoeud; ptnoeud = ptnoeud->DPR_UP, pttrack = pttrack->NEXT)
            put_in_grid (ptnoeud, pttrack->Y, (CONNECTOR *) ptccon->DATA);
    }

    return ptfilnoeud;
}

