/*
 * This file is part of the Alliance CAD System
 * Copyright (C) Laboratoire LIP6 - Dpartement ASIM
 * Universite Pierre et Marie Curie
 * 
 * Home page          : http://www-asim.lip6.fr/alliance/
 * E-mail support     : mailto:alliance-support@asim.lip6.fr
 * 
 * This progam is  free software; you can redistribute it  and/or modify it
 * under the  terms of the GNU  General Public License as  published by the
 * Free Software Foundation;  either version 2 of the License,  or (at your
 * option) any later version.
 * 
 * Alliance VLSI  CAD System  is distributed  in the hope  that it  will be
 * useful, but WITHOUT  ANY WARRANTY; without even the  implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 * 
 * You should have received a copy  of the GNU General Public License along
 * with the GNU C Library; see the  file COPYING. If not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*********************************************************************\

    Fichier     :    strecher_phcut.c

    Date        :    12 octobre 1995
    Version     :    2.0

    Auteur      :    CHAPUT Jean-Paul
    Modifie par :    BEAURIN Oliver

\*********************************************************************/

/*---------------------------------------------------------\
                     includes
\---------------------------------------------------------*/
#include  <stdio.h>
#include  <string.h>
#include  MPH_H
#include  MUT_H

#include  "strecher_support.h"
#include  "strecher_supply.h"
#include  "strecher_intv.h"
#include  "strecher_phcut.h"

/*---------------------------------------------------------\
                     defines
\---------------------------------------------------------*/
#define        C_PHCUT_SZ_BUFFER        8192

/*---------------------------------------------------------\
                     addophcut
\---------------------------------------------------------*/
phcut_list     *addophcut(pthead, ycut, pitchnb)
phcut_list     *pthead;         /* liste triee initiale       */
long            ycut;           /* ordonnee de la coupe       */
long            pitchnb;        /* nombre de pitchs a ajouter */
{
    phcut_list     *pNEW, *pHEAD, **ppcut;

    pHEAD = pthead;
    for (ppcut = &pHEAD; *ppcut != (phcut_list *) NULL;
         ppcut = &(*ppcut)->NEXT)
    {
        if ((*ppcut)->YCUT < ycut)
            continue;

        if ((*ppcut)->YCUT == ycut)
        {
            (*ppcut)->PITCHNB += pitchnb;
            return (pHEAD);
        }

        break;
    }

    pNEW = (phcut_list *) mbkalloc(sizeof(phcut_list));
    pNEW->YCUT = ycut;
    pNEW->PITCHNB = pitchnb;
    pNEW->NEXT = (*ppcut);

    (*ppcut) = pNEW;

    return (pHEAD);
}

/*---------------------------------------------------------\
                     dup_ls_phcut
\---------------------------------------------------------*/
phcut_list     *dup_ls_phcut(pthead)
phcut_list     *pthead;
{
    phcut_list     *pHEAD, *pLAST, *pNEW, *pcut;

    pHEAD = (phcut_list *) NULL;
    pLAST = (phcut_list *) NULL;

    for (pcut = pthead; pcut != (phcut_list *) NULL;
         pcut = pcut->NEXT)
    {
        pNEW = (phcut_list *) mbkalloc(sizeof(phcut_list));

        pNEW->YCUT = pcut->YCUT;
        pNEW->PITCHNB = pcut->PITCHNB;
        pNEW->NEXT = (phcut_list *) NULL;

        if (pLAST != (phcut_list *) NULL)
            pLAST->NEXT = pNEW;

        pLAST = pNEW;

        if (pcut == pthead)
            pHEAD = pNEW;
    }

    return (pHEAD);
} /* end of dup_ls_phcut */

/*---------------------------------------------------------\
                     cmp_ls_phcut
\---------------------------------------------------------*/
long            cmp_ls_phcut(pthead1, pthead2)
phcut_list     *pthead1, *pthead2;
{
    phcut_list     *pcut1, *pcut2;

    for (pcut1 = pthead1, pcut2 = pthead2;
         (pcut1 != (phcut_list *) NULL) && (pcut2 != (phcut_list *) NULL);
         pcut1 = pcut1->NEXT, pcut2 = pcut2->NEXT)
        if ((pcut1->YCUT != pcut2->YCUT) || (pcut1->PITCHNB != pcut2->PITCHNB))
            return (FALSE);

    if ((pcut1 == (phcut_list *) NULL) && (pcut2 == (phcut_list *) NULL))
        return (TRUE);
    else
        return (FALSE);
}

/*---------------------------------------------------------\
                     free_ls_phcut
\---------------------------------------------------------*/
void            free_ls_phcut(pthead)
phcut_list     *pthead;
{
    phcut_list     *pcut, *pDEL;

    pcut = pthead;
    while (pcut != (phcut_list *) NULL)
    {
        pDEL = pcut;
        pcut = pcut->NEXT;

        mbkfree((void *) pDEL);
    }
}

/*---------------------------------------------------------\
                     check_ls_phcut
\---------------------------------------------------------*/
phcut_list     *check_ls_phcut(ptcut, ptallow)
phcut_list     *ptcut;
intv_list      *ptallow;
{
    phcut_list     *pcut, *pHEAD;

    pHEAD = (phcut_list *) NULL;
    for (pcut = ptcut; pcut != (phcut_list *) NULL; pcut = pcut->NEXT)
        if (in_ls_intv(ptallow, pcut->YCUT, pcut->YCUT))
            pHEAD = addophcut(pHEAD, pcut->YCUT, pcut->PITCHNB);
        else
        {
            fflush( stdout );
            fprintf( stderr, "*** strecher warning *** : cut line rejected in %ld.\n", pcut->YCUT );
        }

    return (pHEAD);
}

/*---------------------------------------------------------\
                     ycut_calc
\---------------------------------------------------------*/
long            ycut_calc(pthead, ycut)
phcut_list     *pthead;
long            ycut;
{
    phcut_list     *pcut;
    long            ysum;

    ysum = ycut;
    for (pcut = pthead; pcut != (phcut_list *) NULL;
         pcut = pcut->NEXT)
    {
        if (ycut >= pcut->YCUT)
            ysum += pcut->PITCHNB * SX_PITCH_TALU2;
        else
            return (ysum);
    }

    return (ysum);
} /* end of ycut_calc */


/*---------------------------------------------------------\
                     ycut_norm
\---------------------------------------------------------*/
long            ycut_norm(ysupply_down, tsupply_down, ysupply_up, tsupply_up, ycut)
long            ysupply_down;
char            tsupply_down;
long            ysupply_up;
char            tsupply_up;
long            ycut;
{
    long            width_supply_down, width_supply_up, ynorm;

    /* Largeur de l'alimentation inferieure. */
    switch (tsupply_down)
    {
    case C_SUPPLY_VDD:
        width_supply_down = SX_WIDTH_VDD;
        break;
    case C_SUPPLY_VSS:
    default:
        width_supply_down = SX_WIDTH_VSS;
        break;
    }

    /* Largeur de l'alimentation superieure. */
    switch (tsupply_up)
    {
    case C_SUPPLY_VDD:
        width_supply_up = SX_WIDTH_VDD;
        break;
    case C_SUPPLY_VSS:
    default:
        width_supply_up = SX_WIDTH_VSS;
        break;
    }

    /* Normalisation au pitch superieur. */
    ynorm = ycut + (ysupply_up - width_supply_up / 2
                    + SX_WIDTH_TALU2 / 2 - ycut) % SX_PITCH_TALU2;

    if (ycut > ysupply_up - width_supply_up / 2)
    {
        ynorm = ysupply_up - width_supply_up / 2 + SX_WIDTH_TALU2 / 2;
    }

    if (ycut < ysupply_down + width_supply_down / 2)
    {
        ynorm = ysupply_down + SX_PITCH_TALU2 + width_supply_down / 2 - SX_WIDTH_TALU2 / 2;
    }

    return (ynorm);
} /* end of ycut_norm */


/*---------------------------------------------------------\
                     ycut_analys
\---------------------------------------------------------*/
long            ycut_analys(s_ycut)
char           *s_ycut;
{
    char           *ptr;

    if (!strcmp(s_ycut, "*"))
        return (C_PHCUT_FREE);
    if (!strcmp(s_ycut, "ref_ref"))
        return (C_PHCUT_REF);

    for (ptr = s_ycut; *ptr != (char) 0; ptr++)
        if ((*ptr < '0') || (*ptr > '9'))
            return (C_PHCUT_UNKNOW);

    return (C_PHCUT_USER);
} /* end of ycut_analys */
