/*
 * This file is part of the Alliance CAD System
 * Copyright (C) Laboratoire LIP6 - Dpartement ASIM
 * Universite Pierre et Marie Curie
 * 
 * Home page          : http://www-asim.lip6.fr/alliance/
 * E-mail support     : mailto:alliance-support@asim.lip6.fr
 * 
 * This progam is  free software; you can redistribute it  and/or modify it
 * under the  terms of the GNU  General Public License as  published by the
 * Free Software Foundation;  either version 2 of the License,  or (at your
 * option) any later version.
 * 
 * Alliance VLSI  CAD System  is distributed  in the hope  that it  will be
 * useful, but WITHOUT  ANY WARRANTY; without even the  implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 * 
 * You should have received a copy  of the GNU General Public License along
 * with the GNU C Library; see the  file COPYING. If not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*********************************************************************\

    Fichier     :    strecher_phtalu.c

    Date        :    12 octobre 1995
    Version     :    2.0

    Auteur      :    CHAPUT Jean-Paul
    Modifie par :    BEAURIN Oliver

\*********************************************************************/

/*---------------------------------------------------------\
                     includes
\---------------------------------------------------------*/
#include  <stdio.h>
#include  MPH_H
#include  MUT_H

#include  "dpr.h"
#include  "strecher_support.h"
#include  "strecher_supply.h"
#include  "strecher_intv.h"
#include  "strecher_phcma.h"
#include  "strecher_phtalu.h"

/*---------------------------------------------------------\
                     defines
\---------------------------------------------------------*/
#define        THROUGH_ALU1        1L
#define        THROUGH_ALU2        2L
#define        MAPPING_ALU1        3L
#define        MAPPING_ALU2        4L
#define         MAPCMA_ALU1        5L
#define         MAPCMA_ALU2        6L

/*---------------------------------------------------------\
                     structures
\---------------------------------------------------------*/
typedef struct
{
    long            X1, Y1, X2, Y2;
} rectangle;

/*---------------------------------------------------------\
                     macros
\---------------------------------------------------------*/
#define  maximum(a,b)  ( ((a)<(b)) ? (b) : (a) )
#define  access_intv   ( (intervalle*)(lPINTV->DATA) )

/*---------------------------------------------------------\
                     locals
\---------------------------------------------------------*/
static phfig_list *ptfig_TALU = (phfig_list *) NULL, *ptfig_MODIFY = (phfig_list *) NULL;

/*---------------------------------------------------------\
                     axiale
\---------------------------------------------------------*/
static long     axiale(p_RA, p_RB)
rectangle      *p_RA, *p_RB;
{
    long            DX, DY;

    /* Calcul de DX. */
    DX = (p_RB->X1 - p_RA->X2 > 0) ? (p_RB->X1 - p_RA->X2)
        : (p_RA->X1 - p_RB->X2);
    DX = (DX < 0) ? 0 : DX;

    /* Calcul de DY. */
    DY = (p_RA->Y1 - p_RB->Y2 > 0) ? (p_RA->Y1 - p_RB->Y2)
        : (p_RB->Y1 - p_RA->Y2);
    DY = (DY < 0) ? 0 : DY;

    return (maximum(DX, DY));
} /* end of axiale */

/*---------------------------------------------------------\
                     SegToRect
\---------------------------------------------------------*/
static void     SegToRect(p_R, pseg)
rectangle      *p_R;
phseg_list     *pseg;
{
    if (mc_IsHor(pseg->TYPE))
    {
        /* Conversion d'un segment horizontal. */
        p_R->X1 = pseg->X1;
        p_R->Y1 = pseg->Y1 - pseg->WIDTH / 2;

        p_R->X2 = pseg->X2;
        p_R->Y2 = pseg->Y1 + pseg->WIDTH / 2;
    }
    else
    {
        /* Conversion d'un segment vertical. */
        p_R->X1 = pseg->X1 - pseg->WIDTH / 2;
        p_R->Y1 = pseg->Y1;

        p_R->X2 = pseg->X2 + pseg->WIDTH / 2;
        p_R->Y2 = pseg->Y2;
    }
} /* end of SegToRect */

/*---------------------------------------------------------\
                     axiale_SegToSeg
\---------------------------------------------------------*/
static long     axiale_SegToSeg(psega, psegb)
phseg_list     *psega, *psegb;
{
    rectangle       ra, rb;

    /* Conversion des segments en rectangles. */
    SegToRect(&ra, psega);
    SegToRect(&rb, psegb);

    /* Calcul de la distance axiale entre les deux rectangles. */
    return (axiale(&ra, &rb));
} /* end of axiale_SegToSeg */

/*---------------------------------------------------------\
                     viatoseg
\---------------------------------------------------------*/
static phseg_list *viatoseg(pvia, pseg, LAYER)
phvia_list     *pvia;
phseg_list     *pseg;
char            LAYER;
{
    long            side_via;

    /* Controle du LAYER de conversion. */
    switch (LAYER)
    {
    case ALU1:
        side_via = SX_SIDE_VIA1;
        break;
    case ALU2:
        side_via = SX_SIDE_VIA2;
        break;
    default:
        fflush( stdout );
        fprintf( stderr, "*** strecher bugs *** : in viatoseg, layer is not ALU1 nor ALU2, i'am sorry.\n" );
        exit( 1 );
    }

    /* Conversion. */
    pseg->LAYER = LAYER;
    pseg->TYPE = UP;
    pseg->X1 = pvia->XVIA;
    pseg->Y1 = pvia->YVIA - side_via / 2;
    pseg->X2 = pvia->XVIA;
    pseg->Y2 = pvia->YVIA + side_via / 2;
    pseg->WIDTH = side_via;
    pseg->NAME = (char *) NULL;
    pseg->NEXT = (phseg_list *) NULL;
    pseg->USER = (ptype_list *) NULL;

    return (pseg);
} /* end of viatoseg */


/*---------------------------------------------------------\
                     addphtalu
\---------------------------------------------------------*/
static phtalu_list *addphtalu(pthead, type, x1, x2, ytalu, width, pcma)
phtalu_list    *pthead;
long            type;
long            x1, x2, ytalu, width;
phcma_list     *pcma;
{
    phtalu_list    *ptalu;

    ptalu = (phtalu_list *) mbkalloc(sizeof(phtalu_list));

    /* Initialisation des champs. */
    ptalu->TYPE = type;
    ptalu->X1 = x1;
    ptalu->X2 = x2;
    ptalu->YTALU = ytalu;
    ptalu->WIDTH = width;
    ptalu->NBREF = 0;
    ptalu->PPHSEG = (ptype_list *) NULL;
    ptalu->PPHREF = (ptype_list *) NULL;
    ptalu->INTV = addintv((intv_list *) NULL, x1, x2);
    ptalu->PHCMA = pcma;
    ptalu->NEXT = pthead;

    return (ptalu);
} /* end of addphtalu */


/*---------------------------------------------------------\
                     getphtalu
\---------------------------------------------------------*/
static phtalu_list *getphtalu(pthead, type, ytalu, pcma)
phtalu_list    *pthead;
long            type, ytalu;
phcma_list     *pcma;
{
    phtalu_list    *ptalu;

    for (ptalu = pthead; ptalu != (phtalu_list *) NULL;
         ptalu = ptalu->NEXT)
        if ((ptalu->TYPE == type)
            && (ptalu->YTALU == ytalu)
            && (ptalu->PHCMA == pcma))
            return (ptalu);

    return ((phtalu_list *) NULL);
} /* end of getphtalu */


/*---------------------------------------------------------\
                     alloc_ls_talu
\---------------------------------------------------------*/
void            alloc_ls_talu(ptfig, ybase, NB_transp, user_ptype)
phfig_list     *ptfig;
long            ybase, NB_transp, user_ptype;
{
    phcma_list     *pcma;
    ptype_list     *ptuser1, *ptuser2;
    long            i;

    if (user_ptype & F_PHTALU_TALU2)
    {
        /* Controle de l'existence des structures d'acceuil. */
        ptuser1 = get_ls_data(ptfig, C_SUPP_UTALU2, F_SUPP_UCREAT);

        /* Boucle de creation des transparences TALU2. */
        for (i = 0; i < 2 * NB_transp; i++)
        {
            ptuser1->DATA = (void *) addphtalu(
                                               (phtalu_list *) ptuser1->DATA,
                                               (i % 2) ? THROUGH_ALU2 : MAPPING_ALU2,
                                               ptfig->XAB1 - 3 * SX_DIST_ALU2,
                                               ptfig->XAB2 + 3 * SX_DIST_ALU2,
                                               ybase + (i / 2) * SX_PITCH_TALU2,
                                               (i % 2) ? SX_WIDTH_TALU2 : SX_SIDE_VIA2,
                                               (phcma_list *) NULL);
        }
    }

    if (user_ptype & F_PHTALU_CMA)
    {
        /* Controle de l'existence des structures d'accueil. */
        ptuser1 = get_ls_data(ptfig, C_SUPP_UTALU1, F_SUPP_UCREAT);

        if ((ptuser2 = get_ls_data(ptfig, C_SUPP_UCMA, FALSE)) == (ptype_list *) NULL)
            return;

        /* Boucle sur les transparences ALU1. */
        for (i = 0; i < NB_transp; i++)
        {
            /* Allocation du segment par defaut en ALU1. */
            ptuser1->DATA = (void *) addphtalu(
                                               (phtalu_list *) ptuser1->DATA,
                                               MAPPING_ALU1,
                                               ptfig->XAB1 - 3 * SX_DIST_ALU1,
                                               ptfig->XAB2 + 3 * SX_DIST_ALU1,
                                               ybase + i * SX_PITCH_TALU1,
                                               SX_SIDE_VIA1,
                                               (phcma_list *) NULL);

            /* Boucle sur les CMAs. */
            for (pcma = (phcma_list *) ptuser2->DATA;
                 pcma != (phcma_list *) NULL;
                 pcma = pcma->NEXT)
            {
                ptuser1->DATA = (void *) addphtalu(
                                                   (phtalu_list *) ptuser1->DATA,
                                                   MAPCMA_ALU1,
                                                   ptfig->XAB1 - 3 * SX_DIST_ALU1,
                                                   ptfig->XAB2 + 3 * SX_DIST_ALU1,
                                                   ybase + i * SX_PITCH_TALU1,
                                                   SX_SIDE_VIA1,
                                                   pcma);
            }
        }
    }
} /* end of alloc_ls_talu */

/*---------------------------------------------------------\
                     trunc_ls_talu
\---------------------------------------------------------*/
void            trunc_ls_talu(ptfig)
phfig_list     *ptfig;
{
    phtalu_list    *ptalu;
    ptype_list     *ptuser1;

    /* Controle de l'existence des structures d'acceuil. */
    if ((ptuser1 = get_ls_data(ptfig, C_SUPP_UTALU2, FALSE)) != (ptype_list *) NULL)
        /* Boucle sur les transparences ALU2. */
        for (ptalu = (phtalu_list *) ptuser1->DATA;
             ptalu != (phtalu_list *) NULL;
             ptalu = ptalu->NEXT)
            /* Troncature des intervalles. */
            ptalu->INTV = trunc_ls_intv(ptalu->INTV, ptfig->XAB1, ptfig->XAB2);
}

/*---------------------------------------------------------\
                     ls_talutoseg
\---------------------------------------------------------*/
void            ls_talutoseg(ptfig)
phfig_list     *ptfig;
{
    phtalu_list    *ptalu;
    ptype_list     *ptuser1;

    /* Controle de l'existence des structures d'acceuil. */
    if ((ptuser1 = get_ls_data(ptfig, C_SUPP_UTALU2, FALSE)) != (ptype_list *) NULL)
        /* Boucle sur les transparences ALU2. */
        for (ptalu = (phtalu_list *) ptuser1->DATA;
             ptalu != (phtalu_list *) NULL;
             ptalu = ptalu->NEXT)
            if (ptalu->TYPE == THROUGH_ALU2)
                ls_intvtoseg(ptalu->INTV, ptfig, TALU2, ptalu->WIDTH, ptalu->YTALU);
}

/*---------------------------------------------------------\
                     fmerge_ls_talu
\---------------------------------------------------------*/
static long     fmerge_ls_talu(pthead1, pthead2)
phtalu_list    *pthead1;
phtalu_list    *pthead2;
{
    ptype_list     *ptuser;
    phtalu_list    *ptalu_MDFY_TALU1, *ptalu_MDFY_TALU2;
    phtalu_list    *ptalu_MERGE, *ptalu;
    phcma_list     *pcma_merge = NULL;
    phcma_link_list *pcma_link;
    long            rv;

    ptalu_MDFY_TALU1 = (phtalu_list *) NULL;
    ptalu_MDFY_TALU2 = (phtalu_list *) NULL;
    rv = TRUE;

    /* Controle de l'existence des structures d'accueil. */
    if ((ptuser = get_ls_data(ptfig_MODIFY, C_SUPP_UTALU1, FALSE))
                != (ptype_list *) NULL)
        ptalu_MDFY_TALU1 = (phtalu_list *) ptuser->DATA;

    /* Controle de l'existence des structures d'accueil. */
    if ((ptuser = get_ls_data(ptfig_MODIFY, C_SUPP_UTALU2, FALSE))
                != (ptype_list *) NULL)
        ptalu_MDFY_TALU2 = (phtalu_list *) ptuser->DATA;

    /* Fusion des gabarits ALU2. */
    for (ptalu = pthead2; ptalu != (phtalu_list *) NULL;
         ptalu = ptalu->NEXT)
    {
        /* Ne pas considerer les transparences qui ne sont pas "au dessus" de la cellule feuille. */
        if ((ptalu->YTALU > ptfig_MODIFY->YAB2) || (ptalu->YTALU < ptfig_MODIFY->YAB1))
            continue;

        /* Recherche du gabarit a fusionner. */
        ptalu_MERGE = getphtalu(ptalu_MDFY_TALU2, ptalu->TYPE, ptalu->YTALU, (void *) NULL);

        /* Si le gabarit a ete trouve, fusion des intervalles. */
        if (ptalu_MERGE != (phtalu_list *) NULL)
            ptalu->INTV = merge_ls_intv(ptalu->INTV, ptalu_MERGE->INTV);
        else
            rv = FALSE;
    }

    /* Fusion des gabarits ALU1. */
    for (ptalu = pthead1; ptalu != (phtalu_list *) NULL;
         ptalu = ptalu->NEXT)
    {
        /* Ne pas considerer les transparences qui ne sont pas "au dessus" de la cellule feuille. */
        if ((ptalu->YTALU > ptfig_MODIFY->YAB2) || (ptalu->YTALU < ptfig_MODIFY->YAB1))
            continue;

        if (ptalu->PHCMA != (phcma_list *) NULL)
        {
            /* Recherche du LINK associe au CMA. */
            for (pcma_link = ptalu->PHCMA->LINK;
                 pcma_link != (phcma_link_list *) NULL;
                 pcma_link = pcma_link->NEXT)
            {
                /* Verification de la coincidence entre le modele mis a plat et l'instance. */
                if ((pcma_link->PHFIG != ptfig_MODIFY)
                    || (pcma_link->XFLAT != ptfig_MODIFY->XAB1)
                    || (pcma_link->YFLAT != ptfig_MODIFY->YAB1))
                    continue;

                /* CMA connecte a la transparence. */
                pcma_merge = pcma_link->PHCMA;
            }
        }
        else
            pcma_merge = (phcma_list *) NULL;

        /* Recherche du gabarit a fusionner. */
        ptalu_MERGE = getphtalu(ptalu_MDFY_TALU1, ptalu->TYPE, ptalu->YTALU, pcma_merge);

        /* Si le gabarit a ete trouve, fusion des intervalles. */
        if (ptalu_MERGE != (phtalu_list *) NULL)
        {
            ptalu->INTV = merge_ls_intv(ptalu->INTV, ptalu_MERGE->INTV);
        }
        else
            rv = FALSE;
    }

    return (rv);
} /* end of fmerge_ls_talu */

/*---------------------------------------------------------\
                     fmodify_ls_talu
\---------------------------------------------------------*/
static void     fmodify_ls_talu(p_S, pthead)
phseg_list     *p_S;
phtalu_list    *pthead;
{
    phseg_list      seg_talu;
    phtalu_list    *ptalu;
    phcma_link_list *pcma_link;
    long            distance, dmin = 0, flag_owned;

    if (pthead == (phtalu_list *) NULL)
        return;

    /* Initialisation generale du segment transparence. */
    switch (pthead->TYPE)
    {
    case THROUGH_ALU1:;
    case MAPPING_ALU1:;
    case MAPCMA_ALU1:
        seg_talu.LAYER = ALU1;
        dmin = SX_DIST_ALU1;
        break;
    case THROUGH_ALU2:;
    case MAPPING_ALU2:;
    case MAPCMA_ALU2:
        seg_talu.LAYER = ALU2;
        dmin = SX_DIST_ALU2;
        break;
    }
    seg_talu.TYPE = LEFT;
    seg_talu.NAME = (char *) NULL;
    seg_talu.NEXT = (phseg_list *) NULL;

    /* Boucle sur les transparences. */
    for (ptalu = pthead; ptalu != (phtalu_list *) NULL;
         ptalu = ptalu->NEXT)
    {
        /* Initialisation du segment transparence ALU1. */
        seg_talu.X1 = ptalu->X1;
        seg_talu.X2 = ptalu->X2;
        seg_talu.WIDTH = ptalu->WIDTH;
        seg_talu.Y1 = seg_talu.Y2 = ptalu->YTALU;

        /* Distance transparence-segment. */
        distance = axiale_SegToSeg(&seg_talu, p_S);

        /* Segment trop pres ? */
        if (distance >= dmin)
            continue;

        /* Segment ALU1 appartient a un CMA. */
        if (ptalu->PHCMA != (phcma_list *) NULL)
        {
            flag_owned = FALSE;

            /* Appartient au CMA de ce niveau. */
            if (ptalu->PHCMA == (phcma_list *) p_S->USER)
                if (isconnect(p_S->X1, p_S->Y1,
                              p_S->X2, p_S->Y2,
                              seg_talu.X1, seg_talu.Y1,
                              seg_talu.X2, seg_talu.Y2))
                {
                    /* Liste des segments ALU1 connectes. */
                    ptalu->PPHSEG = addptype(ptalu->PPHSEG, PTYPE_LSEG, (void *) p_S);
                    flag_owned = TRUE;
                }

            /* Appartient au LINK du CMA. */
            for (pcma_link = ptalu->PHCMA->LINK;
                 pcma_link != (phcma_link_list *) NULL;
                 pcma_link = pcma_link->NEXT)
                if (pcma_link->PHCMA == (phcma_list *) p_S->USER)
                    if (isconnect(p_S->X1, p_S->Y1,
                                  p_S->X2, p_S->Y2,
                                  seg_talu.X1, seg_talu.Y1,
                                  seg_talu.X2, seg_talu.Y2))
                        flag_owned = TRUE;

            if (flag_owned)
                continue;
        }


        if (mc_IsHor(p_S->TYPE))
            ptalu->INTV = modify_ls_intv(ptalu->INTV,
                                         p_S->X1 - dmin,
                                         p_S->X2 + dmin);
        else
            ptalu->INTV = modify_ls_intv(ptalu->INTV,
                                         p_S->X1 - p_S->WIDTH / 2 - dmin,
                                         p_S->X1 + p_S->WIDTH / 2 + dmin);
    }
} /* end of fmodify_ls_talu */

/*---------------------------------------------------------\
                     modify_ls_talu
\---------------------------------------------------------*/
void            modify_ls_talu(a_ptfig_TALU, a_ptfig_MODIFY, user_ptype)
phfig_list     *a_ptfig_TALU;
phfig_list     *a_ptfig_MODIFY;
long            user_ptype;
{
    phseg_list     *pseg, seg_via;
    phvia_list     *pvia;
    ptype_list     *ptuser_TALU2, *ptuser_TALU1;
    phtalu_list    *ptalu_TALU1, *ptalu_TALU2;

    ptfig_TALU = a_ptfig_TALU;
    ptfig_MODIFY = a_ptfig_MODIFY;
    ptalu_TALU1 = (phtalu_list *) NULL;
    ptalu_TALU2 = (phtalu_list *) NULL;

    if (user_ptype & F_PHTALU_TALU2)
        /* Controle de l'existence des structures d'accueil. */
        if ((ptuser_TALU2 = get_ls_data(ptfig_TALU, C_SUPP_UTALU2, FALSE))
                    != (ptype_list *) NULL)
            ptalu_TALU2 = (phtalu_list *) ptuser_TALU2->DATA;

    if (user_ptype & F_PHTALU_CMA)
        /* Controle de l'existence des structures d'accueil. */
        if ((ptuser_TALU1 = get_ls_data(ptfig_TALU, C_SUPP_UTALU1, FALSE))
                    != (ptype_list *) NULL)
            ptalu_TALU1 = (phtalu_list *) ptuser_TALU1->DATA;

    /* Tentative de court-circuit du VRD par fusion directe des gabarits des transparences. */
    if ((ptfig_TALU != ptfig_MODIFY) && (fmerge_ls_talu(ptalu_TALU1, ptalu_TALU2)))
        return;

    for (pseg = ptfig_MODIFY->PHSEG; pseg != (phseg_list *) NULL;
         pseg = pseg->NEXT)
    {
        switch (pseg->LAYER)
        {
/*\
PATH ATTENTION :
J'ai enlever le cas des tranaparences : si on streche une colonne
qui contient une cellule deja strechees, les tranparences sont deja
posee et fmodify_ls_talu modifie l'intervalle de possibilite.
Il faut verifier que cela n'a pas de consequences mais je ne pense pas

                RobiN
\*/
        case ALU1:
            fmodify_ls_talu(pseg, ptalu_TALU1);
            break;
        case ALU2:
            fmodify_ls_talu(pseg, ptalu_TALU2);
            break;
        }
    }

    for (pvia = ptfig_MODIFY->PHVIA; pvia != (phvia_list *) NULL;
         pvia = pvia->NEXT)
    {
        switch (pvia->TYPE)
        {
        case CONT_VIA2:
            viatoseg(pvia, &seg_via, ALU2);
            fmodify_ls_talu(&seg_via,
                            ptalu_TALU2);
            break;
        case CONT_VIA:
            viatoseg(pvia, &seg_via, ALU2);
            fmodify_ls_talu(&seg_via,
                            ptalu_TALU2);
        case CONT_DIF_P:;
        case CONT_DIF_N:;
        case CONT_BODY_P:;
        case CONT_BODY_N:;
        case CONT_POLY:
            viatoseg(pvia, &seg_via, ALU1);
            fmodify_ls_talu(&seg_via,
                            ptalu_TALU1);
            break;
        }
    }
} /* end of modify_ls_talu */

/*---------------------------------------------------------\
                     check_refcon
\---------------------------------------------------------*/
static void     check_refcon(pthead, start_damier)
ptype_list     *pthead;
long            start_damier;
{
    /* Macro de recherche du prochain "ref_con" de la liste. ( on ne considere pas les "ref_ref" ) */
#define  mc_nextrefcon(pphead,ppnext)                    \
        for(    (ppnext)=(pphead);                           \
                   ( (ppnext) != (ptype_list*)NULL)              \
                 &&( ((phref_list*)((ppnext)->DATA))->FIGNAME    \
                                                                                  != p_refcon);  \
                        (ppnext)=(ppnext)->NEXT )

#define  access_ref   ( (phref_list*)( pphref->DATA) )
#define  access_ref1  ( (phref_list*)(pphref1->DATA) )
#define  access_ref2  ( (phref_list*)(pphref2->DATA) )

    ptype_list     *pphref, *pphref1,   /* "ref_con" de XREF inferieur. */
                   *pphref2;    /* "ref_con" de XREF superieur. */
    long            damier;
    char           *p_refref, *p_refcon;

    /* Securite : sur-liste vide. */
    if (pthead == (ptype_list *) NULL)
        return;

    p_refref = namealloc("ref_ref");
    p_refcon = namealloc("ref_con");
    damier = start_damier;

    /* Recherche du premier "ref_con". */
    mc_nextrefcon(pthead, pphref1);
    /* Aucun element dans la liste. */
    if ((pphref1 == (ptype_list *) NULL)
        || (access_ref1->FIGNAME != p_refcon))
        return;


    /* Recherche du second "ref_con". */
    mc_nextrefcon(pphref1->NEXT, pphref2);
    /* Pas de second element dans la liste. */
    if ((pphref2 == (ptype_list *) NULL)
        || (access_ref2->FIGNAME != p_refcon))
        return;


    while (TRUE)
    {
        /* Test de la distance VIA2-VIA2. */
        if ((access_ref2->XREF
             - access_ref1->XREF) <
            (SX_DIST_ALU2 + SX_SIDE_VIA2))
        {
            if (damier % 2)
            {
                /* Passage de REF1 en "ref_ref" ("derriere"). */
                access_ref1->FIGNAME = p_refref;
            }
            else
            {
                /* Recherche du "ref_con" suivant. */
                mc_nextrefcon(pphref2->NEXT, pphref);

                /* Passage de REF2 en "ref_ref" ("devant"). */
                access_ref2->FIGNAME = p_refref;

                /* Plus de "ref_con" dans la liste. */
                if ((pphref == (ptype_list *) NULL)
                    || (access_ref->FIGNAME != p_refcon))
                    return;

                pphref2 = pphref;
            }
            damier += 1;
        }
        else
            damier = start_damier;

        /* Recherche du prochain couple de references a comparer. */
        pphref1 = pphref2;
        mc_nextrefcon(pphref2->NEXT, pphref2);
        /* Plus de "ref_con" dans la liste. */
        if ((pphref2 == (ptype_list *) NULL)
            || (access_ref2->FIGNAME != p_refcon))
            return;
    }

#undef  mc_nextrefcon
#undef  access_ref
#undef  access_ref1
#undef  access_ref2
} /* end of check_refcon */

/*---------------------------------------------------------\
                     check_ls_through
\---------------------------------------------------------*/
void            check_ls_through(a_ptfig)
phfig_list     *a_ptfig;
{
    ptype_list     *ptuser2;
    phtalu_list    *ptalu_TALU2, *ptalu2;
    long            value;

    /* Recuperation des gabarits ALU2, si pas de gabarits ALU2, on ne peut rien faire ==> retour. */
    if ((ptuser2 = get_ls_data(a_ptfig, C_SUPP_UTALU2, FALSE))
        != (ptype_list *) NULL)
        ptalu_TALU2 = (phtalu_list *) ptuser2->DATA;
    else
        return;

    /* Verification des distances VIA-VIA. */
    value = 0;
    for (ptalu2 = ptalu_TALU2; ptalu2 != (phtalu_list *) NULL;
         ptalu2 = ptalu2->NEXT)
        if (ptalu2->TYPE == MAPPING_ALU2)
            check_refcon(ptalu2->PPHREF, value++);

} /* end of check_ls_through */


/*---------------------------------------------------------\
                     clean_ls_through
\---------------------------------------------------------*/
void clean_ls_through(a_ptfig)
phfig_list *a_ptfig;
{
#define  access_ref  ( (phref_list*)(pphref->DATA) )

    ptype_list     *ptuser2, *pphref;
    phtalu_list    *ptalu_TALU2, *ptalu2;
    char           *p_refcon;

    p_refcon = namealloc("ref_con");

    /* Recuperation des gabarits ALU2, si pas de gabarits ALU2, on ne peut rien faire ==> retour. */
    if ((ptuser2 = get_ls_data(a_ptfig, C_SUPP_UTALU2, FALSE)) != (ptype_list *) NULL)
        ptalu_TALU2 = (phtalu_list *) ptuser2->DATA;
    else
        return;

    /* Suppression des "ref_ref" devenus inutiles. */
    for (ptalu2 = ptalu_TALU2; ptalu2 != (phtalu_list *) NULL;
         ptalu2 = ptalu2->NEXT)
    {
        pphref = ptalu2->PPHREF;

        while (pphref != (ptype_list *) NULL)
            if (access_ref->FIGNAME != p_refcon)
            {
                /* Destruction de la reference. */
                (void) delphref(a_ptfig, access_ref);

                /* Destruction du pointeur dans la sur-liste. */
                pphref = delptype(pphref, pphref->TYPE);
            }
            else
                pphref = pphref->NEXT;
    }

#undef  access_ref
} /* end of clean_ls_through */

/*---------------------------------------------------------\
                     add_ls_through
\---------------------------------------------------------*/
void            add_ls_through(a_ptfig, ptsupply, user_ptype)
phfig_list     *a_ptfig;
supply_list    *ptsupply;
long            user_ptype;
{
#define  access_seg       ( ( phseg_list*)(  ppseg->DATA) )
#define  access_ref      ( (phref_list*)(pphref->DATA) )

    ptype_list     *ptuser1, *ptuser2, *ppseg;
    ptype_list     *pphref;
    phtalu_list    *ptalu_TALU1, *ptalu_TALU2, *ptalu1, *ptalu2;
    supply_list    *pslice;
    phcma_list     *pcma_TALU1, *pcma;
    phref_list     *pref;
    long            NB_ref;
    char           *type_refcon, *type_refref, *type_ref, name_ref[MBK_SZ_NAME];

    type_refcon = namealloc("ref_con");
    type_refref = namealloc("ref_ref");

    /* Recuperation des gabarits ALU2, si pas de gabarits ALU2, on ne peut rien faire ==> retour. */
    if ((ptuser2 = get_ls_data(a_ptfig, C_SUPP_UTALU2, FALSE)) != (ptype_list *) NULL)
    {
        ptalu_TALU2 = (phtalu_list *) ptuser2->DATA;

        /* Si demande, conversion des gabarits en segments de transparence ALU2. */
        if (user_ptype & F_PHTALU_TALU2)
        {
            trunc_ls_talu(a_ptfig);
            ls_talutoseg(a_ptfig, F_PHTALU_TALU2);
        }
    }
    else
    {
        return;
    }

    if (user_ptype & F_PHTALU_CMA)
    {
        /* Recuperation de la liste des CMAs. */
        if ((ptuser1 = get_ls_data(a_ptfig, C_SUPP_UCMA, FALSE)) != (ptype_list *) NULL)
            pcma_TALU1 = (phcma_list *) ptuser1->DATA;
        else
        {
            return;
        }
        /* Recuperation de la liste des gabarits ALU1. */
        if ((ptuser1 = get_ls_data(a_ptfig, C_SUPP_UTALU1, FALSE)) != (ptype_list *) NULL)
            ptalu_TALU1 = (phtalu_list *) ptuser1->DATA;
        else
        {
            return;
        }
    }
    else
    {
        return;
    }

    /* Boucle de niveau 0: transparences ALU2. */
    for (ptalu2 = ptalu_TALU2; ptalu2 != (phtalu_list *) NULL; ptalu2 = ptalu2->NEXT)
    {
        /* On n'examine que les gabarits de pose de VIA. ( gabarits de type "MAPPING_ALU2" ) */
        if (ptalu2->TYPE != MAPPING_ALU2)
        {
            continue;
        }
        pslice = getnsupply(ptsupply, ptalu2->YTALU);

        /* Precaution : elimination de toute liste precedemment posee ( effacer aussi les "ref_con" ). */
        if (ptalu2->PPHREF != (ptype_list *) NULL)
        {
            freeptype(ptalu2->PPHREF);
            ptalu2->PPHREF = (ptype_list *) NULL;
        }

        /* Boucle de niveau 1: sur les CMAs */
        for (pcma = pcma_TALU1; pcma != (phcma_list *) NULL;
             pcma = pcma->NEXT)
        {
            /* Recuperation du gabarit du CMAs pour l'ordonnee YTALU ( celle de l'ALU2 courant ). */
            ptalu1 = getphtalu(ptalu_TALU1, MAPCMA_ALU1, ptalu2->YTALU, pcma);
            if (ptalu1 == (phtalu_list *) NULL)
            {
                continue;
            }
            /* Nombre de "ref_con" ajoutes, pour ce CMA sur cette transparence. */
            NB_ref = ptalu1->NBREF;

            /* Boucle de niveau 2: segments ALU1 secants au CMA. */
            for (ppseg = ptalu1->PPHSEG; ppseg != (ptype_list *) NULL; ppseg = ppseg->NEXT)
            {
                /* Test de l'encombrement d'un VIA au point d'intersection transparence - segment. */
                if (in_ls_intv(ptalu1->INTV,
                               access_seg->X1 - SX_SIDE_VIA1 / 2,
                               access_seg->X1 + SX_SIDE_VIA1 / 2)
                    && in_ls_intv(ptalu2->INTV,
                                  access_seg->X1 - SX_SIDE_VIA2 / 2,
                                  access_seg->X1 + SX_SIDE_VIA2 / 2))
                {
                    /* Type de la nouvelle reference. */
                    if ((pcma->TYPE == C_PHCMA_CMA) && (user_ptype & F_PHTALU_CMA))
                        type_ref = type_refcon;
                    else
                        type_ref = type_refref;
                }
                else
                    type_ref = type_refref;

                if( ( pphref = pcma->PPHREF ) && ( access_ref ) )
                {
                    if( pslice != getnsupply(ptsupply, access_ref->YREF ) )
                    {
                        fprintf(stderr,"*** strecher warning *** : Reference not add in %s (not in main slice).\n",
                                a_ptfig->NAME );
                        continue;
                    }
                }

                /* Nom de la nouvelle reference. */
                sprintf(name_ref, "%s_%ld", pcma->NAME, index_calc(pslice, NB_ref, ptalu2->YTALU));
                NB_ref += 1;

                pref = addphref(a_ptfig, type_ref, name_ref, access_seg->X1, ptalu2->YTALU);
                pref->USER = (ptype_list *) pcma;

                /* Addition a la liste de tous les "ref_con" et "ref_ref", tous CMAs confondus. */
                ptalu2->PPHREF = addoptype(ptalu2->PPHREF, pref->XREF, (void *) pref);
            }
            /* Fin de la boucle 2: */

            ptalu1->NBREF = NB_ref;
        }
        /* Fin de la boucle 1: */
    }

#undef  access_seg
#undef  access_ref
}

/*---------------------------------------------------------\
                     export_ls_through
\---------------------------------------------------------*/
void            export_ls_through(ptfig_father, ptfig_son, atransf_son)
phfig_list     *ptfig_father;
phfig_list     *ptfig_son;
char            atransf_son;
{
#define  access_seg       ( ( phseg_list*)(  ppseg->DATA) )
#define  access_ref       ( ( phref_list*)(  ppref->DATA) )
#define  access_s_supply  ( (supply_list*)(ptuser3->DATA) )

    ptype_list     *ptuser1, *ptuser2, *ptuser3, *ptuser2_link, *ppref;
    phtalu_list    *ptalu_TALU1, *ptalu_TALU2, *ptalu_TALU2_link, *ptalu1, *ptalu2, *ptalu2_link;
    supply_list    *pslice;
    phcma_list     *pcma_TALU1, *pcma;
    phcma_link_list *pcmalink;
    phref_list     *pref;
    long            NB_ref;
    char           *type_refcon, *type_refref, *type_ref, name_ref[MBK_SZ_NAME];

    type_refcon = namealloc("ref_con");
    type_refref = namealloc("ref_ref");

    /* Recuperation des gabarits ALU2 du LINK, si pas de gabarits ALU2, on ne peut rien faire ==> retour. */
    if ((ptuser2_link = get_ls_data(ptfig_son, C_SUPP_UTALU2, FALSE))
                != (ptype_list *) NULL)
        ptalu_TALU2_link = (phtalu_list *) ptuser2_link->DATA;
    else
        return;

    /* Recuperation des gabarits ALU2, si pas de gabarits ALU2, on ne peut rien faire ==> retour. */
    if ((ptuser2 = get_ls_data(ptfig_father, C_SUPP_UTALU2, FALSE))
                != (ptype_list *) NULL)
        ptalu_TALU2 = (phtalu_list *) ptuser2->DATA;
    else
        return;

    /* Recuperation de la liste des CMAs. ( presence obigatoire ) */
    if ((ptuser1 = get_ls_data(ptfig_father, C_SUPP_UCMA, FALSE))
                != (ptype_list *) NULL)
        pcma_TALU1 = (phcma_list *) ptuser1->DATA;
    else
        return;

    /* Recuperation de la liste des gabarits ALU1. */
    if ((ptuser1 = get_ls_data(ptfig_father, C_SUPP_UTALU1, FALSE))
                != (ptype_list *) NULL)
        ptalu_TALU1 = (phtalu_list *) ptuser1->DATA;
    else
        return;


    if ((ptuser3 = get_ls_data(ptfig_father, C_SUPP_USUPPLY, FALSE))
                == (ptype_list *) NULL)
        return;

    /* Boucle de niveau 0: transparences ALU2. */
    for (ptalu2 = ptalu_TALU2; ptalu2 != (phtalu_list *) NULL;
         ptalu2 = ptalu2->NEXT)
    {
        /* On n'examine que les gabarits de pose de VIA. ( gabarits de type "MAPPING_ALU2" ) */
        if (ptalu2->TYPE != MAPPING_ALU2)
            continue;

        pslice = getnsupply(access_s_supply, ptalu2->YTALU);

        /* Boucle de niveau 1: sur les CMAs */
        for (pcma = pcma_TALU1; pcma != (phcma_list *) NULL;
             pcma = pcma->NEXT)
        {
            /* Recuperation du gabarit du CMAs pour l'ordonnee YTALU ( celle de l'ALU2 courant ). */
            ptalu1 = getphtalu(ptalu_TALU1,
                               MAPCMA_ALU1,
                               ptalu2->YTALU,
                               pcma);
            if (ptalu1 == (phtalu_list *) NULL)
                continue;

            /* Nombre de "ref_con" ajoutes, pour ce CMA sur cette transparence. */
            NB_ref = ptalu1->NBREF;

            /* Boucle de niveau 2: sur les LINK. */
            for (pcmalink = pcma->LINK;
                 pcmalink != (phcma_link_list *) NULL;
                 pcmalink = pcmalink->NEXT)
            {                   /* Rupture de l'indentation. */
                /* Le lien est il associe a cette cellule. */
                if ((pcmalink->PHFIG != ptfig_son)
                    || (pcmalink->XFLAT != ptfig_son->XAB1)
                    || (pcmalink->YFLAT != ptfig_son->YAB1))
                    continue;

                /* Recuperation du gabarit ALU2 du LINK. */
                ptalu2_link = getphtalu(ptalu_TALU2_link,
                                        MAPPING_ALU2,
                                        ptalu2->YTALU,
                                        (phcma_list *) NULL);
                if (ptalu2_link == (phtalu_list *) NULL)
                    continue;

                /* Boucle de niveau 3: exportation des references du LINK. */
                for (ppref = ptalu2_link->PPHREF; ppref != (ptype_list *) NULL;
                     ppref = ppref->NEXT)
                {
                    if ((access_ref->USER == (ptype_list *) pcmalink->PHCMA)
                        && in_ls_intv(ptalu1->INTV,
                                      access_ref->XREF - SX_SIDE_VIA1 / 2,
                                      access_ref->XREF + SX_SIDE_VIA1 / 2)
                        && in_ls_intv(ptalu2->INTV,
                                      access_ref->XREF - SX_SIDE_VIA2 / 2,
                                      access_ref->XREF + SX_SIDE_VIA2 / 2))
                    {
                        /* Nom de la nouvelle reference. */
                        sprintf(name_ref, "%s_%ld", pcma->NAME,
                                index_calc(pslice, NB_ref, ptalu2->YTALU));
                        NB_ref += 1;

                        /* Type de la nouvelle reference. */
                        if (pcma->TYPE == C_PHCMA_CMA)
                            type_ref = type_refcon;
                        else
                            type_ref = type_refref;

                        pref = addphref(ptfig_father,
                                        type_ref,
                                        name_ref,
                                        access_ref->XREF,
                                        ptalu2->YTALU);
                        pref->USER = (ptype_list *) pcma;

                        /* Addition a la liste de tous les "ref_con" et "ref_ref", tous CMAs confondus. */
                        ptalu2->PPHREF = addoptype(ptalu2->PPHREF,
                                                   pref->XREF,
                                                   (void *) pref);
                    }
                }

            }                   /* Retour a l'indentation normale. */
            /* Fin de la boucle 2: ( LINKs ) */

            ptalu1->NBREF = NB_ref;
        }
        /* Fin de la boucle 1: ( CMAs ) */
    }
    /* Fin de la boucle 0: ( TALU2s ) */

#undef  access_seg
#undef  access_ref
#undef  access_s_supply
}
