/*
 * $Author: ac $
 * $Id: bflheap.c,v 1.1 1999/10/21 12:51:51 ac Exp $
 * $Date: 1999/10/21 12:51:51 $
 */
/*
 * $Log: bflheap.c,v $
 * Revision 1.1  1999/10/21 12:51:51  ac
 * Nouvelle version avec les librairies bdd104, abe, abv, abt.
 * Ajout de fpm_network pour accelerer l'optimisation.
 *
 */

/*----------------------------------------------------------------------*/
/*                               Includes                               */
/*----------------------------------------------------------------------*/

#include BFL_H

/*----------------------------------------------------------------------*/
/*                        Constant declarations                         */
/*----------------------------------------------------------------------*/

/*----------------------------------------------------------------------*/
/*                          Type declarations                           */
/*----------------------------------------------------------------------*/

/*----------------------------------------------------------------------*/
/*                        Stucture declarations                         */
/*----------------------------------------------------------------------*/

/*----------------------------------------------------------------------*/
/*                        Variable declarations                         */
/*----------------------------------------------------------------------*/

/*----------------------------------------------------------------------*/
/*                          Macro declarations                          */
/*----------------------------------------------------------------------*/

/*----------------------------------------------------------------------*/
/*                      Static function prototypes                      */
/*----------------------------------------------------------------------*/

/*----------------------------------------------------------------------*/
/*                    Definition of static functions                    */
/*----------------------------------------------------------------------*/

/*----------------------------------------------------------------------*/
/*                    Definition of private functions                   */
/*----------------------------------------------------------------------*/

/*----------------------------------------------------------------------*/
/*                   Definition of exported functions                   */
/*----------------------------------------------------------------------*/

bflheap_t *
bflheapnew(bflcompar_fn comp)
{
	bflheap_t *heap;

	heap = (bflheap_t *) bflmemalloc(sizeof(bflheap_t));
	heap->comp = comp;
	heap->start = 0;
	heap->finish = 0;
	heap->end_of_storage = 0;
	return heap;
}

void
bflheapfree(bflheap_t *heap)
{
	if (heap->start)
		free(heap->start);
	bflmemfree(heap, sizeof(bflheap_t));
}

void
bflheappush(bflheap_t *heap, bflptr_t data)
{
	bflsize_t index, parent;

	if (heap->finish == heap->end_of_storage) {
		bflsize_t size = heap->start ? 2 * bflheapsize(heap) : 32;
		bflptr_t *node = bflalloc(size * sizeof(bflptr_t));
		bflptr_t *ptr = node;
		bflptr_t *first = heap->start;
		while (first != heap->finish) *ptr++ = *first++;
		free(heap->start);
		heap->start = node;
		heap->finish = ptr;
		heap->end_of_storage = node + size;
	}

	index = (heap->finish++ - heap->start);
	parent = (index - 1) / 2;
	while (index > 0 && heap->comp(heap->start[parent], data) < 0) {
		heap->start[index] = heap->start[parent];
		index = parent;
		parent = (index - 1) / 2;
	}
	heap->start[index] = data;
}

bflptr_t
bflheappop(bflheap_t *heap)
{
	bflptr_t  res  = *heap->start;
	bflsize_t hole = 0;
	bflsize_t next = 2;
	bflsize_t prev;
	bflptr_t  last = *--heap->finish;
	bflsize_t size = heap->finish - heap->start;

	while (next < size) {
		if (heap->comp(heap->start[next], heap->start[next - 1]) < 0)
			--next;
		heap->start[hole] = heap->start[next];
		hole = next;
		next = 2 * (next + 1);
	}
	if (next == size) {
		heap->start[hole] = heap->start[next - 1];
		hole = next - 1;
	}

	prev = (hole - 1) / 2;
	while (hole > 0 && heap->comp(heap->start[prev], last) < 0) {
		heap->start[hole] = heap->start[prev];
		hole = prev;
		prev = (hole - 1) / 2;
	}
	heap->start[hole] = last;
	return res;
}

