/*
 * $Author: ac $
 * $Id: fpm_bdd.c,v 1.1 1999/10/21 12:52:03 ac Exp $
 * $Date: 1999/10/21 12:52:03 $
 */
/*
 * $Log: fpm_bdd.c,v $
 * Revision 1.1  1999/10/21 12:52:03  ac
 * Nouvelle version avec les librairies bdd104, abe, abv, abt.
 * Ajout de fpm_network pour accelerer l'optimisation.
 *
 */

/*----------------------------------------------------------------------*/
/*                               Includes                               */
/*----------------------------------------------------------------------*/

#include  MUT_H
#include  BFL_H
#include  AUT_H
#include  ABL_H
#include  BDD_H
#include  ABE_H
#include "fpm_network.h"
#include "fpm_user.h"
#include "fpm_bdd.h"

/*----------------------------------------------------------------------*/
/*                        Constant declarations                         */
/*----------------------------------------------------------------------*/

/*----------------------------------------------------------------------*/
/*                          Type declarations                           */
/*----------------------------------------------------------------------*/

/*----------------------------------------------------------------------*/
/*                        Stucture declarations                         */
/*----------------------------------------------------------------------*/

/*----------------------------------------------------------------------*/
/*                        Variable declarations                         */
/*----------------------------------------------------------------------*/

static chain_list	*FpmHeadName		= (chain_list *)0;
static authtable	*FpmHashTableOrder 	= (authtable  *)0;

/*----------------------------------------------------------------------*/
/*                          Macro declarations                          */
/*----------------------------------------------------------------------*/

/*----------------------------------------------------------------------*/
/*                      Static function prototypes                      */
/*----------------------------------------------------------------------*/

static	bddnode	*fpm_expr2bdd	 __P(( befig_list *BehFig, chain_list *Expr ));
static	bddnode *fpm_abl2bdd	 __P(( befig_list *BehFig, char *Name, chain_list *Expr ));
static	void	 fpm_orderexprbdd __P(( befig_list *BehFig, chain_list *Expr ));
static	void	 fpm_orderablbdd	 __P(( befig_list *BehFig, char *Name, chain_list *Expr ));
static	long	 fpm_sortcompare	 __P(( authelem *ValueArray,
				       long      Index1,
				       long      Index2 ));

/*----------------------------------------------------------------------*/
/*                    Definition of static functions                    */
/*----------------------------------------------------------------------*/

static int
fpm_is_saved_aux( SavedAux, Name )
	chain_list *SavedAux;
	char       *Name;
{
	while ( SavedAux && SavedAux->DATA != Name )
		SavedAux = SavedAux->NEXT;
	return (SavedAux != NULL);
}

static bddnode *
fpm_expr2bdd( befig_list *BehFig, chain_list *Expr )
{
    bddnode       *BddNode;
    bddnode       *BddFirst;
    char          *AtomValue;
    beaux_list    *ScanAux;
    chain_list    *ScanChain;
    long           Oper;
    int            Negative;
    fpm_network_t *Network = FpmUser->NETWORK;

    if (Expr == (chain_list *) 0)
	bflerror(1, "error: null expression");

    if (ABL_ATOM(Expr)) {
	AtomValue = ABL_ATOM_VALUE(Expr);

	if (AtomValue == ABL_ATOM_NAME_ONE)
	    return (BddLocalSystem->ONE);

	if ((AtomValue == ABL_ATOM_NAME_ZERO) ||
	    (AtomValue == ABL_ATOM_NAME_DC))
	    return (BddLocalSystem->ZERO);

	BddNode = fpm_atom_find_node( Network, AtomValue);

	if (BddNode != (bddnode *) 0)
	    return (incbddrefext(BddNode));

	for (ScanAux = BehFig->BEAUX; ScanAux != (beaux_list *) 0;
	     ScanAux = ScanAux->NEXT)
	    if (ScanAux->NAME == AtomValue)
		break;

	if (ScanAux == (beaux_list *) 0)
	    bflerror(1, "error: unknown atom `%s'", AtomValue);

	for (ScanChain = FpmHeadName; ScanChain != (chain_list *) 0;
	     ScanChain = ScanChain->NEXT)
	    if (ScanChain->DATA == AtomValue)
		bflerror(1, "error: combinatory loop on `%s'", AtomValue);

	if (ScanAux->NODE == (void *) 0) {
	    FpmHeadName = addchain(FpmHeadName, AtomValue);
	    ScanAux->NODE = fpm_expr2bdd( BehFig, ScanAux->ABL);
	    FpmHeadName = delchain(FpmHeadName, FpmHeadName);
	}
	return (incbddrefext(ScanAux->NODE));
    }
    
    Oper = ABL_OPER(Expr);
    Expr = ABL_CDR(Expr);

    if (Oper == ABL_NOT) {
	if (ABL_CDR(Expr) != (chain_list *) 0)
	    bflerror(1, "error: illegal use of operator %ld", Oper);

	BddFirst = fpm_expr2bdd( BehFig, ABL_CAR(Expr));
	BddNode = applybddnodenot((bddsystem *) 0, decbddrefext(BddFirst));

	return (BddNode);
    }
    
    if (isablunaryoper(Oper) || (ABL_CDR(Expr) == (chain_list *) 0))
	bflerror(1, "error: illegal use of operator %ld", Oper);

    if ((getabloperpolar(Oper) == ABL_POLAR_POSITIVE) ||
	(ABL_CDDR(Expr) == (chain_list *) 0)) {
	Negative = 0;
    } else {
	Negative = 1;
	Oper = getablopernot(Oper);
    }

    BddFirst = fpm_expr2bdd(BehFig, ABL_CAR(Expr));

    while ((Expr = ABL_CDR(Expr)) != (chain_list *) 0) {
	BddNode = fpm_expr2bdd(BehFig, ABL_CAR(Expr));
	BddFirst = applybddnode((bddsystem *) 0, Oper, decbddrefext(BddFirst),
						       decbddrefext(BddNode));
    }

    if (Negative)
	BddFirst = applybddnodenot((bddsystem *) 0, decbddrefext(BddFirst));

    return (BddFirst);
}

/*----------------------------------------------------------------------*/
/*                              fpm_abl2bdd                              */
/*----------------------------------------------------------------------*/

static bddnode *
fpm_abl2bdd( befig_list *BehFig, char *Name, chain_list *Expr )
{
    bddnode       *BddNode;
    fpm_network_t *Network = FpmUser->NETWORK;

    if (Name != (char *) 0)
	FpmHeadName = addchain(FpmHeadName, Name);

    BddNode = fpm_expr2bdd(BehFig, Expr);

    if (Name != (char *) 0) {
	FpmHeadName = delchain(FpmHeadName, FpmHeadName);

	fpm_expr_new( Network, Name, BddNode );
    }
    return (BddNode);
}

/*----------------------------------------------------------------------*/
/*                            fpm_orderexprbdd                           */
/*----------------------------------------------------------------------*/

static void
fpm_orderexprbdd( befig_list *BehFig, chain_list *Expr )
{
    char       *AtomValue;
    beaux_list *ScanAux;
    chain_list *ScanChain;
    authelem   *Element;
    long        Oper;

    if ( Expr == (chain_list *)0 )
	bflerror( 1, "error: null expression" );

    if ( ABL_ATOM( Expr ) ) {
	AtomValue = ABL_ATOM_VALUE( Expr );

/*
 * if (AtomValue == 0)
 *	bflerror( 1, "error: null atom", AtomValue );
 */
	
        if ( ( AtomValue == ABL_ATOM_NAME_ONE  ) ||
             ( AtomValue == ABL_ATOM_NAME_ZERO ) ||
             ( AtomValue == ABL_ATOM_NAME_DC   ) ) {
          return;
        }

        Element = searchauthelem( FpmHashTableOrder, AtomValue );

        if ( Element != (authelem *)0 ) {
	    Element->VALUE++;
 	    return;
        }

        for ( ScanAux = BehFig->BEAUX; ScanAux != 0; ScanAux = ScanAux->NEXT )
            if ( ScanAux->NAME == AtomValue )
	       break;

        if ( ScanAux == (beaux_list *)0 )
   	    bflerror( 1, "error: unknown atom `%s'", AtomValue );

        for ( ScanChain = FpmHeadName; ScanChain != 0; ScanChain = ScanChain->NEXT )
  	    if ( ScanChain->DATA == AtomValue )
	          bflerror( 1, "error: combinatory loop on `%s'", AtomValue );

	FpmHeadName   = addchain( FpmHeadName, AtomValue );
	fpm_orderexprbdd( BehFig, ScanAux->ABL );
	FpmHeadName   = delchain( FpmHeadName, FpmHeadName );

	return;
    }

    Oper = ABL_OPER( Expr );
    Expr = ABL_CDR( Expr );

    if ( Oper == ABL_NOT ) {
	if ( ABL_CDR( Expr ) != (chain_list *)0 )
	    bflerror( 1, "error: illegal use of operator %ld", Oper );

	fpm_orderexprbdd( BehFig, ABL_CAR( Expr ) );
	return;
    }

    if ( isablunaryoper( Oper ) || ( ABL_CDR( Expr ) == (chain_list *)0 ) )
	bflerror( 1, "error: illegal use of operator %ld", Oper );

    fpm_orderexprbdd( BehFig, ABL_CAR( Expr ) );

    while ( ( Expr = ABL_CDR( Expr ) ) != (chain_list *)0 )
	fpm_orderexprbdd( BehFig, ABL_CAR( Expr ) );
}

/*----------------------------------------------------------------------*/
/*                            fpm_orderablbdd                            */
/*----------------------------------------------------------------------*/

static void
fpm_orderablbdd( befig_list *BehFig, char *Name, chain_list *Expr )
{
    if ( Name != (char *)0 )
	FpmHeadName = addchain( FpmHeadName, Name );

    fpm_orderexprbdd( BehFig, Expr );

    if ( Name != (char *)0 )
	FpmHeadName = delchain( FpmHeadName, FpmHeadName );
}

/*----------------------------------------------------------------------*/
/*                            fpm_sortcompare                            */
/*----------------------------------------------------------------------*/

static long
fpm_sortcompare( ValueArray, Index1, Index2 )
    authelem *ValueArray;
    long      Index1;
    long      Index2;
{
    return( ValueArray[ Index1 ].VALUE - ValueArray[ Index2 ].VALUE );
}

/*----------------------------------------------------------------------*/
/*                   Definition of exported functions                   */
/*----------------------------------------------------------------------*/

static void
fpm_order_bdd(befig_list *BehFig, int Flag, chain_list *SavedAux)
{
    berin_list    *BehRin;
    beaux_list    *BehAux;
    beout_list    *BehOut;
    bebus_list    *BehBus;
    bebux_list    *BehBux;
    bereg_list    *BehReg;
    biabl_list    *BiAbl;
    authelem      *Element;
    authelem      *ValueArray;
    long          *IndexArray;
    long           SizeArray;
    long           Index;
    long           NumberIn;
    bddindex       BddIndex;
    int            AblOrder;
    int            Flatten;
    fpm_network_t *Network = FpmUser->NETWORK;

    AblOrder    = Flag & FPN_ABL_ORDER;
    Flatten     = Flag & FPN_FLATTEN_BEH;
    
    FpmHeadName  = (chain_list *)0;
    NumberIn     = Network->NAME_IN_SIZE;
    
    BehFig->BERIN = (berin_list *)reverse( (chain_list *)BehFig->BERIN );
    
    if ( AblOrder == 0 ) {
        for ( BehRin = BehFig->BERIN; BehRin != NULL; BehRin = BehRin->NEXT ) {
	    BddIndex = Network->NAME_IN_SIZE + BDD_INDEX_MIN;
    
	    fpm_atom_new( Network, BehRin->NAME, -1 );
        }
  
        BehFig->BERIN = (berin_list *)reverse( (chain_list *)BehFig->BERIN );
  
        return;
    }
  
  
    printf( "   - Running the abl ordonancer ...\n" );
      
    FpmHashTableOrder = createauthtable( NumberIn << 1 );
    
    for ( BehRin = BehFig->BERIN; BehRin != NULL; BehRin = BehRin->NEXT )
	addauthelem( FpmHashTableOrder, BehRin->NAME, 0 );
  
    BehFig->BERIN = (berin_list *)reverse( (chain_list *)BehFig->BERIN );
  
    for ( BehAux = BehFig->BEAUX; BehAux != NULL; BehAux = BehAux->NEXT )
        if ( !Flatten || fpm_is_saved_aux( SavedAux, BehAux->NAME ) )
	    if (BehAux->ABL != 0) {
#ifdef DEBUG
		bflwarning("  --> orderabl = %s", BehAux->NAME);
#endif
                fpm_orderablbdd( BehFig, BehAux->NAME, BehAux->ABL );
	    }
    for ( BehOut = BehFig->BEOUT; BehOut != NULL; BehOut = BehOut->NEXT )
	if (BehOut->ABL != 0) {
#ifdef DEBUG
	    bflwarning("  --> orderabl = %s", BehOut->NAME);
#endif
	    fpm_orderablbdd( BehFig, BehOut->NAME, BehOut->ABL );
	}
    
    for ( BehBus = BehFig->BEBUS; BehBus != NULL; BehBus = BehBus->NEXT )
	for ( BiAbl = BehBus->BIABL; BiAbl != NULL; BiAbl = BiAbl->NEXT ) {
	    if ( BiAbl->VALABL != 0 )
		fpm_orderablbdd( BehFig, (char *)0, BiAbl->VALABL );
	    if ( BiAbl->CNDABL != 0 )
		fpm_orderablbdd( BehFig, (char *)0   , BiAbl->CNDABL );
	}
  
    for ( BehBux = BehFig->BEBUX; BehBux != NULL; BehBux = BehBux->NEXT )
	for ( BiAbl = BehBux->BIABL; BiAbl != NULL; BiAbl  = BiAbl->NEXT ) {
	    if ( BiAbl->VALABL != 0 )
		fpm_orderablbdd( BehFig, (char *)0, BiAbl->VALABL );
	    if ( BiAbl->CNDABL != 0 )
		fpm_orderablbdd( BehFig, (char *)0, BiAbl->CNDABL );
	}

    for ( BehReg = BehFig->BEREG; BehReg != NULL; BehReg = BehReg->NEXT )
        for ( BiAbl  = BehReg->BIABL; BiAbl != NULL; BiAbl  = BiAbl->NEXT ) {
	    if ( BiAbl->VALABL != 0 )
		fpm_orderablbdd( BehFig, (char *)0, BiAbl->VALABL );
            if ( BiAbl->CNDABL != 0 )
		fpm_orderablbdd( BehFig, (char *)0, BiAbl->CNDABL );
	}
   
    SizeArray  = FpmHashTableOrder->TABLE_SIZE;
    ValueArray = FpmHashTableOrder->TABLE;
    IndexArray = (long *)autallocblock( sizeof( long ) * SizeArray );
   
    sortautarray( ValueArray, IndexArray, SizeArray, fpm_sortcompare );
   
    for ( Index = 0; Index < SizeArray; Index++ ) {
	Element = &ValueArray[ IndexArray[ Index ] ];
   
	if ( Element->KEY != AUT_HASH_KEY_EMPTY ) {
	    BddIndex = Network->NAME_IN_SIZE + BDD_INDEX_MIN;

	    fpm_atom_new( Network, Element->KEY, BddIndex );

# ifdef DEBUG
	    fprintf( stdout, "Var: %s Order: %ld\n", Element->KEY, Element->VALUE );
# endif
	}
    }

    autfreeblock( (char *)IndexArray );
    destroyauthtable( FpmHashTableOrder );
}

/*----------------------------------------------------------------------*/
/*                              fpm_makebdd                              */
/*----------------------------------------------------------------------*/

/* Ajoute un fpm_network_t a une befig_list et construit les bdds.
 */
void
fpm_make_bdd(befig_list *BehFig, int Flag, chain_list *SavedAux)
{
    berin_list    *BehRin;
    beaux_list    *BehAux;
    beout_list    *BehOut;
    bebus_list    *BehBus;
    bebux_list    *BehBux;
    bereg_list    *BehReg;
    biabl_list    *BiAbl;
    binode_list   *BiNode;
    bddsystem     *BddSystem;
    fpm_network_t *FpmNetwork;
    long           Flatten = Flag & FPN_FLATTEN_BEH;
    long           NumberAtom  = 1;
    long           NumberExpr = 1;

    for ( BehAux = BehFig->BEAUX; BehAux != NULL; BehAux = BehAux->NEXT ) {
	if ( Flatten && ( ! fpm_is_saved_aux( SavedAux, BehAux->NAME ) ) )
	    BehFig->BERIN = beh_rmvberin(BehFig->BERIN, BehAux->NAME );
	NumberExpr += 1;
    }

    for ( BehRin = BehFig->BERIN; BehRin != NULL; BehRin = BehRin->NEXT )
	NumberAtom += 1;

    for ( BehOut = BehFig->BEOUT; BehOut != NULL; BehOut = BehOut->NEXT )
	NumberExpr += 1;

    for ( BehBus = BehFig->BEBUS; BehBus != NULL; BehBus = BehBus->NEXT )
	NumberExpr += 1;

    BddSystem        = createbddsystem( 100, 1000, 100, 5000000 );
    FpmNetwork       = fpm_network_new( BehFig->NAME, BddSystem );
    FpmUser->NETWORK = FpmNetwork;

    fpm_order_bdd( BehFig, Flag, SavedAux );

    printf( "   - Making Bdd ...\n" );

    for ( BehAux = BehFig->BEAUX; BehAux != NULL; BehAux = BehAux->NEXT )
	if (BehAux->NODE == NULL)
	    BehAux->NODE = fpm_abl2bdd(BehFig, BehAux->NAME, BehAux->ABL);
    
    for ( BehOut = BehFig->BEOUT; BehOut != NULL; BehOut = BehOut->NEXT )
	if (BehOut->NODE == NULL)
	    BehOut->NODE = fpm_abl2bdd(BehFig, BehOut->NAME, BehOut->ABL);

    for ( BehBus = BehFig->BEBUS; BehBus != NULL; BehBus = BehBus->NEXT ) {
	BiAbl  = BehBus->BIABL;
	BiNode = BehBus->BINODE;

	while ( BiAbl != NULL ) {
	    if ( BiAbl->CNDABL != (ablexpr *)0 )
		BiNode->CNDNODE = fpm_abl2bdd(BehFig, (char *) 0, BiAbl->CNDABL);
	    if ( BiAbl->VALABL != (ablexpr *)0 )
		BiNode->VALNODE = fpm_abl2bdd(BehFig, BehBus->NAME, BiAbl->VALABL);

	    BiAbl  = BiAbl->NEXT;
	    BiNode = BiNode->NEXT;
	}
    }

    for ( BehBux = BehFig->BEBUX; BehBux != NULL; BehBux = BehBux->NEXT ) {
	BiAbl  = BehBux->BIABL;
	BiNode = BehBux->BINODE;

	while ( BiAbl != NULL ) {
	    if ( BiAbl->CNDABL != (ablexpr *)0 )
		BiNode->CNDNODE = fpm_abl2bdd(BehFig, (char *) 0, BiAbl->CNDABL);
	    if ( BiAbl->VALABL != (ablexpr *)0 )
		BiNode->VALNODE = fpm_abl2bdd(BehFig, BehBux->NAME, BiAbl->VALABL);

	    BiAbl  = BiAbl->NEXT;
	    BiNode = BiNode->NEXT;
	}
    }

    for ( BehReg = BehFig->BEREG; BehReg != NULL; BehReg = BehReg->NEXT ) {
	BiAbl  = BehReg->BIABL;
	BiNode = BehReg->BINODE;

	while ( BiAbl != NULL ) {
	    if ( BiAbl->CNDABL != (ablexpr *)0 )
		BiNode->CNDNODE = fpm_abl2bdd(BehFig, (char *) 0, BiAbl->CNDABL);
	    if ( BiAbl->VALABL != (ablexpr *)0 )
		BiNode->VALNODE = fpm_abl2bdd(BehFig, BehReg->NAME, BiAbl->VALABL);

	    BiAbl  = BiAbl->NEXT;
	    BiNode = BiNode->NEXT;
	}
    }

    BehAux = BehFig->BEAUX;

    while ( BehAux != 0 ) {
	beaux_list *Next = BehAux->NEXT;
	
	if ( Flatten && ( ! fpm_is_saved_aux( SavedAux, BehAux->NAME ) ) ) {
	    if ( BehAux->NODE != 0 )
		fpm_expr_delete( FpmNetwork, fpm_expr_find( FpmNetwork, BehAux->NAME ) );
	    BehFig->BEAUX = beh_rmvbeaux( BehFig->BEAUX, BehAux->NAME, 'Y' );
	}
	BehAux = Next;
    }

    sweepbddsystem( 0 );
}


