/*
 * $Author: ac $
 * $Id: fpm_init.c,v 1.1 1999/10/21 12:52:04 ac Exp $
 * $Date: 1999/10/21 12:52:04 $
 */
/*
 * $Log: fpm_init.c,v $
 * Revision 1.1  1999/10/21 12:52:04  ac
 * Nouvelle version avec les librairies bdd104, abe, abv, abt.
 * Ajout de fpm_network pour accelerer l'optimisation.
 *
 */

/*----------------------------------------------------------------------*/
/*                               Includes                               */
/*----------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include  MUT_H
#include  BFL_H
#include  AUT_H
#include  ABL_H
#include  BDD_H
#include  ABE_H
#include  LAX_H
#include "fpm_network.h"
#include "fpm_user.h"
#include "x4000.h"
#include "virtex.h"
#include "klut.h"

/*----------------------------------------------------------------------*/
/*                        Constant declarations                         */
/*----------------------------------------------------------------------*/

/*----------------------------------------------------------------------*/
/*                          Type declarations                           */
/*----------------------------------------------------------------------*/

/*----------------------------------------------------------------------*/
/*                        Stucture declarations                         */
/*----------------------------------------------------------------------*/

/*----------------------------------------------------------------------*/
/*                        Variable declarations                         */
/*----------------------------------------------------------------------*/

static char *UsageStr[] =
{
	"usage: fpmap [switches] <filename>",
	"",
	"Most frequently used switches:",
	"  -Ttarget                     defines the target architecture",
        "  -O[0-5]                      defines the level of optimization",
	"  -M[0-5]                      defines the area/delay trade-off",
	"  -a                           run the abl ordonnancer",
	"  -f                           flatten the network before optim",
        "  -k[1-9] K-Lut                set the size of the lut",
	"  -o <file>                    place the expr into <file>",
	"  -p <file>                    use <file> as the parameter file",
	"",
	"Available target are:",
	"    X4000         Xilinx X4000 Family",
	"    Virtex        Xilinx Virtex Family",
        "    K-Lut         Generic K-Lut Familly",
      "For more complete information, please read the fpmap man page.\n",
	0
};

fpm_user_t *FpmUser;

/*----------------------------------------------------------------------*/
/*                          Macro declarations                          */
/*----------------------------------------------------------------------*/

/*----------------------------------------------------------------------*/
/*                      Static function prototypes                      */
/*----------------------------------------------------------------------*/

static void fpm_usage __P((void));
static void fpm_print_system __P((void));
static void fpm_print_mbk __P((void));

/*----------------------------------------------------------------------*/
/*                    Definition of static functions                    */
/*----------------------------------------------------------------------*/

static void
fpm_usage()
{
	char **Str = UsageStr;

	while (*Str)
		puts(*Str++);
	exit(1);
}

static void
fpm_print_system()
{
	chain_list *Ptr;

	putchar('\n');
	printf("          *  FPGA environement:\n\n");
	printf("             VHDL file        : %s.vbe\n", FpmUser->FILE_VHDL);
	printf("             Expr file      : %s.vbe\n", FpmUser->FILE_EXPR);
	if (FpmUser->FILE_PARAM)
		printf("             Parameter file   : %s.lax\n", FpmUser->FILE_PARAM);
	else
		printf("             Parameter file   : none\n");
	printf("             Architecture     : %s\n", FpmUser->ARCHI_NAME);
	if (strcmp(FpmUser->ARCHI_NAME, "K-Lut") == 0)
		printf("             Lut size         : %d\n", FpmUser->KLUT_SIZE);
	if ( strcmp(  FpmUser->ARCHI_NAME, "K-Lut" ) ==0 )
		printf( "             Lut size         : %d\n",     FpmUser->KLUT_SIZE );
	printf("             Optimisation     : -O%d\n", FpmUser->LEVEL);
	printf("             Abl ordonancer   : %s\n",
	       (FpmUser->ABL_ORDER) ? "yes" : "no");
	printf("             Area / Delay     : %d%% area / %d%% delay\n",
	       (FpmUser->MODE * 20), ((5 - FpmUser->MODE) * 20));

	if (strcmp(FpmUser->ARCHI_NAME, "X4000") == 0)
		printf("             Iob mode         : %sgenerate iobs\n",
		       (FpmUser->IOB_MODE == 1) ? "" : "do not ");

	for (Ptr = FpmUser->CLOCKS; Ptr != 0; Ptr = Ptr->NEXT) {
		if (Ptr == FpmUser->CLOCKS)
			fputs("             Clocks           : ", stdout);
		else
			fputs("                                ", stdout);
		puts((char *) Ptr->DATA);
	}

	for (Ptr = FpmUser->SAVED_AUX; Ptr != 0; Ptr = Ptr->NEXT) {
		if (Ptr == FpmUser->SAVED_AUX)
			fputs("             Saved Aux        : ", stdout);
		else
			fputs("                                ", stdout);
		puts((char *) Ptr->DATA);
	}
	putchar('\n');
}

static void
fpm_print_mbk()
{
	int index;

	fputs("          *  MBK environement :\n\n", stdout);
	fputs("             MBK_IN_LO        : ", stdout);
	puts(IN_LO);
	fputs("             MBK_OUT_LO       : ", stdout);
	puts(OUT_LO);
	fputs("             MBK_IN_PH        : ", stdout);
	puts(IN_PH);
	fputs("             MBK_OUT_PH       : ", stdout);
	puts(OUT_PH);
	fputs("             MBK_WORK_LIB     : ", stdout);
	puts(WORK_LIB);
	for (index = 0; CATA_LIB[index] != NULL; index++) {
		if (index == 0)
			fputs("             MBK_CATA_LIB     : ", stdout);
		else
			fputs("                                ", stdout);
		puts(CATA_LIB[index]);
	}
	putchar('\n');
}

/*----------------------------------------------------------------------*/
/*                   Definition of exported functions                   */
/*----------------------------------------------------------------------*/

#define MODE_DONE	(1 << 0)
#define LEVEL_DONE	(1 << 1)

void
fpm_init(int Argc, char **Argv)
{
	int Number;
	int Index;
	int Done = 0;
	lax_param_t *lax_param;

	if (Argc < 3)
		fpm_usage();

	FpmUser = mbkalloc(sizeof(fpm_user_t));
	FpmUser->ARCHI_NAME = 0;
	FpmUser->ARCHI_CHECK = 0;
	FpmUser->ARCHI_COST = 0;
	FpmUser->ARCHI_MAPP = 0;
	FpmUser->ARCHI_SAVE = 0;
	FpmUser->LEVEL = 2;
	FpmUser->MODE = 2;
	FpmUser->ABL_ORDER = 0;
	FpmUser->FLATTEN = 0;
	FpmUser->IOB_MODE = 1;
	FpmUser->KLUT_SIZE = 4;
	FpmUser->SAVED_AUX = 0;
	FpmUser->CLOCKS = 0;
	FpmUser->FILE_VHDL = 0;
	FpmUser->FILE_PARAM = 0;
	FpmUser->FILE_EXPR = 0;
	FpmUser->BEFIG = 0;

	for (Number = 1; Number < Argc; Number++) {
		if (Argv[Number][0] == '-') {
			if ((Argv[Number][1] == 'O') &&
			    (Argv[Number][3] == '\0')) {

				FpmUser->LEVEL = Argv[Number][2] - '0';
				Done |= LEVEL_DONE;
				if (FpmUser->LEVEL < 0 || FpmUser->LEVEL > 5)
					fpm_usage();

			} else if ((Argv[Number][1] == 'M') &&
				   (Argv[Number][3] == '\0')) {

				FpmUser->MODE = Argv[Number][2] - '0';
				Done |= MODE_DONE;
				if (FpmUser->MODE < 0 || FpmUser->MODE > 5)
					fpm_usage();

			} else if ( ( Argv[ Number ][ 1 ] == 'k'  ) &&
				    ( Argv[ Number ][ 3 ] == '\0' ) ) {
		
				FpmUser->KLUT_SIZE = Argv[ Number ][ 2 ] - '0';
				if ( FpmUser->KLUT_SIZE < 2 || FpmUser->KLUT_SIZE > 9 )
					fpm_usage();
		
			} else if (Argv[Number][1] == 'T') {
				if (strcmp(Argv[Number] + 2, "Virtex") == 0) {
					FpmUser->ARCHI_NAME     = "Virtex";
					FpmUser->ARCHI_INIT     = virtex_init;
					FpmUser->ARCHI_DONE     = virtex_done;
					FpmUser->ARCHI_CHECK    = virtex_check;
					FpmUser->ARCHI_COST     = virtex_cost;
					FpmUser->ARCHI_CLEAR    = virtex_clear;
					FpmUser->ARCHI_MAPP     = virtex_mapp;
					FpmUser->ARCHI_REPORT   = virtex_report;
					FpmUser->ARCHI_SAVE     = virtex_save;
				} else if ( strcmp( Argv[ Number ] + 2, "X4000" ) == 0 ) {
					FpmUser->ARCHI_NAME	= "X4000";
					FpmUser->ARCHI_INIT     = x4_init;
					FpmUser->ARCHI_DONE     = x4_done;
					FpmUser->ARCHI_CHECK	= x4_check;
					FpmUser->ARCHI_COST	= x4_cost;
					FpmUser->ARCHI_CLEAR	= x4_clear;
					FpmUser->ARCHI_MAPP	= x4_mapp;
					FpmUser->ARCHI_REPORT	= x4_report;
					FpmUser->ARCHI_SAVE	= x4_save;
				} else if ( strcmp( Argv[ Number ] + 2, "K-Lut" ) == 0 ) {
					FpmUser->ARCHI_NAME	= "K-Lut";
					FpmUser->ARCHI_INIT     = klut_init;
					FpmUser->ARCHI_DONE     = klut_done;
					FpmUser->ARCHI_CHECK	= klut_check;
					FpmUser->ARCHI_COST	= klut_cost;
					FpmUser->ARCHI_CLEAR	= klut_clear;
					FpmUser->ARCHI_MAPP	= klut_mapp;
					FpmUser->ARCHI_REPORT	= klut_report;
					FpmUser->ARCHI_SAVE	= klut_save;
				} else
					fpm_usage();
			} else if (Argv[Number][1] == 'p') {
				if (Argv[Number][2] != '\0')
					FpmUser->FILE_PARAM = Argv[Number] + 2;
				else
					FpmUser->FILE_PARAM = Argv[++Number];
			} else if (Argv[Number][1] == 'o') {
				if (Argv[Number][2] != '\0')
					FpmUser->FILE_EXPR = Argv[Number] + 2;
				else
					FpmUser->FILE_EXPR = Argv[++Number];
			} else {
				for (Index = 1; Argv[Number][Index] != '\0'; Index++) {
					switch (Argv[Number][Index]) {
					case 'a':
						FpmUser->ABL_ORDER = 1;
						break;

					case 'f':
						FpmUser->FLATTEN = 1;
						break;

					case 'i':
						FpmUser->IOB_MODE = 1;
						break;

					case 'n':
						FpmUser->IOB_MODE = 0;
						break;

					default:
						fpm_usage();
					}
				}
			}
		} else if (FpmUser->FILE_VHDL == 0)
			FpmUser->FILE_VHDL = Argv[Number];
		else
			fpm_usage();

	}

	if (FpmUser->FILE_VHDL == 0)
		fpm_usage();

	/*
	 * Lecture fichier de parametres
	 */
	if (FpmUser->FILE_PARAM) {
		if ((lax_param = laxloadparam(FpmUser->FILE_PARAM)) == NULL)
			bflerror(1, "aborting ...");

		if (!(Done & LEVEL_DONE))
			FpmUser->LEVEL = lax_param->level;

		if (!(Done & MODE_DONE))
			FpmUser->MODE = lax_param->mode;

		if (lax_param->saved_aux != (chain_list *) 0)
			FpmUser->SAVED_AUX = lax_param->saved_aux;

		if (lax_param->clock != (chain_list *) 0)
			FpmUser->CLOCKS = lax_param->clock;
	}
	if (FpmUser->CLOCKS == 0)
		FpmUser->CLOCKS = addchain(0, namealloc("ck"));

	if (FpmUser->FILE_EXPR == 0) {
		char *Name;

		Name = (char *) mbkalloc(strlen(FpmUser->FILE_VHDL) + 2);
		strcpy(Name, FpmUser->FILE_VHDL);
		strcat(Name, (FpmUser->FLATTEN == 1) ? "o" : "l");
		FpmUser->FILE_EXPR = namealloc(Name);
		free(Name);
	}
	fpm_print_system();
	fpm_print_mbk();
}
