/*
 * $Author: ac $
 * $Id: fpm_network.c,v 1.1 1999/10/21 12:52:06 ac Exp $
 * $Date: 1999/10/21 12:52:06 $
 */
/*
 * $Log: fpm_network.c,v $
 * Revision 1.1  1999/10/21 12:52:06  ac
 * Nouvelle version avec les librairies bdd104, abe, abv, abt.
 * Ajout de fpm_network pour accelerer l'optimisation.
 *
 */

/*----------------------------------------------------------------------*/
/*                               Includes                               */
/*----------------------------------------------------------------------*/

#include  MUT_H
#include  BFL_H
#include  AUT_H
#include  ABL_H
#include  BDD_H
#include  ABE_H
#include "fpm_network.h"

/*----------------------------------------------------------------------*/
/*                        Constant declarations                         */
/*----------------------------------------------------------------------*/

/*----------------------------------------------------------------------*/
/*                          Type declarations                           */
/*----------------------------------------------------------------------*/

/*----------------------------------------------------------------------*/
/*                        Stucture declarations                         */
/*----------------------------------------------------------------------*/

/*----------------------------------------------------------------------*/
/*                        Variable declarations                         */
/*----------------------------------------------------------------------*/

/*----------------------------------------------------------------------*/
/*                          Macro declarations                          */
/*----------------------------------------------------------------------*/

/*----------------------------------------------------------------------*/
/*                      Static function prototypes                      */
/*----------------------------------------------------------------------*/

/*----------------------------------------------------------------------*/
/*                    Definition of static functions                    */
/*----------------------------------------------------------------------*/

static chain_list *
rmvchain(chain_list *head, void *data)
{
	chain_list *cur = (chain_list *)&head;

	while (cur && cur->NEXT != NULL) {
		if (cur->NEXT->DATA == data) {
			chain_list *tmp = cur->NEXT;
			cur->NEXT = cur->NEXT->NEXT;
			tmp->NEXT = HEAD_CHAIN;
			HEAD_CHAIN = tmp;
		} else {
			cur = cur->NEXT;
		} 
	}
	return head;
}

/*----------------------------------------------------------------------*/
/*                   Definition of exported functions                   */
/*----------------------------------------------------------------------*/

fpm_network_t *
fpm_network_new( char *Name, bddsystem *System )
{
	fpm_network_t *Network;
	
	setbddlocalsystem( System );
  
	Network                = fpm_network_alloc();
	Network->NAME          = namealloc( Name );
	Network->MAP_ATOM     = bflmapnew( bflstrcmp, 0 );
	Network->MAP_EXPR    = bflmapnew( bflstrcmp, 0 );
	Network->NAME_IN       = allocbddnamein( 128 );
	Network->NAME_IN_SIZE  = 128;
	Network->INDEX_IN      = allocbddindexin( 128 );
	Network->INDEX_IN_SIZE = 128;
	Network->BDD_SYSTEM    = BddLocalSystem;

	return Network;
}

void
fpm_network_depend( fpm_network_t *Network )
{
	bflmapnode_t *First;
	bflmapnode_t *Last;

	First = bflmapbegin( Network->MAP_ATOM );
	Last  = bflmapend( Network->MAP_ATOM );
	while ( First != Last ) {
		fpm_atom_t  *Atom = (fpm_atom_t *)bflmapdata( First );
		freechain( Atom->BDD_EXPR );
		Atom->BDD_EXPR = (chain_list *)0;
		First = bflmapnext( First );
	}

	First = bflmapbegin( Network->MAP_EXPR );
	Last  = bflmapend( Network->MAP_EXPR );
	while ( First != Last ) {
		fpm_expr_t *Expr = (fpm_expr_t *)bflmapdata( First );
		freechain( Expr->BDD_ATOM );
		Expr->BDD_ATOM = (chain_list *)0;
		fpm_expr_depend( Network, Expr );
		First = bflmapnext( First );
	}
}

void
fpm_network_view( fpm_network_t *Network )
{
	bflmapnode_t *First;
	bflmapnode_t *Last;
	chain_list   *Chain;
	
	setbddlocalsystem( Network->BDD_SYSTEM );
	
	printf( " -->Network[ %s ]\n", Network->NAME );
	printf( "  number atom  : %d\n", bflmapsize( Network->MAP_ATOM ) );
	printf( "  number expr : %d\n", bflmapsize( Network->MAP_EXPR ) );

	First = bflmapbegin( Network->MAP_ATOM );
	Last  = bflmapend( Network->MAP_ATOM );
	while (First != Last ) {
		fpm_atom_t *Atom = (fpm_atom_t *)bflmapdata( First );
		
		printf( "   --> Atom[ %s ]\n", Atom->NAME );
		printf( "   <--\n" );
		First = bflmapnext( First );
	}

	First = bflmapbegin( Network->MAP_EXPR );
	Last  = bflmapend( Network->MAP_EXPR );
	while (First != Last ) {
		fpm_expr_t *FpmExpr = (fpm_expr_t *)bflmapdata( First );
		ablexpr    *AblExpr = convertbddnodeabl( (bddsystem *)0, Network->NAME_IN, Network->INDEX_IN, FpmExpr->BDD );
		
		printf( "  --> Expr[ %s ]\n", FpmExpr->NAME );
		printf( "    --> Expr    : " );
		viewablexprfile( stdout, AblExpr, ABL_VIEW_VHDL );
		printf( "\n" );
		freeablexpr( AblExpr );
		printf( "    --> Support :" );
		for ( Chain = FpmExpr->BDD_ATOM; Chain != 0; Chain = Chain->NEXT )
			printf( " %s", ((fpm_atom_t *)Chain->DATA)->NAME);
		printf( "\n" );
		First = bflmapnext( First );
	}
}

fpm_atom_t *
fpm_atom_new( fpm_network_t *Network, char *Name, int Index )
{
	fpm_atom_t  *Atom;
	char        **NameIn;
	long          NameInSize;      
	long          NumberNameIn;
	bddindex     *IndexIn;
	long          IndexInSize;      

	setbddlocalsystem( Network->BDD_SYSTEM );

	Name = namealloc( Name );

	if ( fpm_atom_find( Network, Name ) != (fpm_atom_t *)0 )
		bflerror( 1, "fpm_atom_new: atom %s already exists", Name );

	Atom             = fpm_atom_alloc();
	Atom->NAME       = Name;
	
	if ( Index > 0 && checkbddindex( (bddsystem *)0, Index, 0 ) )
		Atom->BDD = getbddvarnodebyindex( (bddsystem *)0, Index );
	else
		Atom->BDD = addbddvarlast( Network->BDD_SYSTEM );

	Atom->BDD_EXPR = (chain_list *)0;
	Atom->USER       = (ptype_list *)0;
	
	bflmapinsert( Network->MAP_ATOM, Atom->NAME, Atom );
	
	NumberNameIn = bflmapsize( Network->MAP_ATOM ) - 1;
	NameIn       = Network->NAME_IN;
	NameInSize   = Network->NAME_IN_SIZE;

	if ( NumberNameIn >= NameInSize ) {
		NameInSize <<= 1;
		NameIn = resizebddnamein( NameIn, NameInSize, NameInSize );

		Network->NAME_IN      = NameIn;
		Network->NAME_IN_SIZE = NameInSize;
	}

	NameIn[ NumberNameIn ] = Atom->NAME;

	IndexIn     = Network->INDEX_IN;
	IndexInSize = Network->INDEX_IN_SIZE;
	Index       = Atom->BDD->INDEX - BDD_INDEX_MIN;

	if ( Index >= IndexInSize ) {
		IndexIn = resizebddindexin( IndexIn, IndexInSize, Index << 1 );

		Network->INDEX_IN      = IndexIn;
		Network->INDEX_IN_SIZE = (long)( Index ) << 1;
	}

	IndexIn[ Index ] = (bddindex)NumberNameIn;

	return Atom;
}

fpm_atom_t *
fpm_atom_find( fpm_network_t *Network, char *Name )
{
	bflmapnode_t *MapNode;

	Name    = namealloc( Name );
	MapNode = bflmapfind( Network->MAP_ATOM, Name );

	if ( MapNode == (bflmapnode_t *)0 )
		return (fpm_atom_t *)0;
	else
		return (fpm_atom_t *)bflmapdata( MapNode );
}

bddnode *
fpm_atom_find_node( fpm_network_t *Network, char *Name )
{
	fpm_atom_t *Atom;

	Atom = fpm_atom_find( Network, Name );
	if ( Atom != NULL )
		return Atom->BDD;
	else
		return (bddnode *)0;
}

char *
fpm_atom_name( fpm_network_t *Network, bddindex Index )
{
	if ( Index == BDD_INDEX_ZERO )
		return getablatomzero();

	if ( Index == BDD_INDEX_ONE )
		return getablatomone();

	Index = Index - BDD_INDEX_MIN;

	if ( Network->INDEX_IN != (bddindex *)0 )
		Index = Network->INDEX_IN[ Index ];

	return Network->NAME_IN[ Index ];
}

bddvar
fpm_atom_var( fpm_network_t *Network, fpm_atom_t *Atom )
{
	if ( Atom == NULL )
		return 0;
	else
		return getbddvarbyindex( Network->BDD_SYSTEM, Atom->BDD->INDEX );
}

fpm_expr_t *
fpm_expr_new( fpm_network_t *Network, char *Name, bddnode *Node )
{
	fpm_expr_t *Expr;
	
	setbddlocalsystem( Network->BDD_SYSTEM );

	Name = namealloc( Name );

	if ( fpm_expr_find( Network, Name ) != (fpm_expr_t *)0 )
		bflerror( 1, "fpm_expr_new: expr '%s' already exists", Name );

	Expr            = fpm_expr_alloc();
	Expr->NAME      = Name;
	Expr->BDD       = Node;
	Expr->BDD_ATOM = NULL;
	Expr->USER      = NULL;

	bflmapinsert( Network->MAP_EXPR, Expr->NAME, Expr );
	fpm_expr_depend( Network, Expr );

	return Expr;
}

void
fpm_expr_delete( fpm_network_t *Network, fpm_expr_t *Expr )
{
	chain_list  *Chain;
	fpm_atom_t *Atom;

	if ( Network == NULL || Expr == NULL )
		return;

	for ( Chain = Expr->BDD_ATOM; Chain; Chain = Chain->NEXT ) {
		Atom = (fpm_atom_t *)Chain->DATA;
		Atom->BDD_EXPR = rmvchain( Atom->BDD_EXPR, Expr );
	}
	freechain( Expr->BDD_ATOM );
	decbddrefext( Expr->BDD );
	bflmapremove( Network->MAP_EXPR, Expr->NAME );
	fpm_expr_free( Expr );
}

fpm_expr_t *
fpm_expr_find( fpm_network_t *Network, char *Name )
{
	bflmapnode_t *MapNode;

	Name    = namealloc( Name );
	MapNode = bflmapfind( Network->MAP_EXPR, Name );

	if ( MapNode == (bflmapnode_t *)0 )
		return (fpm_expr_t *)0;
	else
		return (fpm_expr_t *)bflmapdata( MapNode );
}

void
fpm_expr_depend( fpm_network_t *Network, fpm_expr_t *Expr )
{
	chain_list   *Support;
	chain_list   *Chain;
	bddnode     *AtomNode;
	char        *AtomName;
	fpm_atom_t *Atom;

	Support = getbddnodesupport( Network->BDD_SYSTEM, Expr->BDD, BDD_SUPPORT_CHAIN );
	for ( Chain = Support; Chain != NULL; Chain = Chain->NEXT ) {
		AtomNode  = (bddnode *)Chain->DATA;
		AtomName  = fpm_atom_name( Network, AtomNode->INDEX );
		Atom      = fpm_atom_find( Network, AtomName );

		if ( Atom != (fpm_atom_t *)0 ) {
			Atom->BDD_EXPR = addchain( Atom->BDD_EXPR, Expr );
			Expr->BDD_ATOM = addchain( Expr->BDD_ATOM, Atom  );
		} else {
			bflwarning( "fpm_expr_depend: can't find atom with index %d", AtomNode->INDEX );
		}
	}
	freechain( Support );
}

long
fpm_expr_num( fpm_network_t *Network )
{
	return bflmapsize( Network->MAP_EXPR );
}
