/*
 * $Author: alliance $
 * $Id: laxparam.c,v 1.1.1.1 1998/10/21 16:32:42 alliance Exp $
 * $Date: 1998/10/21 16:32:42 $
 */
/*
 * $Log: laxparam.c,v $
 * Revision 1.1.1.1  1998/10/21 16:32:42  alliance
 * fpmap import
 *
 */

/*----------------------------------------------------------------------*/
/*                               Includes                               */
/*----------------------------------------------------------------------*/

# include <stdio.h>
# include <string.h>
# include  MUT_H
# include  BFL_H
# include  LAX_H

/*----------------------------------------------------------------------*/
/*                        Constant declarations                         */
/*----------------------------------------------------------------------*/

/*----------------------------------------------------------------------*/
/*                          Type declarations                           */
/*----------------------------------------------------------------------*/

/*----------------------------------------------------------------------*/
/*                        Stucture declarations                         */
/*----------------------------------------------------------------------*/

/*----------------------------------------------------------------------*/
/*                        Variable declarations                         */
/*----------------------------------------------------------------------*/

/*----------------------------------------------------------------------*/
/*                          Macro declarations                          */
/*----------------------------------------------------------------------*/

/*----------------------------------------------------------------------*/
/*                      Static function prototypes                      */
/*----------------------------------------------------------------------*/

/*----------------------------------------------------------------------*/
/*                    Definition of static functions                    */
/*----------------------------------------------------------------------*/

static char *laxname( Name )
    char   *Name;
{
    size_t  Len;
    char   *Ptr;
    
    if ( strchr( Name, ' ' ) != NULL ) {
	Len = strlen( Name ) + 2;
	Ptr = mbkalloc( Len );
	strncpy( Ptr, Name, Len );
	Name = Ptr;
	Ptr  = strchr( Name, ' ' );
	*Ptr = '(';
	Name[ Len - 2] = ')';
    }
    return Name;
}

static void laxprintsimple( File, Chain )
    FILE *File;
    chain_list *Chain;
{
	fputs( "{\n", File );
	while ( Chain != 0 ) {
	    fprintf( File, "    %s;\n", laxname( (char *)Chain->DATA ) );
	    Chain = Chain->NEXT;
	}
	fputs( "}\n", File );
}

static void laxprintdouble( File, PType )
    FILE *File;
    ptype_list *PType;
{
	fputs( "{\n", File );
	while ( PType != 0 ) {
	    fprintf( File, "    %s: %ld;\n", laxname( (char *)PType->DATA ), PType->TYPE );
	    PType = PType->NEXT;
	}
	fputs( "}\n", File );
}

/*----------------------------------------------------------------------*/
/*                   Definition of exported functions                   */
/*----------------------------------------------------------------------*/

lax_param_t *laxnewparam()
{
    lax_param_t *Param = (lax_param_t *)mbkalloc( sizeof( lax_param_t ) );
    Param->mode			= 2;
    Param->level		= 2;
    Param->max_slope_time	= 1000;
    Param->aux			= 0;
    Param->num_trans_n		= 4;
    Param->num_trans_p		= 4;
    Param->early_po		= 0;
    Param->delay_pi		= 0;
    Param->imp_pi		= 0;
    Param->capa_po		= 0;
    Param->capa_pi		= 0;
    Param->saved_aux		= 0;
    Param->clock		= addchain( 0, namealloc( "ck" ) );

    return Param;
}

void laxsaveparam( Param, Filename )
    lax_param_t *Param;
    char *Filename;
{
    FILE *File;

    if ( ( File = mbkfopen( Filename, "lax", "wt" ) ) == NULL )
	bflsyserror( 1, "can't open file %s.lax", Filename );

    /* Mode */
    fprintf( File, "## mode\n");
    fprintf( File, "#M{%d}\n", Param->mode);
    
    /* Level */
    fprintf( File, "## level\n");
    fprintf( File, "#L{%d}\n", Param->level);

    /* Signals */
    if ( Param->saved_aux != 0 ) {
        fputs( "## auxiliary signals saved\n", File );
	fputs( "#S", File );
	laxprintsimple( File, Param->saved_aux );
    }
    
    /* Clocks */
    if ( Param->clock != 0 ) {
        fputs( "## Clocks\n", File );
	fputs( "#K", File );
	laxprintsimple( File, Param->clock );
    }
    
    /* Delay */
    if ( Param->delay_pi != 0 ) {
	fputs( "## delayed inputs\n", File );
	fputs( "#D", File );
	laxprintdouble( File, Param->delay_pi );
    }

    /* Early */
    if ( Param->early_po != 0 ) {
        fputs( "## early outputs\n", File );
	fputs( "#E", File );
	laxprintsimple( File, Param->early_po );
    }
    
    /* TransN */
    fprintf( File, "## Number Transistor N\n");
    fprintf( File, "#N{%d}\n", Param->num_trans_n);
    
    /* TransP */
    fprintf( File, "## Number Transistor P\n");
    fprintf( File, "#P{%d}\n", Param->num_trans_p);

    /* Slope */
    fprintf( File, "## Fanout factor\n");
    fprintf( File, "#T{%ld}\n", Param->max_slope_time);
	
    /* CapI */
    if ( Param->capa_pi != 0 ) {
	fputs( "## Input Capacitance in fF\n", File );
	fputs( "#F", File );
	laxprintdouble( File, Param->capa_pi );
    }

    /* CapO */
    if ( Param->capa_po != 0 ) {
	fputs( "## Output Capacitance in fF\n", File );
	fputs( "#C", File );
	laxprintdouble( File, Param->capa_po );
    }

    /* ImpI */
    if ( Param->imp_pi != 0 ) {
	fputs( "## Input Impedance in Ohms\n", File );
	fputs( "#I", File );
	laxprintdouble( File, Param->imp_pi );
    }

    /* BufI */
    if ( Param->buff_pi != 0 ) {
	fputs( "## Buffered Input: this is a list of primary inputs\n", File );
	fputs( "## with the number of buffer you want to add.\n", File );
	fputs( "#B", File );
	laxprintdouble( File, Param->buff_pi );
    }

    fclose( File );
    printf( " * Saving parameter file '%s.lax' ...\n", Filename );
}

