#include <assert.h>
#include <stdlib.h>

/*
   ### -------------------------------------------------- ###
   $Author: hcl $
   $Date: 2002/02/25 11:41:13 $
   $Log: ocrAstar.c,v $
   Revision 1.5  2002/02/25 11:41:13  hcl
   A la chasse au bug bug bug

   Revision 1.4  2002/02/22 15:20:49  hcl
   A* is ready

   Revision 1.3  2002/02/21 13:17:57  hcl
   Introducing a new algo (A*, dont yet work...)

   Revision 1.2  2002/02/15 12:06:31  hcl
   First attempt

   Revision 1.1  2002/02/12 15:14:14  hcl
   New algo.

   ### -------------------------------------------------- ###
*/

#include MUT_H
#include MLO_H
#include MLU_H
#include MPH_H
#include MPU_H
#include "ocr.h"
#include "ocrUtil.h"
#include "ocrWRoutingDataBase.h"
#include "ocrWindow.h"
#include "mbk_tree.h"
#include "ocrWRoutingUtil.h"
#include "ocrWRouting.h"
#include "display.h"

extern ocrOption *g_pOption;
#define LEVEL (g_pOption->LEVEL)

/*********************************************************************/
#define TAG_KEEP             1
#define TAG_CLEAN            2
#define TAG_FREED            4
#define TAG_SEEN             8


/*********************************************************************/

/**
 * Variables globales
 **/
ocrWRoutingGrid *grid = NULL;
ocrRoutingParameters *param = NULL;

ocrNaturalInt xs = 0, ys = 0;

/*********************************************************************/

/**
 *
 * Local data structures
 *
 **/


/*********************************************************************/

/**
 * Functions
 */

/* Cost function (Manhattan distance) */
ocrNaturalInt eval (ocrWSegment *segment_source, ocrWSegment *segment_dest) {

    ocrNaturalInt x1s, x2s, y1s, y2s, x1d, x2d, y1d, y2d, dx, dy;

    x1s = getWSegXCoord (param, segment_source, segment_source->P_MIN);
    y1s = getWSegYCoord (param, segment_source, segment_source->P_MIN);
    x2s = getWSegXCoord (param, segment_source, segment_source->P_MAX);
    y2s = getWSegYCoord (param, segment_source, segment_source->P_MAX);

    x1d = getWSegXCoord (param, segment_dest, segment_dest->P_MIN);
    y1d = getWSegYCoord (param, segment_dest, segment_dest->P_MIN);
    x2d = getWSegXCoord (param, segment_dest, segment_dest->P_MAX);
    y2d = getWSegYCoord (param, segment_dest, segment_dest->P_MAX);

    dx = ( (x2s < x1d)? (x1d - x2s) : ( (x1s > x2d) ? (x1s - x2d) : 0 ) );
    dy = ( (y2s < y1d)? (y1d - y2s) : ( (y1s > y2d) ? (y1s - y2d) : 0 ) );

    return (dx + dy);
}

/* kost of the path from segment_source to segment_dest */
inline ocrNaturalInt kost (ocrWSegment *segment_source, ocrWSegment *segment_dest) {

    ocrWSegment *ps;
    ocrNaturalInt dest_offset, p;
    
    ps = segment_source->ROOT;
    dest_offset = segment_dest->OFFSET;

    if (ps) {
        return (
                (ps->OFFSET > dest_offset) ?
                (ps->OFFSET - dest_offset) :
                (dest_offset - ps->OFFSET)
               )
            ;
    } else {
        p = (getWSegDirection (param, segment_source) == ocrHorizontal) ?
            xs : ys;

        return (
                (p > dest_offset) ?
                (p - dest_offset) :
                (dest_offset - p)
               )
            ;
    }
}


void tag_keep (ocrWSegment *path) {

    while (path) {
        path->TAG = TAG_KEEP;
        path = path->ROOT;
    }
    return;
}

/**
 * stack
 **/

ocrWSegment *pop (ocrWSegment **list) {
    ocrWSegment *res;

    assert (*list);

    res = (*list);
    (*list) = ( (*list)->NEXT );
    res->NEXT = NULL;

    return res;
}

void push (ocrWSegment **list, ocrWSegment *elem) {
    
    assert (elem);

    elem->NEXT = (*list);
    (*list) = elem;

    return;
}

/**
 * clean_mem
 **/
void clean_tag () {
    ocrWSegment *seg;
    int i, j, k;

    for (k = 0 ; k < grid->NB_OF_LAYERS ; k += 2)
        for (j = 0 ; j < grid->SIZE_V ; j++)
            for (i = 0 ; i < grid->SIZE_H ; i++) {
                seg = getWSegment (grid, i, j, k);
                seg->TAG = TAG_CLEAN;
                i = seg->P_MAX;
        }

    for (k = 1 ; k < grid->NB_OF_LAYERS ; k += 2)
        for (i = 0 ; i < grid->SIZE_H ; i++)
            for (j = 0 ; j < grid->SIZE_V ; j++) {
                seg = getWSegment (grid, i, j, k);
                seg->TAG = TAG_CLEAN;
                j = seg->P_MAX;
            }
                
    return;
}

void clean_tag_list (ocrWSegment *list) {

    while (list) {
        list->TAG = TAG_CLEAN;
        list = list->NEXT;
    }
    return;
}

/**
 * insert elem keeping the list sorted by increasing cost+h
 **/
ocrWSegment *insert (ocrWSegment *list, ocrWSegment *elem) {

    assert (elem);

    if (!list) {
        elem->NEXT = NULL;
        return elem;
    } else if ( (elem->HCOST) > (list->HCOST) ) {
        list->NEXT = insert (list->NEXT, elem);
        return list;
    } else if ( ((elem->HCOST) == (list->HCOST)) && ((elem->COST) > (list->COST)) ){
        list->NEXT = insert (list->NEXT, elem);
        return list;
    } else {
        elem->NEXT = list;
        return elem;
    }
}


ocrWSegment *_remove (ocrWSegment *list, ocrWSegment *elem) {

    assert (elem);

    if (!list)
        return list;
    if (elem == list)
        return elem->NEXT;
    
    return _remove (list->NEXT, elem);
}
    
ocrWSegment *explore (ocrWSegment *segment_source, ocrWSegment *segment_target) {

    ocrWSegment *visited = NULL;
    ocrWSegment *terra_incognita = NULL;
    ocrNaturalInt p, cost;
    int i;

    assert (segment_source != segment_target);

    display (LEVEL, DEBUG, "d starting exploration\n");

    terra_incognita = segment_source;
    terra_incognita->COST = 0;
    terra_incognita->H = eval (segment_source, segment_target);
    terra_incognita->HCOST = terra_incognita->H;
    terra_incognita->TAG = TAG_SEEN;
    terra_incognita->ROOT = NULL;
    terra_incognita->NEXT = NULL;

    while (terra_incognita) {
        ocrWSegment *node = NULL;
        ocrWSegment *seg = NULL;
        
        node = pop (&terra_incognita);
        push (&visited, node);

        //if (node->seg == segment_target) {
            //erase (terra_incognita->NEXT);
            //terra_incognita->NEXT = NULL;
        //    return node;
        //}

        for (i = -1 ; i <= 1 ; i += 2) {
            switch (i) {

                case -1: if ( ((node->LAYER) ) == 0 )
                            continue;
                        break;
                case  1: if ( ((node->LAYER) + 1) >= (grid->NB_OF_LAYERS) )
                            continue;
                        break;
                    
                default:
                    /*display (LEVEL, ERROR, __FILE__ ":" __LINE__ "exand_eval Oops\n");*/
                    abort ();
            }
            for (p = node->P_MIN ; p <= node->P_MAX ; p++) {
            
                display (LEVEL, DEBUG, "d expanding %ld\n", i);
                seg = getWSegment(grid,
                                  getWSegXCoord (param, node, p),
                                  getWSegYCoord (param, node, p),
                                  node->LAYER + i
                                 );

#ifdef DEBUG
                if (!seg) {
                    display (LEVEL, DEBUG, "oddity: null seg at (%ld, %ld, %ld)\n", getWSegXCoord(param, node, p), getWSegXCoord(param, node, p), node->LAYER + i);
                    continue;
                }
#endif

                if (seg == segment_target) {
                    /* bingo ! */
                    clean_tag_list (visited);
                    clean_tag_list (terra_incognita);
                    seg->ROOT = node;
                    return seg;
                }
            
                if (isObstructed (seg))
                    continue;

                cost = node->COST + kost (node, seg); /* XXX cout du chemin parcouru ??? */
                if ( ((seg->TAG == TAG_SEEN) && (seg->COST > cost)) || (seg->TAG != TAG_SEEN) ) {
                    seg->COST = cost;
                    if (seg->TAG != TAG_SEEN) {
                        seg->H = eval (seg, segment_target);
                        /*display (LEVEL, DEBUG, "astar unseen\n");*/
                    } else {
                        /* il y a sans doute quelque chose a faire ici */
                        /*display (LEVEL, DEBUG, "astar better than already visited\n");*/
                        terra_incognita = _remove (terra_incognita, seg);
                    }
                    seg->ROOT = node;
                    seg->HCOST = seg->H + cost;
                    seg->TAG = TAG_SEEN;
                    terra_incognita = insert (terra_incognita, seg);
                    /*display (LEVEL, DEBUG, "astar one more %ld\n", cost);*/
                } else {
                    /*display (LEVEL, DEBUG, "astar worse than already visited\n");*/
                }
            }
        } /* end for i */
    } /* while */

    /* Failed */
    clean_tag ();

    /*isplay (LEVEL, DEBUG, "d exploration failed\n");*/
    
    
    return NULL;
}

void addseglist (ocrSignal *s, ocrWSegment *l2) {
    ocrWSegment *l1;

    if (!(s->SEGMENT))
        s->SEGMENT = l2;
    else {
        l1 = s->SEGMENT;
        while (l1->NEXT)
            l1 = l1->NEXT;
        l1->NEXT = l2;
    }
    return;
}

ocrNaturalInt make_segments (ocrWSegment *segment_dest,
                    ocrWSegment *segment_source,
                    ocrNaturalInt xsource,
                    ocrNaturalInt ysource,
                    ocrNaturalInt zsource,
                    ocrNaturalInt xdest,
                    ocrNaturalInt ydest,
                    ocrNaturalInt zdest,
                    ocrNaturalInt signal_index,
                    ocrSignal * i_pSignal
                   )
{
    ocrWSegment *seg, *nseg, *root, *aux;
    ocrNaturalInt xp, yp, xn, yn;
    ocrNaturalInt p1, p2, pp1, pp2;
    ocrNaturalInt distance = 0;
    ocrNaturalInt nb_segs = 0;

    /* initialisation */

    aux = i_pSignal->SEGMENT;

    display (LEVEL, DEBUG, "c dest (%ld, %ld, %ld)\n", xdest, ydest, zdest);
    xp = xdest;
    yp = ydest;
    seg = segment_dest;
    nseg = NULL;

    /*

       seg, seg->ROOT
       nseg <= split (seg)

       nseg -> NEXT = prevseg;
       prevseg = nseg;

    */
    
    while (seg->ROOT) {
        root = seg->ROOT;
        xn = getWSegXCoord (param, root, xp);
        yn = getWSegYCoord (param, root, yp);
        display (LEVEL, DEBUG, "p (%ld, %ld, %ld) -> (%ld, %ld, %ld) : d=%ld, c=%ld, h=%ld, hc=%ld\n", xp, yp, seg->LAYER, xn, yn, seg->LAYER, distance, seg->COST, seg->H, seg->HCOST);
        ((getWSegDirection(param, seg) == ocrHorizontal) ? (p1 = xp, p2 = xn) : (p1 = yp, p2 = yn) );
        ( (p1 < p2) ? (pp1 = p1, pp2 = p2) : (pp1 = p2, pp2 = p1) );
        nseg = splitWSegment (param, grid, seg,
                              pp1, pp2,
                              signal_index
                             );
        nseg->NEXT = aux;
        aux = nseg;
        distance += pp2 - pp1;
        nb_segs ++;
        xp = xn;
        yp = yn;
        /*seg->SIGNAL_INDEX = signal_index;*/
        seg = root;
    }

    /*assert (nseg == segment_source);*/

    xn = xsource;
    yn = ysource;
    /*root = seg->ROOT;*/
    ((getWSegDirection(param, seg) == ocrHorizontal) ? (p1 = xp, p2 = xn) : (p1 = yp, p2 = yn) );
    ( (p1 < p2) ? (pp1 = p1, pp2 = p2) : (pp1 = p2, pp2 = p1) );
    nseg = splitWSegment (param, grid, seg,
                          pp1, pp2,
                          signal_index
                         );
    nseg->NEXT = aux;
    i_pSignal->SEGMENT = nseg;

    distance += pp2 - pp1;
    nb_segs ++;
    display (LEVEL, DEBUG, "p (%ld, %ld, %ld) -> (%ld, %ld, %ld) : %ld\n", xp, yp, seg->LAYER, xn, yn, zsource, distance);
    display (LEVEL, DEBUG, "c source (%ld, %ld, %ld)\n", xsource, ysource, zsource);
    display (LEVEL, DEBUG, "p num of segments : %ld\n", nb_segs);
    /*seg->SIGNAL_INDEX = signal_index;*/



    /* ajout a equipot deja existante */
    /*addseglist (i_pSignal, aux);*/
    
    display (LEVEL, DEBUG, "i path length : %ld\n", distance);
    
    return distance;
}

    

ocrNaturalInt check_path (ocrSignal *i_pSignal) {

    ocrWSegment *seg;
    ocrNaturalInt dist;

    seg = i_pSignal->SEGMENT;

    dist = 0;

    while (seg) {
        if ( (seg->SIGNAL_INDEX) != (i_pSignal->INDEX) ) {
            display (LEVEL, ERROR, " ---INVALID Segment->SIGNAL_INDEX\n");
            abort ();
        }

        display (LEVEL, DEBUG, "(%ld, %ld, %ld, %ld)\n", seg->OFFSET, seg->P_MIN, seg->P_MAX, seg->LAYER);

        dist += seg->P_MAX - seg->P_MIN;

        seg = seg->NEXT;
    }

    display (LEVEL, DEBUG, "DIST CHECK --- %ld\n", dist);

    return dist;
}

 


    
ocrNaturalInt find_path_astar (ocrRoutingParameters * p_param,
                      ocrWRoutingGrid * p_grid,
                      ocrNaturalInt xsource,
                      ocrNaturalInt ysource,
                      ocrNaturalInt zsource,
                      ocrNaturalInt xtarget,
                      ocrNaturalInt ytarget,
                      ocrNaturalInt ztarget,
                      ocrNaturalInt signal_index,
                      ocrSignal * i_pSignal)
{
    ocrWSegment *path = NULL;
    ocrWSegment *segment_source = NULL;
    ocrWSegment *segment_dest = NULL;
    ocrNaturalInt dist, check_dist;

    grid = p_grid;
    param = p_param;

    xs = xsource;
    ys = ysource;

    /*clean_tag ();*/

    
    
    segment_source = getWSegment (grid,
                                  xsource,
                                  ysource,
                                  zsource
                                 );
    display (LEVEL, DEBUG, "d source : %p\n", segment_source);
    segment_dest   = getWSegment (grid,
                                  xtarget,
                                  ytarget,
                                  ztarget
                                 );
    display (LEVEL, DEBUG, "d dest : %p\n", segment_dest);
    if ((segment_source != segment_dest) && (!isObstructed(segment_source))) {
        path = explore (segment_source, segment_dest);
    } else {
        path = segment_source;
        //path->TAG = TAG_KEEP;
        path->ROOT = NULL;
        path->NEXT = NULL;
        display (LEVEL, DEBUG, "o kewl direct link\n");
    }

    if (!path) {
        display (LEVEL, VVERB, "o A* failed\n");
        return OCRNATURALINT_MAX;
    }
    
    display (LEVEL, VVERB, "o A* succeeded\n");
    
    /*clean_tag ();*/
    
    dist = make_segments (path,
                          segment_source,
                          xsource,
                          ysource,
                          zsource,
                          xtarget,
                          ytarget,
                          ztarget,
                          signal_index,
                          i_pSignal
                         );


    /*assert (((check_dist = check_path (i_pSignal)), dist == check_dist));*/

    return dist;
}


/**
 *
 **/
/*void init_Astar (ocrRoutingDataBase *db) {
    assert (db);

    grid = db->GRID;
    param = db->PARAM;

    return;    
}
*/
