#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>

#include MUT_H
#include LOG_H
#include "beh.h"
#include "bhl.h"
#include "bvl.h"

#define POW_STA_FIELD 1
#define POW_NRJ_FIELD 2
#define POW_ALODFN    64
#define POW_HSZDFN   307

struct poden                    /* dictionary entry point               */
{
  struct poden *NEXT;           /* next entry                           */
  struct podrd *DATA;           /* data                                 */
  char         *KEY ;           /* key                                  */
};

struct podrd                    /* dictionary record                    */
{
  struct podrd *NEXT   ;        /* next record                          */
  char         *KEY    ;        /* context key                          */
  float         FL0_VAL;        /* data field                           */
  float         FL1_VAL;        /* data field                           */
  float         FL2_VAL;        /* data field                           */
};

static struct poden *POW_DCEHED = NULL;		/* free entries list	*/
static struct podrd *POW_DCRHED = NULL;		/* free records list	*/

/* ###--------------------------------------------------------------### */
/* function	: pow_addent						*/
/* description	: add a new entry point to the dictionary		*/
/* called func.	: mbkalloc						*/
/* ###--------------------------------------------------------------### */

static struct poden *pow_addent (head, key)

struct poden *head;
char         *key;

{
  struct poden *entry;
  int           i    ;

  if (POW_DCEHED == NULL)
    {
    POW_DCEHED = (struct poden *)
                 mbkalloc (sizeof (struct poden) * POW_ALODFN);

    entry = POW_DCEHED;
    for (i=1 ; i<POW_ALODFN ; i++)
      {
      entry->NEXT = entry + 1;
      entry++;
      }
    entry->NEXT = NULL;
    }

  entry       = POW_DCEHED;
  POW_DCEHED  = POW_DCEHED->NEXT;

  entry->NEXT = head;
  entry->DATA = NULL;
  entry->KEY  = key;

  return (entry);
}

/* ###--------------------------------------------------------------### */
/* function	: pow_addrcd						*/
/* description	: add a new record to the dictionary			*/
/* called func.	: mbkalloc						*/
/* ###--------------------------------------------------------------### */

static struct podrd *pow_addrcd (head, key)

struct podrd *head;
char         *key;

{
  struct podrd *recrd;
  int           i    ;

  if (POW_DCRHED == NULL)
    {
    POW_DCRHED = (struct podrd *)
                 mbkalloc (sizeof (struct podrd) * POW_ALODFN);

    recrd = POW_DCRHED;
    for (i=1 ; i<POW_ALODFN ; i++)
      {
      recrd->NEXT = recrd + 1;
      recrd++;
      }
    recrd->NEXT = NULL;
    }

  recrd           = POW_DCRHED;
  POW_DCRHED      = POW_DCRHED->NEXT;

  recrd->NEXT     = head;
  recrd->FL0_VAL  = 0.0;
  recrd->FL1_VAL  = 0.0;
  recrd->FL2_VAL  = 0.0;
  recrd->KEY      = key;

  return (recrd);
}

/* ###--------------------------------------------------------------### */
/* function	: pow_initab						*/
/* description	: create a new dictionary				*/
/* called func.	: mbkalloc						*/
/* ###--------------------------------------------------------------### */

struct poden **pow_initab ()

{
  struct poden **head;
  int                i;

  head = (struct poden **)
         mbkalloc (sizeof(struct poden *) * POW_HSZDFN);

  for (i=0 ; i<POW_HSZDFN ; i++)
    head[i] = NULL;

  return (head);
}

/* ###--------------------------------------------------------------### */
/* function	: pow_addtab						*/
/* description	: save a data in a dictionary				*/
/* called func.	: pow_addent, pow_addrcd				*/
/* ###--------------------------------------------------------------### */

void pow_addtab (head, key_str, ctx_str, field, valu)

struct poden **head;
char          *key_str;
char          *ctx_str;
int            field;
float          valu;

{
  int           found     = 0;
  unsigned int  key       ;
  unsigned int  index     ;
  struct poden *entry_pnt ;
  struct podrd *recrd_pnt ;


  key       = ((unsigned int) key_str) + ((unsigned int) ctx_str);

  index     = key % POW_HSZDFN;
  entry_pnt = head [index];

  while (entry_pnt != NULL)
    {
    if (entry_pnt->KEY == key_str)
      {
      found = 1;
      break;
      }
    entry_pnt = entry_pnt->NEXT;
    }

  if (found == 0)
    {
    head [index] = pow_addent (head [index], key_str); 
    entry_pnt    = head [index]; 
    }

  found     = 0;
  recrd_pnt = entry_pnt->DATA;
  while (recrd_pnt != NULL)
    {
    if (recrd_pnt->KEY == ctx_str)
      {
      found = 1;
      break;
      }
    recrd_pnt = recrd_pnt->NEXT;
    }

  if (found == 0)
    {
    entry_pnt->DATA = pow_addrcd (entry_pnt->DATA, ctx_str); 
    recrd_pnt       = entry_pnt->DATA ;
    }

  switch (field)
    {
    case 0 :
      recrd_pnt->FL0_VAL = valu; break;
    case 1 :
      recrd_pnt->FL1_VAL = valu; break;
    case 2 :
      recrd_pnt->FL2_VAL = valu; break;
    }

}

/* ###--------------------------------------------------------------### */
/* function	: pow_chktab						*/
/* description	: extract a data from a dictionary			*/
/* called func.	: none							*/
/* ###--------------------------------------------------------------### */

float pow_chktab (head, key_str, ctx_str, field)

struct poden **head   ;
char          *key_str;
char          *ctx_str;
int            field  ;

{
  int           found    = 0;
  float         valu     = 0;
  unsigned int  key      ;
  unsigned int  index    ;
  struct poden *entry_pnt;
  struct podrd *recrd_pnt;


  key       = ((unsigned int) key_str) + ((unsigned int) ctx_str);

  index     = key % POW_HSZDFN;
  entry_pnt = head [index];

  while (entry_pnt != NULL)
    {
    if (entry_pnt->KEY == key_str)
      {
      found = 1;
      break;
      }
    entry_pnt = entry_pnt->NEXT;
    }

  if (found == 1)
    {
    found     = 0;
    recrd_pnt = entry_pnt->DATA;
    while (recrd_pnt != NULL)
      {
      if (recrd_pnt->KEY == ctx_str)
        {
        found = 1;
        break;
        }
      recrd_pnt = recrd_pnt->NEXT;
      }
    if (found == 1)
      {
      switch (field)
        {
        case 0 :
          valu = recrd_pnt->FL0_VAL; break;
        case 1 :
          valu = recrd_pnt->FL1_VAL; break;
        case 2 :
          valu = recrd_pnt->FL2_VAL; break;
        }
      }
    }

  return (valu);
}

/* ###--------------------------------------------------------------### */
/* function	: pow_fretab						*/
/* description	: release a dictionary					*/
/* called func.	: mbkfree						*/
/* ###--------------------------------------------------------------### */

void pow_fretab (pt_hash)

struct poden **pt_hash;

{
  struct poden *pt_entry   ;
  struct poden *pt_nxtentry;
  struct podrd *pt_record  ;
  int           i          ;

  if (pt_hash != NULL)
    {
    for (i=0 ; i<POW_HSZDFN ; i++)
      {
      if ((pt_entry = pt_hash [i]) != NULL)
        {
        while (pt_entry != NULL)
          {
          pt_record = pt_entry->DATA;

          while (pt_record->NEXT != NULL)
            pt_record = pt_record->NEXT;

          pt_record->NEXT = POW_DCRHED;
          POW_DCRHED      = pt_entry->DATA;

          pt_nxtentry     = pt_entry->NEXT;
          pt_entry->NEXT  = POW_DCEHED;
          POW_DCEHED      = pt_entry;
          pt_entry        = pt_nxtentry;
          }
        }
      }
    mbkfree (pt_hash);
    }
}
/* ###--------------------------------------------------------------### */
/* function	: pow_backsta  						*/
/* ###--------------------------------------------------------------### */

void pow_backsta ( sta_dic, sta_filename, fig_name )

struct poden **sta_dic;
char          *sta_filename;
char          *fig_name;

{
  FILE	*sta_file;
  char	 line[256];
  char	 direction[256];
  char	 instance[256];
  char   signame[256];
  char	 vector_radical[256];
  int	 index;
  float	 proba;
  float	 density;

  printf("MKPOW : compiling `%s.sta`\n", sta_filename);

  sta_file = mbkfopen( sta_filename, "sta", "r" );

  if ( sta_file == NULL )
    {
    printf("MKPOW : cannot find `%s.sta`\n", sta_filename);
    EXIT(1);
    }
 
   
  while ( fgets(line, 256, sta_file) != NULL)
    {
    if ( sscanf(line,"%s %s : p('1') = %f | density = %f ;", direction, signame,
                      &proba, &density ) == 4 )
      {
        pow_addtab (sta_dic, namealloc(signame), NULL, POW_STA_FIELD, density);
      }
    else if ( sscanf(line,"%s.%s : p('1') = %f | density = %f ;", instance,
              signame, &proba, &density ) == 4 )
      {
      pow_addtab (sta_dic, namealloc(signame), NULL, POW_STA_FIELD, density);
      }
    else if ( sscanf(line, "%s %s %d : p('1') = %f | density = %f ;", direction,
              vector_radical, &index, &proba, &density ) == 5 )
      {
      sprintf(signame, "%s %d", vector_radical, index);
      pow_addtab (sta_dic, namealloc(signame), NULL, POW_STA_FIELD, density);
      }
    else if ( sscanf(line,"%s : p('1') = %f | density = %f ;",
                     signame, &proba, &density ) == 3 )
      {
      pow_addtab (sta_dic, namealloc(signame+strlen(fig_name)+1), NULL, POW_STA_FIELD, density);
      }
    else if ( sscanf(line, "%s %d : p('1') = %f | density = %f ;",
                     vector_radical, &index, &proba, &density ) == 4 )
      {
      sprintf(signame, "%s %d", vector_radical, index);
      pow_addtab (sta_dic, namealloc(signame+strlen(fig_name)+1), NULL, POW_STA_FIELD, density);
      }
      /* LUDO
    else
      {
      if ( (line[0] != '\n') &&
           (line[0] != '#' ))
        {
        printf("WARNING : invalid line in file %s : \n", sta_filename);
        printf("        : %s\n", line);
        }
      }
      */
    }
  fclose(sta_file);
}

/* ###--------------------------------------------------------------### */
/* function	: pow_backnrj  						*/
/* ###--------------------------------------------------------------### */

void pow_backnrj ( nrj_dic, nrj_filename )

struct poden **nrj_dic;
char          *nrj_filename;
{
  FILE	*nrj_file;
  char	 line[256];
  char   signame[256];
  char	 vector_radical[256];
  int	 index;
  float	 nrj;
  float	 capa;

  printf("MKPOW : compiling `%s.nrj` (energy file)\n", nrj_filename);

  nrj_file = mbkfopen( nrj_filename, "nrj", "r" );

  if ( nrj_file == NULL )
    {
      printf("MKPOW : cannot find `%s.nrj`\n", nrj_filename);
      EXIT(1);
    }
 
   
  while ( fgets(line, 256, nrj_file) != NULL)
    {
    if ( sscanf( line, "%s : %f 10e-12 J [ %f 10e-12 F]",
                 signame, &nrj, &capa ) == 3 )
      {
      pow_addtab (nrj_dic, namealloc(signame), NULL, POW_NRJ_FIELD, nrj);
      }
    else if ( sscanf( line, "%s %d : %f 10e-12 J [ %f 10e-12 F]",
                      vector_radical, &index, &nrj, &capa ) == 4 )
      {
      sprintf(signame, "%s %d", vector_radical, index);
      pow_addtab (nrj_dic, namealloc(signame), NULL, POW_NRJ_FIELD, nrj);
      }
    else
      {
      sscanf( line, "%s", signame );
      if (line[0] != '\n')
        {
        printf("WARNING : invalid line in file %s : \n", nrj_filename);
        printf("        : %s\n", line);
        }
      }
    }
  fclose(nrj_file);
}

/* ###--------------------------------------------------------------### */
/* function	: pow_setpower						*/
/* ###--------------------------------------------------------------### */

void pow_setpower (pt_befig, sta_dic, nrj_dic, pow_filename, freq)

struct befig  *pt_befig;
struct poden **sta_dic;
struct poden **nrj_dic;
char          *pow_filename;
float	       freq;

{
  struct bepor         *pt_bepor ;
  struct beout         *pt_beout ;
  struct beaux         *pt_beaux ;
  struct bebus         *pt_bebus ;
  struct bebux         *pt_bebux ;
  struct bereg         *pt_bereg ;
  struct biabl         *pt_biabl ;
  FILE	*pow_file;
  float	 nrj;
  float	 sta;
  float	 pow;
  float	 total_pow = 0.0;

  printf("MKPOW : drives `%s.pow`\n", pow_filename);
  printf( "MKPOW : Frequency %f Mhz\n", freq );

  pow_file = mbkfopen( pow_filename, "pow", "w" );

  if ( pow_file != NULL )
    {
    fprintf(pow_file, "Cycle Frequency : %-6.4f 10e+6 Hz\n",  freq);

    pt_bepor = pt_befig->BEPOR;
    while (pt_bepor != NULL)
      {
      sta = pow_chktab (sta_dic, pt_bepor->NAME, NULL, POW_STA_FIELD) ;
      nrj = pow_chktab (nrj_dic, pt_bepor->NAME, NULL, POW_NRJ_FIELD) ;
# ifdef DEBUG
      fprintf( stdout, "PORT %s %f %f\n", pt_bepor->NAME, sta, nrj );
# endif
      pow = sta * nrj * freq;
      total_pow += pow;
      fprintf(pow_file, "%-15s : %-6.4f 10e-6 W [ %-6.4f 10e-12 J ] [ %-6.4f Tr/Cycle ]\n", pt_bepor->NAME, pow, nrj, sta);
      pt_bepor = pt_bepor->NEXT;
      }

    pt_beaux = pt_befig->BEAUX;
    while (pt_beaux != NULL)
      {
      sta = pow_chktab (sta_dic, pt_beaux->NAME, NULL, POW_STA_FIELD) ;
      nrj = pow_chktab (nrj_dic, pt_beaux->NAME, NULL, POW_NRJ_FIELD) ;
# ifdef DEBUG
      fprintf( stdout, "AUX %s %f %f\n", pt_beaux->NAME, sta, nrj );
# endif
      pow = sta * nrj * freq;

/*---------
fprintf(stdout, "%s sta : %-6.4f  ",  pt_beaux->NAME, sta);
fprintf(stdout, "nrj : %-6.4f  ",  nrj);
fprintf(stdout, "pow : %-6.4f\n",  pow);
----------*/

      total_pow += pow;
      fprintf(pow_file, "%-15s : %-6.4f 10e-6 W [ %-6.4f 10e-12 J ] [ %-6.4f Tr/Cycle ]\n", pt_beaux->NAME, pow, nrj, sta);
      pt_beaux = pt_beaux->NEXT;
      }

    pt_bereg = pt_befig->BEREG;
    while (pt_bereg != NULL)
      {
      sta = pow_chktab (sta_dic, pt_bereg->NAME, NULL, POW_STA_FIELD) ;
      nrj = pow_chktab (nrj_dic, pt_bereg->NAME, NULL, POW_NRJ_FIELD) ;
      pow = sta * nrj * freq;
      total_pow += pow;
      fprintf(pow_file, "%-15s : %-6.4f 10e-6 W [ %-6.4f 10e-12 J ] [ %-6.4f Tr/Cycle ]\n", pt_bereg->NAME, pow, nrj, sta);
      pt_bereg = pt_bereg->NEXT;
      }

/*----------
    pt_bebus = pt_befig->BEBUS;
    while (pt_bebus != NULL)
      {
      sta = pow_chktab (sta_dic, pt_bebus->NAME, NULL, POW_STA_FIELD) ;
      nrj = pow_chktab (nrj_dic, pt_bebus->NAME, NULL, POW_NRJ_FIELD) ;
      pow = sta * nrj * freq;
      total_pow += pow;
      fprintf(pow_file, "%-15s : %-6.4f 10e-6 W [ %-6.4f 10e-12 J ] [ %-6.4f Tr/Cycle ]\n", pt_bebus->NAME, pow, nrj, sta);
      pt_bebus = pt_bebus->NEXT;
      }

    pt_bebux = pt_befig->BEBUX;
    while (pt_bebux != NULL)
      {
      sta = pow_chktab (sta_dic, pt_bebux->NAME, NULL, POW_STA_FIELD) ;
      nrj = pow_chktab (nrj_dic, pt_bebux->NAME, NULL, POW_NRJ_FIELD) ;
      pow = sta * nrj * freq;
      total_pow += pow;
      fprintf(pow_file, "%-15s : %-6.4f 10e-6 W [ %-6.4f 10e-12 J ] [ %-6.4f Tr/Cycle ]\n", pt_bebux->NAME, pow, nrj, sta);
      pt_bebux = pt_bebux->NEXT;
      }
----------*/

    fprintf(pow_file, "Total Power : %-6.4f 10e-6 W\n",  total_pow);
    fclose( pow_file );
    printf( "MKPOW : total power %-6.4f 10e-6 W\n", total_pow);
    }
}



usage()
{
   fprintf(stderr, "Usage: epoc beh_file sta_file nrj_file pow_file freq\n" );
   fprintf(stderr, "       beh_file is the input vhdl behavioral file\n");
   fprintf(stderr, "       sta_file is the input statistics file\n");
   fprintf(stderr, "       nrj_file is the input energy file\n");
   fprintf(stderr, "       pow_file is the output power file\n");
   fprintf(stderr, "       freq     is the cycle frequency in MHz\n");
   exit(1);
}

main(argc, argv)
int argc;
char **argv;
{
  struct befig    *ptbefig;
  struct poden   **sta_dic      ;
  struct poden   **nrj_dic     ;
  unsigned int     comp_mode     = 0         ;  /* beh compiler mode    */

  mbkenv();

  alliancebanner_with_authors( 
      "EPoC", "1.00", "Estimation of POwer Consumption", "1999",
      ALLIANCE_VERSION, "Julien Dunoyer" );

  if (argc != 6)
    usage(argv[0]);

  sta_dic   = pow_initab ();
  nrj_dic   = pow_initab ();


  comp_mode = BVL_TRACE | BVL_KEEPAUX ;

  ptbefig   = vhdlloadbefig (ptbefig, argv [1], comp_mode);

  pow_backsta (sta_dic, argv[2], ptbefig->NAME);
  pow_backnrj (nrj_dic, argv[3]);
  pow_setpower  (ptbefig, sta_dic, nrj_dic, argv[4], atof(argv[5]));

  exit(0);
}

