
/* ###--------------------------------------------------------------### */
/* 									*/
/* file		: gss_debug.c						*/
/* date		: Sep 26 1995						*/
/* version	: v103							*/
/* authors	: Pirouz BAZARGAN SABET					*/
/* content	: high level function					*/
/*									*/
/* ###--------------------------------------------------------------### */

#include <stdio.h>
#include MUT_H
#include MLO_H
#include LOG_H
#include "beh.h"
#include GSS_H
#include "gss_debug.h"

/* ###--------------------------------------------------------------### */
/* function	: gss_debug						*/
/* description	: display any structure					*/
/* called func.	: gss_error, gss_message				*/
/* ###--------------------------------------------------------------### */

void gss_debug (head_pnt, type)

void *head_pnt;				/* structure's pointer		*/
char *type;				/* structure's type		*/

  {
  static int           translate  ();
  static int           splitline  ();
  static void          disp_immd  ();
  static void          get_size   ();
  static void          read_field ();
  static void          push       ();
  static void          pop        ();
  static struct chain *go_forward ();

  char          line   [128];		/* buffer to read a cmd line	*/
  char          buffer [128];		/* buffer to split the cmd line	*/

  char         *words  [ 10];		/* number of words on a line	*/
  int           nmbrs  [ 10];		/* words translated into number	*/
  char          flags  [ 10];		/* set if words is a number	*/
  int           indxs  [ 10];		/* index of words		*/

  struct stack  jtab   [ 10];		/* list of memorized addresses	*/
  int           idx, i;
  int           readflg = 0;
  int           code;
  unsigned int  size;
  char         *pntr   = NULL;
  long          pshtype;
  int           wrdcnt = 1;

  struct stack  stk [STKSIZ_DFN];
  int           stkpnt = -1;

  union value   pnt [MAXCMD_DFN];
  long          typ [MAXCMD_DFN];
  unsigned int  siz [MAXCMD_DFN];

  static char  *str [] = {
                         "_help"     , "_back"     , "_exit"     , "_jump"    ,
                         "_save"     , "_stop"     , "_top"      , "_up"      ,
                         "_display"  ,

                         "character" , "short"     , "integer"   , "long"     ,
                         "float"     , "void"      , "string"    ,

                         "chain"     , "ptype"     ,

                         "lofig"     , "locon"     , "losig"     , "loins"    ,

                         "befig"     , "bereg"     , "bemsg"     , "berin"    ,
                         "beout"     , "bebus"     , "beaux"     , "bedly"    ,
                         "bebux"     , "bepor"     , "begen"     , "biabl"    ,
                         "binode"    , "bequad",

                         "gssseq"    , "gssevt"    ,

                         "lkdfig"    , "lkdins"    , "simsig"    , "bussig" , 
                         "wrbsig"    , "wriaux"    , "wrimsg"    , "wribux"   ,
                         "wrireg"    , "wrssig"    , "depsig"    ,

                         "abl"       , "bdd"       , "gex"       ,

                         "aux_ref"   , "bus_ref"   , "bux_ref"   , "capa"     ,
			 "cndabl"    , "cndnode"   , "cndval"    , "curval"   ,
			 "data"      , "derivate"  , "direction" , "dly_ref"  ,
                         "drvval"   ,
                         "errflg"    , "eval"      , "figname"   , "index"    ,
			 "insname"   , "label"     , "level"     , "message"  ,
			 "mode"      , "modelchain", "msg_ref"   , "name"     ,
			 "namechain" , "next"      , "node"      , "out_ref"  ,
                         "prjbvl"    , "prjval"    , "redlst"   ,
			 "reg_ref"   , "root"      , "sig"       , "signal"   ,
			 "type"     ,
			 "user"      , "valabl"    , "valnbr"    , "valnode"  ,
			 "valu"      , "value"     ,"varlist"    

                         };

	/* ###------------------------------------------------------### */
	/*    initialisation :						*/
	/*    - allocate a buffer for read words			*/
	/*    - break the argument that identifies the structure (type)	*/
	/*      into words						*/
	/*    - search that words among recognized strings		*/
	/* ###------------------------------------------------------### */

  words [0] = buffer;
  get_size (siz);

  wrdcnt        = splitline (words, type);
  idx           = translate (words, wrdcnt, str, nmbrs, flags, indxs);

  typ [idx]     = POINTER_DFN | s_DFN | idx;
  pnt [idx].dat = head_pnt;

	/* ###------------------------------------------------------### */
	/*    process the command line until the _exit command		*/
	/* ###------------------------------------------------------### */

  while ((idx & TYPE_DFN) != _exit_DFN)
    {
	/* ###------------------------------------------------------### */
	/*    if the first word of the line has not been recognized,	*/
	/* print an error message. Otherwise, proccess the command line	*/
	/* (generally it is a request for displaying a specific field).	*/
	/*								*/
	/* At this point :						*/
	/*    - pnt [] contains all available pointers			*/
	/*    - typ [] contains the type of availabale pointers		*/
	/* ###------------------------------------------------------### */

    if (idx == _error_DFN)
      fprintf (stderr, "command not understood\n");
      /* gss_error (...); */
    else
      {

	/* ###------------------------------------------------------### */
	/*    define the type of the structure that may be pushed on	*/
	/* the stack :							*/
	/*    - if the pointer is a VOID pointer the exact type must be	*/
	/*      defined on the command line (last word of the command)	*/
	/* ###------------------------------------------------------### */

      pshtype = POINTER_DFN | s_DFN | void_DFN;
      if ((typ [idx] & TYPE_DFN) == void_DFN)
        {
        if ((wrdcnt >= 2) && (indxs [wrdcnt - 1] != _error_DFN))
          pshtype = POINTER_DFN | s_DFN | indxs [wrdcnt - 1];
        else
          fprintf (stderr, "cannot display void pointer\n");
	  /* gss_error (...); */
        }
      else
       pshtype = typ [idx];

	/* ###------------------------------------------------------### */
	/*    depending on the kind of the first word of the command	*/
	/* activate actions :						*/
	/*    - COMMAND (_help, _top, _up, ...)				*/
	/*    - POINTER							*/
	/*        - for NEXT go forward until the Nth element of the	*/
	/*          list. Then, push it on the stack and read fields	*/
	/*        - for others push and read fields			*/
	/*    - ARRAY							*/
	/*        - push the Nth element of the array and read its	*/
	/*          fields						*/
	/*    - ARRAY OF POINTER					*/
	/*        - push the object which address is the Nth element of	*/
	/*          the array and read its fields			*/
	/* ###------------------------------------------------------### */

      switch (typ [idx] & KIND_DFN)
        {

	/* ###------------------------------------------------------### */
	/*    COMMANDS ...						*/
	/* ###------------------------------------------------------### */

        case COMMAND_DFN :

          switch (typ [idx] & TYPE_DFN)
            {
	/* ###------------------------------------------------------### */
	/*    _top COMMAND : reset the stack pointer, call read_field	*/
	/* to read the structure on the top of stack			*/
	/* ###------------------------------------------------------### */

            case _top_DFN :
              stkpnt  = 0;
              readflg = 1;
              break;

	/* ###------------------------------------------------------### */
	/*    _stop COMMAND : set the stop mark for the structure on	*/
	/* the top of stack						*/
	/* ###------------------------------------------------------### */

            case _stop_DFN :
              stk [stkpnt].mark = 1;
	      fprintf (stdout, "gss_debug : stop mark set on the current structure\n");
	      /* gss_message (...); */
              break;

	/* ###------------------------------------------------------### */
	/*    _back COMMAND : pop structures from the stack until a	*/
	/* marked structure is found. Call read_field to read the	*/
	/* structure on the top of stack				*/
	/* ###------------------------------------------------------### */

            case _back_DFN :
              while ((stkpnt != 0) && (stk[--stkpnt].mark != 1));
              readflg = 1;
              break;

	/* ###------------------------------------------------------### */
	/*    _jump COMMAND : push on the stack the structure saved in	*/
	/* the jump table. Call read_field to read the structure on the	*/
	/* top of stack							*/
	/* ###------------------------------------------------------### */

            case _jump_DFN :
              if ((wrdcnt == 2) && (flags [1] == 1) && (nmbrs [1] < 10))
                {
                push (stk, &stkpnt, jtab[nmbrs[1]].data, jtab[nmbrs[1]].type);
                readflg = 1;
                }
              else
                fprintf (stderr, "command not understood\n");
		/* gss_error (...); */
              break;

	/* ###------------------------------------------------------### */
	/*    _up COMMAND : pop structure from the stack. Call		*/
	/* read_field to read the structure on the top of stack		*/
	/* ###------------------------------------------------------### */

            case _up_DFN :
              if (wrdcnt == 1)
                {
                pop (&stkpnt, 1);
                readflg = 1;
                }
              else
                {
                if ((wrdcnt == 2) && (flags [1] == 1))
                  {
                  pop (&stkpnt, nmbrs [1] + 1);
                  readflg = 1;
                  }
                else
                  fprintf (stderr, "command not understood\n");
		  /* gss_error (...); */
                }
              break;

	/* ###------------------------------------------------------### */
	/*    _save COMMAND : save the structure on the top of stack in	*/
	/* the jump table						*/
	/* ###------------------------------------------------------### */

            case _save_DFN :
              if ((wrdcnt == 2) && (flags [1] == 1) && (nmbrs [1] < 10))
                jtab [nmbrs[1]] = stk [stkpnt];
              else
                fprintf (stderr, "command not understood\n");
		/* gss_error (...); */
              break;

	/* ###------------------------------------------------------### */
	/*    _display COMMAND : display a specified field as an	*/
	/* immediate							*/
	/* ###------------------------------------------------------### */

            case _display_DFN :
              if ((wrdcnt == 3) && (indxs [1] != _error_DFN))
                disp_immd (str, pnt [indxs [1]], indxs [2]);
              else
                fprintf (stderr, "command not understood\n");
		/* gss_error (...); */
              break;

	/* ###------------------------------------------------------### */
	/*    _help COMMAND : display an help on the commands		*/
	/* ###------------------------------------------------------### */

            case _help_DFN :
                fprintf (stdout, "DEBUG HELP :\n\n");
                fprintf (stdout, "	_help	: debug help\n");
                fprintf (stdout, "	_back	: one call back\n");
                fprintf (stdout, "	_top	: top of debug\n");
                fprintf (stdout, "	_jump	: last saved call\n");
                fprintf (stdout, "	_stop	: set stop mark\n");
                fprintf (stdout, "	_up	: debug help\n");
                fprintf (stdout, "	_display: debug help\n");
                fprintf (stdout, "	_save	: for jump usage\n");
                fprintf (stdout, "	_exit	: exit debugger\n");
              break;

            }
          break;

	/* ###------------------------------------------------------### */
	/*   POINTERS ...						*/
	/* ###------------------------------------------------------### */

        case POINTER_DFN :
          if (idx == next_DFN)
            {
            if ((wrdcnt == 1) || ((wrdcnt == 2) && (flags [1] == 0)))
              {
              push (stk, &stkpnt, pnt [idx].dat, pshtype);
              readflg = 1;
              }
            else
              {
              if (((wrdcnt == 3) || (wrdcnt == 2)) && (flags [1] == 1))
                {
                pnt [idx].dat = (void *) go_forward (pnt [idx].dat, nmbrs [1]);
                push (stk, &stkpnt, pnt [idx].dat, pshtype);
                readflg = 1;
                }
              else
                fprintf (stderr, "command not understood\n");
		/* gss_error (...); */
              }
            }
          else
            {
            push (stk, &stkpnt, pnt [idx].dat, pshtype);
            readflg = 1;
            }
          break;

	/* ###------------------------------------------------------### */
	/*   ARRAIES of structure					*/
	/* ###------------------------------------------------------### */

        case ARRAY_DFN :
          if ((wrdcnt > 1) && (flags [1] == 1))
            {
            size = siz [(typ [idx] & TYPE_DFN)];
            pntr = (void *)
                   (((unsigned int) pnt [idx].dat) + (size * nmbrs [1]));
            push (stk, &stkpnt, pntr, pshtype);
            readflg = 1;
            }
          else
            fprintf (stderr, "command not understood\n");
	    /* gss_error (...); */
          break;

	/* ###------------------------------------------------------### */
	/*   ARRAIES of pointers					*/
	/* ###------------------------------------------------------### */

        case ARYOFPNT_DFN :
          if ((wrdcnt > 1) && (flags [1] == 1))
            {
            size = sizeof (void *);
            pntr = * (void **)
                     (((unsigned int) pnt [idx].dat) + (size * nmbrs [1]));
            push (stk, &stkpnt, pntr, pshtype);
            readflg = 1;
            }
          else
            fprintf (stderr, "command not understood\n");
	    /* gss_error (...); */
          break;

        }

      if (readflg == 1)
        {
        readflg = 0;
        read_field (stk [stkpnt], pnt, typ, str);
        }

      }

    printf ("\n\nCOMMAND > ");

    gets (line);
    if (strcmp (line ,"."))
      {
      wrdcnt = splitline (words, line);
      idx    = translate (words, wrdcnt, str, nmbrs, flags, indxs);
      }
    }

  }

/* ###--------------------------------------------------------------### */
/* function	: read_field						*/
/* description	: read the content of a data structure			*/
/* called func.	: none							*/
/* ###--------------------------------------------------------------### */

static void read_field (curpnt, pnt, typ, str)

struct stack   curpnt;			/* current struct. to be read	*/
union value   *pnt   ;			/* child structures or data	*/
long          *typ   ;			/* child struct. or data type	*/
char         **str   ;			/* recognized strings		*/

  {
  extern char         *exprToCharInt ();
  extern struct chain *bddToAbl ();

  int                    i;
  struct chain	        *ptr_abl;
  static char           *buffer      = NULL;
  static unsigned int    buffer_size = 0;
  static struct circuit *circuit_pnt = NULL;

  struct chain   *chain_pnt;
  struct ptype   *ptype_pnt;

  struct lofig   *lofig_pnt;
  struct locon   *locon_pnt;
  struct losig   *losig_pnt;
  struct loins   *loins_pnt;

  struct befig   *befig_pnt;
  struct bereg   *bereg_pnt;
  struct bemsg   *bemsg_pnt;
  struct berin   *berin_pnt;
  struct beout   *beout_pnt;
  struct bebus   *bebus_pnt;
  struct beaux   *beaux_pnt;
  struct beaux   *bedly_pnt;
  struct bebux   *bebux_pnt;
  struct bepor   *bepor_pnt;
  struct bequad  *bequad_pnt;
  struct begen   *begen_pnt;
  struct biabl   *biabl_pnt;
  struct binode  *binode_pnt;

  struct gssseq  *gssseq_pnt;
  struct gssevt  *gssevt_pnt;

  struct lkdfig  *lkdfig_pnt;
  struct lkdins  *lkdins_pnt;
  struct simsig  *simsig_pnt;
  struct bussig  *bussig_pnt;
  struct wrbsig  *wrbsig_pnt;
  struct wriaux  *wriaux_pnt;
  struct wrimsg  *wrimsg_pnt;
  struct wrireg  *wrireg_pnt;
  struct wribux  *wribux_pnt;
  struct wrssig  *wrssig_pnt;
  struct depsig  *depsig_pnt;
  struct redlst  *redlst_pnt;
  struct prjbvl  *prjbvl_pnt;
  struct prjrvl  *prjrvl_pnt;

  char          **string_pnt;
  pNode           bdd_pnt;
  char           *character_pnt;
  int            *integer_pnt;
  float          *float_pnt;

	/* ###------------------------------------------------------### */
	/*    initialization						*/
	/* ###------------------------------------------------------### */

  for (i=0 ; i<MAXCMD_DFN ; i++)
    {
    typ [i]     = _error_DFN;
    pnt [i].dat = NULL;
    }

  if (buffer == NULL)
    {
    buffer      = (char *) mbkalloc (1024);
    buffer_size = 1024;
    }

  typ [_exit_DFN]    = COMMAND_DFN | _exit_DFN   ;
  typ [_up_DFN]      = COMMAND_DFN | _up_DFN     ;
  typ [_top_DFN]     = COMMAND_DFN | _top_DFN    ;
  typ [_stop_DFN]    = COMMAND_DFN | _stop_DFN   ;
  typ [_back_DFN]    = COMMAND_DFN | _back_DFN   ;
  typ [_jump_DFN]    = COMMAND_DFN | _jump_DFN   ;
  typ [_save_DFN]    = COMMAND_DFN | _save_DFN   ;
  typ [_display_DFN] = COMMAND_DFN | _display_DFN;
  typ [_help_DFN]    = COMMAND_DFN | _help_DFN   ;

  switch (curpnt.type & TYPE_DFN)
    {
	/* ###------------------------------------------------------### */
	/*    depending on the type of the current structure :		*/
	/*								*/
	/*       - define the type of each field :			*/
	/*          - the field contains an immediate or a pointer	*/
	/*          - what format must be used to print it		*/
	/*          - kind of data it contains				*/
	/*								*/
	/*       - register the data contained in the field		*/
	/* ###------------------------------------------------------### */

	/* ###------------------------------------------------------### */
	/*    lofig							*/
	/* ###------------------------------------------------------### */

    case lofig_DFN :

      lofig_pnt = (struct lofig *) curpnt.data;

      typ [next_DFN]             = POINTER_DFN   | s_DFN | lofig_DFN;
      typ [modelchain_DFN]       = POINTER_DFN   | s_DFN | chain_DFN;
      typ [locon_DFN]            = POINTER_DFN   | s_DFN | locon_DFN;
      typ [losig_DFN]            = POINTER_DFN   | s_DFN | losig_DFN;
      typ [loins_DFN]            = POINTER_DFN   | s_DFN | loins_DFN;
      typ [name_DFN]             = IMMEDIATE_DFN | s_DFN | string_DFN;
      typ [mode_DFN]             = IMMEDIATE_DFN | c_DFN | character_DFN;
      typ [user_DFN]             = POINTER_DFN   | s_DFN | ptype_DFN;
      typ [next_DFN]             = POINTER_DFN   | s_DFN | lofig_DFN;

      pnt [modelchain_DFN].dat = lofig_pnt->MODELCHAIN;
      pnt [locon_DFN].dat      = lofig_pnt->LOCON;
      pnt [losig_DFN].dat      = lofig_pnt->LOSIG;
      pnt [loins_DFN].dat      = lofig_pnt->LOINS;
      pnt [name_DFN].dat       = lofig_pnt->NAME;
      pnt [mode_DFN].imd       = lofig_pnt->MODE;
      pnt [user_DFN].dat       = lofig_pnt->USER;

      break;

	/* ###------------------------------------------------------### */
	/*    locon							*/
	/* ###------------------------------------------------------### */

    case locon_DFN :

      locon_pnt = (struct locon *) curpnt.data;

      typ [next_DFN]          = POINTER_DFN   | s_DFN | locon_DFN;
      typ [name_DFN]          = IMMEDIATE_DFN | s_DFN | string_DFN;
      typ [sig_DFN]           = POINTER_DFN   | s_DFN | losig_DFN;
      typ [root_DFN]          = POINTER_DFN   | s_DFN | void_DFN;
      typ [direction_DFN]     = IMMEDIATE_DFN | c_DFN | character_DFN;
      typ [type_DFN]          = IMMEDIATE_DFN | c_DFN | character_DFN;
      typ [user_DFN]          = POINTER_DFN   | s_DFN | ptype_DFN;

      pnt [next_DFN].dat      = locon_pnt->NEXT;
      pnt [name_DFN].dat      = locon_pnt->NAME;
      pnt [sig_DFN].dat       = locon_pnt->SIG;
      pnt [root_DFN].dat      = locon_pnt->ROOT;
      pnt [direction_DFN].imd = locon_pnt->DIRECTION;
      pnt [type_DFN].imd      = locon_pnt->TYPE;
      pnt [user_DFN].dat      = locon_pnt->USER;

      break;

	/* ###------------------------------------------------------### */
	/*    losig							*/
	/* ###------------------------------------------------------### */

    case losig_DFN :

      losig_pnt = (struct losig *) curpnt.data;

      typ [next_DFN]             = POINTER_DFN   | s_DFN | losig_DFN;
      typ [namechain_DFN]        = POINTER_DFN   | s_DFN | chain_DFN;
      typ [capa_DFN]             = IMMEDIATE_DFN | f_DFN | float_DFN;
      typ [index_DFN]            = IMMEDIATE_DFN | l_DFN | long_DFN;
      typ [type_DFN]             = IMMEDIATE_DFN | c_DFN | character_DFN;
      typ [user_DFN]             = POINTER_DFN   | s_DFN | ptype_DFN;

      pnt [next_DFN].dat      = losig_pnt->NEXT;
      pnt [namechain_DFN].dat = losig_pnt->NAMECHAIN;
      /* LUDO
      pnt [capa_DFN].flt      = losig_pnt->CAPA;
      */
      pnt [index_DFN].imd     = losig_pnt->INDEX;
      pnt [type_DFN].imd      = losig_pnt->TYPE;
      pnt [user_DFN].dat      = losig_pnt->USER;

      break;

	/* ###------------------------------------------------------### */
	/*    loins							*/
	/* ###------------------------------------------------------### */

    case loins_DFN :

      loins_pnt = (struct loins *) curpnt.data;

      typ [next_DFN]         = POINTER_DFN   | s_DFN | loins_DFN;
      typ [locon_DFN]        = POINTER_DFN   | s_DFN | locon_DFN;
      typ [insname_DFN]      = IMMEDIATE_DFN | s_DFN | string_DFN;
      typ [figname_DFN]      = IMMEDIATE_DFN | s_DFN | string_DFN;
      typ [user_DFN]         = POINTER_DFN   | s_DFN | ptype_DFN;

      pnt [next_DFN].dat     = loins_pnt->NEXT;
      pnt [locon_DFN].dat    = loins_pnt->LOCON;
      pnt [insname_DFN].dat  = loins_pnt->INSNAME;
      pnt [figname_DFN].dat  = loins_pnt->FIGNAME;
      pnt [user_DFN].dat     = loins_pnt->USER;

      break;

	/* ###------------------------------------------------------### */
	/*    befig							*/
	/* ###------------------------------------------------------### */

    case befig_DFN :

      befig_pnt = (struct befig *) curpnt.data;

      typ [next_DFN]   = POINTER_DFN   | s_DFN | befig_DFN;
      typ [name_DFN]   = IMMEDIATE_DFN | s_DFN | string_DFN;
      typ [bereg_DFN]  = POINTER_DFN   | s_DFN | bereg_DFN;
      typ [bemsg_DFN]  = POINTER_DFN   | s_DFN | bemsg_DFN;
      typ [berin_DFN]  = POINTER_DFN   | s_DFN | berin_DFN;
      typ [beout_DFN]  = POINTER_DFN   | s_DFN | beout_DFN;
      typ [bebus_DFN]  = POINTER_DFN   | s_DFN | bebus_DFN;
      typ [beaux_DFN]  = POINTER_DFN   | s_DFN | beaux_DFN;
      typ [bedly_DFN]  = POINTER_DFN   | s_DFN | beaux_DFN;
      typ [bebux_DFN]  = POINTER_DFN   | s_DFN | bebux_DFN;
      typ [bepor_DFN]  = POINTER_DFN   | s_DFN | bepor_DFN;
      typ [begen_DFN]  = POINTER_DFN   | s_DFN | begen_DFN;
      typ [user_DFN]   = POINTER_DFN   | s_DFN | ptype_DFN;
      typ [errflg_DFN] = IMMEDIATE_DFN | d_DFN | character_DFN;
      typ [type_DFN]   = IMMEDIATE_DFN | d_DFN | character_DFN;

      pnt [next_DFN].dat   = befig_pnt->NEXT;
      pnt [name_DFN].dat   = befig_pnt->NAME;
      pnt [bereg_DFN].dat  = befig_pnt->BEREG;
      pnt [bemsg_DFN].dat  = befig_pnt->BEMSG;
      pnt [berin_DFN].dat  = befig_pnt->BERIN;
      pnt [beout_DFN].dat  = befig_pnt->BEOUT;
      pnt [bebus_DFN].dat  = befig_pnt->BEBUS;
      pnt [beaux_DFN].dat  = befig_pnt->BEAUX;
      pnt [bedly_DFN].dat  = befig_pnt->BEDLY;
      pnt [bebux_DFN].dat  = befig_pnt->BEBUX;
      pnt [bepor_DFN].dat  = befig_pnt->BEPOR;
      pnt [begen_DFN].dat  = befig_pnt->BEGEN;
      pnt [user_DFN].dat   = befig_pnt->USER;
      pnt [errflg_DFN].imd = befig_pnt->ERRFLG;
      pnt [type_DFN].imd   = befig_pnt->TYPE;

      circuit_pnt = befig_pnt->CIRCUI;

      break;

	/* ###------------------------------------------------------### */
	/*    bereg							*/
	/* ###------------------------------------------------------### */

    case bereg_DFN :

      bereg_pnt = (struct bereg *) curpnt.data;

      typ [next_DFN]   = POINTER_DFN   | s_DFN | bereg_DFN;
      typ [name_DFN]   = IMMEDIATE_DFN | s_DFN | string_DFN;
      typ [biabl_DFN]  = POINTER_DFN   | s_DFN | biabl_DFN;
      typ [binode_DFN] = POINTER_DFN   | s_DFN | binode_DFN;

      pnt [next_DFN].dat   = bereg_pnt->NEXT;
      pnt [name_DFN].dat   = bereg_pnt->NAME;
      pnt [biabl_DFN].dat  = bereg_pnt->BIABL;
      pnt [binode_DFN].dat = bereg_pnt->BINODE;

      break;

	/* ###------------------------------------------------------### */
	/*    bemsg							*/
	/* ###------------------------------------------------------### */

    case bemsg_DFN :

      bemsg_pnt = (struct bemsg *) curpnt.data;

      typ [next_DFN]    = POINTER_DFN   | s_DFN   | bemsg_DFN;
      typ [level_DFN]   = IMMEDIATE_DFN | c_DFN   | character_DFN;
      typ [label_DFN]   = IMMEDIATE_DFN | s_DFN   | string_DFN;
      typ [message_DFN] = IMMEDIATE_DFN | s_DFN   | string_DFN;
      typ [abl_DFN]     = IMMEDIATE_DFN | ABL_DFN | abl_DFN;
      typ [node_DFN]    = POINTER_DFN   | s_DFN   | void_DFN;

      pnt [next_DFN].dat    = bemsg_pnt->NEXT;
      pnt [level_DFN].imd   = bemsg_pnt->LEVEL;
      pnt [label_DFN].dat   = bemsg_pnt->LABEL;
      pnt [message_DFN].dat = bemsg_pnt->MESSAGE;
      pnt [abl_DFN].dat     = bemsg_pnt->ABL;
      pnt [node_DFN].dat    = bemsg_pnt->NODE;

      break;

	/* ###------------------------------------------------------### */
	/*    beout							*/
	/* ###------------------------------------------------------### */

    case beout_DFN :

      beout_pnt = (struct beout *) curpnt.data;

      typ [next_DFN]    = POINTER_DFN   | s_DFN   | beout_DFN;
      typ [name_DFN]    = IMMEDIATE_DFN | s_DFN   | string_DFN;
      typ [abl_DFN]     = IMMEDIATE_DFN | ABL_DFN | abl_DFN;
      typ [node_DFN]    = POINTER_DFN   | s_DFN   | void_DFN;

      pnt [next_DFN].dat    = beout_pnt->NEXT;
      pnt [name_DFN].dat    = beout_pnt->NAME;
      pnt [abl_DFN].dat     = beout_pnt->ABL;
      pnt [node_DFN].dat    = beout_pnt->NODE;

      break;

	/* ###------------------------------------------------------### */
	/*    bebus							*/
	/* ###------------------------------------------------------### */

    case bebus_DFN :

      bebus_pnt = (struct bebus *) curpnt.data;

      typ [next_DFN]    = POINTER_DFN   | s_DFN | bebus_DFN;
      typ [name_DFN]    = IMMEDIATE_DFN | s_DFN | string_DFN;
      typ [biabl_DFN]   = POINTER_DFN   | s_DFN | biabl_DFN;
      typ [binode_DFN]  = POINTER_DFN   | s_DFN | binode_DFN;
      typ [type_DFN]    = IMMEDIATE_DFN | c_DFN | character_DFN;

      pnt [next_DFN].dat    = bebus_pnt->NEXT;
      pnt [name_DFN].dat    = bebus_pnt->NAME;
      pnt [biabl_DFN].dat   = bebus_pnt->BIABL;
      pnt [binode_DFN].dat  = bebus_pnt->BINODE;
      pnt [type_DFN].imd    = bebus_pnt->TYPE;

      break;

	/* ###------------------------------------------------------### */
	/*    beaux							*/
	/* ###------------------------------------------------------### */

    case beaux_DFN :

      beaux_pnt = (struct beaux *) curpnt.data;

      typ [next_DFN]    = POINTER_DFN   | s_DFN   | beaux_DFN;
      typ [name_DFN]    = IMMEDIATE_DFN | s_DFN   | string_DFN;
      typ [abl_DFN]     = IMMEDIATE_DFN | ABL_DFN | abl_DFN;
      typ [node_DFN]    = POINTER_DFN   | s_DFN   | void_DFN;

      pnt [next_DFN].dat    = beaux_pnt->NEXT;
      pnt [name_DFN].dat    = beaux_pnt->NAME;
      pnt [abl_DFN].dat     = beaux_pnt->ABL;
      pnt [node_DFN].dat    = beaux_pnt->NODE;

      break;

	/* ###------------------------------------------------------### */
	/*    bebux							*/
	/* ###------------------------------------------------------### */

    case bebux_DFN :

      bebux_pnt = (struct bebux *) curpnt.data;

      typ [next_DFN]    = POINTER_DFN   | s_DFN | bebux_DFN;
      typ [name_DFN]    = IMMEDIATE_DFN | s_DFN | string_DFN;
      typ [biabl_DFN]   = POINTER_DFN   | s_DFN | biabl_DFN;
      typ [binode_DFN]  = POINTER_DFN   | s_DFN | binode_DFN;
      typ [type_DFN]    = IMMEDIATE_DFN | c_DFN | character_DFN;

      pnt [next_DFN].dat    = bebux_pnt->NEXT;
      pnt [name_DFN].dat    = bebux_pnt->NAME;
      pnt [biabl_DFN].dat   = bebux_pnt->BIABL;
      pnt [binode_DFN].dat  = bebux_pnt->BINODE;
      pnt [type_DFN].imd    = bebux_pnt->TYPE;

      break;

	/* ###------------------------------------------------------### */
	/*    bepor							*/
	/* ###------------------------------------------------------### */

    case bepor_DFN :

      bepor_pnt = (struct bepor *) curpnt.data;

      typ [next_DFN]      = POINTER_DFN   | s_DFN | bepor_DFN;
      typ [name_DFN]      = IMMEDIATE_DFN | s_DFN | string_DFN;
      typ [direction_DFN] = IMMEDIATE_DFN | c_DFN | character_DFN;
      typ [type_DFN]      = IMMEDIATE_DFN | c_DFN | character_DFN;

      pnt [next_DFN].dat      = bepor_pnt->NEXT;
      pnt [name_DFN].dat      = bepor_pnt->NAME;
      pnt [direction_DFN].imd = bepor_pnt->DIRECTION;
      pnt [type_DFN].imd      = bepor_pnt->TYPE;

      break;

	/* ###------------------------------------------------------### */
	/*    biabl							*/
	/* ###------------------------------------------------------### */

    case biabl_DFN :

      biabl_pnt = (struct biabl *) curpnt.data;

      typ [next_DFN]      = POINTER_DFN   | s_DFN   | biabl_DFN;
      typ [label_DFN]     = IMMEDIATE_DFN | s_DFN   | string_DFN;
      typ [cndabl_DFN]    = IMMEDIATE_DFN | ABL_DFN | abl_DFN;
      typ [valabl_DFN]    = IMMEDIATE_DFN | ABL_DFN | abl_DFN;
      typ [user_DFN]      = POINTER_DFN   | s_DFN   | ptype_DFN;

      pnt [next_DFN].dat   = biabl_pnt->NEXT;
      pnt [label_DFN].dat  = biabl_pnt->LABEL;
      pnt [cndabl_DFN].dat = biabl_pnt->CNDABL;
      pnt [valabl_DFN].dat = biabl_pnt->VALABL;
      pnt [user_DFN].dat   = biabl_pnt->USER;

      break;

	/* ###------------------------------------------------------### */
	/*    binode							*/
	/* ###------------------------------------------------------### */

    case binode_DFN :

      binode_pnt = (struct binode *) curpnt.data;

      typ [next_DFN]      = POINTER_DFN   | s_DFN   | binode_DFN;
      typ [cndnode_DFN]   = POINTER_DFN   | s_DFN   | void_DFN;
      typ [valnode_DFN]   = POINTER_DFN   | s_DFN   | void_DFN;

      pnt [next_DFN].dat      = binode_pnt->NEXT;
      pnt [cndnode_DFN].dat   = binode_pnt->CNDNODE;
      pnt [valnode_DFN].dat   = binode_pnt->VALNODE;

      break;

	/* ###------------------------------------------------------### */
	/*    berin							*/
	/* ###------------------------------------------------------### */

    case berin_DFN :

      berin_pnt = (struct berin *) curpnt.data;

      typ [next_DFN]    = POINTER_DFN   | s_DFN | berin_DFN;
      typ [name_DFN]    = IMMEDIATE_DFN | s_DFN | string_DFN;
      typ [out_ref_DFN] = POINTER_DFN   | s_DFN | chain_DFN;
      typ [bus_ref_DFN] = POINTER_DFN   | s_DFN | chain_DFN;
      typ [aux_ref_DFN] = POINTER_DFN   | s_DFN | chain_DFN;
      typ [reg_ref_DFN] = POINTER_DFN   | s_DFN | chain_DFN;
      typ [bux_ref_DFN] = POINTER_DFN   | s_DFN | chain_DFN;
      typ [msg_ref_DFN] = POINTER_DFN   | s_DFN | chain_DFN;
      typ [dly_ref_DFN] = POINTER_DFN   | s_DFN | chain_DFN;

      pnt [next_DFN].dat    = berin_pnt->NEXT;
      pnt [name_DFN].dat    = berin_pnt->NAME;
      pnt [out_ref_DFN].dat = berin_pnt->OUT_REF;
      pnt [bus_ref_DFN].dat = berin_pnt->BUS_REF;
      pnt [aux_ref_DFN].dat = berin_pnt->AUX_REF;
      pnt [reg_ref_DFN].dat = berin_pnt->REG_REF;
      pnt [bux_ref_DFN].dat = berin_pnt->BUX_REF;
      pnt [msg_ref_DFN].dat = berin_pnt->MSG_REF;
      pnt [dly_ref_DFN].dat = berin_pnt->DLY_REF;

      break;

	/* ###------------------------------------------------------### */
	/*    begen							*/
	/* ###------------------------------------------------------### */

    case begen_DFN :

      begen_pnt = (struct begen *) curpnt.data;

      typ [next_DFN]    = POINTER_DFN   | s_DFN | begen_DFN;
      typ [name_DFN]    = IMMEDIATE_DFN | s_DFN | string_DFN;
      typ [type_DFN]    = IMMEDIATE_DFN | s_DFN | string_DFN;
      typ [value_DFN]   = POINTER_DFN   | s_DFN | void_DFN;

      pnt [next_DFN].dat    = begen_pnt->NEXT;
      pnt [name_DFN].dat    = begen_pnt->NAME;
      pnt [type_DFN].dat    = begen_pnt->TYPE;
      pnt [value_DFN].dat   = begen_pnt->VALUE;

      break;

	/* ###------------------------------------------------------### */
	/*    gssseq							*/
	/* ###------------------------------------------------------### */
    case bequad_DFN :

      bequad_pnt = (struct bequad *) curpnt.data;

      typ [bdd_DFN]     = POINTER_DFN   | s_DFN   | chain_DFN;
      typ [gex_DFN]     = POINTER_DFN   | s_DFN   | void_DFN;
      typ [varlist_DFN] = POINTER_DFN   | s_DFN   | chain_DFN;
      typ [derivate_DFN]= POINTER_DFN   | s_DFN   | void_DFN;
      typ [user_DFN]    = POINTER_DFN   | s_DFN   | void_DFN;

      pnt [bdd_DFN].dat      = bequad_pnt->BDD;
      pnt [gex_DFN].dat      = bequad_pnt->GEX;
      pnt [varlist_DFN].dat  = bequad_pnt->VARLIST;
      pnt [derivate_DFN].dat = bequad_pnt->DERIVATE;
      pnt [user_DFN].dat     = bequad_pnt->USER;

      break;

    case gssseq_DFN :

      gssseq_pnt = (struct gssseq *) curpnt.data;

      typ [next_DFN]       = POINTER_DFN   | s_DFN | gssseq_DFN;
      typ [name_DFN]       = IMMEDIATE_DFN | s_DFN | string_DFN;
      typ [gssevt_DFN]     = POINTER_DFN   | s_DFN | gssevt_DFN;
      typ [errflg_DFN]     = IMMEDIATE_DFN | d_DFN | character_DFN;

      pnt [next_DFN].dat    = gssseq_pnt->NEXT;
      pnt [name_DFN].dat    = gssseq_pnt->NAME;
      pnt [gssevt_DFN].dat  = gssseq_pnt->EVENT;
      pnt [errflg_DFN].imd  = gssseq_pnt->ERRFLG;

      break;

	/* ###------------------------------------------------------### */
	/*    gssevt							*/
	/* ###------------------------------------------------------### */

    case gssevt_DFN :

      gssevt_pnt = (struct gssevt *) curpnt.data;

      typ [next_DFN]      = POINTER_DFN   | s_DFN | gssevt_DFN;
      typ [name_DFN]      = IMMEDIATE_DFN | s_DFN | string_DFN;
      typ [value_DFN]     = POINTER_DFN   | s_DFN | void_DFN;
      typ [type_DFN]      = IMMEDIATE_DFN | t_DFN | character_DFN;
      typ [signal_DFN]    = POINTER_DFN   | s_DFN | void_DFN;

      pnt [next_DFN].dat    = gssevt_pnt->NEXT;
      pnt [name_DFN].dat    = gssevt_pnt->NAME;
      pnt [value_DFN].dat   = gssevt_pnt->VALUE;
      pnt [type_DFN].imd    = gssevt_pnt->TYPE;
      pnt [signal_DFN].dat  = gssevt_pnt->SIGNAL;

      break;

	/* ###------------------------------------------------------### */
	/*    lkdfig							*/
	/* ###------------------------------------------------------### */

    case lkdfig_DFN :

      lkdfig_pnt = (struct lkdfig *) curpnt.data;

      typ [next_DFN]      = POINTER_DFN   | s_DFN | lkdfig_DFN;
      typ [name_DFN]      = IMMEDIATE_DFN | s_DFN | string_DFN;
      typ [lkdins_DFN]    = POINTER_DFN   | s_DFN | lkdins_DFN;
      typ [simsig_DFN]    = POINTER_DFN   | s_DFN | simsig_DFN;
      typ [bussig_DFN]    = POINTER_DFN   | s_DFN | bussig_DFN;

      pnt [next_DFN].dat   = lkdfig_pnt->NEXT;
      pnt [name_DFN].dat   = lkdfig_pnt->NAME;
      pnt [lkdins_DFN].dat = lkdfig_pnt->LKDINS;
      pnt [simsig_DFN].dat = lkdfig_pnt->SIMSIG;
      pnt [bussig_DFN].dat = lkdfig_pnt->BUSSIG;

      break;

	/* ###------------------------------------------------------### */
	/*    lkdins							*/
	/* ###------------------------------------------------------### */

    case lkdins_DFN :

      lkdins_pnt = (struct lkdins *) curpnt.data;

      typ [next_DFN]      = POINTER_DFN   | s_DFN | lkdins_DFN;
      typ [insname_DFN]   = IMMEDIATE_DFN | s_DFN | string_DFN;
      typ [befig_DFN]     = POINTER_DFN   | s_DFN | befig_DFN;
      typ [wrssig_DFN]    = POINTER_DFN   | s_DFN | wrssig_DFN;
      typ [wrbsig_DFN]    = POINTER_DFN   | s_DFN | wrbsig_DFN;
      typ [wriaux_DFN]    = POINTER_DFN   | s_DFN | wriaux_DFN;
      typ [wrimsg_DFN]    = POINTER_DFN   | s_DFN | wrimsg_DFN;
      typ [wribux_DFN]    = POINTER_DFN   | s_DFN | wribux_DFN;
      typ [wrireg_DFN]    = POINTER_DFN   | s_DFN | wrireg_DFN;
      typ [redlst_DFN]    = POINTER_DFN   | s_DFN | redlst_DFN;

      pnt [next_DFN].dat    = lkdins_pnt->NEXT;
      pnt [insname_DFN].dat = lkdins_pnt->INSNAME;
      pnt [befig_DFN].dat   = lkdins_pnt->BEFIG;
      pnt [wrssig_DFN].dat  = lkdins_pnt->WRSSIG;
      pnt [wrbsig_DFN].dat  = lkdins_pnt->WRBSIG;
      pnt [wriaux_DFN].dat  = lkdins_pnt->WRIAUX;
      pnt [wrimsg_DFN].dat  = lkdins_pnt->WRIMSG;
      pnt [wribux_DFN].dat  = lkdins_pnt->WRIBUX;
      pnt [wrireg_DFN].dat  = lkdins_pnt->WRIREG;
      pnt [redlst_DFN].dat  = lkdins_pnt->REDLST;

      circuit_pnt = lkdins_pnt->CIRCUI;

      break;

	/* ###------------------------------------------------------### */
	/*    simsig							*/
	/* ###------------------------------------------------------### */

    case simsig_DFN :

      simsig_pnt = (struct simsig *) curpnt.data;

      typ [next_DFN]      = POINTER_DFN   | s_DFN | simsig_DFN;
      typ [depsig_DFN]    = POINTER_DFN   | s_DFN | depsig_DFN;
      typ [curval_DFN]    = POINTER_DFN   | s_DFN | void_DFN;
      typ [prjval_DFN]    = POINTER_DFN   | s_DFN | void_DFN;

      pnt [next_DFN].dat      = simsig_pnt->NEXT;
      pnt [depsig_DFN].dat    = simsig_pnt->DEPSIG;
      pnt [curval_DFN].dat    = simsig_pnt->CURVAL;
      pnt [prjval_DFN].dat    = simsig_pnt->PRJVAL;

      break;

	/* ###------------------------------------------------------### */
	/*    bussig							*/
	/* ###------------------------------------------------------### */

    case bussig_DFN :

      bussig_pnt = (struct bussig *) curpnt.data;

      typ [next_DFN]      = POINTER_DFN   | s_DFN | bussig_DFN;
      typ [depsig_DFN]    = POINTER_DFN   | s_DFN | depsig_DFN;
      typ [curval_DFN]    = POINTER_DFN   | s_DFN | void_DFN;
      typ [prjval_DFN]    = POINTER_DFN   | s_DFN | void_DFN;
      typ [prjbvl_DFN]    = POINTER_DFN   | s_DFN | prjbvl_DFN;
      typ [type_DFN]      = IMMEDIATE_DFN | c_DFN | character_DFN;

      pnt [next_DFN].dat      = bussig_pnt->NEXT;
      pnt [depsig_DFN].dat    = bussig_pnt->DEPSIG;
      pnt [curval_DFN].dat    = bussig_pnt->CURVAL;
      pnt [prjval_DFN].dat    = bussig_pnt->PRJVAL;
      pnt [prjbvl_DFN].dat    = bussig_pnt->PRJBVL;
      pnt [type_DFN].imd      = bussig_pnt->TYPE;

      break;

	/* ###------------------------------------------------------### */
	/*    wrssig							*/
	/* ###------------------------------------------------------### */

    case wrssig_DFN :

      wrssig_pnt = (struct wrssig *) curpnt.data;

      typ [next_DFN]      = POINTER_DFN   | s_DFN | wrssig_DFN;
      typ [lkdins_DFN]    = POINTER_DFN   | s_DFN | lkdins_DFN;
      typ [beout_DFN]     = POINTER_DFN   | s_DFN | beout_DFN;
      typ [simsig_DFN]    = POINTER_DFN   | s_DFN | simsig_DFN;
      typ [eval_DFN]      = IMMEDIATE_DFN | c_DFN | character_DFN;

      pnt [next_DFN].dat   = wrssig_pnt->NEXT;
      pnt [lkdins_DFN].dat = wrssig_pnt->LKDINS;
      pnt [beout_DFN].dat  = wrssig_pnt->BEOUT;
      pnt [simsig_DFN].dat = wrssig_pnt->SIMSIG;
      pnt [eval_DFN].imd   = wrssig_pnt->EVAL;

      break;

	/* ###------------------------------------------------------### */
	/*    wrbsig							*/
	/* ###------------------------------------------------------### */

    case wrbsig_DFN :

      wrbsig_pnt = (struct wrbsig *) curpnt.data;

      typ [next_DFN]      = POINTER_DFN   | s_DFN | wrbsig_DFN;
      typ [lkdins_DFN]    = POINTER_DFN   | s_DFN | lkdins_DFN;
      typ [bebus_DFN]     = POINTER_DFN   | s_DFN | bebus_DFN;
      typ [bussig_DFN]    = POINTER_DFN   | s_DFN | bussig_DFN;
      typ [prjbvl_DFN]    = POINTER_DFN   | s_DFN | prjbvl_DFN;
      typ [eval_DFN]      = IMMEDIATE_DFN | c_DFN | character_DFN;

      pnt [next_DFN].dat   = wrbsig_pnt->NEXT;
      pnt [lkdins_DFN].dat = wrbsig_pnt->LKDINS;
      pnt [bebus_DFN].dat  = wrbsig_pnt->BEBUS;
      pnt [bussig_DFN].dat = wrbsig_pnt->BUSSIG;
      pnt [prjbvl_DFN].dat = wrbsig_pnt->PRJBVL;
      pnt [eval_DFN].imd   = wrbsig_pnt->EVAL;

      break;

	/* ###------------------------------------------------------### */
	/*    wriaux							*/
	/* ###------------------------------------------------------### */

    case wriaux_DFN :

      wriaux_pnt = (struct wriaux *) curpnt.data;

      typ [next_DFN]      = POINTER_DFN   | s_DFN | wriaux_DFN;
      typ [lkdins_DFN]    = POINTER_DFN   | s_DFN | lkdins_DFN;
      typ [depsig_DFN]    = POINTER_DFN   | s_DFN | depsig_DFN;
      typ [beaux_DFN]     = POINTER_DFN   | s_DFN | beaux_DFN;
      typ [curval_DFN]    = POINTER_DFN   | s_DFN | void_DFN;
      typ [prjval_DFN]    = POINTER_DFN   | s_DFN | void_DFN;
      typ [eval_DFN]      = IMMEDIATE_DFN | c_DFN | character_DFN;

      pnt [next_DFN].dat   = wriaux_pnt->NEXT;
      pnt [lkdins_DFN].dat = wriaux_pnt->LKDINS;
      pnt [depsig_DFN].dat = wriaux_pnt->DEPSIG;
      pnt [beaux_DFN].dat  = wriaux_pnt->BEAUX;
      pnt [curval_DFN].dat = wriaux_pnt->CURVAL;
      pnt [prjval_DFN].dat = wriaux_pnt->PRJVAL;
      pnt [eval_DFN].imd   = wriaux_pnt->EVAL;

      break;

	/* ###------------------------------------------------------### */
	/*    wrimsg							*/
	/* ###------------------------------------------------------### */

    case wrimsg_DFN :

      wrimsg_pnt = (struct wrimsg *) curpnt.data;

      typ [next_DFN]      = POINTER_DFN   | s_DFN | wriaux_DFN;
      typ [lkdins_DFN]    = POINTER_DFN   | s_DFN | lkdins_DFN;
      typ [bemsg_DFN]     = POINTER_DFN   | s_DFN | bemsg_DFN;
      typ [curval_DFN]    = POINTER_DFN   | s_DFN | void_DFN;
      typ [prjval_DFN]    = POINTER_DFN   | s_DFN | void_DFN;
      typ [eval_DFN]      = IMMEDIATE_DFN | c_DFN | character_DFN;

      pnt [next_DFN].dat   = wrimsg_pnt->NEXT;
      pnt [lkdins_DFN].dat = wrimsg_pnt->LKDINS;
      pnt [bemsg_DFN].dat  = wrimsg_pnt->BEMSG;
      pnt [curval_DFN].dat = wrimsg_pnt->CURVAL;
      pnt [prjval_DFN].dat = wrimsg_pnt->PRJVAL;
      pnt [eval_DFN].imd   = wrimsg_pnt->EVAL;

      break;

	/* ###------------------------------------------------------### */
	/*    wrireg							*/
	/* ###------------------------------------------------------### */

    case wrireg_DFN :

      wrireg_pnt = (struct wrireg *) curpnt.data;

      typ [next_DFN]      = POINTER_DFN   | s_DFN | wrireg_DFN;
      typ [lkdins_DFN]    = POINTER_DFN   | s_DFN | lkdins_DFN;
      typ [depsig_DFN]    = POINTER_DFN   | s_DFN | depsig_DFN;
      typ [bereg_DFN]     = POINTER_DFN   | s_DFN | bereg_DFN;
      typ [curval_DFN]    = POINTER_DFN   | s_DFN | void_DFN;
      typ [prjval_DFN]    = POINTER_DFN   | s_DFN | void_DFN;
      typ [prjbvl_DFN]    = POINTER_DFN   | s_DFN | prjbvl_DFN;
      typ [eval_DFN]      = IMMEDIATE_DFN | c_DFN | character_DFN;

      pnt [next_DFN].dat   = wrireg_pnt->NEXT;
      pnt [lkdins_DFN].dat = wrireg_pnt->LKDINS;
      pnt [depsig_DFN].dat = wriaux_pnt->DEPSIG;
      pnt [bereg_DFN].dat  = wrireg_pnt->BEREG;
      pnt [curval_DFN].dat = wrireg_pnt->CURVAL;
      pnt [prjval_DFN].dat = wrireg_pnt->PRJVAL;
      pnt [prjbvl_DFN].dat = wrireg_pnt->PRJBVL;
      pnt [eval_DFN].imd   = wrireg_pnt->EVAL;

      break;

	/* ###------------------------------------------------------### */
	/*    wribux							*/
	/* ###------------------------------------------------------### */

    case wribux_DFN :

      wribux_pnt = (struct wribux *) curpnt.data;

      typ [next_DFN]      = POINTER_DFN   | s_DFN | wribux_DFN;
      typ [lkdins_DFN]    = POINTER_DFN   | s_DFN | lkdins_DFN;
      typ [depsig_DFN]    = POINTER_DFN   | s_DFN | depsig_DFN;
      typ [bebux_DFN]     = POINTER_DFN   | s_DFN | bebux_DFN;
      typ [curval_DFN]    = POINTER_DFN   | s_DFN | void_DFN;
      typ [prjval_DFN]    = POINTER_DFN   | s_DFN | void_DFN;
      typ [prjbvl_DFN]    = POINTER_DFN   | s_DFN | prjbvl_DFN;
      typ [eval_DFN]      = IMMEDIATE_DFN | c_DFN | character_DFN;

      pnt [next_DFN].dat   = wribux_pnt->NEXT;
      pnt [lkdins_DFN].dat = wribux_pnt->LKDINS;
      pnt [depsig_DFN].dat = wribux_pnt->DEPSIG;
      pnt [bebux_DFN].dat  = wribux_pnt->BEBUX;
      pnt [curval_DFN].dat = wribux_pnt->CURVAL;
      pnt [prjval_DFN].dat = wribux_pnt->PRJVAL;
      pnt [prjbvl_DFN].dat = wribux_pnt->PRJBVL;
      pnt [eval_DFN].imd   = wribux_pnt->EVAL;

      break;

	/* ###------------------------------------------------------### */
	/*    depsig							*/
	/* ###------------------------------------------------------### */

    case depsig_DFN :

      depsig_pnt = (struct depsig *) curpnt.data;

      typ [next_DFN]      = POINTER_DFN   | s_DFN | depsig_DFN;
      typ [signal_DFN]    = POINTER_DFN   | s_DFN | void_DFN;
      typ [type_DFN]      = IMMEDIATE_DFN | t_DFN | character_DFN;

      pnt [next_DFN].dat   = depsig_pnt->NEXT;
      pnt [signal_DFN].dat = depsig_pnt->SIGNAL;
      pnt [type_DFN].imd   = depsig_pnt->TYPE;

      break;

	/* ###------------------------------------------------------### */
	/*    redlst							*/
	/* ###------------------------------------------------------### */

    case redlst_DFN :

      redlst_pnt = (struct redlst *) curpnt.data;

      typ [next_DFN]      = POINTER_DFN   | s_DFN | redlst_DFN;
      typ [valu_DFN]      = POINTER_DFN   | s_DFN | void_DFN;

      pnt [next_DFN].dat = redlst_pnt->NEXT;
      pnt [valu_DFN].dat = redlst_pnt->VALU;

      break;

	/* ###------------------------------------------------------### */
	/*    prjbvl							*/
	/* ###------------------------------------------------------### */

    case prjbvl_DFN :

      prjbvl_pnt = (struct prjbvl *) curpnt.data;

      typ [next_DFN]      = POINTER_DFN   | s_DFN | prjbvl_DFN;
      typ [cndval_DFN]    = POINTER_DFN   | s_DFN | void_DFN;
      typ [drvval_DFN]    = POINTER_DFN   | s_DFN | void_DFN;

      pnt [next_DFN].dat   = prjbvl_pnt->NEXT;
      pnt [cndval_DFN].dat = prjbvl_pnt->CNDVAL;
      pnt [drvval_DFN].dat = prjbvl_pnt->DRVVAL;

      break;

	/* ###------------------------------------------------------### */
	/*    ptype							*/
	/* ###------------------------------------------------------### */

    case ptype_DFN :

      ptype_pnt = (struct ptype *) curpnt.data;

      typ [next_DFN]       = POINTER_DFN   | s_DFN | ptype_DFN;
      typ [data_DFN]       = POINTER_DFN   | s_DFN | void_DFN;
      typ [type_DFN]       = IMMEDIATE_DFN | l_DFN | long_DFN;

      pnt [next_DFN].dat   = ptype_pnt->NEXT;
      pnt [data_DFN].dat   = ptype_pnt->DATA;
      pnt [type_DFN].imd   = ptype_pnt->TYPE;

      break;

	/* ###------------------------------------------------------### */
	/*    chain							*/
	/* ###------------------------------------------------------### */

    case chain_DFN :

      chain_pnt = (struct chain *) curpnt.data;

      typ [next_DFN]       = POINTER_DFN   | s_DFN | chain_DFN;
      typ [data_DFN]       = POINTER_DFN   | s_DFN | void_DFN;

      pnt [next_DFN].dat   = chain_pnt->NEXT;
      pnt [data_DFN].dat   = chain_pnt->DATA;

      break;

	/* ###------------------------------------------------------### */
	/*    string							*/
	/* ###------------------------------------------------------### */

    case string_DFN :

      string_pnt = (char **) curpnt.data;

      typ [string_DFN]     = IMMEDIATE_DFN | s_DFN | string_DFN;
      pnt [string_DFN].dat = *string_pnt;

      break;

	/* ###------------------------------------------------------### */
	/*    bdd							*/
	/* ###------------------------------------------------------### */

    case bdd_DFN :

      bdd_pnt = (pNode) curpnt.data;

      typ [bdd_DFN]     = IMMEDIATE_DFN | BDD_DFN | bdd_DFN;
      pnt [bdd_DFN].dat = bdd_pnt;

      break;

	/* ###------------------------------------------------------### */
	/*    character							*/
	/* ###------------------------------------------------------### */

    case character_DFN :

      character_pnt = (char *) curpnt.data;

      typ [character_DFN]     = IMMEDIATE_DFN | x_DFN | character_DFN;
      pnt [character_DFN].imd = *character_pnt;

      break;

	/* ###------------------------------------------------------### */
	/*    integer							*/
	/* ###------------------------------------------------------### */

    case integer_DFN :

      integer_pnt = (int *) curpnt.data;

      typ [integer_DFN]     = IMMEDIATE_DFN | d_DFN | integer_DFN;
      pnt [integer_DFN].imd = *integer_pnt;

      break;

	/* ###------------------------------------------------------### */
	/*    float							*/
	/* ###------------------------------------------------------### */

    case float_DFN :

      float_pnt = (float *) curpnt.data;

      typ [float_DFN]     = IMMEDIATE_DFN | f_DFN | float_DFN;
      pnt [float_DFN].flt = *float_pnt;

      break;
    }

	/* ###------------------------------------------------------### */
	/*    scan the list of recongnized strings to print those	*/
	/* values that are not empty (different form _error_DFN) :	*/
	/*								*/
	/*   - if it contains an immediate, print it following the	*/
	/*     required format						*/
	/*								*/
	/*   - if it contains a pointer, just print the pointer		*/
	/*     indication, ->, and "availbale"				*/
	/*								*/
	/*   - if it contains an array, just print the array		*/
	/*     indication, =>, and "availbale"				*/
	/* ###------------------------------------------------------### */

  printf ("--------- current structure : %s ---------\n\n",
          str [curpnt.type & TYPE_DFN]);

  for (i=0 ; i<MAXCMD_DFN ; i++)
    {
    if ((typ [i] & KIND_DFN) == IMMEDIATE_DFN)
      {
      printf ("   %-15s: ", str [i]);
      switch (typ [i] & FORMAT_DFN)
        {
        case s_DFN :
          if (pnt[i].dat != NULL)
            printf ("%s", pnt[i].dat);
          printf ("\n");
          break;

        case f_DFN :
          printf ("%f\n", pnt[i].flt);
          break;

        case u_DFN :
          printf ("%u\n", pnt[i].imd);
          break;

        case c_DFN :
          printf ("%c\n", pnt[i].imd);
          break;

        case d_DFN :
          printf ("%d\n", pnt[i].imd);
          break;

        case l_DFN :
          printf ("%ld\n", pnt[i].imd);
          break;

        case x_DFN :
          printf ("0x%x\n", pnt[i].imd);
          break;

        case t_DFN :
          if (pnt[i].imd == GSS_EMPTY)
            printf (" GSS_EMPTY\n");
          else
            {
            if ((pnt[i].imd & GSS_SIGDEF) != GSS_EMPTY)
              printf (" GSS_SIGDEF ");

            if ((pnt[i].imd & GSS_BUSDEF) != GSS_EMPTY)
              printf (" GSS_BUSDEF ");

            if ((pnt[i].imd & GSS_BUXDEF) != GSS_EMPTY)
              printf (" GSS_BUXDEF ");

            if ((pnt[i].imd & GSS_REGDEF) != GSS_EMPTY)
              printf (" GSS_REGDEF ");

            if ((pnt[i].imd & GSS_AUXDEF) != GSS_EMPTY)
              printf (" GSS_AUXDEF ");

            if ((pnt[i].imd & GSS_MSGDEF) != GSS_EMPTY)
              printf (" GSS_MSGDEF ");

            if ((pnt[i].imd & GSS_DRVDEF) != GSS_EMPTY)
              printf (" GSS_DRVDEF ");

            if ((pnt[i].imd & GSS_CNDDEF) != GSS_EMPTY)
              printf (" GSS_CNDDEF ");

            printf("\n");
            }
          break;

        case ABL_DFN :
          if (pnt[i].dat != NULL)
            {
            buffer [0] = '\0';
            buffer = exprToCharInt (pnt[i].dat, 1, buffer, &buffer_size);
            printf ("%s", buffer);
            }
          printf ("\n");
          break;

        case BDD_DFN :
          if ((pnt[i].dat != NULL) && (circuit_pnt != NULL))
            {
            buffer [0] = '\0';
            ptr_abl = bddToAbl (pnt[i].dat, circuit_pnt->pNameI);
            buffer  = exprToCharInt (ptr_abl, 1, buffer, &buffer_size);
            printf ("%s", buffer);
            }
          printf ("\n");
          break;
        }
      }

    if ((typ [i] & KIND_DFN) == POINTER_DFN)
      {
      printf ("-> %-15s: ", str [i]);
      if (pnt[i].dat != NULL)
        printf ("available");
      printf ("\n");
      }

    if ( ((typ [i] & KIND_DFN) == ARRAY_DFN   ) ||
         ((typ [i] & KIND_DFN) == ARYOFPNT_DFN) )
      {
      printf ("=> %-15s: ", str [i]);
      if (pnt[i].dat != NULL)
        printf ("available");
      printf ("\n");
      }
    }

  }

/* ###--------------------------------------------------------------### */
/* function	: splitline						*/
/* description	: read a line (the space must have been reserved by the	*/
/*		  caller - *words) from the standard input and split it	*/
/*		  into seperate words. Return the number of words read.	*/
/* called func.	: none							*/
/* ###--------------------------------------------------------------### */

static int splitline (words, line)

char **words;
char  *line ;
  {

  char *heap   = *words;
  int   new    =  1;
  int   wrdcnt =  0;
  int   i      =  0;
  int   j      =  0;

	/* ###------------------------------------------------------### */
	/*    copy the line read from input into the words until a	*/
	/* space has been found. Then, increment the word count and	*/
	/* begin filling the next word					*/
	/* ###------------------------------------------------------### */

  while ((heap[j] = line[i++]) != '\0')
    {
    if ((heap[j] != ' ') && (heap[j] != '\t'))
      {
      new = 0;
      j++;
      }
    else
      {
      if (new != 1)
        {
        heap  [j++]      = '\0';
        words [++wrdcnt] = heap + j;
        new = 1;
        }
      }
    }

  if (new != 1)
    wrdcnt++;

  return (wrdcnt);
  }

/* ###--------------------------------------------------------------### */
/* function	: push							*/
/* description	: push a structure on the stack				*/
/* called func.	: none							*/
/* ###--------------------------------------------------------------### */

static void push (stk, stkpnt, data, type)

struct stack *stk;
int          *stkpnt;
void         *data;
short         type;

  {
  int stkidx = *stkpnt;

  if (stkidx == STKSIZ_DFN)
    fprintf (stderr, "STACK OVERFLOW ---- STACK OVERFLOW !\n");
	  /* gss_error (...); */

  else
    {
    if (data != NULL)
      {
      stkidx++;
      stk [stkidx].data = data;
      stk [stkidx].type = type;
      stk [stkidx].mark = 0;
      *stkpnt           = stkidx;
      }
    else
      fprintf (stderr, "STACK OVERFLOW ---- STACK OVERFLOW !\n");
      /* gss_error (...); */
    }
  }

/* ###--------------------------------------------------------------### */
/* function	: pop							*/
/* description	: pop N structures from the stack			*/
/* called func.	: none							*/
/* ###--------------------------------------------------------------### */

static void pop (stkpnt, count)

int          *stkpnt;
int           count;

  {
  int stkidx = *stkpnt;

  if (stkidx <= count)
    stkidx = 0;
  else
    stkidx -= count;

  *stkpnt = stkidx;
  }

/* ###--------------------------------------------------------------### */
/* function	: go_forward						*/
/* description	: go forward through the NEXT field to find the Nth	*/
/*		  structure in the list and return it			*/
/* called func.	: none							*/
/* ###--------------------------------------------------------------### */

static struct chain *go_forward (pnt, count)

struct chain *pnt;
int           count;

  {
  while ((count > 0) && (pnt != NULL) && (pnt->NEXT != NULL))
    {
    pnt = pnt->NEXT;
    count--;
    }
  return (pnt);
  }

/* ###--------------------------------------------------------------### */
/* function	: translate						*/
/* description	: find the index of each word read from the command	*/
/*		  line. For each word check if it is a number. Return	*/
/*		  the index of the first word.				*/
/* called func.	: none							*/
/* ###--------------------------------------------------------------### */

static int translate (words, wrdcnt, strgs, nmbrs, flags, indxs)

char **words ;				/* list of words		*/
int    wrdcnt;				/* number of words		*/
char **strgs ;				/* list of recognized strings	*/
int   *nmbrs ;				/* words translated to numbers	*/
char  *flags ;				/* set if word is a number	*/
int   *indxs ;				/* words' index in strgs table	*/

  {
  int i , j;

	/* ###------------------------------------------------------### */
	/*    initialize flags and indxs				*/
	/* ###------------------------------------------------------### */

  for (i=0 ; i<10 ; i++)
    {
    flags [i] = 0;
    indxs [i] = _error_DFN;
    }

	/* ###------------------------------------------------------### */
	/*   for each word :						*/
	/*      - check if it is a number				*/
	/*      - try to locate the word among the recognized strings.	*/
	/*        If found save it's index				*/
	/* ###------------------------------------------------------### */

  for (i=0 ; i<wrdcnt ; i++)
    {
    flags [i] = sscanf (words [i], "%u", &nmbrs [i]);
    for (j=0; j<MAXCMD_DFN ; j++)
      {
      if (!strcmp (strgs [j], words [i]))
        {
        indxs [i] = j;
        break;
        }
      }
    }
  return (indxs [0]);
  }

static void get_size (siz)

unsigned int siz [];

  {
  int i;
 
  for (i=0 ; i<MAXCMD_DFN ; i++)
    siz [i] = 0;


  siz [character_DFN] = sizeof (char);
  siz [short_DFN]     = sizeof (short);
  siz [integer_DFN]   = sizeof (int);
  siz [long_DFN]      = sizeof (long);
  siz [string_DFN]    = sizeof (char *);

  siz [chain_DFN]     = sizeof (struct chain);
  siz [ptype_DFN]     = sizeof (struct ptype);

  siz [lofig_DFN]     = sizeof (struct lofig);
  siz [locon_DFN]     = sizeof (struct locon);
  siz [losig_DFN]     = sizeof (struct losig);
  siz [loins_DFN]     = sizeof (struct loins);

  siz [befig_DFN]     = sizeof (struct befig);
  siz [bereg_DFN]     = sizeof (struct bereg);
  siz [bemsg_DFN]     = sizeof (struct bemsg);
  siz [berin_DFN]     = sizeof (struct berin);
  siz [beout_DFN]     = sizeof (struct beout);
  siz [bebus_DFN]     = sizeof (struct bebus);
  siz [beaux_DFN]     = sizeof (struct beaux);
  siz [bedly_DFN]     = sizeof (struct beaux);
  siz [bebux_DFN]     = sizeof (struct bebux);
  siz [bepor_DFN]     = sizeof (struct bepor);
  siz [begen_DFN]     = sizeof (struct begen);
  siz [biabl_DFN]     = sizeof (struct biabl);
  siz [binode_DFN]    = sizeof (struct binode);
  siz [bequad_DFN]    = sizeof (struct bequad);

  siz [gssseq_DFN]     = sizeof (struct gssseq);
  siz [gssevt_DFN]     = sizeof (struct gssevt);

  siz [lkdfig_DFN]    = sizeof (struct lkdfig);
  siz [lkdins_DFN]    = sizeof (struct lkdins);
  siz [simsig_DFN]    = sizeof (struct simsig);
  siz [bussig_DFN]    = sizeof (struct bussig);
  siz [wrbsig_DFN]    = sizeof (struct wrbsig);
  siz [wriaux_DFN]    = sizeof (struct wriaux);
  siz [wrireg_DFN]    = sizeof (struct wrireg);
  siz [wribux_DFN]    = sizeof (struct wribux);
  siz [wrssig_DFN]    = sizeof (struct wrssig);
  siz [redlst_DFN]    = sizeof (struct redlst);
  siz [prjbvl_DFN]    = sizeof (struct prjbvl);

  }

/* ###--------------------------------------------------------------### */
/* function	: disp_immd						*/
/* description	: display a specific field as an immediate		*/
/* called func.	: none							*/
/* ###--------------------------------------------------------------### */

static void disp_immd (str, pnt , type)

char        **str;
union value pnt  ;
int         type ;

  {
  char *lcl_str;

  printf ("   %-15s: ", str [type]);
  switch (type)
    {
    case string_DFN :
      if (((char *) pnt.dat) != NULL)
        printf ("%s", (char *) pnt.dat);
      printf ("\n");
      break;

    case float_DFN :
      printf ("%f\n", pnt.flt);
      break;

    case character_DFN :
      printf ("%c\n", pnt.imd);
      break;

    case short_DFN     :
    case integer_DFN   :
    case long_DFN      :
      printf ("0x%x\n", pnt.imd);
      break;

    default :
      fprintf (stderr, "command not understood\n");
      /* gss_error (...); */
    }
  }
