
/* ###--------------------------------------------------------------### */
/* 									*/
/* file		: gss_history.c						*/
/* date		: Jan 28 1998						*/
/* version	: v100							*/
/* authors	: DUNOYER Julien					*/
/* content	: high-level gss function				*/
/*		  gss_gethistory, gss_addevthis, gss_freevthis		*/
/* ###--------------------------------------------------------------### */

#include MUT_H
#include GSS_H
#include "beh.h"

/* ###--------------------------------------------------------------### */
/* function	: gss_gethistory					*/
/* description	: get events history by signal index and instance	*/
/* called func.	: none							*/
/* ###--------------------------------------------------------------### */

struct evthis *gss_gethistory (instance, indexbdd)

struct lkdins *instance;
short          indexbdd;

  {
  struct evthis *history = NULL;

      /* ###--------------------------------------------------------### */
      /*    if the GSS REDLST Table is expanded, use a right shift      */
      /* ###--------------------------------------------------------### */

  if ( BEH_CIRCUI & BEH_CIRCUI_EXPANDED)
    {
    if (indexbdd > 1)
        history = (instance->REDLST + ((indexbdd - 2) >> 1))->HIST;
    else
        gss_error (0,"gss_gethistory");
    }
  else
    {
    if (indexbdd > 1)
        history = (instance->REDLST + (indexbdd - 2))->HIST;
    else
        gss_error (0,"gss_gethistory");
    }
  return (history); 
  }


/* ###--------------------------------------------------------------### */
/* function	: gss_getlasteventdate					*/
/* description	: get the last event date by signal index and instance	*/
/* called func.	: none							*/
/* ###--------------------------------------------------------------### */

unsigned long  gss_getlasteventdate (instance, indexbdd, date)

struct lkdins *instance;
short          indexbdd;
unsigned long *date;

  {
  struct evthis *history = NULL;

  history = gss_gethistory (instance, indexbdd);
  *date  = history->DATE;

  return (history->COUNT); 
  }


/* ###--------------------------------------------------------------### */
/* function	: gss_getlasteventdelta					*/
/* description	: get the last event delta cycle by index and instance	*/
/* called func.	: none							*/
/* ###--------------------------------------------------------------### */

unsigned long  gss_getlasteventdelta (instance, indexbdd, delta)

struct lkdins *instance;
short          indexbdd;
int           *delta;

  {
  struct evthis *history = NULL;

  history = gss_gethistory (instance, indexbdd);
  *delta  = history->DELTA;

  return (history->COUNT); 
  }


/* ###--------------------------------------------------------------### */
/* function	: gss_addevthis						*/
/* description	: Used to add an event to an history 			*/
/* called funct	: mbkalloc						*/
/* ###--------------------------------------------------------------### */

struct evthis *gss_addevthis (headevthis, date, delta)

struct evthis *headevthis;
unsigned long  date;
int            delta;

  {
  struct evthis *ptevthis;

  if ( headevthis == NULL )
    {
    ptevthis = (struct evthis *)mbkalloc (sizeof(struct evthis));

    ptevthis->DATE  = 0;

    ptevthis->DELTA = 0;

    ptevthis->COUNT = GSS_NOEVENTYET;

    ptevthis->NEXT  = NULL;

    return (ptevthis);
    }

  else if ( GSS_HISTORY_MODE & GSS_LASTEVENT_HISTORY )
    {
    headevthis->DATE  = date;

    headevthis->DELTA = delta;

    headevthis->COUNT = headevthis->COUNT + 1;

    headevthis->NEXT  = NULL;

    return (headevthis);
    }

  else
    {
    ptevthis = (struct evthis *)mbkalloc (sizeof(struct evthis));

    ptevthis->DATE    = headevthis->DATE;
    headevthis->DATE  = date;

    ptevthis->DELTA   = headevthis->DELTA;
    headevthis->DELTA = delta;

    ptevthis->COUNT   = headevthis->COUNT;
    headevthis->COUNT = ptevthis->COUNT + 1;

    ptevthis->NEXT    = headevthis->NEXT;
    headevthis->NEXT  = ptevthis;

    return (headevthis);
    }
  }

/* ###--------------------------------------------------------------### */
/* function	: gss_freevthis						*/
/* description	: free a list of events history structure		*/
/* called funct	: mbkfree						*/
/* ###--------------------------------------------------------------### */

void gss_freevthis (listevthis)

struct evthis  *listevthis;

  {
  struct evthis *ptevthis;

  while (listevthis != NULL)
    {
    ptevthis   = listevthis;
    listevthis = listevthis->NEXT;
    mbkfree (ptevthis);
    }
  }

/* ###--------------------------------------------------------------### */
/* function	: gss_initevthis					*/
/* description	: init (or reinit) a list of events history structure	*/
/* called funct	: mbkfree						*/
/* ###--------------------------------------------------------------### */

void gss_initevthis (listevthis)

struct evthis  *listevthis;

  {
  if (listevthis == NULL)
    {
    gss_error (0,"gss_initevthis");
    }
  else
    {
    listevthis->DATE  = 0;

    listevthis->DELTA = 0;

    listevthis->COUNT = GSS_NOEVENTYET;

    gss_freevthis ( listevthis->NEXT );

    listevthis->NEXT  = NULL;
    }
  }
