
/* ###--------------------------------------------------------------### */
/*									*/
/* file		: prb_compare.c						*/
/* date		: Jun  1 1994						*/
/* version	: v1.00							*/
/* author	: DUNOYER julien					*/
/* description	: This file contains functions relative to the 		*/
/*		  comparaison between prb sequences :			*/
/*                                                                      */
/*  prb_seqcompare     : compare two sequences                          */
/*                                                                      */
/*  prb_seqchncompare  : compare all sequences of a chain list          */
/*                                                                      */
/* ###--------------------------------------------------------------### */

#include <stdio.h>
#include <math.h>
#include <sys/time.h>

#include MUT_H
#include LOG_H
#include MLO_H
#include "beh.h"
#include "bhl.h"
#include PRB_H
#include GSS_H

/* ###--------------------------------------------------------------### */
/*                       defines                                        */
/*                       static variables declaration                   */
/*                       externals                                      */
/* ###--------------------------------------------------------------### */


/* ###--------------------------------------------------------------### */
/* function	: prb_seqcompare					*/
/* description	: compare a prb sequence with the reference prb seq	*/
/* called func.	: none							*/
/* ###--------------------------------------------------------------### */

struct prbseq *prb_seqcompare (pt_refseq, pt_prbseq, cmp_name, cmp_mode)

struct prbseq *pt_refseq;
struct prbseq *pt_prbseq;
char          *cmp_name;
char           cmp_mode;

  {
  struct prbevt     *pt_prbevt  = NULL;
  struct prbevt     *pt_refevt  = NULL;
  struct prbseq     *pt_cmpseq  = NULL;
  struct prbevt     *pt_cmpevt  = NULL;
  float              cmp_probval;
  float              cmp_densval;
  float              cmp_discval;

        /* ###------------------------------------------------------### */
        /*   If there is no prb sequence => fatal error                 */
        /* ###------------------------------------------------------### */

  if (pt_prbseq == NULL)
    {
    prb_toolbug (0, NULL, 0);
    }

        /* ###------------------------------------------------------### */
        /*   If there is no ref sequence => fatal error                 */
        /* ###------------------------------------------------------### */

  if (pt_refseq == NULL)
    {
    prb_toolbug (0, NULL, 0);
    }

  pt_cmpseq = prb_addprbseq(cmp_name);

  pt_refevt = pt_refseq->PRBEVT;

  while (pt_refevt != NULL)
    {
        /* ###------------------------------------------------------### */
        /*   Find the corresponding prbevt                              */
        /* ###------------------------------------------------------### */

        /*   Cette solution est couteuse (en N^2),                      */
        /*   mais on fait au plus simple pour le moment.                */

    pt_prbevt = pt_prbseq->PRBEVT;

    while (pt_prbevt != NULL)
      {
      if  ( (pt_prbevt->NAME == pt_refevt->NAME)
         && (pt_prbevt->MODE == pt_refevt->MODE)
         && (pt_prbevt->TYPE == pt_refevt->TYPE)
         && (pt_prbevt->SIG == pt_refevt->SIG)   )
        {
        break;
        }
      pt_prbevt = pt_prbevt->NEXT;
      }

    if (pt_prbevt == NULL)
      {
      prb_toolbug (3, pt_refevt->NAME, 0);
      }

    switch (cmp_mode)
      {
      case DEF_PRB_DELTA_CMPMODE:
        cmp_discval = (pt_prbevt->VALUE->DISCVAL) - (pt_refevt->VALUE->DISCVAL);
        cmp_densval = (pt_prbevt->VALUE->DENSVAL) - (pt_refevt->VALUE->DENSVAL);
        cmp_probval = (pt_prbevt->VALUE->PROBVAL) - (pt_refevt->VALUE->PROBVAL);
        break;

      case DEF_PRB_DRELA_CMPMODE:
        cmp_discval = ((pt_prbevt->VALUE->DISCVAL) - (pt_refevt->VALUE->DISCVAL))
                    /  (pt_refevt->VALUE->DISCVAL);
        cmp_densval = ((pt_prbevt->VALUE->DENSVAL) - (pt_refevt->VALUE->DENSVAL))
                    /  (pt_refevt->VALUE->DENSVAL);
        cmp_probval = ((pt_prbevt->VALUE->PROBVAL) - (pt_refevt->VALUE->PROBVAL))
                    /  (pt_refevt->VALUE->PROBVAL);
        break;

      case DEF_PRB_DELTABS_CMPMODE:
        cmp_discval = ((pt_prbevt->VALUE->DISCVAL - pt_refevt->VALUE->DISCVAL) < 0.0) ?
                      ((pt_refevt->VALUE->DISCVAL - pt_prbevt->VALUE->DISCVAL))       :
                      ((pt_prbevt->VALUE->DISCVAL - pt_refevt->VALUE->DISCVAL))       ;
        cmp_densval = ((pt_prbevt->VALUE->DENSVAL - pt_refevt->VALUE->DENSVAL) < 0.0) ?
                      ((pt_refevt->VALUE->DENSVAL - pt_prbevt->VALUE->DENSVAL))       :
                      ((pt_prbevt->VALUE->DENSVAL - pt_refevt->VALUE->DENSVAL))       ;
        cmp_probval = ((pt_prbevt->VALUE->PROBVAL - pt_refevt->VALUE->PROBVAL) < 0.0) ?
                      ((pt_refevt->VALUE->PROBVAL - pt_prbevt->VALUE->PROBVAL))       :
                      ((pt_prbevt->VALUE->PROBVAL - pt_refevt->VALUE->PROBVAL))       ;
        break;

      case DEF_PRB_DRELABS_CMPMODE:
        cmp_discval = ((pt_prbevt->VALUE->DISCVAL - pt_refevt->VALUE->DISCVAL) < 0.0) ?
                      (((pt_refevt->VALUE->DISCVAL - pt_prbevt->VALUE->DISCVAL))
                    /  (pt_refevt->VALUE->DISCVAL))                            :
                      (((pt_prbevt->VALUE->DISCVAL - pt_refevt->VALUE->DISCVAL))
                    /  (pt_refevt->VALUE->DISCVAL))                            ;
        cmp_densval = ((pt_prbevt->VALUE->DENSVAL - pt_refevt->VALUE->DENSVAL) < 0.0) ?
                      (((pt_refevt->VALUE->DENSVAL - pt_prbevt->VALUE->DENSVAL))
                    /  (pt_refevt->VALUE->DENSVAL))                            :
                      (((pt_prbevt->VALUE->DENSVAL - pt_refevt->VALUE->DENSVAL))
                    /  (pt_refevt->VALUE->DENSVAL))                            ;
        cmp_probval = ((pt_prbevt->VALUE->PROBVAL - pt_refevt->VALUE->PROBVAL) < 0.0) ?
                      (((pt_refevt->VALUE->PROBVAL - pt_prbevt->VALUE->PROBVAL))
                    /  (pt_refevt->VALUE->PROBVAL))                            :
                      (((pt_prbevt->VALUE->PROBVAL - pt_refevt->VALUE->PROBVAL))
                    /  (pt_refevt->VALUE->PROBVAL))                            ;
        break;

      case DEF_PRB_RMS_CMPMODE:
        printf ("compare with rms    algorithm : not implemented yet\n");
        return (pt_cmpseq);
        break;

      default:
        printf ("Unknown mode : %d\n", cmp_mode);
        printf ("compare with no     algorithm : nothing to do\n");
        return (pt_cmpseq);
        break;
      }

    pt_cmpevt = prb_addprbevt( pt_cmpevt,
                               pt_prbevt->NAME,
                               cmp_discval,
                               cmp_densval,
                               cmp_probval,
                               pt_prbevt->MODE,
                               pt_prbevt->TYPE);

    pt_cmpevt->SIG = pt_prbevt->SIG;

    pt_refevt = pt_refevt->NEXT;
    }

  pt_cmpseq->PRBEVT = (struct prbevt *) reverse (pt_cmpevt);

  return (pt_cmpseq);
  }


        /* ###------------------------------------------------------### */
        /*   Les fonctions de comparaisons de prb_seqchn n'ont pas      */
        /*   ete testees apres les modifs de Janvier 1998 (ajout        */
        /*   d'une structure prbval au lieu d'un pointeur de float).    */
        /*                                                              */
        /*   D'autre part, la comparaison necessite toujours le meme    */
        /*   ordre pour les evenements de toutes les sequences de la    */
        /*   chaine, ce qui n'est plus le cas dans prb_seqcompare.      */
        /* ###------------------------------------------------------### */

/* ###--------------------------------------------------------------### */
/* function	: prb_seqchn_min					*/
/* description	: min the prb sequence of a chain list		*/
/* called func.	: none							*/
/* ###--------------------------------------------------------------### */

static struct prbseq *prb_seqchn_min (pt_seqchn, cmp_name)

chain_list    *pt_seqchn;
char          *cmp_name;

  {
  struct prbseq     *pt_prbseq  = NULL;
  struct prbevt     *pt_prbevt  = NULL;
  struct prbseq     *pt_cmpseq  = NULL;
  struct prbevt     *pt_cmpevt  = NULL;

        /* ###------------------------------------------------------### */
        /*   If there is no sequence chn => fatal error                 */
        /* ###------------------------------------------------------### */

  if (pt_seqchn == NULL)
    {
    prb_toolbug (0, NULL, 0);
    }

        /* ###------------------------------------------------------### */
        /*   pt_cmpseq is the result of the comparaison                 */
        /* ###------------------------------------------------------### */

  pt_cmpseq = prb_addprbseq(cmp_name);

        /* ###------------------------------------------------------### */
        /* The first prbseq element is used all along as the reference  */
        /* ###------------------------------------------------------### */

  pt_prbseq = pt_seqchn->DATA;

  if (pt_prbseq == NULL)
    {
    prb_toolbug (0, NULL, 0);
    }

  pt_prbevt = pt_prbseq->PRBEVT;

  while (pt_prbevt != NULL)
    {
    pt_cmpevt = prb_addprbevt( pt_cmpevt,
                               pt_prbevt->NAME,
                               pt_prbevt->VALUE->DISCVAL,
                               pt_prbevt->VALUE->DENSVAL,
                               pt_prbevt->VALUE->PROBVAL,
                               pt_prbevt->MODE,
                               pt_prbevt->TYPE);

    pt_cmpevt->SIG = pt_prbevt->SIG;

    pt_prbevt = pt_prbevt->NEXT;
    }

  pt_cmpseq->PRBEVT = (struct prbevt *) reverse (pt_cmpevt);

  pt_seqchn = pt_seqchn->NEXT;

  while (pt_seqchn != NULL)
    {
    pt_prbseq = pt_seqchn->DATA;

    if (pt_prbseq == NULL)
      {
      prb_toolbug (0, NULL, 0);
      }

    pt_prbevt = pt_prbseq->PRBEVT;
    pt_cmpevt = pt_cmpseq->PRBEVT;

    while ((pt_prbevt != NULL) && (pt_cmpevt != NULL))
      {
      if (    (pt_prbevt->NAME != pt_cmpevt->NAME)
           || (pt_prbevt->MODE != pt_cmpevt->MODE)
           || (pt_prbevt->TYPE != pt_cmpevt->TYPE)
           || (pt_prbevt->SIG != pt_cmpevt->SIG)   )
        {
        prb_toolbug (0, NULL, 0);
        }

      if (pt_prbevt->VALUE->DISCVAL < pt_cmpevt->VALUE->DISCVAL)
        {
        pt_cmpevt->VALUE->DISCVAL = pt_prbevt->VALUE->DISCVAL;
        }

      if (pt_prbevt->VALUE->DENSVAL < pt_cmpevt->VALUE->DENSVAL)
        {
        pt_cmpevt->VALUE->DENSVAL = pt_prbevt->VALUE->DENSVAL;
        }

      if (pt_prbevt->VALUE->PROBVAL < pt_cmpevt->VALUE->PROBVAL)
        {
        pt_cmpevt->VALUE->PROBVAL = pt_prbevt->VALUE->PROBVAL;
        }

      pt_prbevt = pt_prbevt->NEXT;
      pt_cmpevt = pt_cmpevt->NEXT;
      }

    if ((pt_prbevt != NULL) || (pt_cmpevt != NULL))
      {
      prb_toolbug (0, NULL, 0);
      }

    pt_seqchn = pt_seqchn->NEXT;
    }

  return (pt_cmpseq);
  }



/* ###--------------------------------------------------------------### */
/* function	: prb_seqchn_max					*/
/* description	: max the prb sequence of a chain list		*/
/* called func.	: none							*/
/* ###--------------------------------------------------------------### */

static struct prbseq *prb_seqchn_max (pt_seqchn, cmp_name)

chain_list    *pt_seqchn;
char          *cmp_name;

  {
  struct prbseq     *pt_prbseq  = NULL;
  struct prbevt     *pt_prbevt  = NULL;
  struct prbseq     *pt_cmpseq  = NULL;
  struct prbevt     *pt_cmpevt  = NULL;

        /* ###------------------------------------------------------### */
        /*   If there is no sequence chn => fatal error                 */
        /* ###------------------------------------------------------### */

  if (pt_seqchn == NULL)
    {
    prb_toolbug (0, NULL, 0);
    }

  pt_cmpseq = prb_addprbseq(cmp_name);

        /* ###------------------------------------------------------### */
        /* The first prbseq element is used all along as the reference  */
        /* ###------------------------------------------------------### */

  pt_prbseq = pt_seqchn->DATA;

  if (pt_prbseq == NULL)
    {
    prb_toolbug (0, NULL, 0);
    }

  pt_prbevt = pt_prbseq->PRBEVT;

  while (pt_prbevt != NULL)
    {
    pt_cmpevt = prb_addprbevt( pt_cmpevt,
                               pt_prbevt->NAME,
                               pt_prbevt->VALUE->DISCVAL,
                               pt_prbevt->VALUE->DENSVAL,
                               pt_prbevt->VALUE->PROBVAL,
                               pt_prbevt->MODE,
                               pt_prbevt->TYPE);

    pt_cmpevt->SIG = pt_prbevt->SIG;

    pt_prbevt = pt_prbevt->NEXT;
    }

  pt_cmpseq->PRBEVT = (struct prbevt *) reverse (pt_cmpevt);

  pt_seqchn = pt_seqchn->NEXT;

  while (pt_seqchn != NULL)
    {
    pt_prbseq = pt_seqchn->DATA;

    if (pt_prbseq == NULL)
      {
      prb_toolbug (0, NULL, 0);
      }

    pt_prbevt = pt_prbseq->PRBEVT;
    pt_cmpevt = pt_cmpseq->PRBEVT;

    while ((pt_prbevt != NULL) && (pt_cmpevt != NULL))
      {
      if (    (pt_prbevt->NAME != pt_cmpevt->NAME)
           || (pt_prbevt->MODE != pt_cmpevt->MODE)
           || (pt_prbevt->TYPE != pt_cmpevt->TYPE)
           || (pt_prbevt->SIG != pt_cmpevt->SIG)   )
        {
        prb_toolbug (0, NULL, 0);
        }

      if (pt_prbevt->VALUE->DISCVAL > pt_cmpevt->VALUE->DISCVAL)
        {
        pt_cmpevt->VALUE->DISCVAL = pt_prbevt->VALUE->DISCVAL;
        }

      if (pt_prbevt->VALUE->DENSVAL > pt_cmpevt->VALUE->DENSVAL)
        {
        pt_cmpevt->VALUE->DENSVAL = pt_prbevt->VALUE->DENSVAL;
        }

      if (pt_prbevt->VALUE->PROBVAL > pt_cmpevt->VALUE->PROBVAL)
        {
        pt_cmpevt->VALUE->PROBVAL = pt_prbevt->VALUE->PROBVAL;
        }

      pt_prbevt = pt_prbevt->NEXT;
      pt_cmpevt = pt_cmpevt->NEXT;
      }

    if ((pt_prbevt != NULL) || (pt_cmpevt != NULL))
      {
      prb_toolbug (0, NULL, 0);
      }

    pt_seqchn = pt_seqchn->NEXT;
    }

  return (pt_cmpseq);
  }



/* ###--------------------------------------------------------------### */
/* function	: prb_seqchn_mean					*/
/* description	: mean the prb sequence of a chain list		*/
/* called func.	: none							*/
/* ###--------------------------------------------------------------### */

static struct prbseq *prb_seqchn_mean (pt_seqchn, cmp_name)

chain_list    *pt_seqchn;
char          *cmp_name;

  {
  struct prbseq     *pt_prbseq  = NULL;
  struct prbevt     *pt_prbevt  = NULL;
  struct prbseq     *pt_cmpseq  = NULL;
  struct prbevt     *pt_cmpevt  = NULL;
  unsigned int       cmp_chnlength = 0;

        /* ###------------------------------------------------------### */
        /*   If there is no sequence chn => fatal error                 */
        /* ###------------------------------------------------------### */

  if (pt_seqchn == NULL)
    {
    prb_toolbug (0, NULL, 0);
    }

  pt_cmpseq = prb_addprbseq(cmp_name);

        /* ###------------------------------------------------------### */
        /* The first prbseq element is used all along as the reference  */
        /* ###------------------------------------------------------### */

  pt_prbseq = pt_seqchn->DATA;

  if (pt_prbseq == NULL)
    {
    prb_toolbug (0, NULL, 0);
    }

  pt_prbevt = pt_prbseq->PRBEVT;

  while (pt_prbevt != NULL)
    {
    pt_cmpevt = prb_addprbevt( pt_cmpevt,
                               pt_prbevt->NAME,
                               0.0,
                               0.0,
                               0.0,
                               pt_prbevt->MODE,
                               pt_prbevt->TYPE);

    pt_cmpevt->SIG = pt_prbevt->SIG;

    pt_prbevt = pt_prbevt->NEXT;
    }

  pt_cmpseq->PRBEVT = (struct prbevt *) reverse (pt_cmpevt);

  pt_seqchn = pt_seqchn->NEXT;

  while (pt_seqchn != NULL)
    {
    pt_prbseq = pt_seqchn->DATA;

    if (pt_prbseq == NULL)
      {
      prb_toolbug (0, NULL, 0);
      }

    pt_prbevt = pt_prbseq->PRBEVT;
    pt_cmpevt = pt_cmpseq->PRBEVT;

    while ((pt_prbevt != NULL) && (pt_cmpevt != NULL))
      {
      if (    (pt_prbevt->NAME != pt_cmpevt->NAME)
           || (pt_prbevt->MODE != pt_cmpevt->MODE)
           || (pt_prbevt->TYPE != pt_cmpevt->TYPE)
           || (pt_prbevt->SIG != pt_cmpevt->SIG)   )
        {
        prb_toolbug (0, NULL, 0);
        }

      pt_cmpevt->VALUE->DISCVAL += pt_prbevt->VALUE->DISCVAL;
      pt_cmpevt->VALUE->DENSVAL += pt_prbevt->VALUE->DENSVAL;
      pt_cmpevt->VALUE->PROBVAL += pt_prbevt->VALUE->PROBVAL;

      pt_prbevt = pt_prbevt->NEXT;
      pt_cmpevt = pt_cmpevt->NEXT;
      }

    if ((pt_prbevt != NULL) || (pt_cmpevt != NULL))
      {
      prb_toolbug (0, NULL, 0);
      }

    cmp_chnlength ++;
    pt_seqchn = pt_seqchn->NEXT;
    }

  pt_cmpevt = pt_cmpseq->PRBEVT;

  while (pt_cmpevt != NULL)
    {
    pt_cmpevt->VALUE->DISCVAL = pt_cmpevt->VALUE->DISCVAL / cmp_chnlength;
    pt_cmpevt->VALUE->DENSVAL = pt_cmpevt->VALUE->DENSVAL / cmp_chnlength;
    pt_cmpevt->VALUE->PROBVAL = pt_cmpevt->VALUE->PROBVAL / cmp_chnlength;

    pt_cmpevt = pt_cmpevt->NEXT;
    }

  return (pt_cmpseq);
  }



/* ###--------------------------------------------------------------### */
/* function	: prb_seqchn_stddev					*/
/* description	: compare the prb sequence of a chain list		*/
/* called func.	: none							*/
/* ###--------------------------------------------------------------### */

static struct prbseq *prb_seqchn_stddev (pt_seqchn, pt_meanseq, cmp_name)

chain_list    *pt_seqchn;
struct prbseq *pt_meanseq;
char          *cmp_name;

  {
  struct prbevt     *pt_meanevt = NULL;
  struct prbseq     *pt_prbseq  = NULL;
  struct prbevt     *pt_prbevt  = NULL;
  struct prbseq     *pt_cmpseq  = NULL;
  struct prbevt     *pt_cmpevt  = NULL;
  unsigned int       cmp_chnlength = 0;

        /* ###------------------------------------------------------### */
        /*   If there is no sequence chn => fatal error                 */
        /* ###------------------------------------------------------### */

  if (pt_seqchn == NULL)
    {
    prb_toolbug (0, NULL, 0);
    }

  pt_cmpseq = prb_addprbseq(cmp_name);

        /* ###------------------------------------------------------### */
        /* The mean sequence is used all along as the reference         */
        /* ###------------------------------------------------------### */

  if (pt_meanseq == NULL)
    {
    prb_toolbug (0, NULL, 0);
    }

  pt_meanevt = pt_meanseq->PRBEVT;

  while (pt_meanevt != NULL)
    {
    pt_cmpevt = prb_addprbevt( pt_cmpevt,
                               pt_meanevt->NAME,
                               0.0,
                               0.0,
                               0.0,
                               pt_meanevt->MODE,
                               pt_meanevt->TYPE);

    pt_cmpevt->SIG = pt_meanevt->SIG;

    pt_meanevt = pt_meanevt->NEXT;
    }

  pt_cmpseq->PRBEVT = (struct prbevt *) reverse (pt_cmpevt);

  pt_seqchn = pt_seqchn->NEXT;

  while (pt_seqchn != NULL)
    {
    pt_prbseq = pt_seqchn->DATA;

    if (pt_prbseq == NULL)
      {
      prb_toolbug (0, NULL, 0);
      }

    pt_prbevt  = pt_prbseq->PRBEVT;
    pt_cmpevt  = pt_cmpseq->PRBEVT;
    pt_meanevt = pt_meanseq->PRBEVT;

    while ((pt_prbevt != NULL) && (pt_cmpevt != NULL))
      {
      if (    (pt_prbevt->NAME != pt_cmpevt->NAME)
           || (pt_prbevt->MODE != pt_cmpevt->MODE)
           || (pt_prbevt->TYPE != pt_cmpevt->TYPE)
           || (pt_prbevt->SIG != pt_cmpevt->SIG)   )
        {
        prb_toolbug (0, NULL, 0);
        }

      pt_cmpevt->VALUE->DISCVAL += (pt_prbevt->VALUE->DISCVAL - pt_meanevt->VALUE->DISCVAL)
                          * (pt_prbevt->VALUE->DISCVAL - pt_meanevt->VALUE->DISCVAL);
      pt_cmpevt->VALUE->DENSVAL += (pt_prbevt->VALUE->DENSVAL - pt_meanevt->VALUE->DENSVAL)
                          * (pt_prbevt->VALUE->DENSVAL - pt_meanevt->VALUE->DENSVAL);
      pt_cmpevt->VALUE->PROBVAL += (pt_prbevt->VALUE->PROBVAL - pt_meanevt->VALUE->PROBVAL)
                          * (pt_prbevt->VALUE->PROBVAL - pt_meanevt->VALUE->PROBVAL);

      pt_prbevt  = pt_prbevt->NEXT;
      pt_cmpevt  = pt_cmpevt->NEXT;
      pt_meanevt = pt_meanevt->NEXT;
      }

    if ((pt_prbevt != NULL) || (pt_cmpevt != NULL))
      {
      prb_toolbug (0, NULL, 0);
      }

    cmp_chnlength ++;
    pt_seqchn = pt_seqchn->NEXT;
    }

  pt_cmpevt = pt_cmpseq->PRBEVT;

  while (pt_cmpevt != NULL)
    {
    pt_cmpevt->VALUE->DISCVAL = sqrt (pt_cmpevt->VALUE->DISCVAL / cmp_chnlength);
    pt_cmpevt->VALUE->DENSVAL = sqrt (pt_cmpevt->VALUE->DENSVAL / cmp_chnlength);
    pt_cmpevt->VALUE->PROBVAL = sqrt (pt_cmpevt->VALUE->PROBVAL / cmp_chnlength);

    pt_cmpevt = pt_cmpevt->NEXT;
    }

  return (pt_cmpseq);
  }




/* ###--------------------------------------------------------------### */
/* function	: prb_seqchncompare					*/
/* description	: compare the prb sequence of a chain list		*/
/* called func.	: none							*/
/* ###--------------------------------------------------------------### */

struct prbseq *prb_seqchncompare (pt_seqchn, cmp_name, cmp_mode)

chain_list    *pt_seqchn;
char          *cmp_name;
char           cmp_mode;

  {
  struct prbseq     *pt_prbseq  = NULL;
  struct prbseq     *pt_cmpseq  = NULL;

        /* ###------------------------------------------------------### */
        /*   If there is no sequence chn => fatal error                 */
        /* ###------------------------------------------------------### */

  switch (cmp_mode)
    {
    case DEF_PRB_MINIMUM_CMPMODE:
      pt_cmpseq = prb_seqchn_min (pt_seqchn, cmp_name);
      break;

    case DEF_PRB_MAXIMUM_CMPMODE:
      pt_cmpseq = prb_seqchn_max (pt_seqchn, cmp_name);
      break;

    case DEF_PRB_MEAN_CMPMODE:
      pt_cmpseq = prb_seqchn_mean (pt_seqchn, cmp_name);
      break;

    case DEF_PRB_STDDEV_CMPMODE:
      pt_prbseq = prb_seqchn_mean (pt_seqchn, namealloc ("mean"));
      pt_cmpseq = prb_seqchn_stddev (pt_seqchn, pt_prbseq, cmp_name);
      break;

    default:
      printf ("compare with no     algorithm : nothing to do\n");
      break;
    }

  return (pt_cmpseq);
  }

