
/* ###--------------------------------------------------------------### */
/*									*/
/* file		: prb_debug.c						*/
/* date		: May 19 1994						*/
/* version	: v100							*/
/* author	: Julien DUNOYER					*/
/* description	: high level function					*/
/* modified	: adapted from Pirouz BAZARGAN SABET's pat library	*/
/*									*/
/* ###--------------------------------------------------------------### */

#include <stdio.h>
#include <stdlib.h>
#include PRB_H

struct typoin				/* structure used by prb_debug	*/
  {					/* to store its state		*/
  short  type;				/* code of the current structure*/
  short  mark;				/* stop mark			*/
  void  *data;				/* pointer of the structure	*/
  };

#define VHD_STKSIZ     200

#define VHD__errorDFN   -1
#define VHD__exitDFN     0
#define VHD__upDFN       1
#define VHD__topDFN      2
#define VHD__stopDFN     3
#define VHD__backDFN     4

#define VHD_prbseqDFN    5
#define VHD_nextDFN      6
#define VHD_nameDFN      7
#define VHD_errflgDFN    8
#define VHD_flagDFN      9
#define VHD_densvalDFN   10
#define VHD_probvalDFN   11
#define VHD_scanvalDFN   12

#define VHD_prbevtDFN    13
#define VHD_prbiniDFN    14
#define VHD_prbinoDFN    15
#define VHD_typeDFN      16
#define VHD_sigDFN       17
#define VHD_MAXDFN       18

/*###--------------------------------------------------------###*/

static int vhd_hash (str)

char *str;

  {
  int code = 0;

  while(*str != '\0')
    code =  code << 1 ^ *str++;

  return (code);
  }

/*###--------------------------------------------------------###*/

static int vhd_getcmd (prvcmd)

char prvcmd[3][20];

  {
  char readstr[60];
  char comd0[20];
  char comd1[20];
  char comd2[20];
  int  code;

  printf ("\n\n\n COMMAND >> ");
  comd0[0] = '\0';
  comd1[0] = '\0';
  comd2[0] = '\0';
  gets   (readstr);
  sscanf (readstr,"%s%s%s",comd0,comd1,comd2);

  if (strcmp(comd0,"."))
    {
    strcpy (prvcmd[0], comd0);
    strcpy (prvcmd[1], comd1);
    strcpy (prvcmd[2], comd2);
    }
  code = vhd_hash (prvcmd[0]);

  printf ("\n\n\n\n\n");
  return (code);
  }

/*###--------------------------------------------------------###*/

void prb_debug (head_pnt, stru_name)

void  *head_pnt;
char  *stru_name;

  {

  int		 i;
  int		 state;
  int		 newsta0;
  int		 readcmd = 0;
  char		 readtab[3][20];
  int		 stk_pnt = 0;
  int            type = VHD__exitDFN;

  struct typoin  current_pnt;
  struct typoin  stk[VHD_STKSIZ];

  char           empty[4];
  char           avail[12];

  int		 key[VHD_MAXDFN];
  char		*tab[VHD_MAXDFN];
  void		*nxt[VHD_MAXDFN];
  short		 typ[VHD_MAXDFN];

  struct prbseq *prbseq_pnt;
  struct prbevt *prbevt_pnt;
  struct prbini *prbini_pnt;
  struct prbino *prbino_pnt;

  key[VHD_prbseqDFN ] 	= vhd_hash("prbseq");
  key[VHD_nextDFN ] 	= vhd_hash("next");
  key[VHD_nameDFN ] 	= vhd_hash("name");
  key[VHD_errflgDFN ] 	= vhd_hash("errflg");
  key[VHD_flagDFN ] 	= vhd_hash("flag");
  key[VHD_densvalDFN ] 	= vhd_hash("densval");
  key[VHD_probvalDFN ] 	= vhd_hash("probval");
  key[VHD_scanvalDFN ] 	= vhd_hash("scanval");
  
  key[VHD_prbevtDFN ] 	= vhd_hash("prbevt");
  key[VHD_prbiniDFN ] 	= vhd_hash("prbini");
  key[VHD_prbinoDFN ] 	= vhd_hash("prbino");
  key[VHD_typeDFN ] 	= vhd_hash("type");
  key[VHD_sigDFN ] 	= vhd_hash("sig");

	/* ###------------------------------------------------------### */
	/*    Set of predefined commands				*/
	/* ###------------------------------------------------------### */

  key[VHD__exitDFN] = vhd_hash ("_exit");
  key[VHD__upDFN]   = vhd_hash ("_up");
  key[VHD__topDFN]  = vhd_hash ("_top");
  key[VHD__stopDFN] = vhd_hash ("_stop");
  key[VHD__backDFN] = vhd_hash ("_back");

  strcpy (empty,"");
  strcpy (avail,"AVAILABLE");

  for (i=0 ; i<VHD_MAXDFN ; i++)
    typ[i] = i;

	/* ###------------------------------------------------------### */
	/*    Find the type of the head structure			*/
	/* ###------------------------------------------------------### */

  readcmd = vhd_hash (stru_name);
  for (i=0 ; i<VHD_MAXDFN ; i++)
    {
    if (readcmd == key[i])
      {
      type = typ[i];
      break;
      }
    }

	/* ###------------------------------------------------------### */
	/*    Exit if the head structure is empty			*/
	/* ###------------------------------------------------------### */

  if (head_pnt == NULL)
    type = VHD__exitDFN;

  current_pnt.data = head_pnt;
  current_pnt.type = type;
  state            = type;

  while (state != VHD__exitDFN)
    {
	/* ###------------------------------------------------------### */
	/*    Print structure's field until the exit command is read	*/
	/* ###------------------------------------------------------### */

    for (i=0 ; i<VHD_MAXDFN ; i++)
      {
      tab[i] = empty;
      nxt[i] = NULL;
      }

	/* ###------------------------------------------------------### */
	/*    _exit and _stop commands are allways available		*/
	/* ###------------------------------------------------------### */

    tab[VHD__exitDFN] = avail;
    tab[VHD__stopDFN] = avail;

	/* ###------------------------------------------------------### */
	/*    _up, _top, and _back commands are available only if the	*/
	/* stack is not empty						*/
	/* ###------------------------------------------------------### */

    if (stk_pnt != 0)
      {
      tab[VHD__upDFN] = avail;
      tab[VHD__topDFN] = avail;
      tab[VHD__backDFN] = avail;
      }

    switch (state)
      {

    case (VHD_prbseqDFN):

		/* ###--------- prbseq ---------### */

      prbseq_pnt = (struct prbseq *)(current_pnt.data);

      if (prbseq_pnt->NEXT != NULL)
        {
        tab[VHD_nextDFN] = avail;
        nxt[VHD_nextDFN] = (void *)prbseq_pnt->NEXT;
        typ[VHD_nextDFN] = VHD_prbseqDFN;
        }
      if (prbseq_pnt->PRBINI != NULL)
        {
        tab[VHD_prbiniDFN] = avail;
        nxt[VHD_prbiniDFN] = (void *)prbseq_pnt->PRBINI;
        typ[VHD_prbiniDFN] = VHD_prbiniDFN;
        }
      if (prbseq_pnt->PRBINO != NULL)
        {
        tab[VHD_prbinoDFN] = avail;
        nxt[VHD_prbinoDFN] = (void *)prbseq_pnt->PRBINO;
        typ[VHD_prbinoDFN] = VHD_prbinoDFN;
        }
      if (prbseq_pnt->PRBEVT != NULL)
        {
        tab[VHD_prbevtDFN] = avail;
        nxt[VHD_prbevtDFN] = (void *)prbseq_pnt->PRBEVT;
        typ[VHD_prbevtDFN] = VHD_prbevtDFN;
        }

      printf ("-> next         : %s\n",tab[VHD_nextDFN]);
      printf ("-> prbini       : %s\n",tab[VHD_prbiniDFN]);
      printf ("-> prbino       : %s\n",tab[VHD_prbinoDFN]);
      printf ("-> prbevt       : %s\n",tab[VHD_prbevtDFN]);

      printf ("   name         : %s\n",prbseq_pnt->NAME);

      printf ("   scanval      : %c\n",prbseq_pnt->SCANVAL);
      printf ("   densval      : %f\n",prbseq_pnt->DENSVAL);
      printf ("   probval      : %f\n",prbseq_pnt->PROBVAL);

      printf ("   flag         : %d\n",prbseq_pnt->FLAG);
      printf ("   errflg       : %d\n",prbseq_pnt->ERRFLG);

      break;


    case (VHD_prbiniDFN):

		/* ###--------- prbini ---------### */

      prbini_pnt = (struct prbini *)(current_pnt.data);

      if (prbini_pnt->NEXT != NULL)
        {
        tab[VHD_nextDFN] = avail;
        nxt[VHD_nextDFN] = (void *)prbini_pnt->NEXT;
        typ[VHD_nextDFN] = VHD_prbiniDFN;
        }
      printf ("-> next         : %s\n",tab[VHD_nextDFN]);
      printf ("   name         : %s\n",prbini_pnt->NAME);

      break;

    case (VHD_prbinoDFN):

                /* ###--------- prbino ---------### */

      prbino_pnt = (struct prbino *)(current_pnt.data);

      if (prbino_pnt->NEXT != NULL)
        {
        tab[VHD_nextDFN] = avail;
        nxt[VHD_nextDFN] = (void *)prbino_pnt->NEXT;
        typ[VHD_nextDFN] = VHD_prbinoDFN;
        }
      printf ("-> next         : %s\n",tab[VHD_nextDFN]);
      printf ("   name         : %s\n",prbino_pnt->NAME);

      break;

    case (VHD_prbevtDFN):

		/* ###--------- prbevt ---------### */

      prbevt_pnt = (struct prbevt *)(current_pnt.data);

      if (prbevt_pnt->NEXT != NULL)
        {
        tab[VHD_nextDFN] = avail;
        nxt[VHD_nextDFN] = (void *)prbevt_pnt->NEXT;
        typ[VHD_nextDFN] = VHD_prbevtDFN;
        }
      printf ("-> next          : %s\n",tab[VHD_nextDFN]);
      printf ("   name          : %s\n",prbevt_pnt->NAME);
      printf ("   densval       : %f\n",prbevt_pnt->VALUE->DENSVAL);
      printf ("   probval       : %f\n",prbevt_pnt->VALUE->PROBVAL);
      printf ("   mode          : %c\n",prbevt_pnt->MODE);
      printf ("   type          : %d\n",prbevt_pnt->TYPE);
      if (prbevt_pnt->SIG != NULL)
        {
        tab[VHD_sigDFN] = empty;
        nxt[VHD_sigDFN] = (void *)prbevt_pnt->SIG;
        }
      printf ("-> sig          : pas pret\n");


      break;



      }

	/* ###------------------------------------------------------### */
	/*    Reading new command					*/
	/* ###------------------------------------------------------### */

    readcmd = vhd_getcmd (readtab);

    for (i=0 ; i<VHD_MAXDFN ; i++)
      {
      if (readcmd == key[i])
        break;
      }
    if ((i != VHD_MAXDFN+1) && (tab[i] == avail))
      newsta0 = i;
    else
      newsta0 = -1;

	/* ###------------------------------------------------------### */
	/*    Analysing new command					*/
	/* ###------------------------------------------------------### */

    switch (newsta0)
      {
      case (VHD__upDFN):
        current_pnt = stk[--stk_pnt] ;
        break;

      case (VHD__backDFN):
        while ((--stk_pnt != 0) && (stk[stk_pnt].mark != 1));
        current_pnt = stk[stk_pnt] ;
        break;

      case (VHD__stopDFN):
        current_pnt.mark = 1;
        break;

      case (VHD__topDFN):
        current_pnt = stk[0] ;
        stk_pnt     = 0 ;
        break;

      case (VHD__exitDFN):
        current_pnt.data = NULL;
        current_pnt.type = VHD__exitDFN;
        current_pnt.mark = 0;
        break;

      case (VHD__errorDFN):
        break;

      default:
        if (stk_pnt < VHD_STKSIZ)
          {
          stk[stk_pnt++]   = current_pnt;
          current_pnt.type = typ[newsta0];
          current_pnt.mark = 0;
          current_pnt.data = nxt[newsta0];
          }
        else
          printf ("==> STACK OVERFLOW -- STACK OVERFLOW <==\n");
        break;

      }

    state  = current_pnt.type;
    }
  }
