/* ###--------------------------------------------------------------### */
/*									*/
/* file		: prb_driver.c						*/
/* date		: Jun  1 1994						*/
/* version	: v1.00							*/
/* author	: DUNOYER julien					*/
/* description	: This file contains driving function for probabilities	*/
/*		  prb_driver 						*/
/* ###--------------------------------------------------------------### */

#include <stdio.h>
#include <sys/time.h>

#include MUT_H
#include PRB_H

/* ###--------------------------------------------------------------### */
/*                       defines                                        */
/*                       static variables declaration                   */
/*                       externals                                      */
/* ###--------------------------------------------------------------### */

/* ###--------------------------------------------------------------### */
/* function	: prb_driver						*/
/* description	: drive a file resulting of the simulation cycle	*/
/* called func.	: prb_toolbug, prb_error				*/
/* ###--------------------------------------------------------------### */

int prb_driver (pt_prbseq)

struct prbseq     *pt_prbseq ;	/* current sequence		*/

  {
  time_t         clock       ;
  int            max_err = 0 ;
  FILE          *fp          ;
  struct prbevt *pt_prbevt   ;
  struct prbval *value       ;
  char           printname[256]    ;

  if (pt_prbseq == NULL)
    {
    prb_toolbug (5, NULL, 0);
    }

  fp = mbkfopen (pt_prbseq->NAME, PRB_OUT_SUFFIX, WRITE_TEXT);

  if (fp == NULL)
    {
    max_err++;
    prb_error (2, pt_prbseq->NAME, 0.0, ' ', 0);
    }

  else
    {
    time (&clock);

    (void)fprintf (fp,"#\t\tCore file generated from prb_driver\n");
    (void)fprintf (fp,"#\t\tdate : %s \n\n", ctime(&clock));

    (void)fprintf (fp,"#\t\tSequence : %s \n\n",pt_prbseq->NAME);

    pt_prbevt = pt_prbseq->PRBEVT;
    while (pt_prbevt != NULL)
      {
      if (pt_prbevt->NAME != NULL)
        {

        if (vectorindex(pt_prbevt->NAME) == -1)
          {
          sprintf (printname, "%s", vectorradical(pt_prbevt->NAME));
          }

        else
          {
          sprintf (printname, "%s(%d)", vectorradical(pt_prbevt->NAME),
                                        vectorindex(pt_prbevt->NAME));
          }

        switch (pt_prbevt->MODE )
          {
          case 'I' :
            (void)fprintf(fp, "in       %-15s : ", printname);
            break;
          case 'O' :
            (void)fprintf(fp, "out      %-15s : ", printname);
            break;
          case 'B' :
            (void)fprintf(fp, "inout    %-15s : ", printname);
            break;
          case 'Z' :
            (void)fprintf(fp, "tristate %-15s : ", printname);
            break;
          case 'T' :
            (void)fprintf(fp, "transc   %-15s : ", printname);
            break;
          case 'P' :
            (void)fprintf(fp, "port     %-15s : ", printname);
            break;
          case 'S' :
            (void)fprintf(fp, "signal   %-15s : ", printname);
            break;
          case 'A' :
            (void)fprintf(fp, "assert   %-15s : ", printname);
            break;
          case 'R' :
            (void)fprintf(fp, "register %-15s : ", printname);
            break;
          case 'C' :
            (void)fprintf(fp, "clock    %-15s : ", printname);
            break;
          default  :
            (void)fprintf(fp, "unknown  %-15s : ", printname);
            break;
          }

        value = pt_prbevt->VALUE;

        (void)fprintf(fp,"p('1') = %-6.4f ", value->PROBVAL);
        (void)fprintf(fp," | ");

        (void)fprintf(fp,"density = %-6.4f ", value->DENSVAL);
        (void)fprintf(fp,";\n");
        }
      pt_prbevt = pt_prbevt->NEXT;
      }

    fclose (fp);
    }
  return (max_err);
  }
