/* ###--------------------------------------------------------------### */
/* file		: prb_makderdeder.c					*/
/* date		: Jan 24 1998						*/
/* version	: v111							*/
/* authors	: Julien DUNOYER					*/
/* description	: high level function					*/
/* ###--------------------------------------------------------------### */

#include <stdio.h>
#include MUT_H
#include LOG_H
#include "beh.h"
#include "bhl.h"
#include PRB_H
#include GSS_H

static struct circuit    *GLOBAL_CIRCUIT;
static struct dct_entry **PRIMVAR_DICT;
static struct dct_entry **DERIVATE_DICT;

/*##------------------------------------------------------------------##*/
/*  Function : prb_ddrchoice()                                     	*/
/*  contents : make the choice between BYVAR and BYTIME for the		*/
/*             construction of the dynamic derivate for a given varlist */
/*  called func. :                                                      */
/*##------------------------------------------------------------------##*/

static unsigned int prb_ddrchoice(pt_varlist)
struct chain *pt_varlist;
  {
  struct chain *pt = NULL;
  unsigned int  compteur;
  unsigned int  increment;

  compteur = 0;
  for (pt = pt_varlist; pt; pt = pt->NEXT)
    {
    if ((unsigned int)gss_chktab(PRIMVAR_DICT,pt->DATA, NULL, 0) == 1)
      {
      compteur = compteur + 1 ;
      }
    else
      {
      compteur = compteur - 0 ;
      }
    }
  if (compteur > 1)
    {
    return (1);
    }
  else
    {
    return (0);
    }
  }


/*##------------------------------------------------------------------##*/
/*  Function : prb_appendbeder()                                        */
/*  contents : append a derivate list at the end of another derivate    */
/*             list.                                                    */
/*  called func. :                                                      */
/*##------------------------------------------------------------------##*/

static struct beder *prb_appendbeder(pt1, pt2)
struct beder *pt1,*pt2;
  {
  struct beder *pt;

  if (pt1 == NULL)
    {
    return (pt2);
    }

  else
    {
    for (pt = pt1; pt->NEXT; pt = pt->NEXT);

    pt->NEXT = pt2;
    return (pt1);
    }
  }

/* ###--------------------------------------------------------------### */
/* function	: prb_timesimplifybdd					*/
/* description	: simplify a bytime derivate bdd on the set of		*/
/*                variable of a varlist.				*/
/* called func.	: 							*/
/* ###--------------------------------------------------------------### */

static struct node *prb_timesimplifybdd (pt_bdd, head_varlist)

struct node     *pt_bdd  ;
struct chain  *head_varlist;
  {
  struct chain    *pt_varlist  = NULL;
  struct node     *pt_compbdd = NULL;

/* printf("prb_timesimplifybdd : entree\n"); */

/*   head_varlist = supportIndexBdd (pt_bdd, 0); */

  pt_varlist   = head_varlist;
  while (pt_varlist != NULL)
    {
/* printf("on traite la variable %d\n", pt_varlist->DATA); */

    pt_compbdd = createNodeTermBdd ((unsigned int)pt_varlist->DATA);
    pt_bdd = composeBdd(pt_bdd,
                        pt_compbdd,
                        (unsigned int)pt_varlist->DATA | (~(~0 << 1)) );

    pt_varlist = pt_varlist->NEXT;
    }

  return(pt_bdd);
  }

/*##------------------------------------------------------------------##*/
/*  Function : prb_dupbeder()                                           */
/*  contents : duplicate a derivate list and return a pointer on the    */
/*             new structure. This duplication is not a pure hard copy  */
/*             since bytime derivates (INDEX = 0) are omitted.          */
/*  called func. : reverse(), beh_addbeder(),                           */
/*##------------------------------------------------------------------##*/

static struct beder *prb_dupbeder(pt_derlist)

struct beder *pt_derlist;
  {
  struct beder *pt_duplist = NULL;         /* Returned chain pointer       */


  while(pt_derlist != NULL)
    {
    if (pt_derlist->INDEX > 1)
      {
      pt_duplist = beh_addbeder(pt_duplist, pt_derlist->BDD,
                                pt_derlist->INDEX, pt_derlist->TIME);
      }
    pt_derlist = pt_derlist->NEXT;
    }

  pt_duplist = (struct beder *)reverse( (struct chain *) pt_duplist);

  return(pt_duplist);
  }


/*##------------------------------------------------------------------##*/
/*  Function : prb_duptimebeder()                                       */
/*  contents : duplicate a derivate list and return a pointer on the    */
/*             new structure. This duplication is not a pure hard copy  */
/*             since a special mix of bytime derivates ( INDEX=0 )      */
/*             and byvar derivates (INDEX>1) is created.                */
/*  called func. : reverse(), beh_addbeder(),                           */
/*##------------------------------------------------------------------##*/

static struct beder *prb_duptimebeder(pt_derlist)

struct beder *pt_derlist;
  {
  struct node     *simplify_bdd = NULL;
  struct beder    *pt_duplist   = NULL;
  struct chain    *der_varlist  = NULL;
  struct beder    *tmp_derlist  = NULL;

/* printf("prb_duptimebeder : entree\n"); */

/*   on fait la liste des variables non primaires pour simplify */

  tmp_derlist = pt_derlist;
  while(tmp_derlist != NULL)
    {
    if ( ( tmp_derlist->INDEX > 1 )
    &&   ((unsigned int)gss_chktab(PRIMVAR_DICT, tmp_derlist->INDEX,
                                   NULL, 0 ) != 1 ) )
      {
/* printf("on ajoute la variable %d\n", tmp_derlist->INDEX); */
      der_varlist = addchain (der_varlist, tmp_derlist->INDEX);
      }
    tmp_derlist = tmp_derlist->NEXT;
    }

  tmp_derlist = pt_derlist;
  while(tmp_derlist != NULL)
    {

/*   Si la derivee est une byvar et que son index n'est pas une  */
/*   variable primaire, alors on garde cette derivee.            */
/*   Si la derivee est une byvtime, on la simplifie pour ne plus */
/*   considerer que les variables primaires.                     */
/*   Les derivees byvar des var primaires sont laissees de cote. */

    if ( ( tmp_derlist->INDEX > 1 ) &&
       ((unsigned int)gss_chktab(PRIMVAR_DICT,tmp_derlist->INDEX, NULL, 0) != 1))
      {
      pt_duplist = beh_addbeder(pt_duplist, tmp_derlist->BDD,
                                tmp_derlist->INDEX, tmp_derlist->TIME);
      }
    else if ( tmp_derlist->INDEX < 1 )
      {
      simplify_bdd = prb_timesimplifybdd (tmp_derlist->BDD, der_varlist);

      pt_duplist = beh_addbeder(pt_duplist,
                                simplify_bdd,
                                tmp_derlist->INDEX,
                                tmp_derlist->TIME);
      }
    tmp_derlist = tmp_derlist->NEXT;
    }

  freechain(der_varlist);
  pt_duplist = (struct beder *)reverse( (struct chain *) pt_duplist);

  return(pt_duplist);
  }


/*##------------------------------------------------------------------##*/
/*  Function : prb_expansebeder()                                       */
/*  contents : expanse a derivate list and return a              	*/
/*             header on the expanded list.                             */
/*  called func. : reverse(), beh_addbeder(),                           */
/*##------------------------------------------------------------------##*/

static struct beder *prb_expansebeder(head_derlist, pt_varlist)

struct beder    *head_derlist;
struct chain    *pt_varlist;
  {
  struct beder *pt_ddrlist  = NULL;
  struct beder *dup_ddrlist = NULL;
  struct beder *tmp_ddrlist = NULL;
  struct beder *save_ddrlist = NULL;
  int           count        = 0;

/* Ce qu'il faut faire : - Trouver la derivee du signal concerne           */
/*                       - Prendre sa derivee et la composer avec derlist */
/*                       - Prendre le delai de cette derivee et l'ajouter */
/*                       - Prendre les elmts de derlist et les expanser   */


  for (pt_ddrlist = head_derlist; pt_ddrlist ; pt_ddrlist=pt_ddrlist->NEXT)
    {     
    if ( ! cst_IsInSetElement(pt_varlist, pt_ddrlist->INDEX) ) 
      {
      dup_ddrlist = prb_dupbeder( PRB_DERIVATE(gss_chktab(DERIVATE_DICT,
                                               pt_ddrlist->INDEX,
                                               NULL, GSS_PNT0FIELD)));

      for (tmp_ddrlist=dup_ddrlist; tmp_ddrlist ; tmp_ddrlist=tmp_ddrlist->NEXT)
        {     
        tmp_ddrlist->BDD=applyBinBdd (AND, tmp_ddrlist->BDD, pt_ddrlist->BDD);
        tmp_ddrlist->TIME += pt_ddrlist->TIME;
        }

      pt_ddrlist = prb_appendbeder ( pt_ddrlist, dup_ddrlist );
      }
    else
      {
      save_ddrlist = beh_addbeder( save_ddrlist,
                                   pt_ddrlist->BDD,
                                   pt_ddrlist->INDEX,
                                   pt_ddrlist->TIME );
      }
    }
  freechain(head_derlist);

  return(save_ddrlist);
  }


/*##------------------------------------------------------------------##*/
/*  Function : prb_exptimebeder()                                       */
/*  contents : expanse a derivate list and return a              	*/
/*             header on the expanded list.                             */
/*  called func. : reverse(), beh_addbeder(),                           */
/*##------------------------------------------------------------------##*/

static struct beder *prb_exptimebeder(head_derlist, pt_varlist)

struct beder    *head_derlist;
struct chain    *pt_varlist;
  {
  struct beder *pt_ddrlist  = NULL;
  struct beder *dup_ddrlist = NULL;
  struct beder *tmp_ddrlist = NULL;
  struct beder *save_ddrlist = NULL;

/* printf("prb_exptimebeder : entree\n"); */
/* Ce qu'il faut faire : - Pour chaque derivee de la liste des derivees   */
/*                           Si ce n'est pas une entree de la Supergate   */
/*                             on dupplique sa liste de derivees          */
/*                             on compose chaque BDD avec celui           */
/*                               de la derivee en cours de traitement     */
/*                             on ajoute a chaque delai le delai          */
/*                               de la derivee en cours de traitement     */
/*                             on renvoie cette liste duppliquee en fin   */
/*                               de la liste globale pour plus tard       */
/*                           Si c'est une entree de la Supergate          */
/*                             on sauve cette derivee.                    */


  for (pt_ddrlist = head_derlist; pt_ddrlist ; pt_ddrlist=pt_ddrlist->NEXT)
    {     
/* printf("prb_exptimebeder : boucle externe\n"); */
    if (pt_ddrlist->INDEX < 1)
      {
/* printf("prb_exptimebeder : boucle externe : c'est une ByTime \n"); */
      save_ddrlist = beh_addbeder( save_ddrlist,
                                   pt_ddrlist->BDD,
                                   pt_ddrlist->INDEX,
                                   pt_ddrlist->TIME );
      }
    else if ( ! cst_IsInSetElement(pt_varlist, pt_ddrlist->INDEX) ) 
      {
/* printf("prb_exptimebeder : boucle externe : ce n'est pas une entree de spg\n"); */
      dup_ddrlist = prb_duptimebeder( PRB_DERIVATE(gss_chktab(DERIVATE_DICT,
                                                   pt_ddrlist->INDEX,
                                                   NULL, GSS_PNT0FIELD)));
/* printf("prb_exptimebeder : boucle externe : on a dupplique avec succes\n"); */


      for (tmp_ddrlist=dup_ddrlist; tmp_ddrlist ; tmp_ddrlist=tmp_ddrlist->NEXT)
        {     
/* printf("prb_exptimebeder : boucle interne\n"); */
        tmp_ddrlist->BDD=applyBinBdd (AND, tmp_ddrlist->BDD, pt_ddrlist->BDD);
        tmp_ddrlist->TIME += pt_ddrlist->TIME;
        }

      pt_ddrlist = prb_appendbeder ( pt_ddrlist, dup_ddrlist );
      }
    else
      {
/* printf("prb_exptimebeder : boucle externe : c'est une entree de spg\n"); */
      save_ddrlist = beh_addbeder( save_ddrlist,
                                   pt_ddrlist->BDD,
                                   pt_ddrlist->INDEX,
                                   pt_ddrlist->TIME );
      }
    }
  freechain(head_derlist);

  return(save_ddrlist);
  }


/* ###--------------------------------------------------------------### */
/* function	: prb_expansevarlist					*/
/* description	: expanse a varlist on the set of variable of a varlist	*/
/* called func.	: 							*/
/* ###--------------------------------------------------------------### */

static struct chain *prb_expansevarlist (pt_varlist, pt_varexp)

struct chain    *pt_varlist;
struct chain    *pt_varexp;
  {
  struct chain  *loc_varlist;

  for (loc_varlist = pt_varlist; loc_varlist; loc_varlist = loc_varlist->NEXT )
    {
    if ( ! cst_IsInSetElement(pt_varexp, loc_varlist->DATA) ) 
      {
      }
    }

  return(pt_varlist);
  }

/* ###--------------------------------------------------------------### */
/* function	: prb_expansebdd					*/
/* description	: expanse a bdd on the set of variable of a varlist.	*/
/* called func.	: 							*/
/* ###--------------------------------------------------------------### */

static struct node *prb_expansebdd (pt_bdd, pt_varexp)

struct node     *pt_bdd  ;
struct chain    *pt_varexp;
  {
  struct chain    *pt_varlist  = NULL;
  struct node     *pt_compbdd = NULL;
  struct chain    *head_varlist  = NULL;

  head_varlist = supportIndexBdd (pt_bdd, 0);

  pt_varlist   = head_varlist;
  while (pt_varlist != NULL)
    {
    if ( ! cst_IsInSetElement(pt_varexp, pt_varlist->DATA) ) 
      {
      pt_compbdd = PRB_BDD(gss_chktab(DERIVATE_DICT,
                                      pt_varlist->DATA,
                                      NULL, GSS_PNT0FIELD));

      pt_compbdd = prb_expansebdd(pt_compbdd, pt_varexp);

      pt_bdd = composeBdd(pt_bdd,
                          pt_compbdd,
                          pt_varlist->DATA);
      }

    pt_varlist = pt_varlist->NEXT;
    }

  freechain(head_varlist);

  return(pt_bdd);
  }

/* ###--------------------------------------------------------------### */
/* function	: prb_exptimebdd					*/
/* description	: expanse a bdd on the set of variable of a varlist.	*/
/* called func.	: 							*/
/* ###--------------------------------------------------------------### */

static struct node *prb_exptimebdd (pt_bdd, pt_varexp, head_varlist)

struct node     *pt_bdd  ;
struct chain    *pt_varexp;
struct chain    *head_varlist;
  {
  struct chain    *pt_varlist  = NULL;
  struct node     *pt_compbdd = NULL;

  while (head_varlist != NULL)
    {
    if ( ! cst_IsInSetElement(pt_varexp,
                        (((unsigned int)(head_varlist->DATA)) & (~0 << 1)) )) 
      {
      pt_compbdd = PRB_BDD(gss_chktab(DERIVATE_DICT,
                                      head_varlist->DATA,
                                      NULL, GSS_PNT0FIELD));

      pt_varlist = PRB_VARLIST(gss_chktab(DERIVATE_DICT,
                                          head_varlist->DATA,
                                          NULL, GSS_PNT0FIELD));

      pt_compbdd = prb_exptimebdd(pt_compbdd, pt_varexp, pt_varlist);

      pt_bdd = composeBdd(pt_bdd,
                          pt_compbdd,
                          head_varlist->DATA);
      }

    head_varlist = head_varlist->NEXT;
    }

  return(pt_bdd);
  }

/* ###--------------------------------------------------------------### */
/* function	: prb_derbyvox						*/
/* description	: build a derdeder for one signal			*/
/*            	  For BYVOX only construction :  			*/
/*            	  On derive par variable au 1er ordre. Puis on expanse	*/
/*                la fonction d'observabilite de chaque derivee part.	*/
/* called func.	: 							*/
/* ###--------------------------------------------------------------### */

static struct beder *prb_derbyvox (pt_derlist, pt_varlist, mode)

struct beder    *pt_derlist;
struct chain    *pt_varlist;
unsigned char    mode;
  {
  struct beder  *head_ddrlist;
  struct beder  *tmp_ddrlist;
  struct beder  *pt_ddrlist;

/* au cas ou la derivee serait construite by time et aussi by var    */
/* on verifie que l'index est superieur a 1 avant d'expanser le bdd  */

  head_ddrlist  = prb_dupbeder(pt_derlist);

  if ( mode & DEF_PRB_BYVOXDDR )
    {
    for (pt_ddrlist = head_ddrlist; pt_ddrlist ; pt_ddrlist=pt_ddrlist->NEXT)
      {
      if (pt_derlist->INDEX > 1)
        {
        pt_ddrlist->BDD = prb_expansebdd(pt_ddrlist->BDD, pt_varlist);
        }
      }
    }

  return (head_ddrlist);
  }

/* ###--------------------------------------------------------------### */
/* function	: prb_derbyvax						*/
/* description	: build a derder for one signal				*/
/*            	  For BYVAX only construction :  			*/
/*            	  On derive par variable recursivement. Puis on expanse	*/
/*                la fonction d'observabilite de chaque derivee part.	*/
/* called func.	: 							*/
/* ###--------------------------------------------------------------### */

static struct beder *prb_derbyvax (pt_derlist, pt_varlist, mode)

struct beder    *pt_derlist;
struct chain    *pt_varlist;
unsigned char    mode;
  {
  struct beder  *head_ddrlist = NULL;
  struct beder  *tmp_ddrlist = NULL;
  struct beder  *pt_ddrlist = NULL;

  if ( mode & DEF_PRB_BYVAXDDR )
    {

/* Ici commence la premiere partie :                               */
/*           duplication puis developpement recursif par variable  */
/*           la dupplication filtre les derivees by time (INDEX 1) */


    head_ddrlist = prb_expansebeder (prb_dupbeder (pt_derlist), pt_varlist);


/* Ici commence la seconde partie : expansion des fonctions d'obser: */

    for (pt_ddrlist = head_ddrlist; pt_ddrlist ; pt_ddrlist=pt_ddrlist->NEXT)
      {
      pt_ddrlist->BDD = prb_expansebdd(pt_ddrlist->BDD, pt_varlist);
      }
    }

  return (head_ddrlist);
  }

/* ###--------------------------------------------------------------### */
/* function	: prb_derfinal						*/
/* description	: build a derder for one signal				*/
/*            	  For FINAL only construction :  			*/
/*            	  On derive par variable recursivement. Puis on expanse	*/
/*                la fonction d'observabilite de chaque derivee part.	*/
/*                Mais il y a une finesse car les maillons qui ont     	*/
/*                plusieurs entrees primaires prennent une derivee 	*/
/*                bytime plutot que byvar			  	*/
/*                pour combiner avec l'observabilite.                	*/
/* called func.	: 							*/
/* ###--------------------------------------------------------------### */

static struct beder *prb_derfinal (pt_derlist, pt_varlist, mode)

struct beder    *pt_derlist;
struct chain    *pt_varlist;
unsigned char    mode;
  {
  struct beder  *head_ddrlist = NULL;
  struct beder  *tmp_ddrlist = NULL;
  struct beder  *pt_ddrlist = NULL;
  struct chain  *tmp_varlist = NULL;

  if ( mode & DEF_PRB_FINALDDR )
    {
/* printf("prb_derfinal : entree\n"); */
/* Ici commence la premiere partie :                               */
/*           duplication puis developpement recursif par variable  */
/*           le developpement recursif peut donner des  bytime     */
/*           et des byvar, mais une bytime n'est plus developpee.  */


    head_ddrlist = prb_exptimebeder (prb_duptimebeder(pt_derlist), pt_varlist);


/* printf("prb_derfinal : expansion des derivees effectuee\n"); */
/* Ici commence la seconde partie : expansion des fonctions d'obser: */

    for (pt_ddrlist = head_ddrlist; pt_ddrlist ; pt_ddrlist=pt_ddrlist->NEXT)
      {
      tmp_varlist = supportIndexBdd (pt_ddrlist->BDD, 0);
      pt_ddrlist->BDD = prb_exptimebdd(pt_ddrlist->BDD, pt_varlist, tmp_varlist);
      freechain(tmp_varlist);
      }
/* printf("prb_derfinal : expansion des BDDs     effectuee\n"); */
    }

  return (head_ddrlist);
  }

/* ###--------------------------------------------------------------### */
/* function	: prb_makderdeder					*/
/* description	: build the list of derdeder for all signals		*/
/*                6 modes are possible : - BYVARDDR			*/
/*                                       - BYTIMEDDR			*/
/*                                       - BYVOXDDR (VAR OBS eXpanded)	*/
/*                                       - BYVAXDDR (VAR ALL eXpanded)	*/
/*                                       - MIXEDDDR (TIME & VOX)	*/
/*                                       - FINALDDR (TIME & VAX)	*/
/* called func.	:             						*/
/* ###--------------------------------------------------------------### */

void prb_makderdeder (pt_befig, mode)

struct befig    *pt_befig;
unsigned char    mode;

  {
  struct beaux  *pt_beaux;
  struct beout  *pt_beout;
  struct bebus  *pt_bebus;
  struct bebux  *pt_bebux;
  struct bereg  *pt_bereg;
  struct bemsg  *pt_bemsg;
  struct bepor  *pt_bepor;
  struct binode *pt_binod;
  struct chain  *list_der;


  GLOBAL_CIRCUIT = pt_befig->CIRCUI;

	/* ###------------------------------------------------------### */
	/*   check that the BEFIG is ready to support the derivation	*/
	/* ###------------------------------------------------------### */

  if ((pt_befig->TYPE & BEH_NODE_QUAD)
  &&  (pt_befig->TYPE & BEH_NODE_USER))
    {

    if (mode & DEF_PRB_BYVOXDDR)
      {
      DERIVATE_DICT = gss_initab();

	/* ###------------------------------------------------------### */
	/*   Add in DERIVATE_DIC internal simple signals		*/
	/* ###------------------------------------------------------### */
    
      pt_beaux = pt_befig->BEAUX;
      while (pt_beaux != NULL)
         {
         gss_addtab (DERIVATE_DICT,
                     searchInputCct(pt_befig->CIRCUI, pt_beaux->NAME),
                     NULL,GSS_PNT0FIELD,
                     pt_beaux->NODE);

         pt_beaux       = pt_beaux->NEXT;
         }

	/* ###------------------------------------------------------### */
	/*   Add in DERIVATE_DIC internal delayed signals		*/
	/* ###------------------------------------------------------### */

      pt_beaux = pt_befig->BEDLY;
      while (pt_beaux != NULL)
         {
         gss_addtab (DERIVATE_DICT,
                     searchInputCct(pt_befig->CIRCUI, pt_beaux->NAME),
                     NULL,GSS_PNT0FIELD,
                     pt_beaux->NODE);

         pt_beaux       = pt_beaux->NEXT;
         }


	/* ###------------------------------------------------------### */
	/*   simple output ports					*/
	/* ###------------------------------------------------------### */

      pt_beout = pt_befig->BEOUT;
      while (pt_beout != NULL)
        {
        if (PRB_SUPERFLAG (pt_beout->NODE) & DEF_PRB_SUPERGATE)
          {
          PRB_SUPERDERDEDER (pt_beout->NODE) =
             prb_derbyvox (PRB_DERIVATE (pt_beout->NODE),
                           PRB_SUPERVARLIST (pt_beout->NODE), mode);
          }
        else
          {
          PRB_SUPERDERDEDER (pt_beout->NODE) =
             prb_derbyvox (PRB_DERIVATE (pt_beout->NODE),
                           PRB_SUPERVARLIST (pt_beout->NODE), mode);
          }
        PRB_SUPERFLAG (pt_beout->NODE) |= mode;
        pt_beout = pt_beout->NEXT;
        }

	/* ###------------------------------------------------------### */
	/*   bussed output ports					*/
	/* ###------------------------------------------------------### */

      pt_bebus = pt_befig->BEBUS;
      while (pt_bebus != NULL)
        {
        pt_binod = pt_bebus->BINODE;
        while (pt_binod != NULL)
           {
           if (PRB_SUPERFLAG (pt_binod->CNDNODE) & DEF_PRB_SUPERGATE)
             {
             PRB_SUPERDERDEDER (pt_binod->CNDNODE) =
               prb_derbyvox (PRB_DERIVATE (pt_binod->CNDNODE),
                             PRB_SUPERVARLIST (pt_binod->CNDNODE), mode);
             }
           else
             {
             PRB_SUPERDERDEDER (pt_binod->CNDNODE) =
               prb_derbyvox (PRB_DERIVATE (pt_binod->CNDNODE),
                             PRB_SUPERVARLIST (pt_binod->CNDNODE), mode);
             }

           if (PRB_SUPERFLAG (pt_binod->VALNODE) & DEF_PRB_SUPERGATE)
             {
             PRB_SUPERDERDEDER (pt_binod->VALNODE) =
               prb_derbyvox (PRB_DERIVATE (pt_binod->VALNODE),
                             PRB_SUPERVARLIST (pt_binod->VALNODE), mode);
             }
           else
             {
             PRB_SUPERDERDEDER (pt_binod->VALNODE) =
               prb_derbyvox (PRB_DERIVATE (pt_binod->VALNODE),
                             PRB_SUPERVARLIST (pt_binod->VALNODE), mode);
             }

           PRB_SUPERFLAG (pt_binod->CNDNODE) |= mode;
           PRB_SUPERFLAG (pt_binod->VALNODE) |= mode;
           pt_binod = pt_binod->NEXT;
           }

        pt_bebus = pt_bebus->NEXT;
        }

	/* ###------------------------------------------------------### */
	/*   bussed internal signals					*/
	/* ###------------------------------------------------------### */

      pt_bebux = pt_befig->BEBUX;
      while (pt_bebux != NULL)
        {
        pt_binod = pt_bebux->BINODE;
        while (pt_binod != NULL)
           {
           if (PRB_SUPERFLAG (pt_binod->CNDNODE) & DEF_PRB_SUPERGATE)
             {
             PRB_SUPERDERDEDER (pt_binod->CNDNODE) =
               prb_derbyvox (PRB_DERIVATE (pt_binod->CNDNODE),
                             PRB_SUPERVARLIST (pt_binod->CNDNODE), mode);
             }
           else
             {
             PRB_SUPERDERDEDER (pt_binod->CNDNODE) =
               prb_derbyvox (PRB_DERIVATE (pt_binod->CNDNODE),
                             PRB_SUPERVARLIST (pt_binod->CNDNODE), mode);
             }

           if (PRB_SUPERFLAG (pt_binod->VALNODE) & DEF_PRB_SUPERGATE)
             {
             PRB_SUPERDERDEDER (pt_binod->VALNODE) =
               prb_derbyvox (PRB_DERIVATE (pt_binod->VALNODE),
                             PRB_SUPERVARLIST (pt_binod->VALNODE), mode);
             }
           else
             {
             PRB_SUPERDERDEDER (pt_binod->VALNODE) =
               prb_derbyvox (PRB_DERIVATE (pt_binod->VALNODE),
                             PRB_SUPERVARLIST (pt_binod->VALNODE), mode);
             }

           PRB_SUPERFLAG (pt_binod->CNDNODE) |= mode;
           PRB_SUPERFLAG (pt_binod->VALNODE) |= mode;
           pt_binod = pt_binod->NEXT;
           }
        pt_bebux = pt_bebux->NEXT;
        }

	/* ###------------------------------------------------------### */
	/*   internal registers						*/
	/* ###------------------------------------------------------### */

      pt_bereg = pt_befig->BEREG;
      while (pt_bereg != NULL)
        {
        pt_binod = pt_bereg->BINODE;
        while (pt_binod != NULL)
           {
           if (PRB_SUPERFLAG (pt_binod->CNDNODE) & DEF_PRB_SUPERGATE)
             {
             PRB_SUPERDERDEDER (pt_binod->CNDNODE) =
               prb_derbyvox (PRB_DERIVATE (pt_binod->CNDNODE),
                             PRB_SUPERVARLIST (pt_binod->CNDNODE), mode);
             }
           else
             {
             PRB_SUPERDERDEDER (pt_binod->CNDNODE) =
               prb_derbyvox (PRB_DERIVATE (pt_binod->CNDNODE),
                             PRB_SUPERVARLIST (pt_binod->CNDNODE), mode);
             }

           if (PRB_SUPERFLAG (pt_binod->VALNODE) & DEF_PRB_SUPERGATE)
             {
             PRB_SUPERDERDEDER (pt_binod->VALNODE) =
               prb_derbyvox (PRB_DERIVATE (pt_binod->VALNODE),
                             PRB_SUPERVARLIST (pt_binod->VALNODE), mode);
             }
           else
             {
             PRB_SUPERDERDEDER (pt_binod->VALNODE) =
               prb_derbyvox (PRB_DERIVATE (pt_binod->VALNODE),
                             PRB_SUPERVARLIST (pt_binod->VALNODE), mode);
             }

           PRB_SUPERFLAG (pt_binod->CNDNODE) |= mode;
           PRB_SUPERFLAG (pt_binod->VALNODE) |= mode;
           pt_binod = pt_binod->NEXT;
           }
        pt_bereg = pt_bereg->NEXT;
        }

	/* ###------------------------------------------------------### */
	/*   internal simple signals					*/
	/* ###------------------------------------------------------### */

      pt_beaux = pt_befig->BEAUX;
      while (pt_beaux != NULL)
        {
        if (PRB_SUPERFLAG (pt_beaux->NODE) & DEF_PRB_SUPERGATE)
          {
          PRB_SUPERDERDEDER (pt_beaux->NODE) =
             prb_derbyvox (PRB_DERIVATE (pt_beaux->NODE),
                           PRB_SUPERVARLIST (pt_beaux->NODE), mode);
          }
        else
          {
          PRB_SUPERDERDEDER (pt_beaux->NODE) =
             prb_derbyvox (PRB_DERIVATE (pt_beaux->NODE),
                           PRB_SUPERVARLIST (pt_beaux->NODE), mode);
          }

        PRB_SUPERFLAG (pt_beaux->NODE) |= mode;
        pt_beaux = pt_beaux->NEXT;
        }


	/* ###------------------------------------------------------### */
	/*   delayed internal signals					*/
	/* ###------------------------------------------------------### */

      pt_beaux = pt_befig->BEDLY;
      while (pt_beaux != NULL)
        {
        if (PRB_SUPERFLAG (pt_beaux->NODE) & DEF_PRB_SUPERGATE)
          {
          PRB_SUPERDERDEDER (pt_beaux->NODE) =
             prb_derbyvox (PRB_DERIVATE (pt_beaux->NODE),
                           PRB_SUPERVARLIST (pt_beaux->NODE), mode);
          }
        else
          {
          PRB_SUPERDERDEDER (pt_beaux->NODE) =
             prb_derbyvox (PRB_DERIVATE (pt_beaux->NODE),
                           PRB_SUPERVARLIST (pt_beaux->NODE), mode);
          }

        PRB_SUPERFLAG (pt_beaux->NODE) |= mode;
        pt_beaux = pt_beaux->NEXT;
        }

	/* ###------------------------------------------------------### */
	/*   assertions							*/
	/* ###------------------------------------------------------### */

      pt_bemsg = pt_befig->BEMSG;
      while (pt_bemsg != NULL)
        {
        if (PRB_SUPERFLAG (pt_bemsg->NODE) & DEF_PRB_SUPERGATE)
          {
          PRB_SUPERDERDEDER (pt_bemsg->NODE) =
             prb_derbyvox (PRB_DERIVATE (pt_bemsg->NODE),
                           PRB_SUPERVARLIST (pt_bemsg->NODE), mode);
          }
        else
          {
          PRB_SUPERDERDEDER (pt_bemsg->NODE) =
             prb_derbyvox (PRB_DERIVATE (pt_bemsg->NODE),
                           PRB_SUPERVARLIST (pt_bemsg->NODE), mode);
          }

        PRB_SUPERFLAG (pt_bemsg->NODE) |= mode;
        pt_bemsg = pt_bemsg->NEXT;
        }
      }

	/*   Dans le mode BYTIME, les supergates ne sont pas exploitees	*/
	/*   pour le calcul des densites de transitions dynamiques	*/
	/*   => On prend la PRB_DERIVATE plutot que PRB_SUPERDERIVATE	*/

    else if (mode & DEF_PRB_BYTIMEDDR)
      {

	/* ###------------------------------------------------------### */
	/*   simple output ports					*/
	/* ###------------------------------------------------------### */

      pt_beout = pt_befig->BEOUT;
      while (pt_beout != NULL)
        {
        if (PRB_SUPERFLAG (pt_beout->NODE) & DEF_PRB_SUPERGATE)
          {
          PRB_SUPERDERDEDER (pt_beout->NODE) =
                           PRB_DERIVATE (pt_beout->NODE);
          }
        else
          {
          PRB_SUPERDERDEDER (pt_beout->NODE) =
                           PRB_DERIVATE (pt_beout->NODE);
          }

        PRB_SUPERFLAG (pt_beout->NODE) |= mode;
        pt_beout = pt_beout->NEXT;
        }

	/* ###------------------------------------------------------### */
	/*   bussed output ports					*/
	/* ###------------------------------------------------------### */

      pt_bebus = pt_befig->BEBUS;
      while (pt_bebus != NULL)
        {
        pt_binod = pt_bebus->BINODE;
        while (pt_binod != NULL)
           {
           if (PRB_SUPERFLAG (pt_binod->CNDNODE) & DEF_PRB_SUPERGATE)
             {
             PRB_SUPERDERDEDER (pt_binod->CNDNODE) =
                           PRB_DERIVATE (pt_binod->CNDNODE);
             }
           else
             {
             PRB_SUPERDERDEDER (pt_binod->CNDNODE) =
                           PRB_DERIVATE (pt_binod->CNDNODE);
             }

           if (PRB_SUPERFLAG (pt_binod->VALNODE) & DEF_PRB_SUPERGATE)
             {
             PRB_SUPERDERDEDER (pt_binod->VALNODE) =
                           PRB_DERIVATE (pt_binod->VALNODE);
             }
           else
             {
             PRB_SUPERDERDEDER (pt_binod->VALNODE) =
                           PRB_DERIVATE (pt_binod->VALNODE);
             }

           PRB_SUPERFLAG (pt_binod->CNDNODE) |= mode;
           PRB_SUPERFLAG (pt_binod->VALNODE) |= mode;
           pt_binod = pt_binod->NEXT;
           }

        pt_bebus = pt_bebus->NEXT;
        }

	/* ###------------------------------------------------------### */
	/*   bussed internal signals					*/
	/* ###------------------------------------------------------### */

      pt_bebux = pt_befig->BEBUX;
      while (pt_bebux != NULL)
        {
        pt_binod = pt_bebux->BINODE;
        while (pt_binod != NULL)
           {
           if (PRB_SUPERFLAG (pt_binod->CNDNODE) & DEF_PRB_SUPERGATE)
             {
             PRB_SUPERDERDEDER (pt_binod->CNDNODE) =
                           PRB_DERIVATE (pt_binod->CNDNODE);
             }
           else
             {
             PRB_SUPERDERDEDER (pt_binod->CNDNODE) =
                           PRB_DERIVATE (pt_binod->CNDNODE);
             }

           if (PRB_SUPERFLAG (pt_binod->VALNODE) & DEF_PRB_SUPERGATE)
             {
             PRB_SUPERDERDEDER (pt_binod->VALNODE) =
                           PRB_DERIVATE (pt_binod->VALNODE);
             }
           else
             {
             PRB_SUPERDERDEDER (pt_binod->VALNODE) =
                           PRB_DERIVATE (pt_binod->VALNODE);
             }

           PRB_SUPERFLAG (pt_binod->CNDNODE) |= mode;
           PRB_SUPERFLAG (pt_binod->VALNODE) |= mode;
           pt_binod = pt_binod->NEXT;
           }
        pt_bebux = pt_bebux->NEXT;
        }

	/* ###------------------------------------------------------### */
	/*   internal registers						*/
	/* ###------------------------------------------------------### */

      pt_bereg = pt_befig->BEREG;
      while (pt_bereg != NULL)
        {
        pt_binod = pt_bereg->BINODE;
        while (pt_binod != NULL)
           {
           if (PRB_SUPERFLAG (pt_binod->CNDNODE) & DEF_PRB_SUPERGATE)
             {
             PRB_SUPERDERDEDER (pt_binod->CNDNODE) =
                           PRB_DERIVATE (pt_binod->CNDNODE);
             }
           else
             {
             PRB_SUPERDERDEDER (pt_binod->CNDNODE) =
                           PRB_DERIVATE (pt_binod->CNDNODE);
             }

           if (PRB_SUPERFLAG (pt_binod->VALNODE) & DEF_PRB_SUPERGATE)
             {
             PRB_SUPERDERDEDER (pt_binod->VALNODE) =
                           PRB_DERIVATE (pt_binod->VALNODE);
             }
           else
             {
             PRB_SUPERDERDEDER (pt_binod->VALNODE) =
                           PRB_DERIVATE (pt_binod->VALNODE);
             }

           PRB_SUPERFLAG (pt_binod->CNDNODE) |= mode;
           PRB_SUPERFLAG (pt_binod->VALNODE) |= mode;
           pt_binod = pt_binod->NEXT;
           }
        pt_bereg = pt_bereg->NEXT;
        }

	/* ###------------------------------------------------------### */
	/*   internal simple signals					*/
	/* ###------------------------------------------------------### */

      pt_beaux = pt_befig->BEAUX;
      while (pt_beaux != NULL)
        {
        if (PRB_SUPERFLAG (pt_beaux->NODE) & DEF_PRB_SUPERGATE)
          {
          PRB_SUPERDERDEDER (pt_beaux->NODE) =
                           PRB_DERIVATE (pt_beaux->NODE);
          }
        else
          {
          PRB_SUPERDERDEDER (pt_beaux->NODE) =
                           PRB_DERIVATE (pt_beaux->NODE);
          }

        PRB_SUPERFLAG (pt_beaux->NODE) |= mode;
        pt_beaux = pt_beaux->NEXT;
        }


	/* ###------------------------------------------------------### */
	/*   delayed internal signals					*/
	/* ###------------------------------------------------------### */

      pt_beaux = pt_befig->BEDLY;
      while (pt_beaux != NULL)
        {
        if (PRB_SUPERFLAG (pt_beaux->NODE) & DEF_PRB_SUPERGATE)
          {
          PRB_SUPERDERDEDER (pt_beaux->NODE) =
                           PRB_DERIVATE (pt_beaux->NODE);
          }
        else
          {
          PRB_SUPERDERDEDER (pt_beaux->NODE) =
                           PRB_DERIVATE (pt_beaux->NODE);
          }

        PRB_SUPERFLAG (pt_beaux->NODE) |= mode;
        pt_beaux = pt_beaux->NEXT;
        }

	/* ###------------------------------------------------------### */
	/*   assertions							*/
	/* ###------------------------------------------------------### */

      pt_bemsg = pt_befig->BEMSG;
      while (pt_bemsg != NULL)
        {
        if (PRB_SUPERFLAG (pt_bemsg->NODE) & DEF_PRB_SUPERGATE)
          {
          PRB_SUPERDERDEDER (pt_bemsg->NODE) =
                           PRB_DERIVATE (pt_bemsg->NODE);
          }
        else
          {
          PRB_SUPERDERDEDER (pt_bemsg->NODE) =
                           PRB_DERIVATE (pt_bemsg->NODE);
          }

        PRB_SUPERFLAG (pt_bemsg->NODE) |= mode;
        pt_bemsg = pt_bemsg->NEXT;
        }
      }

    else if (mode & DEF_PRB_MIXEDDDR)
      {
      PRIMVAR_DICT = gss_initab();

	/* ###------------------------------------------------------### */
	/*   record each input bepor in the PRIMVAR_DIC			*/
	/* ###------------------------------------------------------### */

      for (pt_bepor = pt_befig->BEPOR; pt_bepor; pt_bepor = pt_bepor->NEXT)
        {
        if ((pt_bepor->DIRECTION == 'I') ||
            (pt_bepor->DIRECTION == 'B') ||
            (pt_bepor->DIRECTION == 'T'))
           {
           gss_addtab (PRIMVAR_DICT,
                       searchInputCct(pt_befig->CIRCUI, pt_bepor->NAME),
                       NULL,0,1);
           }
        }
	/* ###------------------------------------------------------### */
	/*   record each bereg in the PRIMVAR_DIC			*/
	/* ###------------------------------------------------------### */

      for (pt_bereg = pt_befig->BEREG; pt_bereg; pt_bereg = pt_bereg->NEXT)
        {
        gss_addtab (PRIMVAR_DICT,
                    searchInputCct(pt_befig->CIRCUI, pt_bereg->NAME),
                    NULL,0,1);
        }

	/* ###------------------------------------------------------### */
	/*   simple output ports					*/
	/* ###------------------------------------------------------### */

      pt_beout = pt_befig->BEOUT;
      while (pt_beout != NULL)
        {
        if (prb_ddrchoice(PRB_SUPERVARLIST (pt_beout->NODE)))
          {
          PRB_SUPERDERDEDER (pt_beout->NODE) =
                           PRB_DERIVATE (pt_beout->NODE);
          PRB_SUPERFLAG (pt_beout->NODE) |= DEF_PRB_BYTIMEDDR;
          }
        else
          {
          PRB_SUPERDERDEDER (pt_beout->NODE) =
             prb_derbyvox (PRB_DERIVATE (pt_beout->NODE),
                           PRB_SUPERVARLIST (pt_beout->NODE), DEF_PRB_BYVOXDDR);
          PRB_SUPERFLAG (pt_beout->NODE) |= DEF_PRB_BYVOXDDR;
          }

        pt_beout = pt_beout->NEXT;
        }

	/* ###------------------------------------------------------### */
	/*   bussed output ports					*/
	/* ###------------------------------------------------------### */

      pt_bebus = pt_befig->BEBUS;
      while (pt_bebus != NULL)
        {
        pt_binod = pt_bebus->BINODE;
        while (pt_binod != NULL)
          {
          if (prb_ddrchoice(PRB_SUPERVARLIST (pt_binod->CNDNODE)))
            {
            PRB_SUPERDERDEDER (pt_binod->CNDNODE) =
                             PRB_DERIVATE (pt_binod->CNDNODE);
            PRB_SUPERFLAG (pt_binod->CNDNODE) |= DEF_PRB_BYTIMEDDR;
            }
          else
            {
            PRB_SUPERDERDEDER (pt_binod->CNDNODE) =
               prb_derbyvox (PRB_DERIVATE (pt_binod->CNDNODE),
                             PRB_SUPERVARLIST (pt_binod->CNDNODE), DEF_PRB_BYVOXDDR);
            PRB_SUPERFLAG (pt_binod->CNDNODE) |= DEF_PRB_BYVOXDDR;
            }

          if (prb_ddrchoice(PRB_SUPERVARLIST (pt_binod->VALNODE)))
            {
            PRB_SUPERDERDEDER (pt_binod->VALNODE) =
                             PRB_DERIVATE (pt_binod->VALNODE);
            PRB_SUPERFLAG (pt_binod->VALNODE) |= DEF_PRB_BYTIMEDDR;
            }
          else
            {
            PRB_SUPERDERDEDER (pt_binod->VALNODE) =
               prb_derbyvox (PRB_DERIVATE (pt_binod->VALNODE),
                             PRB_SUPERVARLIST (pt_binod->VALNODE), DEF_PRB_BYVOXDDR);
            PRB_SUPERFLAG (pt_binod->VALNODE) |= DEF_PRB_BYVOXDDR;
            }

          pt_binod = pt_binod->NEXT;
          }

        pt_bebus = pt_bebus->NEXT;
        }

	/* ###------------------------------------------------------### */
	/*   bussed internal signals					*/
	/* ###------------------------------------------------------### */

      pt_bebux = pt_befig->BEBUX;
      while (pt_bebux != NULL)
        {
        pt_binod = pt_bebux->BINODE;
        while (pt_binod != NULL)
          {
          if (prb_ddrchoice(PRB_SUPERVARLIST (pt_binod->CNDNODE)))
            {
            PRB_SUPERDERDEDER (pt_binod->CNDNODE) =
                             PRB_DERIVATE (pt_binod->CNDNODE);
            PRB_SUPERFLAG (pt_binod->CNDNODE) |= DEF_PRB_BYTIMEDDR;
            }
          else
            {
            PRB_SUPERDERDEDER (pt_binod->CNDNODE) =
               prb_derbyvox (PRB_DERIVATE (pt_binod->CNDNODE),
                             PRB_SUPERVARLIST (pt_binod->CNDNODE), DEF_PRB_BYVOXDDR);
            PRB_SUPERFLAG (pt_binod->CNDNODE) |= DEF_PRB_BYVOXDDR;
            }

          if (prb_ddrchoice(PRB_SUPERVARLIST (pt_binod->VALNODE)))
            {
            PRB_SUPERDERDEDER (pt_binod->VALNODE) =
                             PRB_DERIVATE (pt_binod->VALNODE);
            PRB_SUPERFLAG (pt_binod->VALNODE) |= DEF_PRB_BYTIMEDDR;
            }
          else
            {
            PRB_SUPERDERDEDER (pt_binod->VALNODE) =
               prb_derbyvox (PRB_DERIVATE (pt_binod->VALNODE),
                             PRB_SUPERVARLIST (pt_binod->VALNODE), DEF_PRB_BYVOXDDR);
            PRB_SUPERFLAG (pt_binod->VALNODE) |= DEF_PRB_BYVOXDDR;
            }

          pt_binod = pt_binod->NEXT;
          }
        pt_bebux = pt_bebux->NEXT;
        }

	/* ###------------------------------------------------------### */
	/*   internal registers						*/
	/* ###------------------------------------------------------### */

      pt_bereg = pt_befig->BEREG;
      while (pt_bereg != NULL)
        {
        pt_binod = pt_bereg->BINODE;
        while (pt_binod != NULL)
          {
          if (prb_ddrchoice(PRB_SUPERVARLIST (pt_binod->CNDNODE)))
            {
            PRB_SUPERDERDEDER (pt_binod->CNDNODE) =
                             PRB_DERIVATE (pt_binod->CNDNODE);
            PRB_SUPERFLAG (pt_binod->CNDNODE) |= DEF_PRB_BYTIMEDDR;
            }
          else
            {
            PRB_SUPERDERDEDER (pt_binod->CNDNODE) =
               prb_derbyvox (PRB_DERIVATE (pt_binod->CNDNODE),
                             PRB_SUPERVARLIST (pt_binod->CNDNODE), DEF_PRB_BYVOXDDR);
            PRB_SUPERFLAG (pt_binod->CNDNODE) |= DEF_PRB_BYVOXDDR;
            }

          if (prb_ddrchoice(PRB_SUPERVARLIST (pt_binod->VALNODE)))
            {
            PRB_SUPERDERDEDER (pt_binod->VALNODE) =
                             PRB_DERIVATE (pt_binod->VALNODE);
            PRB_SUPERFLAG (pt_binod->VALNODE) |= DEF_PRB_BYTIMEDDR;
            }
          else
            {
            PRB_SUPERDERDEDER (pt_binod->VALNODE) =
               prb_derbyvox (PRB_DERIVATE (pt_binod->VALNODE),
                             PRB_SUPERVARLIST (pt_binod->VALNODE), DEF_PRB_BYVOXDDR);
            PRB_SUPERFLAG (pt_binod->VALNODE) |= DEF_PRB_BYVOXDDR;
            }

          pt_binod = pt_binod->NEXT;
          }
        pt_bereg = pt_bereg->NEXT;
        }

	/* ###------------------------------------------------------### */
	/*   internal simple signals					*/
	/* ###------------------------------------------------------### */

      pt_beaux = pt_befig->BEAUX;
      while (pt_beaux != NULL)
        {
        if (prb_ddrchoice(PRB_SUPERVARLIST (pt_beaux->NODE)))
          {
          PRB_SUPERDERDEDER (pt_beaux->NODE) =
                           PRB_DERIVATE (pt_beaux->NODE);
          PRB_SUPERFLAG (pt_beaux->NODE) |= DEF_PRB_BYTIMEDDR;
          }
        else
          {
          PRB_SUPERDERDEDER (pt_beaux->NODE) =
             prb_derbyvox (PRB_DERIVATE (pt_beaux->NODE),
                           PRB_SUPERVARLIST (pt_beaux->NODE), DEF_PRB_BYVOXDDR);
          PRB_SUPERFLAG (pt_beaux->NODE) |= DEF_PRB_BYVOXDDR;
          }

        pt_beaux = pt_beaux->NEXT;
        }


	/* ###------------------------------------------------------### */
	/*   delayed internal signals					*/
	/* ###------------------------------------------------------### */

      pt_beaux = pt_befig->BEDLY;
      while (pt_beaux != NULL)
        {
        if (prb_ddrchoice(PRB_SUPERVARLIST (pt_beaux->NODE)))
          {
          PRB_SUPERDERDEDER (pt_beaux->NODE) =
                           PRB_DERIVATE (pt_beaux->NODE);
          PRB_SUPERFLAG (pt_beaux->NODE) |= DEF_PRB_BYTIMEDDR;
          }
        else
          {
          PRB_SUPERDERDEDER (pt_beaux->NODE) =
             prb_derbyvox (PRB_DERIVATE (pt_beaux->NODE),
                           PRB_SUPERVARLIST (pt_beaux->NODE), DEF_PRB_BYVOXDDR);
          PRB_SUPERFLAG (pt_beaux->NODE) |= DEF_PRB_BYVOXDDR;
          }

        pt_beaux = pt_beaux->NEXT;
        }

	/* ###------------------------------------------------------### */
	/*   assertions							*/
	/* ###------------------------------------------------------### */

      pt_bemsg = pt_befig->BEMSG;
      while (pt_bemsg != NULL)
        {
        if (prb_ddrchoice(PRB_SUPERVARLIST (pt_bemsg->NODE)))
          {
          PRB_SUPERDERDEDER (pt_bemsg->NODE) =
                           PRB_DERIVATE (pt_bemsg->NODE);
          PRB_SUPERFLAG (pt_bemsg->NODE) |= DEF_PRB_BYTIMEDDR;
          }
        else
          {
          PRB_SUPERDERDEDER (pt_bemsg->NODE) =
             prb_derbyvox (PRB_DERIVATE (pt_bemsg->NODE),
                           PRB_SUPERVARLIST (pt_bemsg->NODE), DEF_PRB_BYVOXDDR);
          PRB_SUPERFLAG (pt_bemsg->NODE) |= DEF_PRB_BYVOXDDR;
          }

        pt_bemsg = pt_bemsg->NEXT;
        }
      }

	/*   Dans le mode BYVAR, les supergates sont exploitees		*/
	/*   pour le calcul des densites de transitions dynamiques	*/
	/*   => On prend la PRB_SUPERDERIVATE plutot que PRB_DERIVATE	*/

    else if (mode & DEF_PRB_BYVARDDR)
      {

	/* ###------------------------------------------------------### */
	/*   simple output ports					*/
	/* ###------------------------------------------------------### */

      pt_beout = pt_befig->BEOUT;
      while (pt_beout != NULL)
        {
        if (PRB_SUPERFLAG (pt_beout->NODE) & DEF_PRB_SUPERGATE)
          {
          PRB_SUPERDERDEDER (pt_beout->NODE) =
                           PRB_SUPERDERIVATE (pt_beout->NODE);
          }
        else
          {
          PRB_SUPERDERDEDER (pt_beout->NODE) =
                           PRB_SUPERDERIVATE (pt_beout->NODE);
          }

        PRB_SUPERFLAG (pt_beout->NODE) |= mode;
        pt_beout = pt_beout->NEXT;
        }

	/* ###------------------------------------------------------### */
	/*   bussed output ports					*/
	/* ###------------------------------------------------------### */

      pt_bebus = pt_befig->BEBUS;
      while (pt_bebus != NULL)
        {
        pt_binod = pt_bebus->BINODE;
        while (pt_binod != NULL)
           {
           if (PRB_SUPERFLAG (pt_binod->CNDNODE) & DEF_PRB_SUPERGATE)
             {
             PRB_SUPERDERDEDER (pt_binod->CNDNODE) =
                           PRB_SUPERDERIVATE (pt_binod->CNDNODE);
             }
           else
             {
             PRB_SUPERDERDEDER (pt_binod->CNDNODE) =
                           PRB_SUPERDERIVATE (pt_binod->CNDNODE);
             }

           if (PRB_SUPERFLAG (pt_binod->VALNODE) & DEF_PRB_SUPERGATE)
             {
             PRB_SUPERDERDEDER (pt_binod->VALNODE) =
                           PRB_SUPERDERIVATE (pt_binod->VALNODE);
             }
           else
             {
             PRB_SUPERDERDEDER (pt_binod->VALNODE) =
                           PRB_SUPERDERIVATE (pt_binod->VALNODE);
             }

           PRB_SUPERFLAG (pt_binod->CNDNODE) |= mode;
           PRB_SUPERFLAG (pt_binod->VALNODE) |= mode;
           pt_binod = pt_binod->NEXT;
           }

        pt_bebus = pt_bebus->NEXT;
        }

	/* ###------------------------------------------------------### */
	/*   bussed internal signals					*/
	/* ###------------------------------------------------------### */

      pt_bebux = pt_befig->BEBUX;
      while (pt_bebux != NULL)
        {
        pt_binod = pt_bebux->BINODE;
        while (pt_binod != NULL)
           {
           if (PRB_SUPERFLAG (pt_binod->CNDNODE) & DEF_PRB_SUPERGATE)
             {
             PRB_SUPERDERDEDER (pt_binod->CNDNODE) =
                           PRB_SUPERDERIVATE (pt_binod->CNDNODE);
             }
           else
             {
             PRB_SUPERDERDEDER (pt_binod->CNDNODE) =
                           PRB_SUPERDERIVATE (pt_binod->CNDNODE);
             }

           if (PRB_SUPERFLAG (pt_binod->VALNODE) & DEF_PRB_SUPERGATE)
             {
             PRB_SUPERDERDEDER (pt_binod->VALNODE) =
                           PRB_SUPERDERIVATE (pt_binod->VALNODE);
             }
           else
             {
             PRB_SUPERDERDEDER (pt_binod->VALNODE) =
                           PRB_SUPERDERIVATE (pt_binod->VALNODE);
             }

           PRB_SUPERFLAG (pt_binod->CNDNODE) |= mode;
           PRB_SUPERFLAG (pt_binod->VALNODE) |= mode;
           pt_binod = pt_binod->NEXT;
           }
        pt_bebux = pt_bebux->NEXT;
        }

	/* ###------------------------------------------------------### */
	/*   internal registers						*/
	/* ###------------------------------------------------------### */

      pt_bereg = pt_befig->BEREG;
      while (pt_bereg != NULL)
        {
        pt_binod = pt_bereg->BINODE;
        while (pt_binod != NULL)
           {
           if (PRB_SUPERFLAG (pt_binod->CNDNODE) & DEF_PRB_SUPERGATE)
             {
             PRB_SUPERDERDEDER (pt_binod->CNDNODE) =
                           PRB_SUPERDERIVATE (pt_binod->CNDNODE);
             }
           else
             {
             PRB_SUPERDERDEDER (pt_binod->CNDNODE) =
                           PRB_SUPERDERIVATE (pt_binod->CNDNODE);
             }

           if (PRB_SUPERFLAG (pt_binod->VALNODE) & DEF_PRB_SUPERGATE)
             {
             PRB_SUPERDERDEDER (pt_binod->VALNODE) =
                           PRB_SUPERDERIVATE (pt_binod->VALNODE);
             }
           else
             {
             PRB_SUPERDERDEDER (pt_binod->VALNODE) =
                           PRB_SUPERDERIVATE (pt_binod->VALNODE);
             }

           PRB_SUPERFLAG (pt_binod->CNDNODE) |= mode;
           PRB_SUPERFLAG (pt_binod->VALNODE) |= mode;
           pt_binod = pt_binod->NEXT;
           }
        pt_bereg = pt_bereg->NEXT;
        }

	/* ###------------------------------------------------------### */
	/*   internal simple signals					*/
	/* ###------------------------------------------------------### */

      pt_beaux = pt_befig->BEAUX;
      while (pt_beaux != NULL)
        {
        if (PRB_SUPERFLAG (pt_beaux->NODE) & DEF_PRB_SUPERGATE)
          {
          PRB_SUPERDERDEDER (pt_beaux->NODE) =
                           PRB_SUPERDERIVATE (pt_beaux->NODE);
          }
        else
          {
          PRB_SUPERDERDEDER (pt_beaux->NODE) =
                           PRB_SUPERDERIVATE (pt_beaux->NODE);
          }

        PRB_SUPERFLAG (pt_beaux->NODE) |= mode;
        pt_beaux = pt_beaux->NEXT;
        }


	/* ###------------------------------------------------------### */
	/*   delayed internal signals					*/
	/* ###------------------------------------------------------### */

      pt_beaux = pt_befig->BEDLY;
      while (pt_beaux != NULL)
        {
        if (PRB_SUPERFLAG (pt_beaux->NODE) & DEF_PRB_SUPERGATE)
          {
          PRB_SUPERDERDEDER (pt_beaux->NODE) =
                           PRB_SUPERDERIVATE (pt_beaux->NODE);
          }
        else
          {
          PRB_SUPERDERDEDER (pt_beaux->NODE) =
                           PRB_SUPERDERIVATE (pt_beaux->NODE);
          }

        PRB_SUPERFLAG (pt_beaux->NODE) |= mode;
        pt_beaux = pt_beaux->NEXT;
        }

	/* ###------------------------------------------------------### */
	/*   assertions							*/
	/* ###------------------------------------------------------### */

      pt_bemsg = pt_befig->BEMSG;
      while (pt_bemsg != NULL)
        {
        if (PRB_SUPERFLAG (pt_bemsg->NODE) & DEF_PRB_SUPERGATE)
          {
          PRB_SUPERDERDEDER (pt_bemsg->NODE) =
                           PRB_SUPERDERIVATE (pt_bemsg->NODE);
          }
        else
          {
          PRB_SUPERDERDEDER (pt_bemsg->NODE) =
                           PRB_SUPERDERIVATE (pt_bemsg->NODE);
          }

        PRB_SUPERFLAG (pt_bemsg->NODE) |= mode;
        pt_bemsg = pt_bemsg->NEXT;
        }
      }

	/*   Dans le mode BYVAX, on construit les chemins de transfert.	*/
	/*   Il y a resolution des correlations spatiales, mais pas	*/
	/*   des correlations spatio-temporelles.			*/

    else if (mode & DEF_PRB_BYVAXDDR)
      {
      DERIVATE_DICT = gss_initab();

	/* ###------------------------------------------------------### */
	/*   Add in DERIVATE_DIC internal simple signals		*/
	/* ###------------------------------------------------------### */
    
      pt_beaux = pt_befig->BEAUX;
      while (pt_beaux != NULL)
         {
         gss_addtab (DERIVATE_DICT,
                     searchInputCct(pt_befig->CIRCUI, pt_beaux->NAME),
                     NULL,GSS_PNT0FIELD,
                     pt_beaux->NODE);

         pt_beaux       = pt_beaux->NEXT;
         }

	/* ###------------------------------------------------------### */
	/*   Add in DERIVATE_DIC internal delayed signals		*/
	/* ###------------------------------------------------------### */

      pt_beaux = pt_befig->BEDLY;
      while (pt_beaux != NULL)
         {
         gss_addtab (DERIVATE_DICT,
                     searchInputCct(pt_befig->CIRCUI, pt_beaux->NAME),
                     NULL,GSS_PNT0FIELD,
                     pt_beaux->NODE);

         pt_beaux       = pt_beaux->NEXT;
         }


	/* ###------------------------------------------------------### */
	/*   simple output ports					*/
	/* ###------------------------------------------------------### */

      pt_beout = pt_befig->BEOUT;
      while (pt_beout != NULL)
        {
        if (PRB_SUPERFLAG (pt_beout->NODE) & DEF_PRB_SUPERGATE)
          {
          PRB_SUPERDERDEDER (pt_beout->NODE) =
             prb_derbyvax (PRB_DERIVATE (pt_beout->NODE),
                           PRB_SUPERVARLIST (pt_beout->NODE), mode);
          }
        else
          {
          PRB_SUPERDERDEDER (pt_beout->NODE) =
             prb_derbyvax (PRB_DERIVATE (pt_beout->NODE),
                           PRB_SUPERVARLIST (pt_beout->NODE), mode);
          }
        PRB_SUPERFLAG (pt_beout->NODE) |= mode;
        pt_beout = pt_beout->NEXT;
        }

	/* ###------------------------------------------------------### */
	/*   bussed output ports					*/
	/* ###------------------------------------------------------### */

      pt_bebus = pt_befig->BEBUS;
      while (pt_bebus != NULL)
        {
        pt_binod = pt_bebus->BINODE;
        while (pt_binod != NULL)
           {
           if (PRB_SUPERFLAG (pt_binod->CNDNODE) & DEF_PRB_SUPERGATE)
             {
             PRB_SUPERDERDEDER (pt_binod->CNDNODE) =
               prb_derbyvax (PRB_DERIVATE (pt_binod->CNDNODE),
                             PRB_SUPERVARLIST (pt_binod->CNDNODE), mode);
             }
           else
             {
             PRB_SUPERDERDEDER (pt_binod->CNDNODE) =
               prb_derbyvax (PRB_DERIVATE (pt_binod->CNDNODE),
                             PRB_SUPERVARLIST (pt_binod->CNDNODE), mode);
             }

           if (PRB_SUPERFLAG (pt_binod->VALNODE) & DEF_PRB_SUPERGATE)
             {
             PRB_SUPERDERDEDER (pt_binod->VALNODE) =
               prb_derbyvax (PRB_DERIVATE (pt_binod->VALNODE),
                             PRB_SUPERVARLIST (pt_binod->VALNODE), mode);
             }
           else
             {
             PRB_SUPERDERDEDER (pt_binod->VALNODE) =
               prb_derbyvax (PRB_DERIVATE (pt_binod->VALNODE),
                             PRB_SUPERVARLIST (pt_binod->VALNODE), mode);
             }

           PRB_SUPERFLAG (pt_binod->CNDNODE) |= mode;
           PRB_SUPERFLAG (pt_binod->VALNODE) |= mode;
           pt_binod = pt_binod->NEXT;
           }

        pt_bebus = pt_bebus->NEXT;
        }

	/* ###------------------------------------------------------### */
	/*   bussed internal signals					*/
	/* ###------------------------------------------------------### */

      pt_bebux = pt_befig->BEBUX;
      while (pt_bebux != NULL)
        {
        pt_binod = pt_bebux->BINODE;
        while (pt_binod != NULL)
           {
           if (PRB_SUPERFLAG (pt_binod->CNDNODE) & DEF_PRB_SUPERGATE)
             {
             PRB_SUPERDERDEDER (pt_binod->CNDNODE) =
               prb_derbyvax (PRB_DERIVATE (pt_binod->CNDNODE),
                             PRB_SUPERVARLIST (pt_binod->CNDNODE), mode);
             }
           else
             {
             PRB_SUPERDERDEDER (pt_binod->CNDNODE) =
               prb_derbyvax (PRB_DERIVATE (pt_binod->CNDNODE),
                             PRB_SUPERVARLIST (pt_binod->CNDNODE), mode);
             }

           if (PRB_SUPERFLAG (pt_binod->VALNODE) & DEF_PRB_SUPERGATE)
             {
             PRB_SUPERDERDEDER (pt_binod->VALNODE) =
               prb_derbyvax (PRB_DERIVATE (pt_binod->VALNODE),
                             PRB_SUPERVARLIST (pt_binod->VALNODE), mode);
             }
           else
             {
             PRB_SUPERDERDEDER (pt_binod->VALNODE) =
               prb_derbyvax (PRB_DERIVATE (pt_binod->VALNODE),
                             PRB_SUPERVARLIST (pt_binod->VALNODE), mode);
             }

           PRB_SUPERFLAG (pt_binod->CNDNODE) |= mode;
           PRB_SUPERFLAG (pt_binod->VALNODE) |= mode;
           pt_binod = pt_binod->NEXT;
           }
        pt_bebux = pt_bebux->NEXT;
        }

	/* ###------------------------------------------------------### */
	/*   internal registers						*/
	/* ###------------------------------------------------------### */

      pt_bereg = pt_befig->BEREG;
      while (pt_bereg != NULL)
        {
        pt_binod = pt_bereg->BINODE;
        while (pt_binod != NULL)
           {
           if (PRB_SUPERFLAG (pt_binod->CNDNODE) & DEF_PRB_SUPERGATE)
             {
             PRB_SUPERDERDEDER (pt_binod->CNDNODE) =
               prb_derbyvax (PRB_DERIVATE (pt_binod->CNDNODE),
                             PRB_SUPERVARLIST (pt_binod->CNDNODE), mode);
             }
           else
             {
             PRB_SUPERDERDEDER (pt_binod->CNDNODE) =
               prb_derbyvax (PRB_DERIVATE (pt_binod->CNDNODE),
                             PRB_SUPERVARLIST (pt_binod->CNDNODE), mode);
             }

           if (PRB_SUPERFLAG (pt_binod->VALNODE) & DEF_PRB_SUPERGATE)
             {
             PRB_SUPERDERDEDER (pt_binod->VALNODE) =
               prb_derbyvax (PRB_DERIVATE (pt_binod->VALNODE),
                             PRB_SUPERVARLIST (pt_binod->VALNODE), mode);
             }
           else
             {
             PRB_SUPERDERDEDER (pt_binod->VALNODE) =
               prb_derbyvax (PRB_DERIVATE (pt_binod->VALNODE),
                             PRB_SUPERVARLIST (pt_binod->VALNODE), mode);
             }

           PRB_SUPERFLAG (pt_binod->CNDNODE) |= mode;
           PRB_SUPERFLAG (pt_binod->VALNODE) |= mode;
           pt_binod = pt_binod->NEXT;
           }
        pt_bereg = pt_bereg->NEXT;
        }

	/* ###------------------------------------------------------### */
	/*   internal simple signals					*/
	/* ###------------------------------------------------------### */

      pt_beaux = pt_befig->BEAUX;
      while (pt_beaux != NULL)
        {
        if (PRB_SUPERFLAG (pt_beaux->NODE) & DEF_PRB_SUPERGATE)
          {
          PRB_SUPERDERDEDER (pt_beaux->NODE) =
             prb_derbyvax (PRB_DERIVATE (pt_beaux->NODE),
                           PRB_SUPERVARLIST (pt_beaux->NODE), mode);
          }
        else
          {
          PRB_SUPERDERDEDER (pt_beaux->NODE) =
             prb_derbyvax (PRB_DERIVATE (pt_beaux->NODE),
                           PRB_SUPERVARLIST (pt_beaux->NODE), mode);
          }

        PRB_SUPERFLAG (pt_beaux->NODE) |= mode;
        pt_beaux = pt_beaux->NEXT;
        }


	/* ###------------------------------------------------------### */
	/*   delayed internal signals					*/
	/* ###------------------------------------------------------### */

      pt_beaux = pt_befig->BEDLY;
      while (pt_beaux != NULL)
        {
        if (PRB_SUPERFLAG (pt_beaux->NODE) & DEF_PRB_SUPERGATE)
          {
          PRB_SUPERDERDEDER (pt_beaux->NODE) =
             prb_derbyvax (PRB_DERIVATE (pt_beaux->NODE),
                           PRB_SUPERVARLIST (pt_beaux->NODE), mode);
          }
        else
          {
          PRB_SUPERDERDEDER (pt_beaux->NODE) =
             prb_derbyvax (PRB_DERIVATE (pt_beaux->NODE),
                           PRB_SUPERVARLIST (pt_beaux->NODE), mode);
          }

        PRB_SUPERFLAG (pt_beaux->NODE) |= mode;
        pt_beaux = pt_beaux->NEXT;
        }

	/* ###------------------------------------------------------### */
	/*   assertions							*/
	/* ###------------------------------------------------------### */

      pt_bemsg = pt_befig->BEMSG;
      while (pt_bemsg != NULL)
        {
        if (PRB_SUPERFLAG (pt_bemsg->NODE) & DEF_PRB_SUPERGATE)
          {
          PRB_SUPERDERDEDER (pt_bemsg->NODE) =
             prb_derbyvax (PRB_DERIVATE (pt_bemsg->NODE),
                           PRB_SUPERVARLIST (pt_bemsg->NODE), mode);
          }
        else
          {
          PRB_SUPERDERDEDER (pt_bemsg->NODE) =
             prb_derbyvax (PRB_DERIVATE (pt_bemsg->NODE),
                           PRB_SUPERVARLIST (pt_bemsg->NODE), mode);
          }

        PRB_SUPERFLAG (pt_bemsg->NODE) |= mode;
        pt_bemsg = pt_bemsg->NEXT;
        }
      }

	/*   Dans le mode FINAL, on construit les chemins de transfert.	*/
	/*   Il y a resolution des correlations spatiales et des	*/
	/*   correlations spatio-temporelles (partiellement).		*/

    else if (mode & DEF_PRB_FINALDDR)
      {
      PRIMVAR_DICT = gss_initab();

	/* ###------------------------------------------------------### */
	/*   record each input bepor in the PRIMVAR_DIC			*/
	/* ###------------------------------------------------------### */

      for (pt_bepor = pt_befig->BEPOR; pt_bepor; pt_bepor = pt_bepor->NEXT)
        {
        if ((pt_bepor->DIRECTION == 'I') ||
            (pt_bepor->DIRECTION == 'B') ||
            (pt_bepor->DIRECTION == 'T'))
           {
           gss_addtab (PRIMVAR_DICT,
                       searchInputCct(pt_befig->CIRCUI, pt_bepor->NAME),
                       NULL,0,1);
           }
        }
	/* ###------------------------------------------------------### */
	/*   record each bereg in the PRIMVAR_DIC			*/
	/* ###------------------------------------------------------### */

      for (pt_bereg = pt_befig->BEREG; pt_bereg; pt_bereg = pt_bereg->NEXT)
        {
        gss_addtab (PRIMVAR_DICT,
                    searchInputCct(pt_befig->CIRCUI, pt_bereg->NAME),
                    NULL,0,1);
        }

      DERIVATE_DICT = gss_initab();


	/* ###------------------------------------------------------### */
	/*   Add in DERIVATE_DIC internal simple signals		*/
	/* ###------------------------------------------------------### */
    
      pt_beaux = pt_befig->BEAUX;
      while (pt_beaux != NULL)
         {
         gss_addtab (DERIVATE_DICT,
                     searchInputCct(pt_befig->CIRCUI, pt_beaux->NAME),
                     NULL,GSS_PNT0FIELD,
                     pt_beaux->NODE);

         pt_beaux       = pt_beaux->NEXT;
         }

	/* ###------------------------------------------------------### */
	/*   Add in DERIVATE_DIC internal delayed signals		*/
	/* ###------------------------------------------------------### */

      pt_beaux = pt_befig->BEDLY;
      while (pt_beaux != NULL)
         {
         gss_addtab (DERIVATE_DICT,
                     searchInputCct(pt_befig->CIRCUI, pt_beaux->NAME),
                     NULL,GSS_PNT0FIELD,
                     pt_beaux->NODE);

         pt_beaux       = pt_beaux->NEXT;
         }


	/* ###------------------------------------------------------### */
	/*   simple output ports					*/
	/* ###------------------------------------------------------### */

      pt_beout = pt_befig->BEOUT;
      while (pt_beout != NULL)
        {
        if (PRB_SUPERFLAG (pt_beout->NODE) & DEF_PRB_SUPERGATE)
          {
          PRB_SUPERDERDEDER (pt_beout->NODE) =
             prb_derfinal (PRB_DERIVATE (pt_beout->NODE),
                           PRB_SUPERVARLIST (pt_beout->NODE), mode);
          }
        else
          {
          PRB_SUPERDERDEDER (pt_beout->NODE) =
             prb_derfinal (PRB_DERIVATE (pt_beout->NODE),
                           PRB_SUPERVARLIST (pt_beout->NODE), mode);
          }
        PRB_SUPERFLAG (pt_beout->NODE) |= mode;
        pt_beout = pt_beout->NEXT;
        }

	/* ###------------------------------------------------------### */
	/*   bussed output ports					*/
	/* ###------------------------------------------------------### */

      pt_bebus = pt_befig->BEBUS;
      while (pt_bebus != NULL)
        {
        pt_binod = pt_bebus->BINODE;
        while (pt_binod != NULL)
           {
           if (PRB_SUPERFLAG (pt_binod->CNDNODE) & DEF_PRB_SUPERGATE)
             {
             PRB_SUPERDERDEDER (pt_binod->CNDNODE) =
               prb_derfinal (PRB_DERIVATE (pt_binod->CNDNODE),
                             PRB_SUPERVARLIST (pt_binod->CNDNODE), mode);
             }
           else
             {
             PRB_SUPERDERDEDER (pt_binod->CNDNODE) =
               prb_derfinal (PRB_DERIVATE (pt_binod->CNDNODE),
                             PRB_SUPERVARLIST (pt_binod->CNDNODE), mode);
             }

           if (PRB_SUPERFLAG (pt_binod->VALNODE) & DEF_PRB_SUPERGATE)
             {
             PRB_SUPERDERDEDER (pt_binod->VALNODE) =
               prb_derfinal (PRB_DERIVATE (pt_binod->VALNODE),
                             PRB_SUPERVARLIST (pt_binod->VALNODE), mode);
             }
           else
             {
             PRB_SUPERDERDEDER (pt_binod->VALNODE) =
               prb_derfinal (PRB_DERIVATE (pt_binod->VALNODE),
                             PRB_SUPERVARLIST (pt_binod->VALNODE), mode);
             }

           PRB_SUPERFLAG (pt_binod->CNDNODE) |= mode;
           PRB_SUPERFLAG (pt_binod->VALNODE) |= mode;
           pt_binod = pt_binod->NEXT;
           }

        pt_bebus = pt_bebus->NEXT;
        }

	/* ###------------------------------------------------------### */
	/*   bussed internal signals					*/
	/* ###------------------------------------------------------### */

      pt_bebux = pt_befig->BEBUX;
      while (pt_bebux != NULL)
        {
        pt_binod = pt_bebux->BINODE;
        while (pt_binod != NULL)
           {
           if (PRB_SUPERFLAG (pt_binod->CNDNODE) & DEF_PRB_SUPERGATE)
             {
             PRB_SUPERDERDEDER (pt_binod->CNDNODE) =
               prb_derfinal (PRB_DERIVATE (pt_binod->CNDNODE),
                             PRB_SUPERVARLIST (pt_binod->CNDNODE), mode);
             }
           else
             {
             PRB_SUPERDERDEDER (pt_binod->CNDNODE) =
               prb_derfinal (PRB_DERIVATE (pt_binod->CNDNODE),
                             PRB_SUPERVARLIST (pt_binod->CNDNODE), mode);
             }

           if (PRB_SUPERFLAG (pt_binod->VALNODE) & DEF_PRB_SUPERGATE)
             {
             PRB_SUPERDERDEDER (pt_binod->VALNODE) =
               prb_derfinal (PRB_DERIVATE (pt_binod->VALNODE),
                             PRB_SUPERVARLIST (pt_binod->VALNODE), mode);
             }
           else
             {
             PRB_SUPERDERDEDER (pt_binod->VALNODE) =
               prb_derfinal (PRB_DERIVATE (pt_binod->VALNODE),
                             PRB_SUPERVARLIST (pt_binod->VALNODE), mode);
             }

           PRB_SUPERFLAG (pt_binod->CNDNODE) |= mode;
           PRB_SUPERFLAG (pt_binod->VALNODE) |= mode;
           pt_binod = pt_binod->NEXT;
           }
        pt_bebux = pt_bebux->NEXT;
        }

	/* ###------------------------------------------------------### */
	/*   internal registers						*/
	/* ###------------------------------------------------------### */

      pt_bereg = pt_befig->BEREG;
      while (pt_bereg != NULL)
        {
        pt_binod = pt_bereg->BINODE;
        while (pt_binod != NULL)
           {
           if (PRB_SUPERFLAG (pt_binod->CNDNODE) & DEF_PRB_SUPERGATE)
             {
             PRB_SUPERDERDEDER (pt_binod->CNDNODE) =
               prb_derfinal (PRB_DERIVATE (pt_binod->CNDNODE),
                             PRB_SUPERVARLIST (pt_binod->CNDNODE), mode);
             }
           else
             {
             PRB_SUPERDERDEDER (pt_binod->CNDNODE) =
               prb_derfinal (PRB_DERIVATE (pt_binod->CNDNODE),
                             PRB_SUPERVARLIST (pt_binod->CNDNODE), mode);
             }

           if (PRB_SUPERFLAG (pt_binod->VALNODE) & DEF_PRB_SUPERGATE)
             {
             PRB_SUPERDERDEDER (pt_binod->VALNODE) =
               prb_derfinal (PRB_DERIVATE (pt_binod->VALNODE),
                             PRB_SUPERVARLIST (pt_binod->VALNODE), mode);
             }
           else
             {
             PRB_SUPERDERDEDER (pt_binod->VALNODE) =
               prb_derfinal (PRB_DERIVATE (pt_binod->VALNODE),
                             PRB_SUPERVARLIST (pt_binod->VALNODE), mode);
             }

           PRB_SUPERFLAG (pt_binod->CNDNODE) |= mode;
           PRB_SUPERFLAG (pt_binod->VALNODE) |= mode;
           pt_binod = pt_binod->NEXT;
           }
        pt_bereg = pt_bereg->NEXT;
        }

	/* ###------------------------------------------------------### */
	/*   internal simple signals					*/
	/* ###------------------------------------------------------### */

      pt_beaux = pt_befig->BEAUX;
      while (pt_beaux != NULL)
        {
        if (PRB_SUPERFLAG (pt_beaux->NODE) & DEF_PRB_SUPERGATE)
          {
          PRB_SUPERDERDEDER (pt_beaux->NODE) =
             prb_derfinal (PRB_DERIVATE (pt_beaux->NODE),
                           PRB_SUPERVARLIST (pt_beaux->NODE), mode);
          }
        else
          {
          PRB_SUPERDERDEDER (pt_beaux->NODE) =
             prb_derfinal (PRB_DERIVATE (pt_beaux->NODE),
                           PRB_SUPERVARLIST (pt_beaux->NODE), mode);
          }

        PRB_SUPERFLAG (pt_beaux->NODE) |= mode;
        pt_beaux = pt_beaux->NEXT;
        }


	/* ###------------------------------------------------------### */
	/*   delayed internal signals					*/
	/* ###------------------------------------------------------### */

      pt_beaux = pt_befig->BEDLY;
      while (pt_beaux != NULL)
        {
        if (PRB_SUPERFLAG (pt_beaux->NODE) & DEF_PRB_SUPERGATE)
          {
          PRB_SUPERDERDEDER (pt_beaux->NODE) =
             prb_derfinal (PRB_DERIVATE (pt_beaux->NODE),
                           PRB_SUPERVARLIST (pt_beaux->NODE), mode);
          }
        else
          {
          PRB_SUPERDERDEDER (pt_beaux->NODE) =
             prb_derfinal (PRB_DERIVATE (pt_beaux->NODE),
                           PRB_SUPERVARLIST (pt_beaux->NODE), mode);
          }

        PRB_SUPERFLAG (pt_beaux->NODE) |= mode;
        pt_beaux = pt_beaux->NEXT;
        }

	/* ###------------------------------------------------------### */
	/*   assertions							*/
	/* ###------------------------------------------------------### */

      pt_bemsg = pt_befig->BEMSG;
      while (pt_bemsg != NULL)
        {
        if (PRB_SUPERFLAG (pt_bemsg->NODE) & DEF_PRB_SUPERGATE)
          {
          PRB_SUPERDERDEDER (pt_bemsg->NODE) =
             prb_derfinal (PRB_DERIVATE (pt_bemsg->NODE),
                           PRB_SUPERVARLIST (pt_bemsg->NODE), mode);
          }
        else
          {
          PRB_SUPERDERDEDER (pt_bemsg->NODE) =
             prb_derfinal (PRB_DERIVATE (pt_bemsg->NODE),
                           PRB_SUPERVARLIST (pt_bemsg->NODE), mode);
          }

        PRB_SUPERFLAG (pt_bemsg->NODE) |= mode;
        pt_bemsg = pt_bemsg->NEXT;
        }
      }

    else
      {
      prb_error (0, NULL);
      }
    }

  else
    {
    prb_error (0, NULL);
    }
  }
