
/* ###--------------------------------------------------------------### */
/* 									*/
/* file		: prb_maksupabl.c					*/
/* date		: May 13 1995						*/
/* version	: v000							*/
/* authors	: DUNOYER Julien					*/
/* content	: high level function					*/
/*									*/
/* ###--------------------------------------------------------------### */


#include <stdio.h>
#include <sys/time.h>
#include <sys/resource.h>
#include MUT_H
#include MLO_H
#include "beh.h"
#include "bhl.h"
#include LOG_H
#include COX_H
#include GSS_H
#include PRB_H
#include "cox_colorset.h"

#define PRB_BASICABL_DEF 81
#define PRB_SUPERABL_DEF 82
#define PRB_COLORS_DEF   83
#define PRB_OPERABL_DEF  84
#define PRB_TERMABL_DEF  85

static char         *SPG_INSNAME;
struct dct_entry   **SPG_DICTIONNARY;
static struct chain *prb_createsuperabl();



/* ###--------------------------------------------------------------### */
/* function     : prb_abl2str                                           */
/* description  : return a string corresponding to an expression        */
/* called func. :                                                       */
/* ###--------------------------------------------------------------### */

static char *prb_abl2str (expr, chaine, size_pnt)

struct chain *expr;                     /* expression                   */
char         *chaine;                   /* target string                */
int          *size_pnt;                 /* size of available space      */

  {
  int           operator;
  char         *oper    ;
  struct chain *operand ;
  extern char  *operToChar ();
  static char  *str_z = NULL;
  static char  *str_o = NULL;
  static char  *str_d = NULL;

  if (str_z == NULL)
    {
    str_z = namealloc ("'0'");
    str_o = namealloc ("'1'");
    str_d = namealloc ("'d'");
    }

        /* ###------------------------------------------------------### */
        /*    if there is not enough space left allocate a bigger block */
        /* ###------------------------------------------------------### */

  if (*size_pnt < (strlen (chaine) + 50))
    {
    *size_pnt = *size_pnt + 256;
    chaine  = (char *) realloc (chaine, *size_pnt);
    }


        /* ###------------------------------------------------------### */
        /*    if the expression is a terminal (atom) add its name to    */
        /* the previous string                                          */
        /* ###------------------------------------------------------### */

  if (expr->NEXT == NULL)
    {
    if ((expr->DATA == (void *) str_z) ||
        (expr->DATA == (void *) str_o) ||
        (expr->DATA == (void *) str_d)   )
      strcat (chaine, expr->DATA);
    else
      strcat (chaine, expr->DATA);
    }

  else
    {
        /* ###------------------------------------------------------### */
        /*    if the expression is not a terminal :                     */
        /*      - for unary operators (not, stable) add                 */
        /*        "operator ( operand )" to the previous string         */
        /*                                                              */
        /*      - for binary operators (and, or, nor, xor, ...) add     */
        /*        "(operand operator operand operator operand ...)"     */
        /* ###------------------------------------------------------### */

    operator = (int) ((struct chain *) expr->DATA)->DATA;
    operand  = (struct chain *)expr->NEXT->DATA;

    if (operator == STABLE)
      {
      strcat (chaine, operand->DATA);
      strcat (chaine, "'STABLE");
      }
    else
      {
      if (operator == NOT)
        {
        strcat (chaine, "not (");
        chaine = prb_abl2str (operand, chaine, size_pnt);
        }
      else
        {
        oper = operToChar (operator);
        strcat (chaine,"(");
        while (expr = expr->NEXT)
          {
          chaine = prb_abl2str (expr->DATA, chaine, size_pnt);
          if (expr->NEXT)
            {
            strcat (chaine, " ");
            strcat (chaine, oper);
            strcat (chaine, " ");
            }
          }
        }
      strcat (chaine, ")");
      }
    }

  return (chaine);
  }


/* ###--------------------------------------------------------------### */
/* function     : prb_createatom                                        */
/* description  : create SHARED terminals (atoms) abl.			*/
/* called func. : addchain, gss_chktab, gss_addtab			*/
/* ###--------------------------------------------------------------### */
  
static struct chain *prb_createatom(terminal, dico)

char              *terminal ;
struct dct_entry **dico;

  {
  struct chain *res_abl ;

    /* ###-------------------------------------### */
    /* ### search a previously record terminal ### */
    /* ###-------------------------------------### */

  res_abl = (struct chain*)
            gss_chktab (dico, terminal, PRB_TERMABL_DEF, GSS_PNT0FIELD);

  if(! res_abl)
    {
    res_abl = addchain (NULL, (void *)namealloc(terminal));
    gss_addtab (dico, terminal, PRB_TERMABL_DEF, GSS_PNT0FIELD, res_abl);
    }

  return (res_abl);
  }

/* ###--------------------------------------------------------------### */
/* function     : prb_createexpr                                        */
/* description  : create SHARED operators abl.				*/
/* called func. : addchain, gss_chktab, gss_addtab			*/
/* ###--------------------------------------------------------------### */
  
static struct chain *prb_createexpr(operator, dico)

short              operator ;
struct dct_entry **dico;

  {
  struct chain *res_abl ;

    /* ###-------------------------------------### */
    /* ### search a previously record operator ### */
    /* ###-------------------------------------### */

  res_abl = (struct chain*)
            gss_chktab (dico, operator, PRB_OPERABL_DEF, GSS_PNT0FIELD);

  if(! res_abl)
    {
    res_abl = addchain (NULL, operator);
    gss_addtab (dico, operator, PRB_OPERABL_DEF, GSS_PNT0FIELD, res_abl);
    }

  return (addchain (NULL, (void *)res_abl));
  }

/* ###--------------------------------------------------------------### */
/* function     : prb_freabl                                            */
/* description  : free recursively an abl keeping atoms only		*/
/* called func. : delchain, freechain, prb_freebl                       */
/* ###--------------------------------------------------------------### */
  
static void prb_freabl(sce_abl)

struct chain *sce_abl ;

{
struct chain *tmp_abl ;

    /* ###-------------------------------------### */
    /* ###   dont free TERMINALS & OPERATORS   ### */
    /* ###     because they are SHARED ABL.    ### */
    /* ###-------------------------------------### */

if(! ATOM(sce_abl))
  {
  tmp_abl= sce_abl;
  delchain(CAR(sce_abl),CAR(sce_abl));
  while((sce_abl = CDR(sce_abl)))
    prb_freabl(CAR(sce_abl));
  freechain(tmp_abl);
  }
}


/* ###--------------------------------------------------------------### */
/* function     : prb_dupabl                                            */
/* description  : duplicate recursively an abl 				*/
/* called func. : ATOM,CDR,VALUE_ATOM,prb_createatom, prb_createexpr    */
/* ###--------------------------------------------------------------### */
  
static struct chain *prb_dupabl(sce_abl, dico)

struct chain *sce_abl ;
struct dct_entry **dico;

{

struct chain *res_abl;

if(ATOM(sce_abl))
  {
  res_abl = prb_createatom (VALUE_ATOM(sce_abl), dico);
  }

else 
  {
  res_abl = prb_createexpr (OPER(sce_abl), dico);

  while (sce_abl = CDR(sce_abl))
    {
    CDR(res_abl) =
       addchain(CDR(res_abl),(void *)prb_dupabl(CAR(sce_abl),dico));
    }
  }
return (res_abl);
}


/* ###--------------------------------------------------------------### */
/* function     : prb_superabl	 					*/
/* description  : Return a superabl for a signal			*/
/* called func  : prb_creatsuperabl, gss_addtab, gss_chktab		*/
/* ###--------------------------------------------------------------### */

struct chain *prb_superabl(sig_name, spg_dico, cox_dico)

char              *sig_name;
struct dct_entry **spg_dico;
struct dct_entry **cox_dico;

{
struct chain *super_abl = NULL;
struct chain *basic_abl = NULL;
struct chain *extracted_set = NULL;

    /* ###-------------------------------------### */
    /* ###     search the original abl,        ### */
    /* ###     search the extracted set,       ### */
    /* ###     then create the SUPERABL,       ### */
    /* ###     and record it in spg_dico.      ### */
    /* ###-------------------------------------### */

  basic_abl =(struct chain*) gss_chktab (spg_dico,
                                         sig_name,
                                         PRB_BASICABL_DEF,
                                         GSS_PNT0FIELD);

  if (basic_abl != NULL)
    {
    extracted_set = (struct chain *) gss_chktab (cox_dico,
                                                 sig_name,
                                                 COX_VAL_EXTRACTED,
                                                 GSS_PNT0FIELD);
    if (extracted_set == NULL)
      extracted_set = (struct chain *) gss_chktab (cox_dico,
                                                   concatname(SPG_INSNAME,
                                                              sig_name),
                                                   COX_VAL_EXTRACTED,
                                                   GSS_PNT0FIELD);
    if (extracted_set == NULL)
      {
      super_abl = basic_abl;
      }

    else
      {
      super_abl = prb_createsuperabl (basic_abl,
                                      extracted_set,
                                      spg_dico,
                                      cox_dico);

      }
    }

return (super_abl);
}


/* ###--------------------------------------------------------------### */
/* function     : prb_createsuperabl 					*/
/* description  : Recursively Construct a superabl for an abl		*/
/*		  and with a color list for recursive reference.	*/
/* called func  : prb_createsupabl,gss_chktab,gss_addtab,gss_chktab	*/
/* ###--------------------------------------------------------------### */

static struct chain *prb_createsuperabl(sce_abl,extracted_set,spg_dico,cox_dico)

struct chain      *sce_abl;
struct chain      *extracted_set;
struct dct_entry **spg_dico;
struct dct_entry **cox_dico;

  {
  struct chain *spg_abl = NULL;
  struct chain *tmp_abl = NULL;
  struct chain *propagate_set = NULL;

    /* ###-------------------------------------### */
    /* ### if the abl is a terminal,           ### */
    /* ### try to explode it in it's superabl  ### */
    /* ### and then continue the traversal     ### */
    /* ### algorithme with this new abl.       ### */
    /* ###-------------------------------------### */

  if(ATOM (sce_abl))
    {
    /* ###-------------------------------------### */
    /* ### => find the propagated colors set.  ### */
    /* ###-------------------------------------### */

    propagate_set = (struct chain*) gss_chktab (cox_dico,
                                                VALUE_ATOM(sce_abl),
                                                COX_VAL_PROPAGATE,
                                                GSS_PNT0FIELD);

    if (propagate_set == NULL)
      propagate_set=(struct chain*) gss_chktab (cox_dico,
                                                concatname(SPG_INSNAME,
                                                           VALUE_ATOM(sce_abl)),
                                                COX_VAL_PROPAGATE,
                                                GSS_PNT0FIELD);
      

     /* ###--------------------------------------------------### */
     /* ### if extracted colors set and propagated colors    ### */
     /* ###    set are not disjunctives                      ### */
     /* ###          => dupplicate the terminal superabl,    ### */
     /* ###             & continue the traversal algorithme. ### */
     /* ### else     => dupplicate the terminal abl.         ### */
     /* ###--------------------------------------------------### */

    if(! cst_DisjunctiveSet (propagate_set , extracted_set) )
      {
      if ( PRB_SUPERABL_MODE & PRB_EXPLODING_SUPERABL )
        {
        spg_abl = prb_superabl (VALUE_ATOM (sce_abl), spg_dico, cox_dico);
        }
      else
        {
        spg_abl =(struct chain*) gss_chktab (spg_dico,
                                             VALUE_ATOM (sce_abl),
                                             PRB_BASICABL_DEF,
                                             GSS_PNT0FIELD);
        }

      spg_abl = prb_dupabl (spg_abl, spg_dico);
      spg_abl = prb_createsuperabl (spg_abl, extracted_set, spg_dico, cox_dico);
      }
    else
      {
      spg_abl = sce_abl;
      }
    }

   /* ###--------------------------------------### */
   /* ### if the abl is an expression,         ### */ 
   /* ### duplicate the expression and create  ### */ 
   /* ### the superabl of it's operands with   ### */ 
   /* ### the same list of color for reference.### */ 
   /* ###--------------------------------------### */

  else
    {
    tmp_abl = sce_abl;

    spg_abl = prb_createexpr (OPER(tmp_abl), spg_dico);

    while (tmp_abl = CDR(tmp_abl))
      {
      CDR(spg_abl) = addchain(CDR(spg_abl),
                              (void*)prb_createsuperabl(CAR(tmp_abl),
                                                        extracted_set,
                                                        spg_dico,
                                                        cox_dico));
      }
    }

  return (spg_abl);
  }


/* ###--------------------------------------------------------------### */
/* function     : prb_maksupabl 					*/
/* description  : Make a superabl for all signals of a befig		*/
/* called func  : prb_superabl,cox_simulator				*/
/* ###--------------------------------------------------------------### */

void prb_maksupabl(ptr_lofig, ptr_befig)

struct lofig *ptr_lofig;
struct befig *ptr_befig;
  {
  struct dct_entry  **spg_dico;
  struct dct_entry  **cox_dico;

  struct bemsg       *ptr_bemsg ;
  struct beaux       *ptr_beaux ;
  struct beaux       *ptr_bedly;
  struct beout       *ptr_beout ;
  char               *name;

  struct rusage       rusage1, rusage2;
  double              delta_utime = 0.0;

	/* ###------------------------------------------------------### */
	/*    prb_maksupabl use the USER fields of NODEs.		*/
	/*    NODES may be of QUAD type.				*/
	/* ###------------------------------------------------------### */

  if ( (ptr_befig->TYPE & BEH_NODE_USER)
  || (!(ptr_befig->TYPE & BEH_NODE_QUAD)))
    {
    prb_error(0, NULL);
    }

  else
    {
    SPG_DICTIONNARY = gss_initab();

	/* ###------------------------------------------------------### */
	/*    Record each BASIC ABL in the SPG_DICTIONNARY.		*/
	/* ###------------------------------------------------------### */

    SPG_INSNAME = ptr_befig->NAME;

    ptr_beaux = ptr_befig->BEAUX;
    while (ptr_beaux != NULL)
      {
      name = ptr_beaux->NAME;

      gss_addtab (SPG_DICTIONNARY, name, PRB_BASICABL_DEF, GSS_PNT0FIELD, ptr_beaux->ABL);
      ptr_beaux       = ptr_beaux->NEXT;
      }

    ptr_bedly = ptr_befig->BEDLY;
    while (ptr_bedly != NULL)
      {
      name = ptr_bedly->NAME;

      gss_addtab (SPG_DICTIONNARY, name, PRB_BASICABL_DEF, GSS_PNT0FIELD, ptr_bedly->ABL);
      ptr_bedly       = ptr_bedly->NEXT;
      }

    ptr_bemsg = ptr_befig->BEMSG;
    while (ptr_bemsg != NULL)
      {
      name = ptr_bemsg->LABEL;

      gss_addtab (SPG_DICTIONNARY, name, PRB_BASICABL_DEF, GSS_PNT0FIELD, ptr_bemsg->ABL);

      ptr_bemsg       = ptr_bemsg->NEXT;
      }

    ptr_beout = ptr_befig->BEOUT;
    while (ptr_beout != NULL)
      {
      name = ptr_beout->NAME;

      gss_addtab(SPG_DICTIONNARY,name,PRB_BASICABL_DEF,GSS_PNT0FIELD,ptr_beout->ABL);
      ptr_beout       = ptr_beout->NEXT;
      }

	/* ###------------------------------------------------------### */
	/*    Complete the Cox simulation => SuperAbl, Bdd, Varlist.	*/
	/* ###------------------------------------------------------### */

    GSS_MAXDELTA = 100000;

    getrusage(RUSAGE_SELF,&rusage1);

    cox_simulator(ptr_lofig, ptr_befig);

    getrusage(RUSAGE_SELF,&rusage2);

    delta_utime  =
           (rusage2.ru_utime.tv_sec*1000.0+rusage2.ru_utime.tv_usec/1000.0)
         - (rusage1.ru_utime.tv_sec*1000.0+rusage1.ru_utime.tv_usec/1000.0);

    fprintf(stdout, "TEMPS DE COX SIMULATION\n");
    fprintf(stdout, "  +   DE COX EXPANSION  : %.2f\n",delta_utime);

    ptr_befig->TYPE |= BEH_NODE_USER;
    }
  }
