
/* ###--------------------------------------------------------------### */
/* file		: scg109.h						*/
/* date		: Aug 10 1997						*/
/* version	: v109							*/
/* author	: Nizar ABDALLAH, Pirouz BAZARGAN SABET, Julien DUNOYER	*/
/* contents	: defines and structure definitions used in SCG library	*/
/* ###--------------------------------------------------------------### */

#ifndef SCG_SCGDEF
#define SCG_SCGDEF

	/* ###------------------------------------------------------### */
	/*    basic defines						*/
	/* ###------------------------------------------------------### */

#define SCG_DFN_TRABLOCK	1024	/* size of block		*/
#define SCG_DFN_ENTBLOCK	 256	/* size of block		*/
#define SCG_DFN_WAVBLOCK	 256	/* size of block		*/

#define SCG_TRA_NULL		0x00	/* transcation flag: no action	*/

#define SCG_TRA_DELAFTER	0x01	/* transcation flag: del after	*/
#define SCG_TRA_DELBEFORE	0x02	/* transcation flag: del before	*/

#define SCG_TRA_ENTFIRST	0x04	/* transcation flag: 1 of list	*/

#define SCG_TRA_COLLECTOR	0x08	/* transcation flag: del before	*/

	/* ###------------------------------------------------------### */
	/*    complexe defines						*/
	/* ###------------------------------------------------------### */

#define SCG_TRA_DEFAULT		SCG_TRA_DELAFTER | SCG_TRA_DELBEFORE

	/* ###------------------------------------------------------### */
	/*    structure definitions					*/
	/* ###------------------------------------------------------### */

struct shdul			/* scheduler				*/
  {
  struct shdul   *NEXT     ;	/* next scheduler			*/
  void           *LABEL    ;	/* scheduler's label			*/
  struct shent  **TTABLE   ;	/* time table				*/
  struct shwav  **ITABLE   ;	/* identifier table			*/
  unsigned int    CURTIME  ;	/* current absolute time		*/
  };

struct shtra			/* transaction				*/
  {
  struct shtra   *NEXT     ;	/* next     transaction			*/
  struct shtra   *PREV     ;	/* previous transaction			*/
  struct shtra   *WAVE     ;	/* waveform for the same identifier	*/
  unsigned int    DATE     ;	/* transaction's date			*/
  void           *IDENT    ;	/* main   identifier			*/
  void           *SCDID    ;	/* second identifier			*/
  unsigned int    UFLAG    ;	/* user's flags				*/
  void           *COND     ;	/* generic condition value		*/
  void           *VALU     ;	/* generic driven value			*/
  unsigned short  FLAGS    ;	/* flags				*/
  };

struct shent			/* scheduler entry			*/
  {
  struct shent   *NEXT     ;	/* next entry				*/
  unsigned int    DATE     ;	/* entry's date (high order bits)	*/
  struct shtra   *TRANS [8];	/* list of transactions			*/
  };

struct shwav			/* waveform				*/
  {
  struct shwav   *NEXT     ;	/* next waveform			*/
  void           *IDENT    ;	/* main identifier			*/
  struct shtra   *TRANS    ;	/* list of transactions			*/
  };

	/* ###------------------------------------------------------### */
	/*    global variables						*/
	/* ###------------------------------------------------------### */

extern struct shtra  *SCG_FREE_SHTRA   ;
extern struct shent  *SCG_FREE_SHENT   ;
extern struct shwav  *SCG_FREE_SHWAV   ;

	/* ###------------------------------------------------------### */
	/*    functions							*/
	/* ###------------------------------------------------------### */

extern void          scg_bug      ();
extern int           scg_error    ();
extern void          scg_message  ();

extern void          scg_debug    ();

extern unsigned int  scg_hash     ();
extern struct shtra *scg_addshtra ();
extern struct shwav *scg_addshwav ();
extern struct shent *scg_addshent ();
extern struct shtra *scg_rmventry ();
extern void          scg_insentry ();
extern void          scg_inswave  ();

extern struct shdul *scg_CreateScheduler        ();
extern void          scg_AddTranscation         ();
extern struct shtra *scg_GetCurrentTransactions ();
extern int           scg_GotoNextTime           ();
extern int           scg_GetNextTime            ();
extern void          scg_FreeTransactions       ();
extern unsigned int  scg_GetCurrentTime         ();

#endif
