/*
 * This file is part of the Alliance CAD System
 * Copyright (C) Laboratoire LIP6 - Dpartement ASIM
 * Universite Pierre et Marie Curie
 *
 * Home page          : http://www-asim.lip6.fr/alliance/
 * E-mail support     : mailto:alliance-support@asim.lip6.fr
 *
 * This progam is  free software; you can redistribute it  and/or modify it
 * under the  terms of the GNU  General Public License as  published by the
 * Free Software Foundation;  either version 2 of the License,  or (at your
 * option) any later version.
 *
 * Alliance VLSI  CAD System  is distributed  in the hope  that it  will be
 * useful, but WITHOUT  ANY WARRANTY; without even the  implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 *
 * You should have received a copy  of the GNU General Public License along
 * with the GNU C Library; see the  file COPYING. If not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * Tool    : Logic Synthesis - CMOS Cells Compiler
 * Date    : 1991,92
 * Author  : Nathalie Dictus
 * Modified by Czo <Olivier.Sirol@lip6.fr> 1996,97
 */




#ident "$Id: c4_genicon.c,v 1.2 2000/11/07 16:15:17 czo Exp $"

/************************************************************************/
/* DEA MEMI 93-94                                                   CAO */
/* Eric PLAYE                                                   dea9307 */
/*                                                                      */
/* PROJECT : C4                                                         */
/************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>

#include MUT_H
#include LOG_H
#include BEH_H
#include BHL_H
#include BVL_H
#include ICN_H
#include IAC_H
#include ICU_H

struct IconGate *genIcon ();
static int noNOTdepthExpr ();
struct IconGate *boxIcon ();
struct IconGate *operIcon ();


/****************************************************************/
	/*        GENICON FUNCTION                                      */
/****************************************************************/

struct IconGate *
genIcon (figure, store_it)
     struct befig *figure;
     int *store_it;		/* 1 if the figure's icon have to be stored, 0 else */
{
  struct IconGate *figure_icon;
  struct berin *inputs = figure->BERIN;
  struct beout *output = figure->BEOUT;

  if (output->NEXT)
    {
      fprintf (stderr, "error (genIcon) : actual version can't ");
      fprintf (stderr, "generate multiple outputs cell's icon\n");
      return NULL;
    }

  if (noNOTdepthExpr (output->ABL) > 1)
    figure_icon = boxIcon (figure->NAME,
			   inputs, output->NAME, store_it,
			   (OPER (output->ABL) == NOT) ||
			   (OPER (output->ABL) == NOR) ||
			   (OPER (output->ABL) == NAND) ||
			   (OPER (output->ABL) == NXOR));
  else
    {
      if ((OPER (output->ABL) == NOT) && !ATOM (CADR (output->ABL)))
	{
	  switch (OPER (CADR (output->ABL)))
	    {
	    case OR:
	    case NOR:
	      figure_icon = operIcon (figure->NAME,
				      inputs, output->NAME, store_it, NOR);
	      break;
	    case AND:
	    case NAND:
	      figure_icon = operIcon (figure->NAME,
				      inputs, output->NAME, store_it, NAND);
	      break;
	    case XOR:
	    case NXOR:
	      figure_icon = operIcon (figure->NAME,
				      inputs, output->NAME, store_it, NXOR);
	      break;
	    }
	}
      else
	figure_icon = operIcon (figure->NAME,
				inputs, output->NAME, store_it,
				OPER (output->ABL));
    }

  return figure_icon;
}

/****************************************************************/
	/*        NONOTDEPTHEXPR FUNCTION                               */
	/*                                                              */
	/* RESULT :                                                     */
	/*        return the depth of an abl expression given in        */
	/*        argument, without considering the operator 'not'.     */
	/* IMPLEMENTATION :                                             */
	/*     - if the operator (prefixed notation) is 'NOT',          */
	/*       simply return the depth-without-not of its linked      */
	/*       expression;                                            */
	/*     - else, for each sub-expression, calculate the depth-    */
	/*       without-not and return the greatest, working around    */
	/*       the atomic expression (null depth);                    */
	/*     - in case of non-logical operator or unknown one, print  */
	/*       an error message and return -1 (negative depth).       */
/****************************************************************/

static int
noNOTdepthExpr (abl)
     struct chain *abl;
{
  struct chain *sub_abl;
  int max_depth = 0;
  int depth;


  switch (OPER (abl))
    {
    case NOT:
      return noNOTdepthExpr (CADR (abl));
    case OR:
    case AND:
    case XOR:
    case NOR:
    case NAND:
    case NXOR:
      for (sub_abl = CDR (abl); sub_abl; sub_abl = sub_abl->NEXT)
	{
	  if (ATOM (CAR (sub_abl)))
	    continue;
	  if ((depth = noNOTdepthExpr (CAR (sub_abl))) < 0)
	    return -1;
	  if (depth > max_depth)
	    max_depth = depth;
	}
      return max_depth + 1;
    default:
      fprintf (stderr, "error (noNOTdepthExpr) : operator not in");
      fprintf (stderr, "(NOT, OR, AND, XOR, NOR, NAND, NXOR)\n");
      return -1;
    }
}

/****************************************************************/
	/*        BASIC GENICON'S FUNCTION :                            */
	/*        BOXICON                                               */
/****************************************************************/

struct IconGate *
boxIcon (cell_name, inputs_names, output_name, store_it, negative_polarity)
     char *cell_name;
     struct berin *inputs_names;
     char *output_name;
     int store_it;
     int negative_polarity;
{
  struct IconGate *box_icon;
  struct berin *input;
  long x, y;


  /*
     // Create an empty icon structure, formally :
   */

  x = 200;
  box_icon = addicongate (cell_name,
			  x, 0,	/* model name posisiton   */
			  SAX_CENTER, SAX_CENTER, SAX_HORIZONTAL,
			  x, -100,	/* instance name position */
			  SAX_CENTER, SAX_UP, SAX_HORIZONTAL,
			  x, 100,	/* label position         */
			  SAX_CENTER, SAX_DOWN, SAX_HORIZONTAL);
  if (!box_icon)
    return geticongate (cell_name);

  /*
     // Dispose the input connectors, formally 200 pts far away from
     // the west side of the shape and spaced between them by 200 pts.
   */

  x = -200;
  y = 0;
  for (input = inputs_names; input; input = input->NEXT)
    {
      if (isvdd (input->NAME) || isvss (input->NAME))
	continue;
      y -= 100;
      addiconcon (box_icon, input->NAME,
		  x, y, SAX_WEST, IN,
		  SAX_RIGHT, SAX_DOWN, SAX_HORIZONTAL, x, y - 50);
      addiconline (box_icon, x, y, 0, y);
      y -= 100;
    }

  /*
     // Let trace the shape.
   */

  x = 400;
  {
    addiconline (box_icon, 0, 0, x, 0);
    addiconline (box_icon, x, 0, x, y);
    addiconline (box_icon, x, y, 0, y);
    addiconline (box_icon, 0, y, 0, 0);
  }

  /*
     // Now, 'y' variable contents the effective heigth of shape.
     // Let correct the vertical positions of model name and label.
   */

  ICONGATE_Y_INSNAME (box_icon) += y;
  y /= 2;
  ICONGATE_Y_MODELNAME (box_icon) += y;

  /*
     // Ouput connector is linked to the shape at mid-heigth.
     // When it occurs, a circle is provided as negative polarity symbol.
     // Formally, the connector is placed 200 pts easter from the shape
     // (including the possible negative characteristic).
   */

  if (negative_polarity)
    {
      addiconcircle (box_icon, x, y - 50, 100, 100);
      x += 100;
    }
  x += 200;
  addiconcon (box_icon, output_name,
	      x, y, SAX_EAST, OUT,
	      SAX_LEFT, SAX_DOWN, SAX_HORIZONTAL, x, y - 50);
  addiconline (box_icon, x, y, x - 200, y);

  /*
     // Finishing touch.
   */

  ICONGATE_SCALE (box_icon) = SCALE_X / 10;
  update_icon (box_icon);
  if (store_it)
    saveicon (box_icon);

  return box_icon;
}

/****************************************************************/
	/*        BASIC GENICON'S FUNCTION :                            */
	/*        OPERICON                                              */
/****************************************************************/

struct IconGate *
operIcon (cell_name, inputs_names, output_name, store_it, operator)
     char *cell_name;
     struct berin *inputs_names;
     char *output_name;
     int store_it;
     int operator;
{
  struct IconGate *oper_icon;
  struct berin *input;
  long x, y;
  int inputs_nb_odd;


  x = 200;
  oper_icon = addicongate (cell_name,
			   x, 0,	/* model name posisiton   */
			   SAX_CENTER, SAX_CENTER, SAX_HORIZONTAL,
			   x, -300,	/* instance name position */
			   SAX_CENTER, SAX_UP, SAX_HORIZONTAL,
			   x, 300,	/* label position         */
			   SAX_CENTER, SAX_DOWN, SAX_HORIZONTAL);
  if (!oper_icon)
    return geticongate (cell_name);

  x = -200;
  y = 0;
  inputs_nb_odd = 0;
  for (input = inputs_names; input; input = input->NEXT)
    {
      if (isvdd (input->NAME) || isvss (input->NAME))
	continue;
      y -= 100;
      addiconcon (oper_icon, input->NAME,
		  x, y, SAX_WEST, IN,
		  SAX_RIGHT, SAX_DOWN, SAX_HORIZONTAL, x, y - 50);
      addiconline (oper_icon, x, y, 0, y);
      y -= 100;
      inputs_nb_odd = !inputs_nb_odd;
    }

  y /= 2;
  ICONGATE_Y_INSNAME (oper_icon) += y;
  ICONGATE_Y_MODELNAME (oper_icon) += y;
  ICONGATE_Y_LABEL (oper_icon) += y;

  x = 400;
  if ((operator == NOT) ||
      (operator == NOR) || (operator == NAND) || (operator == NXOR))
    {
      addiconcircle (oper_icon, x, y - 50, 100, 100);
      x += 100;
    }
  x += 200;
  addiconcon (oper_icon, output_name,
	      x, y, SAX_EAST, OUT,
	      SAX_LEFT, SAX_DOWN, SAX_HORIZONTAL, x, y - 50);
  addiconline (oper_icon, x, y, x - 200, y);

  switch (operator)
    {
    case NOT:
      addiconline (oper_icon, 0, y, 100, y);
      addiconline (oper_icon, 100, y - 150, 100, y + 150);
      addiconline (oper_icon, 100, y - 150, 400, y);
      addiconline (oper_icon, 100, y + 150, 400, y);
      break;
    case OR:
    case NOR:
      if (inputs_nb_odd)
	addiconline (oper_icon, 0, y, 50, y);
      else
	{
	  addiconline (oper_icon, 0, y - 100, 40, y - 100);
	  addiconline (oper_icon, 0, y + 100, 40, y + 100);
	}
      addiconline (oper_icon, 0, y + 200, 0, 0);
      addiconarc (oper_icon, -790, y - 425, 850, 850, -281, 562);
      addiconline (oper_icon, 0, y - 200, 0, y * 2);
      addiconline (oper_icon, 0, y + 200, 100, y + 200);
      addiconarc (oper_icon, -225, y - 440, 650, 650, -900, 674);
      addiconarc (oper_icon, -225, y - 200, 650, 650, 900, -674);
      addiconline (oper_icon, 0, y - 200, 100, y - 200);
      break;
    case AND:
    case NAND:
      addiconline (oper_icon, 0, 0, 0, y * 2);
      addiconline (oper_icon, 0, y + 200, 200, y + 200);
      addiconline (oper_icon, 0, y - 200, 200, y - 200);
      addiconarc (oper_icon, 0, y - 200, 400, 400, -900, 1800);
      break;
    case XOR:
    case NXOR:
      if (inputs_nb_odd)
	addiconline (oper_icon, 0, y, 50, y);
      else
	{
	  addiconline (oper_icon, 0, y - 100, 40, y - 100);
	  addiconline (oper_icon, 0, y + 100, 40, y + 100);
	}
      addiconline (oper_icon, 0, y + 200, 0, 0);
      addiconarc (oper_icon, -790, y - 425, 850, 850, -281, 562);
      addiconline (oper_icon, 0, y - 200, 0, y * 2);
      addiconline (oper_icon, 50, y + 200, 100, y + 200);
      addiconarc (oper_icon, -225, y - 440, 650, 650, -900, 674);
      addiconarc (oper_icon, -225, y - 200, 650, 650, 900, -674);
      addiconline (oper_icon, 50, y - 200, 100, y - 200);
      addiconarc (oper_icon, -740, y - 425, 850, 850, -281, 562);
      break;
    default:
      x = 400;
      y *= 2;
      addiconline (oper_icon, 0, 0, x, 0);
      addiconline (oper_icon, x, 0, x, y);
      addiconline (oper_icon, x, y, 0, y);
      addiconline (oper_icon, 0, y, 0, 0);
    }

  ICONGATE_SCALE (oper_icon) = SCALE_X / 10;
  update_icon (oper_icon);
  if (store_it)
    saveicon (oper_icon);

  return oper_icon;
}



