/*
 * This file is part of the Alliance CAD System
 * Copyright (C) Laboratoire LIP6 - Dpartement ASIM
 * Universite Pierre et Marie Curie
 * 
 * Home page          : http://www-asim.lip6.fr/alliance/
 * E-mail support     : mailto:alliance-support@asim.lip6.fr
 * 
 * This progam is  free software; you can redistribute it  and/or modify it
 * under the  terms of the GNU  General Public License as  published by the
 * Free Software Foundation;  either version 2 of the License,  or (at your
 * option) any later version.
 * 
 * Alliance VLSI  CAD System  is distributed  in the hope  that it  will be
 * useful, but WITHOUT  ANY WARRANTY; without even the  implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 * 
 * You should have received a copy  of the GNU General Public License along
 * with the GNU C Library; see the  file COPYING. If not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* ###--------------------------------------------------------------### */
/*                                                                      */
/* file         : genscan.c                                             */
/* date         : JUL 15 1996                                           */
/* author       : GUETTAF  Amar                                         */
/* description  : Scan-Path generator main program .                    */
/*                                                                      */
/* ###--------------------------------------------------------------### */


#include <stdio.h>
#include <string.h>
#include MUT_H 
#include MLO_H
#include PPT_H
#include PAT_H
#include "genscan.h"
#include "utile.h"
#include "environment.h"
#include "scan_path.h"


lofig_list *ptfig;     /* pointer to the global netlist */

int mem_nbr     = 0;      /* number of Dff in the netlist */
int nbr_scan    = 0;      /* number of dffs in the scan chain */

struct chain *scan_ptr = NULL;/* list of the ffs in the scan path */
struct chain *scan_order = NULL;/* list of the ff in the scanpath_order file */
struct chain *mem_ptr = NULL;/* list of memory cells */

char *info_filename = NULL;
char *genpat_filename = NULL;
char *bistgenpat_filename = NULL;
char *scanorder_filename = NULL;


main(argc,argv)
int argc;
char *argv[];
{
    locon_list *ptrc;
    losig_list *ptr_sig, *scan_signal;
    chain_list *ptr, *ptr1, *ptr2, *ptr3;
	
    int i,j;
    int nbr_sig1;         /* number of signals in the input netlist */
    int nbr_gates1;       /* number of gates in the input netlist */

	char *input_file, *output_file;
	char *insname;		
	int comb_circuit = 0; /* true if the circuit doesn't contain register gates */
        int Fixed_Scout=0;
	int Extract_Scan=0;
	int _exchange = 0;
        char *options; 

	alliancebanner("GenScan","1.2.2","ScanPath generator","1995",ALCVER);

	if ((argc < 3) || (argc >6)) 
		Usage(argv[0]);
	else
	{
		input_file = argv[1];
		output_file = argv[2];

		for (i = 3; i < argc ; i++)
		{
			options = argv[i];
			switch (*++options)
			{
				case 'n':
					if (*++options =='\0')
						Fixed_Scout = 1;
					else 
						Usage(argv[0]);
				break;
				case 's' :
                                                if (*++options =='\0')
                                                {
                                                        if ((argv[++i] != NULL) && (*argv[i] != '-'))
                                                        {
                                                                scanorder_filename = argv[i];
                                                        }
                                                        else Usage(argv[0]);

                                                }
                                                else
                                                        Usage(argv[0]);
                                        break;
				case 'e' :
					if (*++options =='\0')
						Extract_Scan = 1;
					else 
						Usage(argv[0]);
				break;

				default :
					Usage(argv[0]);
				break;
			}


		}
	}
	
	
	mbkenv();
	if((ptfig = getlofig(input_file,'A'))==NULL)
	{ 
		fprintf(stderr,"ERROR : unable to open figure %s \n",input_file);
		exit(1);
	}
	
			
	get_environment_variables();
	
	get_special_signals(ptfig);

	max_index = get_max_index(ptfig);

	mem_ptr = get_mem_cells(ptfig);

	if ( mem_ptr == NULL) comb_circuit = 1;

	if ( Extract_Scan == 0)
	{
		insert_scan(ptfig,comb_circuit,scanorder_filename,Fixed_Scout);
        	ptfig->NAME = namealloc(output_file);
        	fprintf(stdout,"\n");
        	fprintf(stdout," saving figure file %s ......\n\n",ptfig->NAME);
        	savelofig(ptfig);
	}
	else 
		get_scan(ptfig,scan_in,scan_out);

exit(0);
}




