/*
 *               ,,,
 *              (o o)
 * ####=====oOO--(_)--OOO=========================================####
 * ##                                                               ##
 * ##   This file is part of FPT : A Formal Proof Toolkit           ##
 * ##   (C) 1996, MASI, CAO-VLSI Team                               ##
 * ##                                                               ##
 * ##   File            :  fpt_befig.c                              ##
 * ##   Author          :  Olivier SIROL                            ##
 * ##   Date            :  Jun 1996                                 ##
 * ##                                                               ##
 * ##   E-mail support  :  cao-vlsi@masi.ibp.fr                     ##
 * ##                                                               ##
 * ####===========================================================####
 *
 *
 */

#ident "$Id: fpt_befig.c,v 1.30 1997/05/23 14:27:36 sirol Exp $"


/*
 * ####===========================================================####
 * ##                                                               ##
 * ##                         Include Files                         ##
 * ##                                                               ##
 * ####===========================================================####
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include MUT_H
#include AUT_H
#include ABL_H
#include BDD_H
#include LOG_H
#include BEH_H
#include BVL_H

#include "fpt_public.h"



/*
 * ####===========================================================####
 * ##                                                               ##
 * ##                           Constants                           ##
 * ##                                                               ##
 * ####===========================================================####
 */
/*
 * ####===========================================================####
 * ##                                                               ##
 * ##                             Types                             ##
 * ##                                                               ##
 * ####===========================================================####
 */
/*
 * ####===========================================================####
 * ##                                                               ##
 * ##                           Variables                           ##
 * ##                                                               ##
 * ####===========================================================####
 */

static chain_list *BEFIG_HeadName = (chain_list *) 0;
static befig_list *BEFIG_BehFigure = (befig_list *) 0;
static char *BEFIG_DelayedName = (char *) 0;
static authtable *BEFIG_RenameHashTable = (authtable *) 0;
static authtable *BEFIG_AuxHashTable = (authtable *) 0;
static authtable *BEFIG_AuxNameHashTable = (authtable *) 0;


/*
 * ####===========================================================####
 * ##                                                               ##
 * ##                           Functions                           ##
 * ##                                                               ##
 * ####===========================================================####
 */


static long FptGetAblNumberAtom ();
static long FptGetExprNumberAtom ();


static ablexpr *FptSumBiAbl( BiAbl )

  biabl_list *BiAbl;
{
  ablexpr *Sum;
  ablexpr *Expr;

  Sum = createabloper( ABL_OR );

  while ( BiAbl != (biabl_list *)0 )
  {
    Expr = createablbinexpr( ABL_AND, 
                             dupablexpr( BiAbl->VALABL ),
                             dupablexpr( BiAbl->CNDABL ) );
    addablhexpr( Sum, Expr );

    BiAbl = BiAbl->NEXT;
  }

  if ( ABL_CDR( Sum ) == (ablexpr *)0 )
  {
    freeablexpr( Sum );
    Sum = createablatom( "'0'" );
  }
  else
  if ( ABL_CDDR( Sum ) == (ablexpr *)0 )
  {
    Expr = Sum;
    Sum  = ABL_CADR( Sum );
    ABL_CADR( Expr ) = (ablexpr *)0;
    freeablexpr( Expr );
  }

  return( Sum );
}


int
loadfptbefig (ptbefig, InputFileName, FigureName)
     struct befig **ptbefig;
     char *InputFileName;
     char **FigureName;

{
  char *Name;
  int res;
  static FilesLoaded =1;
  bebus_list *BehBus;
  bebux_list *BehBux;


  printf ("===> Compiling '%s' ...\n", InputFileName);

  *ptbefig = vhdlloadbefig ((befig_list *) 0, InputFileName, 2);

  if ((*ptbefig)->ERRFLG)
    displayfpterror ("Befig %s is bad!\n", InputFileName);

  (*FigureName) = (*ptbefig)->NAME;
  
  Name = autallocblock (strlen ((*ptbefig)->NAME) + 7);
  sprintf (Name, "%s(fig%1d)", (*ptbefig)->NAME, FilesLoaded++);
  (*ptbefig)->NAME = namealloc (Name);
  autfreeblock (Name);


  /* Gestion des bus : on les transforme en multiplex */

  if ((*ptbefig)->BEBUS != NULL)
    displayfptwarning ("figure '%s' BEBUS are converted to multplexors.\n", (*ptbefig)->NAME);
  if ((*ptbefig)->BEBUX != NULL)
    displayfptwarning ("figure '%s' BEBUX are converted to multplexors.\n", (*ptbefig)->NAME);

  for ( BehBus  = (*ptbefig)->BEBUS;
        BehBus != (bebus_list *)0;
        BehBus  = BehBus->NEXT )
  {
    (*ptbefig)->BEOUT = beh_addbeout( (*ptbefig)->BEOUT, BehBus->NAME, 
                                     FptSumBiAbl( BehBus->BIABL ), (void *)0 );
  }

  beh_frebebus( (*ptbefig)->BEBUS );
  (*ptbefig)->BEBUS = (bebus_list *)0;

  for ( BehBux  = (*ptbefig)->BEBUX;
        BehBux != (bebux_list *)0;
        BehBux  = BehBux->NEXT )
  {
    (*ptbefig)->BEAUX = beh_addbeaux( (*ptbefig)->BEAUX, BehBux->NAME,
                                     FptSumBiAbl( BehBux->BIABL ), (void *)0 );
  }

  beh_frebebux( (*ptbefig)->BEBUX );
  (*ptbefig)->BEBUX = (bebux_list *)0;

  /* fin Gestion des bus  */

  /* Plante si ABL loop */
  res = FptBehStatistics (*ptbefig);

  displayfptverbose ("Befig '%s' (%s.vbe) loaded.\n\n", (*ptbefig)->NAME, InputFileName);

  return (res);
}


/*================================================================####
 * Fpt Beh Abl Number Atom
 *     
 */


static long
FptGetAblNumberAtom (Name, Expr)

     char *Name;
     chain_list *Expr;
{
  authelem *HashElement;
  long Count;

  HashElement = searchauthelem (BEFIG_AuxNameHashTable, Name);

  if (HashElement != (authelem *) 0)
    {
      return (HashElement->VALUE);
    }

  Count = FptGetExprNumberAtom (Expr);

  addauthelem (BEFIG_AuxNameHashTable, Name, Count);

  return (Count);
}


/*================================================================####
 * Fpt Beh Expr Number Atom
 *     
 */


static long
FptGetExprNumberAtom (Expr)

     chain_list *Expr;
{
  authelem *HashElement;
  char *Atom;
  long Count;

  if (ABL_ATOM (Expr))
    {
      Atom = ABL_ATOM_VALUE (Expr);

      HashElement = searchauthelem (BEFIG_AuxHashTable, Atom);

      if (HashElement != (authelem *) 0)
	{
	  Expr = (chain_list *) (HashElement->VALUE);
	  Count = FptGetAblNumberAtom (Atom, Expr);

	  return (Count);
	}

      return (1);
    }

  Count = 0;

  while ((Expr = ABL_CDR (Expr)) != (chain_list *) 0)
    {
      Count += FptGetExprNumberAtom (ABL_CAR (Expr));
    }

  return (Count);
}

/*================================================================####
 * Fpt Beh Statistics
 *     
 */


int
FptBehStatistics (BehFigure)

     befig_list *BehFigure;
{
  berin_list *BehIn;
  beout_list *BehOut;
  beaux_list *BehAux;
  beaux_list *BehDly;
  bereg_list *BehReg;

  /*  biabl_list *BiAbl;
   * long Count = 0;
   * long Generation = 0;
   */

  long Transition = 0;
  int NumberIn = 0;
  int NumberReg = 0;
  int NumberAux = 0;
  int NumberDly = 0;
  int NumberOut = 0;

  for (BehReg = BehFigure->BEREG;
       BehReg != (bereg_list *) 0;
       BehReg = BehReg->NEXT)
    {
      NumberReg++;
    }

  for (BehAux = BehFigure->BEAUX;
       BehAux != (beaux_list *) 0;
       BehAux = BehAux->NEXT)
    {
      NumberAux++;
    }

  for (BehDly = BehFigure->BEDLY;
       BehDly != (beaux_list *) 0;
       BehDly = BehDly->NEXT)
    {
      NumberDly++;
    }
  
  if (NumberDly > 1)
	  displayfpterror ("'%s' contains multiple delayed signals, this is not handled yet.\n", BehFigure->NAME );
  
  for (BehIn = BehFigure->BERIN;
       BehIn != (berin_list *) 0;
       BehIn = BehIn->NEXT)
    {
      NumberIn++;
    }
  
      NumberIn = NumberIn - NumberReg - NumberAux - NumberDly;
      
  for (BehOut = BehFigure->BEOUT;
       BehOut != (beout_list *) 0;
       BehOut  = BehOut->NEXT)
    {
      NumberOut++;
    }
  


  /* Plante si ABL loop */
  /*
  BEFIG_AuxHashTable = createauthtable (1000);
  BEFIG_AuxNameHashTable = createauthtable (1000);

  for (BehAux = BehFigure->BEAUX;
       BehAux != (beaux_list *) 0;
       BehAux = BehAux->NEXT)
    {
      addauthelem (BEFIG_AuxHashTable, BehAux->NAME, BehAux->ABL);
    }

  for (BehAux = BehFigure->BEAUX;
       BehAux != (beaux_list *) 0;
       BehAux = BehAux->NEXT)
    {
      FptGetAblNumberAtom (BehAux->NAME, BehAux->ABL);
    }

  for (BehOut = BehFigure->BEOUT;
       BehOut != (beout_list *) 0;
       BehOut = BehOut->NEXT)
    {
      Count = FptGetAblNumberAtom (BehOut->NAME, BehOut->ABL);

      Generation += Count;
    }

  for (BehReg = BehFigure->BEREG;
       BehReg != (bereg_list *) 0;
       BehReg = BehReg->NEXT)
    {
      for (BiAbl = BehReg->BIABL;
	   BiAbl != (biabl_list *) 0;
	   BiAbl = BiAbl->NEXT)
	{
	  if (BiAbl->VALABL != (chain_list *) 0)
	    {
	      Count = FptGetAblNumberAtom (BehReg->NAME, BiAbl->VALABL);

	      Transition += Count;
	    }
	}
    }

  destroyauthtable (BEFIG_AuxHashTable);
  destroyauthtable (BEFIG_AuxNameHashTable);
  */


  displayfptverbose ("     Stats :\n");
  displayfptverbose ("          Fig type : %s\n", ((Transition == 0) ? "Combinatory" : "Sequential"));
  displayfptverbose ("            Inputs : %d\n", NumberIn);
  displayfptverbose ("         Registers : %d\n", NumberReg);
  displayfptverbose ("           Outputs : %d\n", NumberOut);

  /*  displayfptverbose ("   Generation func : %ld\n", Generation);
  displayfptverbose ("   Transition func : %ld\n", Transition);
  return (Transition == 0);
  Avoir des reg == avoir des fonc de trans et donc etre sequentiel?
  */

  return (NumberReg == 0);
}




/*================================================================####
 * Fpt Abl 2 Bdd
 *     
 */

static bddnode *
FptExpr2BddNode (Expr)

     chain_list *Expr;
{
  bddnode *BddNode;
  bddnode *BddFirst;
  char *AtomValue;
  beaux_list *ScanAux;
  chain_list *ScanChain;
  beout_list *ScanOut;
  long Oper;
  int Negative;

  if (Expr == (chain_list *) 0)
    {
      displayfpterror ("abl2bdd : null expression.\n");
    }

  if (ABL_ATOM (Expr))
    {
      AtomValue = ABL_ATOM_VALUE (Expr);

      if (AtomValue == ABL_ATOM_NAME_ONE)
	{
	  return (BddLocalSystem->ONE);
	}
/*
 *      if ((AtomValue == ABL_ATOM_NAME_ZERO) ||
 *        (AtomValue == ABL_ATOM_NAME_DC))
 *      {
 *        return (BddLocalSystem->ZERO);
 *      }
 */

      if (AtomValue == ABL_ATOM_NAME_ZERO)
	{
	  return (BddLocalSystem->ZERO);
	}

      if (AtomValue == ABL_ATOM_NAME_DC)
	{
	  displayfpterror ("Found atom whith 'Dont Care' value, this is not handled yet.\n");
	}


      BddNode = searchbddcircuitin ((bddcircuit *) 0, AtomValue);

      if (BddNode != (bddnode *) 0)
	{
	  return (incbddrefext (BddNode));
	}

      for (ScanAux = BEFIG_BehFigure->BEAUX;
	   ScanAux != (beaux_list *) 0;
	   ScanAux = ScanAux->NEXT)
	{
	  if (ScanAux->NAME == AtomValue)
	    break;
	}

      if (ScanAux == (beaux_list *) 0)
	{
	  for (ScanOut = BEFIG_BehFigure->BEOUT;
	       ScanOut != (beout_list *) 0;
	       ScanOut = ScanOut->NEXT)
	    {
	      if (ScanOut->NAME == AtomValue)
		break;
	    }

	  if (ScanOut == (beout_list *) 0)
	    {
	      displayfpterror ("abl2bdd : unknown atom %s !\n", AtomValue);
	    }
	}

      for (ScanChain = BEFIG_HeadName;
	   ScanChain != (chain_list *) 0;
	   ScanChain = ScanChain->NEXT)
	{
	  if (ScanChain->DATA == AtomValue)
	    {
	      displayfpterror ("abl2bdd : combinatory loop on %s !\n", AtomValue);
	    }
	}
      if (ScanAux != (beaux_list *) 0)
	{
	  if (ScanAux->NODE == (void *) 0)
	    {
	      BEFIG_HeadName = addchain (BEFIG_HeadName, AtomValue);
	      ScanAux->NODE = FptExpr2BddNode (ScanAux->ABL);
	      BEFIG_HeadName = delchain (BEFIG_HeadName, BEFIG_HeadName);
	    }
	  BddNode = ScanAux->NODE;
	}
      else
	{
	  if (ScanOut->NODE == (void *) 0)
	    {
	      BEFIG_HeadName = addchain (BEFIG_HeadName, AtomValue);
	      ScanOut->NODE = FptExpr2BddNode (ScanOut->ABL);
	      BEFIG_HeadName = delchain (BEFIG_HeadName, BEFIG_HeadName);
	    }

	  BddNode = ScanOut->NODE;
	}
      return (incbddrefext (BddNode));
    }

  Oper = ABL_OPER (Expr);
  Expr = ABL_CDR (Expr);

  if (Oper == ABL_NOT)
    {
      if (ABL_CDR (Expr) != (chain_list *) 0)
	{
	  displayfpterror ("abl2bdd : illegal use of operator %d\n", Oper);
	}

      BddFirst = FptExpr2BddNode (ABL_CAR (Expr));
      BddNode = applybddnodenot ((bddsystem *) 0,
				 decbddrefext (BddFirst));

      return (BddNode);
    }

  if (Oper == ABL_STABLE)
    {
      Expr = ABL_CAR (Expr);

      if (!ABL_ATOM (Expr))
	{
	  displayfpterror ("abl2bdd : illegal use of STABLE operator\n");
	}

      AtomValue = ABL_ATOM_VALUE (Expr);
      BddFirst = searchbddcircuitin ((bddcircuit *) 0, AtomValue);

      if (BddFirst == (bddnode *) 0)
	{
	  displayfpterror ("abl2bdd : unknown atom %s !\n", AtomValue);
	}

      BEFIG_DelayedName = autresizeblock (BEFIG_DelayedName, 0, strlen (AtomValue) + 9);
      sprintf (BEFIG_DelayedName, "%s'delayed", AtomValue);
      AtomValue = namealloc (BEFIG_DelayedName);

      BddNode = searchbddcircuitin ((bddcircuit *) 0, AtomValue);

      if (BddNode == (bddnode *) 0)
	{
	  displayfpterror ("abl2bdd : unknown atom %s !\n", AtomValue);
	}

      BddNode = applybddnode ((bddsystem *) 0, ABL_NXOR, BddNode, BddFirst);

      return (BddNode);
    }


  if ((isablunaryoper (Oper)) ||
      (ABL_CDR (Expr) == (chain_list *) 0))
    {
      displayfpterror ("abl2bdd : illegal use of operator %d\n", Oper);
    }

  if ((getabloperpolar (Oper) == ABL_POLAR_POSITIVE) ||
      (ABL_CDDR (Expr) == (chain_list *) 0))
    {
      Negative = 0;
    }
  else
    {
      Negative = 1;
      Oper = getablopernot (Oper);
    }

  BddFirst = FptExpr2BddNode (ABL_CAR (Expr));

  while ((Expr = ABL_CDR (Expr)) != (chain_list *) 0)
    {
      BddNode = FptExpr2BddNode (ABL_CAR (Expr));
      BddFirst = applybddnode ((bddsystem *) 0, Oper,
			       decbddrefext (BddFirst),
			       decbddrefext (BddNode));
    }

  if (Negative)
    {
      BddFirst = applybddnodenot ((bddsystem *) 0,
				  decbddrefext (BddFirst));
    }

  return (BddFirst);
}




/*================================================================####
 * Beh Abl 2 Bdd
 *     
 */

static bddnode *
FptAbl2BddNode (Name, Expr)

     char *Name;
     chain_list *Expr;
{
  bddnode *BddNode;

  if (Name != (char *) 0)
    {
      BEFIG_HeadName = addchain (BEFIG_HeadName, Name);
    }

  BddNode = FptExpr2BddNode (Expr);

  if (Name != (char *) 0)
    {
      BEFIG_HeadName = delchain (BEFIG_HeadName, BEFIG_HeadName);

      BddNode = addbddcircuitout ((bddcircuit *) 0, Name, BddNode);
    }

  return (BddNode);
}


/*================================================================####
 * createfptbdd
 *     
 */

static void
createfptbdd (BehFigure)

     befig_list *BehFigure;
{
  beaux_list *BehAux;
  beout_list *BehOut;
  berin_list *BehRin;
  bebus_list *BehBus;
  bereg_list *BehReg;
  biabl_list *BiAbl;
  binode_list *BiNode;

  for (BehRin = BehFigure->BERIN;
       BehRin != (berin_list *) 0;
       BehRin = BehRin->NEXT)
    {
      addbddcircuitin ((bddcircuit *) 0, BehRin->NAME, 0, BDD_IN_MODE_FIRST);
    }

  BEFIG_BehFigure = BehFigure;
  BEFIG_HeadName = (chain_list *) 0;

/*  initablatomname (); */

  /* for (BehDly = BehFigure->BEDLY;
       BehDly != (beaux_list *) 0;
       BehDly = BehDly->NEXT)
    {
      BehDly->NODE = FptAbl2BddNode (BehDly->NAME, BehDly->ABL);
    } */

  for (BehAux = BehFigure->BEAUX;
       BehAux != (beaux_list *) 0;
       BehAux = BehAux->NEXT)
    {
      BehAux->NODE = FptAbl2BddNode (BehAux->NAME, BehAux->ABL);
    }

  for (BehOut = BehFigure->BEOUT;
       BehOut != (beout_list *) 0;
       BehOut = BehOut->NEXT)
    {
      BehOut->NODE = FptAbl2BddNode (BehOut->NAME, BehOut->ABL);
    }


  for (BehBus = BehFigure->BEBUS;
       BehBus != (bebus_list *) 0;
       BehBus = BehBus->NEXT)
    {
      BiAbl = BehBus->BIABL;
      BiNode = BehBus->BINODE;

      while (BiAbl != (biabl_list *) 0)
	{
	  BiNode->CNDNODE = FptAbl2BddNode ((char *) 0, BiAbl->CNDABL);
	  BiNode->VALNODE = FptAbl2BddNode (BehBus->NAME, BiAbl->VALABL);

	  BiAbl = BiAbl->NEXT;
	  BiNode = BiNode->NEXT;
	}
    }

  for (BehReg = BehFigure->BEREG;
       BehReg != (bereg_list *) 0;
       BehReg = BehReg->NEXT)
    {
      BiAbl = BehReg->BIABL;
      BiNode = BehReg->BINODE;

      while (BiAbl != (biabl_list *) 0)
	{
	  BiNode->CNDNODE = FptAbl2BddNode ((char *) 0, BiAbl->CNDABL);
	  BiNode->VALNODE = FptAbl2BddNode ((char *) 0, BiAbl->VALABL);

	  BiAbl = BiAbl->NEXT;
	  BiNode = BiNode->NEXT;
	}
    }

  for (BehAux = BehFigure->BEAUX;
       BehAux != (beaux_list *) 0;
       BehAux = BehAux->NEXT)
    {
      decbddrefext (BehAux->NODE);
      delbddcircuitout ((bddcircuit *) 0, BehAux->NAME);
    }

/*
   for ( BehReg  = BehFigure->BEREG;
   BehReg != (bereg_list *)0;
   BehReg  = BehReg->NEXT )
   {
   delbddcircuitout( (bddcircuit *)0, BehReg->NAME );
   }
 */

  beh_frebeaux (BehFigure->BEAUX);
  BehFigure->BEAUX = (beaux_list *) 0;
}



/*================================================================####
 * createfptcircuit
 *     
 */


bddcircuit *
createfptcircuit (BehFigure1, BehFigure2, BddSystem)

     befig_list *BehFigure1;
     befig_list *BehFigure2;
     bddsystem *BddSystem;
{
  bddcircuit *BddCircuit;
  long NumberInput;
  long NumberOutput;
  beaux_list *BehAux;
  berin_list *BehRin;
  beout_list *BehOut;
  bereg_list *BehReg;
  bebus_list *BehBus;

  printf ("===> Making Bdds, removing auxiliary signals.\n");

  NumberInput = NumberOutput = 1;

  for (BehAux = BehFigure1->BEAUX;
       BehAux != (beaux_list *) 0;
       BehAux = BehAux->NEXT)
    {
      BehFigure1->BERIN = beh_rmvberin (BehFigure1->BERIN, BehAux->NAME);

      NumberOutput++;
    }

  for (BehAux = BehFigure2->BEAUX;
       BehAux != (beaux_list *) 0;
       BehAux = BehAux->NEXT)
    {
      BehFigure2->BERIN = beh_rmvberin (BehFigure2->BERIN, BehAux->NAME);

      NumberOutput++;
    }

  for (BehRin = BehFigure1->BERIN;
       BehRin != (berin_list *) 0;
       BehRin = BehRin->NEXT)
    {
      NumberInput++;
    }

  for (BehOut = BehFigure1->BEOUT;
       BehOut != (beout_list *) 0;
       BehOut = BehOut->NEXT)
    {
      NumberOutput++;
    }

  for (BehReg = BehFigure1->BEREG;
       BehReg != (bereg_list *) 0;
       BehReg = BehReg->NEXT)
    {
      NumberOutput++;
    }

  for (BehBus = BehFigure1->BEBUS;
       BehBus != (bebus_list *) 0;
       BehBus = BehBus->NEXT)
    {
      NumberOutput++;
    }

  BddCircuit = createbddcircuit ("_formal_proof_", NumberInput,
				 NumberOutput, BddSystem);

  BehFigure2->CIRCUI = BehFigure1->CIRCUI = BddCircuit;

  createfptbdd (BehFigure1);
  createfptbdd (BehFigure2);

  displayfptverbose ("Bdd system has %ld bdd nodes. Reordering ...\n", BddSystem->NUMBER_NODE);
  reorderbddsystemwindow( (bddsystem *)0 );

  printf ("===> Bdd system created (%ld bdd nodes).\n", BddSystem->NUMBER_NODE);
  return (BddCircuit);
}




/*================================================================####
 * fptRenameAtom
 *     
 */


static void
fptRenameAtom (Expr)

     chain_list *Expr;
{
  authelem *Element;
  char *AtomValue;

  if (ABL_ATOM (Expr))
    {
      AtomValue = ABL_ATOM_VALUE (Expr);
      Element = searchauthelem (BEFIG_RenameHashTable, AtomValue);

      if ((Element != (authelem *) 0) &&
	  (Element->VALUE != 0))
	{
	  ABL_ATOM_VALUE (Expr) = (char *) (Element->VALUE);
	}
    }
  else
    {
      while ((Expr = ABL_CDR (Expr)) != (chain_list *) 0)
	{
	  fptRenameAtom (ABL_CAR (Expr));
	}
    }
}

/*================================================================####
 * fptRenameRegisterintelligent
 *     
 */


long
fptRenameRegisterintelligent (BehFigure1, BehFigure2)

     befig_list *BehFigure1;
     befig_list *BehFigure2;
{
  befig_list *BehFigure;
  berin_list *BehRin;
  beout_list *BehOut;
  beaux_list *BehAux;
  bebux_list *BehBux;
  bebus_list *BehBus;
  bereg_list *BehReg;
  biabl_list *BiAbl;
  authelem *Element;
  char Buffer[128];
  long NumberReg1;
  long NumberReg2;
  int Found;

  NumberReg1 = 0;

  for (BehReg = BehFigure1->BEREG;
       BehReg != (bereg_list *) 0;
       BehReg = BehReg->NEXT)
    {
      NumberReg1++;
    }

  NumberReg2 = 0;

  for (BehReg = BehFigure2->BEREG;
       BehReg != (bereg_list *) 0;
       BehReg = BehReg->NEXT)
    {
      NumberReg2++;
    }

  if (NumberReg2 < NumberReg1)
    {
      BehFigure = BehFigure1;
      BehFigure1 = BehFigure2;
      BehFigure2 = BehFigure;
    }

  BEFIG_RenameHashTable = createauthtable (NumberReg1 << 1);
  Found = 0;

  for (BehReg = BehFigure1->BEREG;
       BehReg != (bereg_list *) 0;
       BehReg = BehReg->NEXT)
    {
      addauthelem (BEFIG_RenameHashTable, BehReg->NAME, 0);
    }

  for (BehReg = BehFigure2->BEREG;
       BehReg != (bereg_list *) 0;
       BehReg = BehReg->NEXT)
    {
      Element = searchauthelem (BEFIG_RenameHashTable, BehReg->NAME);

      if ((Element != (authelem *) 0) &&
	  (Element->VALUE == 0))
	{
	  sprintf (Buffer, "2.%s", BehReg->NAME);

	  BehReg->NAME = namealloc (Buffer);
	  Element->VALUE = (long) BehReg->NAME;

	  Found = 1;
	}
    }

  if (Found)
    {
      for (BehRin = BehFigure2->BERIN;
	   BehRin != (berin_list *) 0;
	   BehRin = BehRin->NEXT)
	{
	  Element = searchauthelem (BEFIG_RenameHashTable, BehRin->NAME);

	  if ((Element != (authelem *) 0) &&
	      (Element->VALUE != 0))
	    {
	      BehRin->NAME = (char *) Element->VALUE;
	    }
	}

      for (BehOut = BehFigure2->BEOUT;
	   BehOut != (beout_list *) 0;
	   BehOut = BehOut->NEXT)
	{
	  fptRenameAtom (BehOut->ABL);
	}

      for (BehBus = BehFigure2->BEBUS;
	   BehBus != (bebus_list *) 0;
	   BehBus = BehBus->NEXT)
	{
	  BiAbl = BehBus->BIABL;

	  for (BiAbl = BehBus->BIABL;
	       BiAbl != (biabl_list *) 0;
	       BiAbl = BiAbl->NEXT)
	    {
	      if (BiAbl->VALABL != (chain_list *) 0)
		{
		  fptRenameAtom (BiAbl->VALABL);
		}
	    }
	}

      for (BehBux = BehFigure2->BEBUX;
	   BehBux != (bebux_list *) 0;
	   BehBux = BehBux->NEXT)
	{
	  BiAbl = BehBux->BIABL;

	  for (BiAbl = BehBux->BIABL;
	       BiAbl != (biabl_list *) 0;
	       BiAbl = BiAbl->NEXT)
	    {
	      if (BiAbl->VALABL != (chain_list *) 0)
		{
		  fptRenameAtom (BiAbl->VALABL);
		}
	    }
	}

      for (BehReg = BehFigure2->BEREG;
	   BehReg != (bereg_list *) 0;
	   BehReg = BehReg->NEXT)
	{
	  BiAbl = BehReg->BIABL;

	  for (BiAbl = BehReg->BIABL;
	       BiAbl != (biabl_list *) 0;
	       BiAbl = BiAbl->NEXT)
	    {
	      if (BiAbl->VALABL != (chain_list *) 0)
		{
		  fptRenameAtom (BiAbl->VALABL);
		}
	    }
	}

      for (BehAux = BehFigure2->BEAUX;
	   BehAux != (beaux_list *) 0;
	   BehAux = BehAux->NEXT)
	{
	  fptRenameAtom (BehAux->ABL);
	}
    }

  destroyauthtable (BEFIG_RenameHashTable);

  return (NumberReg1 + NumberReg2);
}






/*================================================================####
 * renamefptregister
 *     
 */


long
renamefptregister (BehFigure1, BehFigure2)

     befig_list *BehFigure1;
     befig_list *BehFigure2;
{
  berin_list *BehRin;
  beout_list *BehOut;
  beaux_list *BehAux;
  bebux_list *BehBux;
  bebus_list *BehBus;
  bereg_list *BehReg;
  biabl_list *BiAbl;
  authelem *Element;
  long NumberReg1;
  long NumberReg2;
  char *tmpstr;
  char *oldname;

  NumberReg1 = 0;

  for (BehReg = BehFigure1->BEREG;
       BehReg != (bereg_list *) 0;
       BehReg = BehReg->NEXT)
    {
      NumberReg1++;
    }

  NumberReg2 = 0;

  for (BehReg = BehFigure2->BEREG;
       BehReg != (bereg_list *) 0;
       BehReg = BehReg->NEXT)
    {
      NumberReg2++;
    }

  if (NumberReg2 > NumberReg1)
    {
      displayfptwarning ("You should swap input files : '%s' contains more registers than '%s'.\n", BehFigure2->NAME, BehFigure1->NAME);
    }

  BEFIG_RenameHashTable = createauthtable (NumberReg2 << 1);

  for (BehReg = BehFigure2->BEREG;
       BehReg != (bereg_list *) 0;
       BehReg = BehReg->NEXT)
    {

      oldname = namealloc (BehReg->NAME);

      tmpstr = mbkalloc (strlen (BehReg->NAME) + 5 + 1);
      strcpy (tmpstr, "fig2_");
      strcat (tmpstr, BehReg->NAME);
      BehReg->NAME = namealloc (tmpstr);
      mbkfree (tmpstr);

      addauthelem (BEFIG_RenameHashTable, oldname, (long) BehReg->NAME);

    }

  for (BehRin = BehFigure2->BERIN;
       BehRin != (berin_list *) 0;
       BehRin = BehRin->NEXT)
    {
      Element = searchauthelem (BEFIG_RenameHashTable, BehRin->NAME);

      if ((Element != (authelem *) 0) &&
	  (Element->VALUE != 0))
	{
	  BehRin->NAME = (char *) Element->VALUE;
	}
    }

  for (BehOut = BehFigure2->BEOUT;
       BehOut != (beout_list *) 0;
       BehOut = BehOut->NEXT)
    {
      fptRenameAtom (BehOut->ABL);
    }

  for (BehBus = BehFigure2->BEBUS;
       BehBus != (bebus_list *) 0;
       BehBus = BehBus->NEXT)
    {
      BiAbl = BehBus->BIABL;

      for (BiAbl = BehBus->BIABL;
	   BiAbl != (biabl_list *) 0;
	   BiAbl = BiAbl->NEXT)
	{
	  if (BiAbl->VALABL != (chain_list *) 0)
	    {
	      fptRenameAtom (BiAbl->VALABL);
	    }
	}
    }

  for (BehBux = BehFigure2->BEBUX;
       BehBux != (bebux_list *) 0;
       BehBux = BehBux->NEXT)
    {
      BiAbl = BehBux->BIABL;

      for (BiAbl = BehBux->BIABL;
	   BiAbl != (biabl_list *) 0;
	   BiAbl = BiAbl->NEXT)
	{
	  if (BiAbl->VALABL != (chain_list *) 0)
	    {
	      fptRenameAtom (BiAbl->VALABL);
	    }
	}
    }

  for (BehReg = BehFigure2->BEREG;
       BehReg != (bereg_list *) 0;
       BehReg = BehReg->NEXT)
    {
      BiAbl = BehReg->BIABL;

      for (BiAbl = BehReg->BIABL;
	   BiAbl != (biabl_list *) 0;
	   BiAbl = BiAbl->NEXT)
	{
	  if (BiAbl->VALABL != (chain_list *) 0)
	    {
	      fptRenameAtom (BiAbl->VALABL);
	    }
	}
    }

  for (BehAux = BehFigure2->BEAUX;
       BehAux != (beaux_list *) 0;
       BehAux = BehAux->NEXT)
    {
      fptRenameAtom (BehAux->ABL);
    }

  destroyauthtable (BEFIG_RenameHashTable);

  printf ("===> Renamed %ld registers in '%s' whith prefix 'fig2_'.\n",
	  NumberReg2, BehFigure2->NAME);

  return (NumberReg1 + NumberReg2);


}
