/*
 *               ,,,
 *              (o o)
 * ####=====oOO--(_)--OOO=========================================####
 * ##                                                               ##
 * ##   This file is part of FPT : A Formal Proof Toolkit           ##
 * ##   (C) 1996, MASI, CAO-VLSI Team                               ##
 * ##                                                               ##
 * ##   File            :  fpt_error.c                              ##
 * ##   Author          :  Olivier SIROL                            ##
 * ##   Date            :  Jun 1996                                 ##
 * ##                                                               ##
 * ##   E-mail support  :  cao-vlsi@masi.ibp.fr                     ##
 * ##                                                               ##
 * ####===========================================================####
 *
 *
 */

#ident "$Id: fpt_error.c,v 1.18 1997/05/23 14:27:36 sirol Exp $"


/*
 * ####===========================================================####
 * ##                                                               ##
 * ##                         Include Files                         ##
 * ##                                                               ##
 * ####===========================================================####
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <varargs.h>

#include MUT_H
#include AUT_H
#include ABL_H
#include BDD_H
#include LOG_H
#include BEH_H
#include BVL_H



#include "fpt_public.h"


/*
 * ####===========================================================####
 * ##                                                               ##
 * ##                           Constants                           ##
 * ##                                                               ##
 * ####===========================================================####
 */
/*
 * ####===========================================================####
 * ##                                                               ##
 * ##                             Types                             ##
 * ##                                                               ##
 * ####===========================================================####
 */
/*
 * ####===========================================================####
 * ##                                                               ##
 * ##                           Variables                           ##
 * ##                                                               ##
 * ####===========================================================####
 */

static int FPT_Verbose_Flag = 0;
static int FPT_Debug_Flag = 0;
static int FPT_Number_of_Warnings = 0;

/*
 * ####===========================================================####
 * ##                                                               ##
 * ##                           Functions                           ##
 * ##                                                               ##
 * ####===========================================================####
 */



/*================================================================####
 * Set or Clear global Flags
 *     
 */

void
setfptverboseflag ()

{
  FPT_Verbose_Flag = 1;
}

void
clearfptverboseflag ()

{
  FPT_Verbose_Flag = 0;
}

void
setfptdebugflag ()

{
  FPT_Debug_Flag = 1;
}

void
clearfptdebugflag ()

{
  FPT_Debug_Flag = 0;
}


int
getfptverboseflag ()

{
  return (FPT_Verbose_Flag);
}

int
getfptdebugflag ()

{
  return (FPT_Debug_Flag);
}


int
getfptwarnings ()

{
  return (FPT_Number_of_Warnings);
}



/***************************************************************
    C'est la nouvelle ecriture (ansi) pour une fonction 
    avec nb variable darguments

# include <stdarg.h>

# ifdef __STDC__

void
FptDisplayError (char *fmt, ...)

{
  va_list args;

  va_start(args, fmt);
  fprintf (stderr, "## Error : ");
  vfprintf (stderr, fmt, args);
  fprintf (stderr, "\n");
  va_end(args);

  EXIT(1);
}

# else
# endif
********************************************************************/




/*================================================================####
 * FptDisplayDebugSource
 *     
 */

void
FptDisplayDebugSource (File, Line)
     char *File;
     int Line;

{
  if (FPT_Debug_Flag)
    fprintf (stderr, "DEBUG FILE ==> file '%s', line %d\n", File, Line);
    fflush(stderr);
}




/*================================================================####
 * FptDisplayDebug
 *     
 */

void
FptDisplayDebug (va_alist)
     va_dcl

{
  va_list args;
  char *fmt;

  if (FPT_Debug_Flag)
    {

      va_start (args);
      fmt = va_arg (args, char *);
      fprintf (stdout, "DEBUG INFO ==> ");
      vfprintf (stdout, fmt, args);
      va_end (args);
      fflush(stdout);
    }
}




/*================================================================####
 * FptDisplayVerbose
 *     
 */

void
FptDisplayVerbose (va_alist)
     va_dcl

{
  va_list args;
  char *fmt;

  if (FPT_Verbose_Flag)
    {

      va_start (args);
      fmt = va_arg (args, char *);
      /* fprintf (stdout, "==> "); */
      vfprintf (stdout, fmt, args);
      va_end (args);
      fflush(stdout);
    }
}

/*================================================================####
 * FptDisplayWarning
 *     
 */

void
FptDisplayWarning (va_alist)
     va_dcl

{
  va_list args;
  char *fmt;

  FPT_Number_of_Warnings++;
  va_start (args);
  fmt = va_arg (args, char *);
  fprintf (stderr, "## WARNING ## ");
  vfprintf (stderr, fmt, args);
  va_end (args);
  fflush(stderr);
}


/*================================================================####
 * FptDisplayError
 *     
 */

void
FptDisplayError (va_alist)
     va_dcl

{
  va_list args;
  char *fmt;

  va_start (args);
  fmt = va_arg (args, char *);
  fprintf (stderr, "##  ERROR  ## ");
  vfprintf (stderr, fmt, args);
  va_end (args);
  fflush(stderr);

  EXIT (1);
}
