/*
 *               ,,,
 *              (o o)
 * ####=====oOO--(_)--OOO=========================================####
 * ##                                                               ##
 * ##   This file is part of FPT : A Formal Proof Toolkit           ##
 * ##   (C) 1996, MASI, CAO-VLSI Team                               ##
 * ##                                                               ##
 * ##   File            :  fpt_pattern.c                            ##
 * ##   Author          :  Olivier SIROL                            ##
 * ##   Date            :  Jun 1996                                 ##
 * ##                                                               ##
 * ##   E-mail support  :  cao-vlsi@masi.ibp.fr                     ##
 * ##                                                               ##
 * ####===========================================================####
 *
 *
 */

#ident "$Id: fpt_pattern.c,v 1.40 1997/05/23 14:27:36 sirol Exp $"

/*
 * ####===========================================================####
 * ##                                                               ##
 * ##                         Include Files                         ##
 * ##                                                               ##
 * ####===========================================================####
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include MUT_H
#include AUT_H
#include ABL_H
#include BDD_H
#include LOG_H
#include BEH_H
#include BVL_H

#include PAT_H
#include PPT_H

#include "fpt_public.h"

/*
 * ####===========================================================####
 * ##                                                               ##
 * ##                           Constants                           ##
 * ##                                                               ##
 * ####===========================================================####
 */

/*
 * ####===========================================================####
 * ##                                                               ##
 * ##                             Types                             ##
 * ##                                                               ##
 * ####===========================================================####
 */
/*
 * ####===========================================================####
 * ##                                                               ##
 * ##                           Variables                           ##
 * ##                                                               ##
 * ####===========================================================####
 */

static char *PAT_PatArray;
static char *PAT_GlobalPat;
static int *PAT_Bdd2PatIndex;
static int PAT_NUMBER_NAME_IN;
static int PAT_Value;
static chain_list *PAT_FirstPat;
static FILE *PAT_PatternFile;

/*
 * ####===========================================================####
 * ##                                                               ##
 * ##                           Functions                           ##
 * ##                                                               ##
 * ####===========================================================####
 */

/*----------------------------------------------------------------####
 * Getting initial state by name
 *     
 */

/*================================================================####
 * FptGetReset
 *     
 */

bddnode *
FptGetReset (BddSystem, BddCircuit, BehFigure1, ResetName, ResetHI)
     bddsystem *BddSystem;
     bddcircuit *BddCircuit;
     befig_list *BehFigure1;
     char *ResetName;
     int ResetHI;
{
  bereg_list *BehReg;
  binode_list *BiNode;
  bddnode *BddNode;
  bddvar Variable;
  bddnode *BddVariable;
  bddnode *BddInitial;
  bddnode *BddSubst;
  chain_list *Expr;

  BddInitial = BddSystem->ONE;

  if (ResetHI)
    BddSubst = BddSystem->ONE;
  else
    BddSubst = BddSystem->ZERO;


  for (BehReg = BehFigure1->BEREG;
       BehReg != (bereg_list *) 0;
       BehReg = BehReg->NEXT)
    {
      BiNode = BehReg->BINODE;

      while (BiNode != (binode_list *) 0)
	{
	  BddNode = BiNode->VALNODE;

	  BddVariable = searchbddcircuitin ((bddcircuit *) 0, ResetName);
	  if (BddVariable == NULL)
	    {
	      displayfptwarning ("Don't know how to find the initial state.\n");
	      displayfpterror ("Reset signal '%s' not found in '%s'\n",
			       ResetName, BehFigure1->NAME);
	    }
	  Variable = getbddvarbyindex ((bddsystem *) 0, BddVariable->INDEX);

	  BddNode = restrictbddnode ((bddsystem *) 0, BddNode, Variable, BddSubst);

	  if (BddNode == BddSystem->ONE)
	    {
	      BddInitial = applybddnode (BddSystem, ABL_AND,
					 BddInitial,
		       searchbddcircuitin ((bddcircuit *) 0, BehReg->NAME));

	    }
	  else
	    {
              if (BddNode != BddSystem->ZERO)
	         displayfptwarning("Register '%s' of '%s' forced to '0'.\n", 
		                    BehReg->NAME,BehFigure1->NAME);

	      BddInitial = applybddnode (BddSystem, ABL_AND,
					 BddInitial,
					 applybddnodenot (BddSystem, searchbddcircuitin ((bddcircuit *) 0, BehReg->NAME)));

	    }
	  if (getfptdebugflag ())
	    {
	      Expr = convertbddcircuitabl (BddCircuit, BddNode);
	      printf ("'%s' a %s implique  '%s' <= ", ResetName,
		      (ResetHI ? "'1'" : "'0'"), BehReg->NAME);
	      viewablexpr (Expr, ABL_VIEW_VHDL);
	      printf ("\n");
	      freeablexpr (Expr);
	    }
	  BiNode = BiNode->NEXT;
	}

    }
  return (BddInitial);
}

/*================================================================####
 * getfptinitstatebyname
 *     
 */

bddnode *
getfptinitstatebyname (BddSystem, BddCircuit,
		       BehFigure1, BehFigure2, ResetName, ResetMode)
     bddsystem *BddSystem;
     bddcircuit *BddCircuit;
     befig_list *BehFigure1;
     befig_list *BehFigure2;
     char *ResetName;
     int ResetMode;

{
  bddnode *BddInitial1;
  bddnode *BddInitial2;
  bddnode *BddReset;
  chain_list *Expr;

  PAT_FirstPat = (chain_list *) 0;

  if (ResetMode)		/* si 1 : reset actif a l'etat haut */
    {
      BddInitial1 = FptGetReset (BddSystem, BddCircuit, BehFigure1, ResetName, 1);
      BddInitial2 = FptGetReset (BddSystem, BddCircuit, BehFigure2, ResetName, 1);
      PAT_FirstPat = addchain (PAT_FirstPat, ResetName);
    }
  else
    {
      BddInitial1 = FptGetReset (BddSystem, BddCircuit, BehFigure1, ResetName, 0);
      BddInitial2 = FptGetReset (BddSystem, BddCircuit, BehFigure2, ResetName, 0);
      PAT_FirstPat = addchain (PAT_FirstPat, NULL);
    }

  BddReset = applybddnode ((bddsystem *) 0, ABL_AND,
			   decbddrefext (BddInitial1),
			   decbddrefext (BddInitial2));

  if (getfptverboseflag ())
    {
      Expr = convertbddcircuitabl (BddCircuit, BddReset);
      printf ("\nInitial state of product machine :\n");
      viewablexpr (Expr, ABL_VIEW_VHDL);
      printf ("\n\n");
      freeablexpr (Expr);
    }

  return (BddReset);

}




/*----------------------------------------------------------------####
 * Getting initial state by pattern file
 *     
 */

/*================================================================####
 * FptGetPatReset
 *     
 */

bddnode *
FptGetPatReset (BddSystem, BddCircuit, BehFigure1, patfile1, rename_regs)
     bddsystem *BddSystem;
     bddcircuit *BddCircuit;
     befig_list *BehFigure1;
     char *patfile1;
     int rename_regs;

{
  bddnode *BddVariable;
  bddnode *BddInitial;

  struct paseq *pt_paseq = NULL;
  struct paiol *pt_paiol;
  struct paevt *pt_evt;
  struct papat *pt_papat;
  char *initstr;
  char *tmpstr;
  char *reg_to_search;
  int i;

  bereg_list *BehReg;
  authtable *RegHashTable;
  authelem *Element;

  BddInitial = BddSystem->ONE;

  displayfptverbose ("===> loading pat file '%s.pat'.\n", patfile1);
  if (rename_regs == NORENAME)
    PAT_FirstPat = (chain_list *) 0;

  pt_paseq = pat_lodpaseq (patfile1, pt_paseq, 0);

  if ((pt_paseq == NULL) || (pt_paseq->ERRFLG != 0))
    displayfpterror ("Error loading '%s.pat'.\n", patfile1);


  initstr = namealloc (PAT_INIT_STR);

  pt_papat = pt_paseq->CURPAT;

  /*   for (pt_ini = pt_papat->PAINI; pt_ini; pt_ini = pt_ini->NEXT)
   *  {
   *   printf("INI: %s=%c\n",pt_ini->NAME,pt_ini->VALUE);
   *  }
   *   exit(123);
   */


  if ((pt_papat != NULL) && (pt_papat->LABEL == initstr))
    {
      RegHashTable = createauthtable (pt_paseq->PATNBR << 1);

      for (pt_evt = pt_papat->PAEVT; pt_evt; pt_evt = pt_evt->NEXT)
	{

	  /* 
	   * get the name for this index 
	   * a vrai dire jen ai un peu marre des hash tables :-) 
	   */

	  for (i = 0, pt_paiol = pt_paseq->PAIOL;
	       i < pt_evt->INDEX;
	       pt_paiol = pt_paiol->NEXT, i++);
	  {
	    if (pt_paiol->MODE == 'R')
	      {

		if (rename_regs == RENAME)
		  {
		    /* si cest fig2 et on doit renomer ... */

		    tmpstr = mbkalloc (strlen (pt_paiol->NAME) + 5 + 1);
		    strcpy (tmpstr, "fig2_");
		    strcat (tmpstr, pt_paiol->NAME);
		    reg_to_search = namealloc (tmpstr);
		    mbkfree (tmpstr);
		  }
		else
		  reg_to_search = pt_paiol->NAME;

		addauthelem (RegHashTable, reg_to_search, 0);

		BddVariable = searchbddcircuitin ((bddcircuit *) 0, reg_to_search);
		if (BddVariable == NULL)
		  {
		    displayfptwarning ("Probable error in pat file '%s.pat'.\n", patfile1);
		    displayfpterror ("Register '%s' of pattern file '%s.pat' not found in '%s'\n",
				     vhdlablname (pt_paiol->NAME),
				     patfile1,
				     BehFigure1->NAME);
		  }
		if (getfptdebugflag ())
		  {
		    printf ("Etat init implique '%s' <= '%d'\n",
			    reg_to_search,
			    (pt_evt->USRVAL == '+' ? 1 : 0));
		  }
		if (pt_evt->USRVAL == '+')
		  {
		    BddInitial = applybddnode (BddSystem, ABL_AND,
					       BddInitial,
					       BddVariable);

		  }
		else
		  {
		    BddInitial = applybddnode (BddSystem, ABL_AND,
					       BddInitial,
				  applybddnodenot (BddSystem, BddVariable));

		  }
	      }
	    else
	      {
		if ((pt_paiol->MODE == 'I') && (rename_regs == NORENAME))
		  {
		    displayfptdebug ("FirstPatPaiol found in : %s <= %c\n", pt_paiol->NAME, pt_evt->USRVAL);
		    /* recuperation des entrees, utilise par FptGetFirstPat(); */
		    /* on renvoie la liste des IN a 1 */
		    BddVariable = searchbddcircuitin ((bddcircuit *) 0, pt_paiol->NAME);
		    if (BddVariable == NULL)
		      {
			displayfptwarning ("Probable error in pat file '%s.pat'.\n", patfile1);
			displayfpterror ("Input '%s' of pattern file '%s.pat' not found in '%s'\n",
					 vhdlablname (pt_paiol->NAME),
					 patfile1,
					 BehFigure1->NAME);
		      }
		    else
		      {
			if (pt_evt->USRVAL == '1')
			  {
			    displayfptdebug ("FirstPatPaiol : %s\n", pt_paiol->NAME);
			    PAT_FirstPat = addchain (PAT_FirstPat, pt_paiol->NAME);
			  }
		      }
		  }
	      }
	  }
	}

      /* On verifie quon a trouve tous les bereg */

      for (BehReg = BehFigure1->BEREG;
	   BehReg != (bereg_list *) 0;
	   BehReg = BehReg->NEXT)
	{
	  Element = searchauthelem (RegHashTable, BehReg->NAME);

	  if (Element == (authelem *) 0)
	    {
	      displayfptwarning ("Probable error in pat file '%s.pat'.\n", patfile1);
	      displayfpterror ("Register '%s' of '%s' not found in pattern file '%s.pat'.\n",
			       BehReg->NAME,
			       BehFigure1->NAME,
			       patfile1);
	    }
	}

      destroyauthtable (RegHashTable);
    }
  else
    {
      displayfptwarning ("Probable error in pat file '%s.pat'.\n", patfile1);
      displayfpterror ("The first pattern label must be '%s'.\n", PAT_INIT_STR);

    }

/*   pat_frepaseq(pt_paseq); le bug qui tue... */

  return (BddInitial);
}






/*================================================================####
 * getfptinitstatebypat
 *     
 */

bddnode *
getfptinitstatebypat (BddSystem, BddCircuit,
		      BehFigure1, BehFigure2,
		      patfile1, patfile2)
     bddsystem *BddSystem;
     bddcircuit *BddCircuit;
     befig_list *BehFigure1;
     befig_list *BehFigure2;
     char *patfile1;
     char *patfile2;

{
  bddnode *BddInitial1;
  bddnode *BddInitial2;
  bddnode *BddReset;
  chain_list *Expr;

  BddInitial1 = FptGetPatReset (BddSystem, BddCircuit, BehFigure1, patfile1, NORENAME);
  BddInitial2 = FptGetPatReset (BddSystem, BddCircuit, BehFigure2, patfile2, RENAME);

  BddReset = applybddnode ((bddsystem *) 0, ABL_AND,
			   decbddrefext (BddInitial1),
			   decbddrefext (BddInitial2));

  if (getfptverboseflag ())
    {
      Expr = convertbddcircuitabl (BddCircuit, BddReset);
      printf ("\nInitial state of product machine :\n");
      viewablexpr (Expr, ABL_VIEW_VHDL);
      printf ("\n\n");
      freeablexpr (Expr);
    }
  return (BddReset);

}

/*================================================================####
 * FptPrintValue
 *     
 */

void
FptPrintValue (BddNode, index)
     bddnode *BddNode;
     bddindex index;

{
  bddindex BddIndex;

  if (BddNode->INDEX < BDD_INDEX_MIN)
    {
      if (BddNode->INDEX == BDD_INDEX_ONE)
	{
	  fprintf (PAT_PatternFile, "%d", PAT_Value);
	}
    }
  else
    {
      BddIndex = BddNode->INDEX;

      if (BddIndex == index)
	PAT_Value = 1;
      FptPrintValue (BddNode->HIGH, index);
      if (BddIndex == index)
	PAT_Value = 0;
      FptPrintValue (BddNode->LOW, index);
    }
}




/*================================================================####
 * FptGetFirstPat
 *     
 */

void
FptGetFirstPat ()

{
  chain_list *pt_first;
  bddnode *BddNode;



  for (pt_first = PAT_FirstPat;
       pt_first != NULL;
       pt_first = pt_first->NEXT)
    {
      BddNode = searchbddcircuitin ((bddcircuit *) 0, pt_first->DATA);
      if (BddNode != NULL)
	{
	  if ((BddNode->INDEX < PAT_NUMBER_NAME_IN)
	      && (PAT_Bdd2PatIndex[BddNode->INDEX] != -1))
	    {
	      displayfptdebug ("FirstPat : %s\n", pt_first->DATA);
	      PAT_PatArray[PAT_Bdd2PatIndex[BddNode->INDEX]] = '1';
	    }
	}
    }
  displayfptdebug ("Pat init : %s\n\n\n", PAT_PatArray);
}


/*================================================================####
 * FptGetOnePat
 *     
 */

void
FptGetOnePat (BddNode)
     bddnode *BddNode;

{
  bddindex BddIndex;

  if (BddNode->INDEX < BDD_INDEX_MIN)
    {
      if (BddNode->INDEX == BDD_INDEX_ONE)
	{
	  PAT_GlobalPat = mbkstrdup (PAT_PatArray);
	  displayfptdebug ("Pat      : %s\n\n\n", PAT_GlobalPat);
	}
    }
  else
    {
      BddIndex = BddNode->INDEX;

      if ((BddIndex < PAT_NUMBER_NAME_IN)
	  && (PAT_Bdd2PatIndex[BddIndex] != -1))
	{
	  PAT_PatArray[PAT_Bdd2PatIndex[BddIndex]] = '1';
	}
      FptGetOnePat (BddNode->HIGH);

      if ((BddIndex < PAT_NUMBER_NAME_IN)
	  && (PAT_Bdd2PatIndex[BddIndex] != -1))
	{
	  PAT_PatArray[PAT_Bdd2PatIndex[BddIndex]] = '0';
	}
      FptGetOnePat (BddNode->LOW);

    }
}


/*================================================================####
 * FptGetPatList
 *    
 */

chain_list *
FptGetPatList (ptbefig1, ptbefig2, BddFautif, Reached_aller,
	       MP_Transfunc, BddReset, patvide)

     befig_list *ptbefig1;
     befig_list *ptbefig2;
     bddnode *BddFautif;
     chain_list *Reached_aller;
     fpttransfunc *MP_Transfunc;
     bddnode *BddReset;
     char *patvide;

{
  chain_list *pat_list;
  chain_list *ptReached;

  bddnode *BddPatern;

  bddnode *BddNode;
  bereg_list *ptbereg;

  bddassoc *BddAssoc;

  int firstone = 1;
  int preimg_counter = 0;
  int pat_counter = 0;




  pat_list = (chain_list *) 0;
  BddAssoc = addbddassoc ((bddsystem *) 0);

  for (ptbereg = ptbefig1->BEREG;
       ptbereg != (bereg_list *) 0;
       ptbereg = ptbereg->NEXT)
    {
      BddNode = searchbddcircuitin ((bddcircuit *) 0, ptbereg->NAME);

      addbddnodeassoc ((bddsystem *) 0, BddAssoc,
		       getbddvarbyindex ((bddsystem *) 0, BddNode->INDEX),
		       BddLocalSystem->ZERO);

    }

  for (ptbereg = ptbefig2->BEREG;
       ptbereg != (bereg_list *) 0;
       ptbereg = ptbereg->NEXT)
    {
      BddNode = searchbddcircuitin ((bddcircuit *) 0, ptbereg->NAME);

      addbddnodeassoc ((bddsystem *) 0, BddAssoc,
		       getbddvarbyindex ((bddsystem *) 0, BddNode->INDEX),
		       BddLocalSystem->ZERO);

    }


  BddPatern = incbddrefext (BddFautif);

  ptReached = Reached_aller;

  displayfptnode ("Preim nodes avant");

  if (ptReached != NULL)
    {
      firstone = 1;

      do
	{


	  if (getfptdebugflag ())
	    {
	      printf ("#######################    PAT %d      ######################\n", pat_counter);
	      printf ("Reached aller :\n");
	      fptViewBddState (ptReached->DATA);
	      printf ("\n");
	    }

	  pat_counter++;

	  if (getfptdebugflag ())
	    {
	      printf ("BddPatern :\n");
	      fptViewBddState (BddPatern);
	      printf ("\n");
	      /* fptViewBddNode (BddPatern);
	       * printf ("\n"); 
	       */
	    }

	  BddPatern = applybddnode ((bddsystem *) 0, ABL_AND,
				    decbddrefext (BddPatern),
				    (bddnode *) ptReached->DATA);

	  if (getfptdebugflag ())
	    {
	      printf ("BddPatern and reachead aller:\n");
	      fptViewBddState (BddPatern);
	      printf ("\n");
	    }
	  BddPatern = satisfybddnodeassoc ((bddsystem *) 0, decbddrefext (BddPatern), BddAssoc);

	  if (getfptdebugflag ())
	    {
	      printf ("BddPatern apres satisfy:\n");
	      fptViewBddState (BddPatern);
	      printf ("\n");
	    }

	  if (firstone)
	    firstone = 0;
	  else
	    {
	      strcpy (PAT_PatArray, patvide);
	      FptGetOnePat (BddPatern);
	      pat_list = addchain (pat_list, PAT_GlobalPat);
	    }

	  BddPatern = missbddnodeassocon ((bddsystem *) 0, decbddrefext (BddPatern), BddAssoc);

	  if (getfptdebugflag ())
	    {
	      printf ("BddPatern apres miss:\n");
	      fptViewBddState (BddPatern);
	      printf ("\n");
	    }



	  /* pas besoin de calculer la preimage si on sarrete */

	  if (ptReached->NEXT != NULL)
	    {
	      BddPatern = preimagefpttransfunc (MP_Transfunc, decbddrefext (BddPatern));

	      if (getfptdebugflag ())
		{
		  fprintf (stdout, "PreImage %d :\n", preimg_counter);
		  fptViewBddState (BddPatern);
		}

	      preimg_counter++;
	    }
	  else
	    {
	      /* Cest le prmier pat (reset) */
	      strcpy (PAT_PatArray, patvide);
	      FptGetFirstPat ();
	      pat_list = addchain (pat_list, mbkstrdup (PAT_PatArray));

	    }

	  if (getfptdebugflag ())
	    {
	      addbddcircuitout ((bddcircuit *) 0, "pat", BddPatern);
	      displayfptverbose ("============ pi cs ru reset fini  tmp pat\n");
	      testbddcircuit ((bddcircuit *) 0);
	    }

	  ptReached = ptReached->NEXT;

	}

      while (ptReached != NULL);
    }

  decbddrefext (BddPatern);
  displayfptnode ("Preim nodes apres");
  
  return (pat_list);
}




/*================================================================####
 * GetClock
 *     
 */

static char *
GetClock (Expr)

     chain_list *Expr;
{
  char *Value;

  if (ABL_ATOM (Expr))
    {
      Value = ABL_ATOM_VALUE (Expr);

      if ((Value != getablatomone ()) &&
	  (Value != getablatomzero ()))
	{
	  return (Value);
	}
    }
  else
    {
      while ((Expr = ABL_CDR (Expr)))
	{
	  Value = GetClock (ABL_CAR (Expr));

	  if (Value != (char *) 0)
	    return (Value);
	}
    }

  return ((char *) 0);
}






/*================================================================####
 * createfptpattern
 *     
 */

void
createfptpattern (ptbefig1,
		  ptbefig2,
		  BddFautif,
		  BeoutFautif,
		  Reached_aller,
		  MP_Transfunc,
		  BddReset,
		  ClockName,
		  VddName,
		  inputfilename1,
		  inputfilename2,
		  fname_sext,
		  file2load1,
		  file2load2,
		  fig1name,
		  fig2name)


     befig_list *ptbefig1;
     befig_list *ptbefig2;
     bddnode *BddFautif;
     beout_list *BeoutFautif;
     chain_list *Reached_aller;
     fpttransfunc *MP_Transfunc;
     bddnode *BddReset;
     char *ClockName;
     char *VddName;
     char *inputfilename1;
     char *inputfilename2;
     char *fname_sext;
     char *file2load1;
     char *file2load2;
     char *fig1name;
     char *fig2name;

{

  bddnode *BddNode;
  bddnode *BddVariable;

  chain_list *pat_list;
  chain_list *pt_pat;

  bepor_list *ptbepor;
  bepor_list *ptbepor_tmp;
  bereg_list *ptbereg;

  long NumberPort1 = 0;
  int i;
  int left, right;
  int index;
  int ClockIndex;
  int VddIndex;

  char *StdPat;
  char *DisplayPat;
  char *papat;
  char *name;
  time_t clock;
  char *mode;

  char fname[256];

  char *blank_space;
  char *sig_name;
  char name_tmp[200];
  char number[200];
  char END = 0;






  strcpy (fname, fname_sext);
  strcat (fname, ".pat");

  printf ("===> Creating '%s' pattern file.\n", fname);

  /* getting good clock */

  if (ClockName == NULL)
    ClockName = GetClock (ptbefig1->BEREG->BIABL->CNDABL);

  BddVariable = searchbddcircuitin ((bddcircuit *) 0, ClockName);
  if (BddVariable == NULL)
    {
      displayfptwarning ("Don't know how to make a sequential pattern file.\n");
      displayfpterror ("Clock signal '%s' not found in '%s'\n",
		       ClockName, ptbefig1->NAME);
    }

  displayfptdebug ("ClockName = %s\n", ClockName);

  BddVariable = searchbddcircuitin ((bddcircuit *) 0, VddName);
  if (BddVariable == NULL)
    {
      displayfptwarning ("VDD signal '%s' not found in '%s'\n",
			 VddName, ptbefig1->NAME);
      displayfptwarning ("Ignoring missing power supply.\n");
    }
  PAT_PatternFile = fopen (fname, "w");
  if (PAT_PatternFile == NULL)
    displayfpterror ("can't write file '%s'\n", fname);
  /*
   * PAT_PatternFile = stdout; 
   */

  for (ptbepor = ptbefig1->BEPOR;
       ptbepor != (bepor_list *) 0;
       ptbepor = ptbepor->NEXT)
    {
      NumberPort1++;
    }

  DisplayPat = autallocblock (NumberPort1 + 1);
  memset (DisplayPat, '-', NumberPort1);

  PAT_PatArray = autallocblock (NumberPort1 + 1);
  memset (PAT_PatArray, '*', NumberPort1);

  PAT_NUMBER_NAME_IN = BddLocalCircuit->NUMBER_NAME_IN;
  PAT_Bdd2PatIndex = (int *) autallocblock ((PAT_NUMBER_NAME_IN)
					    * sizeof (PAT_Bdd2PatIndex));
  memset (PAT_Bdd2PatIndex, -1, (PAT_NUMBER_NAME_IN)
	  * sizeof (PAT_Bdd2PatIndex));

  displayfptdebug ("############### PAT_NUMBER_NAME_IN = %d\n", PAT_NUMBER_NAME_IN);

  /*
   * see bvl_drive.c 
   */

  time (&clock);
  fprintf (PAT_PatternFile, "-- Pattern file generated by YEAP\n");
  fprintf (PAT_PatternFile, "-- date : %s", ctime (&clock));
  fprintf (PAT_PatternFile, "-- \n");
  fprintf (PAT_PatternFile, "# Formal proof between befig '%s.vbe' and '%s.vbe'\n", inputfilename1, inputfilename2);
  fprintf (PAT_PatternFile, "#\n");
  fprintf (PAT_PatternFile, "# Beh Figure 1 %s : %s\n", ptbefig1->NAME, file2load1);
  fprintf (PAT_PatternFile, "# Beh Figure 2 %s : %s\n", ptbefig2->NAME, file2load2);
  fprintf (PAT_PatternFile, "#\n");
  fprintf (PAT_PatternFile, "# Error on output '%s'\n", BeoutFautif->NAME);
  fprintf (PAT_PatternFile, "-- \n");
  fprintf (PAT_PatternFile, "-- Use the following test sequence :\n");
  fprintf (PAT_PatternFile, "--  command1  > asimut -b %s %s good_%s\n",
	   inputfilename1, fname_sext, fname_sext);
  fprintf (PAT_PatternFile, "--  command2  > asimut -b %s good_%s bad_%s\n",
	   inputfilename2, fname_sext, fname_sext);
  fprintf (PAT_PatternFile, "-- \n");
  fprintf (PAT_PatternFile, "--  The first test should run ok (while patterns are unprocessed),\n");
  fprintf (PAT_PatternFile, "--  and the second should fail on mostly last pattern.\n");
  fprintf (PAT_PatternFile, "-- \n\n\n");



/*----------------------------------------------------------------####
 * Port declaration (see bvl_drive.c)
 *     
 */

  ptbepor = ptbefig1->BEPOR;
  if (ptbepor != NULL)
    {
      ptbefig1->BEPOR = (struct bepor *) reverse ((chain_list *)ptbefig1->BEPOR);

      index = 0;
      ptbepor = ptbefig1->BEPOR;
      while (ptbepor != NULL)
	{

	  switch (ptbepor->DIRECTION)
	    {
	    case 'I':
	      mode = namealloc ("in");
	      DisplayPat[index] = ' ';
	      break;
	    case 'O':
	    case 'Z':
	    case 'B':
	    case 'T':
	      mode = namealloc ("out");
	      DisplayPat[index] = '?';
	      break;
	    default:
	      displayfpterror ("port '%s' of befig '%s' has unknwon mode\n",
			       ptbepor->NAME, ptbefig1->NAME);
	    }

	  BddNode = searchbddcircuitin ((bddcircuit *) 0, ptbepor->NAME);
	  if (BddNode != NULL)
	    {
	      if (!searchbddcircuitout ((bddcircuit *) 0, ptbepor->NAME))
		{
		  /* les index des in sont tous < NUMBER_NAME_IN */
		  if ((BddNode->INDEX) > PAT_NUMBER_NAME_IN - 1)
		    displayfpterror ("pat 2 bdd index out of range error\n");

		  PAT_Bdd2PatIndex[BddNode->INDEX] = index;
		  PAT_PatArray[index] = '0';
		  if (ptbepor->NAME == ClockName)
		    ClockIndex = index;
		  if (ptbepor->NAME == VddName)
		    VddIndex = index;

		  displayfptdebug ("BEPORT : %-20s, Index : %d  --> %d\n",
				   ptbepor->NAME, BddNode->INDEX,
				   PAT_Bdd2PatIndex[BddNode->INDEX]);
		}
	    }
/*
 * ptbepor = (bepor_list *) bvl_vectnam (ptbepor,&left,&right,&name,0); 
 */

	  END = 0;
	  ptbepor_tmp = ptbepor;

	  left = right = -1;
	  sig_name = ptbepor_tmp->NAME;
	  name = (char *) mbkalloc (strlen (sig_name) + 1);
	  strcpy (name, sig_name);
	  blank_space = strchr (name, ' ');
	  if (blank_space != NULL)
	    {
	      strcpy (number, blank_space);
	      left = atoi (number);
	      right = left;
	      *blank_space = '\0';
	    }
	  while (!END)
	    {
	      if (ptbepor_tmp->NEXT != NULL)
		{
		  strcpy (name_tmp, (ptbepor_tmp->NEXT)->NAME);
		  blank_space = strchr (name_tmp, ' ');
		  if (blank_space != NULL)
		    {
		      strcpy (number, blank_space);
		      *blank_space = '\0';
		      if (!strcmp (name, name_tmp))
			{
			  right = atoi (number);
			  ptbepor_tmp = ptbepor_tmp->NEXT;

			  index++;
			  DisplayPat[index] = '0';
			  BddNode = searchbddcircuitin ((bddcircuit *) 0,
							ptbepor_tmp->NAME);
			  if (BddNode != NULL)
			    {
			      if (!searchbddcircuitout ((bddcircuit *) 0, ptbepor_tmp->NAME))
				{
				  /* les index des in sont tous < NUMBER_NAME_IN */
				  if ((BddNode->INDEX) > PAT_NUMBER_NAME_IN - 1)
				    displayfpterror ("pat 2 bdd index out of range error\n");

				  PAT_Bdd2PatIndex[BddNode->INDEX] = index;
				  PAT_PatArray[index] = '0';
				  if (ptbepor->NAME == ClockName)
				    ClockIndex = index;
				  if (ptbepor->NAME == VddName)
				    VddIndex = index;

				  displayfptdebug ("BEPORT in second loop : %-20s, Index : %d  --> %d\n",
					  ptbepor_tmp->NAME, BddNode->INDEX,
					  PAT_Bdd2PatIndex[BddNode->INDEX]);
				}
			    }
			}
		      else
			END = 1;
		    }
		  else
		    END = 1;
		}
	      else
		END = 1;
	    }

	  ptbepor = ptbepor_tmp;

	  if (left != -1)
	    {
	      fprintf (PAT_PatternFile, "%s\t%s(%d %s %d)\t%s;;\n", mode, vhdlablname (name),
		       left, (left >= right) ? "downto" : "to", right, "B");
	    }
	  else
	    {
	      fprintf (PAT_PatternFile, "%s\t%s\t%s;;\n", mode, vhdlablname (name), "B");
	    }

	  ptbepor = ptbepor->NEXT;
	  index++;
	}

      ptbefig1->BEPOR = (struct bepor *) reverse ((chain_list *)ptbefig1->BEPOR);
    }




  fprintf (PAT_PatternFile, "\n\n\n\nbegin\n\n");


/*----------------------------------------------------------------####
 * Etat init des reg
 *     
 */

  fprintf (PAT_PatternFile, "# The 3 first patterns (reset) might introduce errors. In this case,\n");
  fprintf (PAT_PatternFile, "# you will have to REPLACE these by  the following init sequence\n");
  fprintf (PAT_PatternFile, "# to force registers to initial state.\n");
  fprintf (PAT_PatternFile, "# You MUST do it 2 times (for BOTH simulation).\n");
  fprintf (PAT_PatternFile, "#\n");
  fprintf (PAT_PatternFile, "# Register initialisation of %s\n", ptbefig1->NAME);

  for (ptbereg = ptbefig1->BEREG;
       ptbereg != (bereg_list *) 0;
       ptbereg = ptbereg->NEXT)
    {

      BddNode = searchbddcircuitin ((bddcircuit *) 0, ptbereg->NAME);
      if (BddNode != NULL)
	{
	  fprintf (PAT_PatternFile, "# %s.%s <= B\"", fig1name, vhdlablname (ptbereg->NAME));
	  FptPrintValue (BddReset, BddNode->INDEX);
	  fprintf (PAT_PatternFile, "\" ;\n");
	}

    }


  fprintf (PAT_PatternFile, "#\n");
  fprintf (PAT_PatternFile, "# Register initialisation of %s\n", ptbefig2->NAME);

  for (ptbereg = ptbefig2->BEREG;
       ptbereg != (bereg_list *) 0;
       ptbereg = ptbereg->NEXT)
    {

      BddNode = searchbddcircuitin ((bddcircuit *) 0, ptbereg->NAME);
      if (BddNode != NULL)
	{
	  /* +5 pour degager fig2_ */
	  fprintf (PAT_PatternFile, "# %s.%s <= B\"", fig2name, vhdlablname ((ptbereg->NAME) + 5));
	  FptPrintValue (BddReset, BddNode->INDEX);
	  fprintf (PAT_PatternFile, "\" ;\n");
	}

    }


  fprintf (PAT_PatternFile, "#\n");
  fprintf (PAT_PatternFile, "\n\n\n\n");

/*
 * gene chain list de patterns 
 */
  StdPat = mbkstrdup (PAT_PatArray);

  pat_list = FptGetPatList (ptbefig1, ptbefig2,
			    BddFautif, Reached_aller,
			    MP_Transfunc, BddReset,
			    StdPat);

  for (pt_pat = pat_list;
       pt_pat != NULL;
       pt_pat = pt_pat->NEXT)
    {
      papat = (char *) pt_pat->DATA;
      for (i = 0; i < 3; i++)
	{
	  fprintf (PAT_PatternFile, "  : ");
	  for (index = 0; index < NumberPort1; index++)
	    {
	      switch (DisplayPat[index])
		{
		case ' ':
		  fprintf (PAT_PatternFile, " ");
		  break;
		case '?':
		  fprintf (PAT_PatternFile, " ?");
		  break;
		default:
		  break;
		}

	      if (index == ClockIndex)
		{
		  if (i == 1)
		    fprintf (PAT_PatternFile, "1");
		  else
		    fprintf (PAT_PatternFile, "0");
		}
	      else
		{

		  if (index == VddIndex)
		    fprintf (PAT_PatternFile, "1");
		  else
		    {
		      switch (papat[index])
			{
			case '0':
			  fprintf (PAT_PatternFile, "0");
			  break;
			case '1':
			  fprintf (PAT_PatternFile, "1");
			  break;
			case '*':
			default:
			  fprintf (PAT_PatternFile, "*");
			  break;
			}
		    }
		}
	    }
	  fprintf (PAT_PatternFile, " ;\n");
	}

    }
  fprintf (PAT_PatternFile, "\n\nend;\n\n");

/*----------------------------------------------------------------####
 * savepat 
 *     
 */

  displayfptverbose ("===> Saving pat file '%s'.\n", fname);
  if (fclose (PAT_PatternFile))
    displayfpterror ("Error writing file '%s'.\n", fname);

}
