/*
 *               ,,,
 *              (o o)
 * ####=====oOO--(_)--OOO=========================================####
 * ##                                                               ##
 * ##   This file is part of FPT : A Formal Proof Toolkit           ##
 * ##   (C) 1996, MASI, CAO-VLSI Team                               ##
 * ##                                                               ##
 * ##   File            :  fpt_transfunc.c                          ##
 * ##   Author          :  Olivier SIROL                            ##
 * ##   Date            :  Jun 1996                                 ##
 * ##                                                               ##
 * ##   E-mail support  :  cao-vlsi@masi.ibp.fr                     ##
 * ##                                                               ##
 * ####===========================================================####
 *
 *
 */

#ident "$Id: fpt_transfunc.c,v 1.35 1997/05/23 14:27:36 sirol Exp $"


/*
 * ####===========================================================####
 * ##                                                               ##
 * ##                         Include Files                         ##
 * ##                                                               ##
 * ####===========================================================####
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include MUT_H
#include AUT_H
#include ABL_H
#include BDD_H
#include LOG_H
#include BEH_H
#include BVL_H

#include "fpt_public.h"



/*
 * ####===========================================================####
 * ##                                                               ##
 * ##                           Constants                           ##
 * ##                                                               ##
 * ####===========================================================####
 */


/*
 * ####===========================================================####
 * ##                                                               ##
 * ##                             Types                             ##
 * ##                                                               ##
 * ####===========================================================####
 */
/*
 * ####===========================================================####
 * ##                                                               ##
 * ##                           Variables                           ##
 * ##                                                               ##
 * ####===========================================================####
 */

static fptvarfunc *TRANSFUNC_VarFunc = (fptvarfunc *) 0;
static long TRANSFUNC_NumberVar = 0;
static long TRANSFUNC_IndexVar = 0;
static char *TRANSFUNC_StateArray = (char *) 0;

/*
 * ####===========================================================####
 * ##                                                               ##
 * ##                           Functions                           ##
 * ##                                                               ##
 * ####===========================================================####
 */





/*================================================================####
 * unionfptstate
 *
 */

bddnode *
unionfptstate (Bdd1, Bdd2)

     bddnode *Bdd1;
     bddnode *Bdd2;
{
  return (applybddnode ((bddsystem *) 0, ABL_OR, Bdd1, Bdd2));

}



/*================================================================####
 * removefptstate
 *
 */

bddnode *
removefptstate (Bdd1, Bdd2)

     bddnode *Bdd1;
     bddnode *Bdd2;
{

  bddnode *Bddtmp;

  Bddtmp = applybddnodenot ((bddsystem *) 0, Bdd2);
  return (applybddnode ((bddsystem *) 0, ABL_AND, Bdd1, decbddrefext (Bddtmp)));

}



/*================================================================####
 * intersectfptstate
 *
 */

bddnode *
intersectfptstate (Bdd1, Bdd2)

     bddnode *Bdd1;
     bddnode *Bdd2;
{


  return (applybddnode ((bddsystem *) 0, ABL_AND, Bdd1, Bdd2));

}



/*================================================================####
 * Alloc Function Transition
 *
 */

fpttransfunc *
allocfpttransfunc ()
{
  return ((fpttransfunc *) autallocblock (sizeof (fpttransfunc)));
}



/*================================================================####
 * Bdd Alloc Variable Function
 *
 */

fptvarfunc *
allocfptvarfunc (Number)

     long Number;
{
  return ((fptvarfunc *) autallocblock (sizeof (fptvarfunc) * Number));
}




/*================================================================####
 * Free Function Transition
 *
 */

void
freefpttransfunc (TransFunc)

     fpttransfunc *TransFunc;
{
  autfreeblock ((char *) TransFunc);
}



/*================================================================####
 * Free Variable Function
 *
 */

void
freefptvarfunc (VarFunc)

     fptvarfunc *VarFunc;
{
  autfreeblock ((char *) VarFunc);
}




/*================================================================####
 * Resize Variable Function
 *
 */

fptvarfunc *
resizefptvarfunc (VarFunc, OldNum, NewNum)

     fptvarfunc *VarFunc;
     long OldNum;
     long NewNum;
{
  return ((fptvarfunc *) autresizeblock ((char *) VarFunc,
					 sizeof (fptvarfunc) * OldNum,
					 sizeof (fptvarfunc) * NewNum));
}




/*================================================================####
 * Local PreImage Transition Function    
 *
 */

bddnode *
loc_preimagefpttransfunc (StateSet)

     bddnode *StateSet;
{
  fptvarfunc *VarFunc;
  bddnode *BddSetIn;
  bddnode *BddSetOut;
  bddnode *BddResult;
  bddnode *BddHigh;
  bddnode *BddLow;
  static long counter = 0;

  counter++;

  if (StateSet->INDEX < BDD_INDEX_MIN)
    {
      return (StateSet);
    }

  VarFunc = &TRANSFUNC_VarFunc[TRANSFUNC_IndexVar];

  BddSetIn = decbddrefext (applybddnodenot ((bddsystem *) 0, StateSet));
  BddSetIn = applybddnode ((bddsystem *) 0, ABL_OR, VarFunc->VAR, BddSetIn);
  BddSetOut = applybddnode ((bddsystem *) 0, ABL_NAND, VarFunc->VAR, StateSet);

  TRANSFUNC_IndexVar++;

  if (TRANSFUNC_IndexVar >= TRANSFUNC_NumberVar)
    {
/*
 * StateSet => Rn,  Fn
 */
      if (BddSetIn->INDEX == BDD_INDEX_ONE)
	{
	  displayfptdebug ("Calc pre-image loop : %ld\n", counter);
	  counter = 0;
	  BddResult = incbddrefext (VarFunc->FUNC);
	}
      else
/*
 * StateSet => ! Rn, ! Fn
 */
      if (BddSetOut->INDEX == BDD_INDEX_ONE)
	{
	  BddResult = applybddnodenot ((bddsystem *) 0, VarFunc->FUNC);
	}
      else
	{
	  BddResult = BddLocalSystem->ONE;
	}
    }
  else
    {
/*
 * StateSet => Ri, Fi . Img( StateSet )
 */
      if (BddSetIn->INDEX == BDD_INDEX_ONE)
	{
	  BddResult = decbddrefext (loc_preimagefpttransfunc (StateSet));
	  BddResult = applybddnode ((bddsystem *) 0, ABL_AND, VarFunc->FUNC, BddResult);
	}
      else
/*
 * StateSet => ! Ri, ! Fi . Img( StateSet )
 */
      if (BddSetOut->INDEX == BDD_INDEX_ONE)
	{
	  BddLow = loc_preimagefpttransfunc (StateSet);
	  BddHigh = applybddnodenot ((bddsystem *) 0, VarFunc->FUNC);

	  BddResult = applybddnode ((bddsystem *) 0, ABL_AND,
				    decbddrefext (BddHigh),
				    decbddrefext (BddLow));
	}
      else
	{
/*
 *  Fi . Img( StateSet . Ri ) +  ! Fi . Img( StateSet . ! Ri )
 */
	  BddSetOut = applybddnodenot ((bddsystem *) 0, decbddrefext (BddSetOut));
	  BddSetIn = applybddnodenot ((bddsystem *) 0, decbddrefext (BddSetIn));

	  BddHigh = applybddnodenot ((bddsystem *) 0, VarFunc->FUNC);
	  BddLow = loc_preimagefpttransfunc (BddSetIn);
	  BddLow = applybddnode ((bddsystem *) 0, ABL_AND,
				 decbddrefext (BddHigh),
				 decbddrefext (BddLow));

	  BddHigh = decbddrefext (loc_preimagefpttransfunc (BddSetOut));
	  BddHigh = applybddnode ((bddsystem *) 0, ABL_AND, VarFunc->FUNC, BddHigh);

	  BddResult = applybddnode ((bddsystem *) 0, ABL_OR,
				    decbddrefext (BddHigh),
				    decbddrefext (BddLow));
	}
    }

  TRANSFUNC_IndexVar--;

  decbddrefext (BddSetIn);
  decbddrefext (BddSetOut);

  return (BddResult);
}




/*================================================================####
 * PreImage Transition Function
 *
 */

bddnode *
preimagefpttransfunc (TransFunc, StateSet)

     fpttransfunc *TransFunc;
     bddnode *StateSet;
{
  bddnode *BddResult;

  if (getfptverboseflag ())
    {
      fprintf (stdout, "+");
      fflush (stdout);
    }

  setbddlocalsystem (TransFunc->BDD_SYSTEM);

  TRANSFUNC_VarFunc = TransFunc->VAR_FUNC;
  TRANSFUNC_NumberVar = TransFunc->NUMBER_VAR;
  TRANSFUNC_IndexVar = 0;

  if (TRANSFUNC_NumberVar == 0)
    {
      displayfpterror ("bad var 0\n");
    }

  BddResult = loc_preimagefpttransfunc (StateSet);

  return (BddResult);
}




/*================================================================####
 * Local Image Transition Function
 *
 */

bddnode *
loc_imagefpttransfunc (StateSet)

     bddnode *StateSet;
{
  fptvarfunc *VarFunc;
  bddnode *BddSetIn;
  bddnode *BddSetOut;
  bddnode *BddResult;
  bddnode *BddHigh;
  bddnode *BddLow;
  static long counter = 0;

  counter++;

  if (StateSet->INDEX == BDD_INDEX_ZERO)
    {
      return (StateSet);
    }

  VarFunc = &TRANSFUNC_VarFunc[TRANSFUNC_IndexVar];

  BddSetIn = decbddrefext (applybddnodenot ((bddsystem *) 0, StateSet));
  BddSetIn = applybddnode ((bddsystem *) 0, ABL_OR, VarFunc->FUNC, BddSetIn);
  BddSetOut = applybddnode ((bddsystem *) 0, ABL_NAND, VarFunc->FUNC, StateSet);

  TRANSFUNC_IndexVar++;

  if (TRANSFUNC_IndexVar >= TRANSFUNC_NumberVar)
    {
/*
 * StateSet => Fn,  Rn
 */
      if (BddSetIn->INDEX == BDD_INDEX_ONE)
	{
	  displayfptdebug ("Calc image loop : %ld\n", counter);
	  counter = 0;
	  BddResult = incbddrefext (VarFunc->VAR);
	}
      else
/*
 * StateSet => ! Fn, ! Rn
 */
      if (BddSetOut->INDEX == BDD_INDEX_ONE)
	{
	  BddResult = applybddnodenot ((bddsystem *) 0, VarFunc->VAR);
	}
      else
	{
	  BddResult = BddLocalSystem->ONE;
	}
    }
  else
    {
/*
 * StateSet => Fi, Ri . Img( StateSet )
 */
      if (BddSetIn->INDEX == BDD_INDEX_ONE)
	{
	  BddResult = decbddrefext (loc_imagefpttransfunc (StateSet));
	  BddResult = applybddnode ((bddsystem *) 0, ABL_AND, VarFunc->VAR, BddResult);
	}
      else
/*
 * StateSet => ! Fi, ! Ri . Img( StateSet )
 */
      if (BddSetOut->INDEX == BDD_INDEX_ONE)
	{
	  BddLow = loc_imagefpttransfunc (StateSet);
	  BddHigh = applybddnodenot ((bddsystem *) 0, VarFunc->VAR);

	  BddResult = applybddnode ((bddsystem *) 0, ABL_AND,
				    decbddrefext (BddHigh),
				    decbddrefext (BddLow));
	}
      else
	{
/*
 *  Ri . Img( StateSet . Fi ) +  ! Ri . Img( StateSet . ! Fi )
 */
	  BddSetOut = applybddnodenot ((bddsystem *) 0, decbddrefext (BddSetOut));
	  BddSetIn = applybddnodenot ((bddsystem *) 0, decbddrefext (BddSetIn));

	  BddHigh = applybddnodenot ((bddsystem *) 0, VarFunc->VAR);
	  BddLow = loc_imagefpttransfunc (BddSetIn);
	  BddLow = applybddnode ((bddsystem *) 0, ABL_AND,
				 decbddrefext (BddHigh),
				 decbddrefext (BddLow));

	  BddHigh = decbddrefext (loc_imagefpttransfunc (BddSetOut));
	  BddHigh = applybddnode ((bddsystem *) 0, ABL_AND, VarFunc->VAR, BddHigh);

	  BddResult = applybddnode ((bddsystem *) 0, ABL_OR,
				    decbddrefext (BddHigh),
				    decbddrefext (BddLow));
	}
    }

  TRANSFUNC_IndexVar--;

  decbddrefext (BddSetIn);
  decbddrefext (BddSetOut);

  return (BddResult);
}




/*================================================================####
 * Image Transition Function
 *
 */

bddnode *
imagefpttransfunc (TransFunc, StateSet)

     fpttransfunc *TransFunc;
     bddnode *StateSet;
{
  bddnode *BddResult;

  if (getfptverboseflag ())
    {
      fprintf (stdout, "+");
      fflush (stdout);
    }

  setbddlocalsystem (TransFunc->BDD_SYSTEM);

  TRANSFUNC_VarFunc = TransFunc->VAR_FUNC;
  TRANSFUNC_NumberVar = TransFunc->NUMBER_VAR;
  TRANSFUNC_IndexVar = 0;

  if (TRANSFUNC_NumberVar == 0)
    {
      displayfpterror ("bad var 0\n");
    }

  BddResult = loc_imagefpttransfunc (StateSet);

  return (BddResult);
}




/*================================================================####
 * Create Transition Function
 *
 */

fpttransfunc *
createfpttransfunc (BddSystem, MaxVar)

     bddsystem *BddSystem;
     bddvar MaxVar;
{
  fpttransfunc *TransFunc;


  TRANSFUNC_StateArray = autallocblock (BddLocalCircuit->NUMBER_NAME_IN + 1);
  memset (TRANSFUNC_StateArray, '*', BddLocalCircuit->NUMBER_NAME_IN);


  setbddlocalsystem (BddSystem);

  if (MaxVar >= BDD_MAX_VAR)
    {
      displayfpterror ("bad var : %ld \n", MaxVar);
    }

  TransFunc = allocfpttransfunc ();

  TransFunc->BDD_SYSTEM = BddLocalSystem;

  TransFunc->MAX_VAR = MaxVar;
  TransFunc->VAR_FUNC = allocfptvarfunc (MaxVar);

  return (TransFunc);
}




/*================================================================####
 * Add Transition Function
 *
 */

void
addfpttransfunc (TransFunc, VarNode, FuncNode)

     fpttransfunc *TransFunc;
     bddnode *VarNode;
     bddnode *FuncNode;
{
  fptvarfunc *VarFunc;
  long MaxVar;
  long NumberVar;

  NumberVar = TransFunc->NUMBER_VAR;
  MaxVar = TransFunc->MAX_VAR;

  if (NumberVar >= BDD_MAX_VAR)
    {
      displayfpterror ("no more var \n");
    }

  if (NumberVar == MaxVar)
    {
      MaxVar = MaxVar * FPT_VAR_STRETCH_FACTOR;

      if (MaxVar >= BDD_MAX_VAR)
	{
	  MaxVar = BDD_MAX_VAR - 1;
	}

      VarFunc = resizefptvarfunc (TransFunc->VAR_FUNC,
				  TransFunc->MAX_VAR, MaxVar);

      TransFunc->VAR_FUNC = VarFunc;
      TransFunc->MAX_VAR = MaxVar;
    }
  else
    {
      VarFunc = TransFunc->VAR_FUNC;
    }

  VarFunc[NumberVar].FUNC = incbddrefext (FuncNode);
  VarFunc[NumberVar].VAR = VarNode;

  TransFunc->NUMBER_VAR++;
}

/*================================================================####
 * Destroy Transition Function
 *
 */

void
destroyfpttransfunc (TransFunc)

     fpttransfunc *TransFunc;
{
  fptvarfunc *VarFunc;
  long Variable;
  long NumberVar;

  NumberVar = TransFunc->NUMBER_VAR;
  VarFunc = TransFunc->VAR_FUNC;

  for (Variable = 0; Variable < NumberVar; Variable++)
    {
      decbddrefext (VarFunc[Variable].FUNC);
    }

  freefptvarfunc (TransFunc->VAR_FUNC);
  freefpttransfunc (TransFunc);
  autfreeblock (TRANSFUNC_StateArray);

}





/*================================================================####
 * fptViewBddState                    
 *
 */


void
fptViewBddState (BddNode)

     bddnode *BddNode;
{
  bddindex BddIndex;

  if (BddNode->INDEX < BDD_INDEX_MIN)
    {
      if (BddNode->INDEX == BDD_INDEX_ONE)
	{
	  fprintf (stdout, "%s\n", TRANSFUNC_StateArray);
	}
    }
  else
    {
      BddIndex = BddNode->INDEX - BDD_INDEX_MIN;

      TRANSFUNC_StateArray[BddIndex] = '1';
      fptViewBddState (BddNode->HIGH);

      TRANSFUNC_StateArray[BddIndex] = '0';
      fptViewBddState (BddNode->LOW);

      TRANSFUNC_StateArray[BddIndex] = '*';
    }
}






/*================================================================####
 * create product machine and transfunc
 *
 */


fpttransfunc *
createfptproductmachinetransfunc (ptbefig1, ptbefig2, NumberReg)

     befig_list *ptbefig1;
     befig_list *ptbefig2;
     long NumberReg;
{
  fpttransfunc *TransFunc;
  bddnode *BddNode;
  beout_list *ptbeout;
  bereg_list *BehReg;
  binode_list *BiNode;



  for (ptbeout = ptbefig1->BEOUT;
       ptbeout != (beout_list *) 0;
       ptbeout = ptbeout->NEXT)
    {
      BddNode = searchbddcircuitout ((bddcircuit *) 0, ptbeout->NAME);

      BddNode = applybddnode ((bddsystem *) 0, ABL_XOR, BddNode,
			      decbddrefext (ptbeout->NODE));
      ptbeout->NODE = BddNode;

      addbddcircuitout ((bddcircuit *) 0, ptbeout->NAME, BddNode);
    }


  if (NumberReg != 0)
    {
      TransFunc = createfpttransfunc ((bddsystem *) 0, NumberReg);


      for (BehReg = ptbefig1->BEREG;
	   BehReg != (bereg_list *) 0;
	   BehReg = BehReg->NEXT)
	{
	  BiNode = BehReg->BINODE;

	  BddNode = searchbddcircuitin ((bddcircuit *) 0, BehReg->NAME);
	  addfpttransfunc (TransFunc, BddNode, BiNode->VALNODE);

	}

      for (BehReg = ptbefig2->BEREG;
	   BehReg != (bereg_list *) 0;
	   BehReg = BehReg->NEXT)
	{
	  BiNode = BehReg->BINODE;

	  BddNode = searchbddcircuitin ((bddcircuit *) 0, BehReg->NAME);
	  addfpttransfunc (TransFunc, BddNode, BiNode->VALNODE);

	}
    }


  if (getfptdebugflag()) 
     viewfpttransfunc (TransFunc);


  return (TransFunc);
}





/*================================================================####
 * sequential formal proof
 *
 */


beout_list *
prooffptsequential (ptbefig1, ptbefig2,
		    BddInitialState,
		    ListOfReached,
		    BddFautifaRenvoyer,
		    TransFunc,
		    flagCreatePat)

     befig_list *ptbefig1;
     befig_list *ptbefig2;
     bddnode *BddInitialState;
     chain_list **ListOfReached;
     bddnode **BddFautifaRenvoyer;
     fpttransfunc *TransFunc;
     int flagCreatePat;

{
  bddnode *BddCurrentSet;
  bddnode *BddReachedSet;
  bddnode *BddImageSet;
  bddnode *BddNode;
  beout_list *ptbeout;
  beout_list *ptbeout2;
  int img_counter = 0;
  int FirstTime;

  displayfptnode ("\nnodes proof debut");


  BddCurrentSet = incbddrefext (BddInitialState);
  BddReachedSet = incbddrefext (BddLocalSystem->ZERO);
  FirstTime = 1;

  (*ListOfReached) = (chain_list *) 0;

  do
    {

      if (FirstTime)
	{
	  FirstTime = 0;
	  BddImageSet = incbddrefext (BddCurrentSet);
	}
      else
	{
	  BddImageSet = imagefpttransfunc (TransFunc, BddCurrentSet);
	  img_counter++;
	  if (getfptdebugflag ())
	    {
	      printf ("\nImage Set %d :\n", img_counter);
	      fptViewBddState (BddImageSet);
	      fptViewBddNode (BddImageSet);
	      printf ("\n");
	    }

	}

      BddCurrentSet = removefptstate (decbddrefext (BddImageSet), BddReachedSet);
      BddReachedSet = unionfptstate (decbddrefext (BddReachedSet), BddCurrentSet);

      if (flagCreatePat)
	(*ListOfReached) = addchain ((*ListOfReached), incbddrefext (BddCurrentSet));

      if (getfptdebugflag ())
	{
	  addbddcircuitout ((bddcircuit *) 0, "CS", BddCurrentSet);
	  addbddcircuitout ((bddcircuit *) 0, "RS", BddReachedSet);
	  displayfptverbose (" cs, rs\n");
	  testbddcircuit ((bddcircuit *) 0);
	}

      if (BddCurrentSet != BddLocalSystem->ZERO)
	{

	  if (getfptdebugflag ())
	    {
	      printf ("\nImage disjointe %d :\n", img_counter);
	      fptViewBddState (BddCurrentSet);
	      fptViewBddNode (BddCurrentSet);
	      printf ("\n");
	    }


	  for (ptbeout = ptbefig1->BEOUT;
	       ptbeout != (beout_list *) 0;
	       ptbeout = ptbeout->NEXT)
	    {

	      BddNode = cofactorbddnode ((bddsystem *) 0, ptbeout->NODE, BddCurrentSet);

	      if (BddNode != BddLocalSystem->ZERO)
		{
		  displayfptverbose ("\n");
		  displayfptwarning ("Error on output : '%s'\n", ptbeout->NAME);
		  if (getfptverboseflag ())
		    {
		      printf ("\nCofactor of output '%s' by current state : ", ptbeout->NAME);
		      fptViewBddNode (BddNode);

		      printf ("\nOutput '%s' of '%s' : ", ptbeout->NAME, ptbefig1->NAME);
		      viewablexpr (ptbeout->ABL, ABL_VIEW_VHDL);
		      printf ("\n\n");

		      for (ptbeout2 = ptbefig2->BEOUT;
			   ptbeout2->NAME != ptbeout->NAME;
			   ptbeout2 = ptbeout2->NEXT);
		      printf ("Output '%s' of '%s' : ", ptbeout2->NAME, ptbefig2->NAME);
		      viewablexpr (ptbeout2->ABL, ABL_VIEW_VHDL);
		      printf ("\n\n");
		    }


		  (*BddFautifaRenvoyer) = incbddrefext (BddNode);

		  if (getfptdebugflag ())
		    {
		      printf ("\nFautif :\n");
		      fptViewBddState (*BddFautifaRenvoyer);
		      printf ("\n");
		      addbddcircuitout ((bddcircuit *) 0, "CF", BddNode);
		      addbddcircuitout ((bddcircuit *) 0, "CS", BddCurrentSet);
		      addbddcircuitout ((bddcircuit *) 0, "RS", BddReachedSet);
		      displayfptverbose (" CF, CS, RS\n");
		      testbddcircuit ((bddcircuit *) 0);
		    }

		  decbddrefext (BddReachedSet);
		  decbddrefext (BddCurrentSet);
		  decbddrefext (BddNode);

		  return (ptbeout);
		}
	      decbddrefext (BddNode);
	    }
	}
      decbddrefext (BddCurrentSet);
    }
  while (BddCurrentSet != BddLocalSystem->ZERO);

  decbddrefext (BddCurrentSet);
  decbddrefext (BddReachedSet);

  displayfptnode ("\nnodes proof fin");

  return (NULL);
}






/*================================================================####
 * formal proof for combinatory onlyi (stateset=1)
 *
 */


beout_list *
prooffptcombinatory (ptbefig1, ptbefig2)

     befig_list *ptbefig1;
     befig_list *ptbefig2;

{
  bddnode *BddNode;
  beout_list *ptbeout;
  beout_list *ptbeout2;


  displayfptnode ("Comb nodes avant");

  for (ptbeout = ptbefig1->BEOUT;
       ptbeout != (beout_list *) 0;
       ptbeout = ptbeout->NEXT)
    {
      BddNode = cofactorbddnode ((bddsystem *) 0, ptbeout->NODE, BddLocalSystem->ONE);

      if (BddNode != BddLocalSystem->ZERO)
	{
	  displayfptwarning ("Error on output : '%s'\n", ptbeout->NAME);
	  if (getfptverboseflag ())
	    {
	      printf ("\nOutput '%s' of '%s' : ", ptbeout->NAME, ptbefig1->NAME);
	      viewablexpr (ptbeout->ABL, ABL_VIEW_VHDL);
	      printf ("\n\n");

	      for (ptbeout2 = ptbefig2->BEOUT;
		   ptbeout2->NAME != ptbeout->NAME;
		   ptbeout2 = ptbeout2->NEXT);
	      printf ("Output '%s' of '%s' : ", ptbeout2->NAME, ptbefig2->NAME);
	      viewablexpr (ptbeout2->ABL, ABL_VIEW_VHDL);
	      printf ("\n\n");
	    }
	  decbddrefext (BddNode);
	  return (ptbeout);
	}
      decbddrefext (BddNode);
    }
  displayfptnode ("Comb nodes apres");

  return (NULL);
}
