/*
 *               ,,,
 *              (o o)
 * ####=====oOO--(_)--OOO=========================================####
 * ##                                                               ##
 * ##   This file is part of FPT : A Formal Proof Toolkit           ##
 * ##   (C) 1996, MASI, CAO-VLSI Team                               ##
 * ##                                                               ##
 * ##   File            :  fpt_verify.c                             ##
 * ##   Author          :  Olivier SIROL                            ##
 * ##   Date            :  Jun 1996                                 ##
 * ##                                                               ##
 * ##   E-mail support  :  cao-vlsi@masi.ibp.fr                     ##
 * ##                                                               ##
 * ####===========================================================####
 *
 *
 */

#ident "$Id: fpt_verify.c,v 1.26 1997/05/23 14:27:36 sirol Exp $"


/*
 * ####===========================================================####
 * ##                                                               ##
 * ##                         Include Files                         ##
 * ##                                                               ##
 * ####===========================================================####
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include MUT_H
#include AUT_H
#include ABL_H
#include BDD_H
#include LOG_H
#include BEH_H
#include BVL_H

#include "fpt_public.h"



/*
 * ####===========================================================####
 * ##                                                               ##
 * ##                           Constants                           ##
 * ##                                                               ##
 * ####===========================================================####
 */
/*
 * ####===========================================================####
 * ##                                                               ##
 * ##                             Types                             ##
 * ##                                                               ##
 * ####===========================================================####
 */
/*
 * ####===========================================================####
 * ##                                                               ##
 * ##                           Variables                           ##
 * ##                                                               ##
 * ####===========================================================####
 */
/*
 * ####===========================================================####
 * ##                                                               ##
 * ##                           Functions                           ##
 * ##                                                               ##
 * ####===========================================================####
 */



/*================================================================####
 * verifyfptdelayed
 *     
 */

int
verifyfptdelayed (BehFigure1, BehFigure2)

     befig_list *BehFigure1;
     befig_list *BehFigure2;
{
  beaux_list *BehDly;
  authtable *HashTable;
  authelem *Element;
  long NumberDly1;
  long NumberDly2;
  int Error = 0;

  NumberDly1 = 0;

  for (BehDly = BehFigure1->BEDLY;
       BehDly != (beaux_list *) 0;
       BehDly = BehDly->NEXT)
    {
      NumberDly1++;
    }

  NumberDly2 = 0;

  for (BehDly = BehFigure2->BEDLY;
       BehDly != (beaux_list *) 0;
       BehDly = BehDly->NEXT)
    {
      NumberDly2++;
    }

  if (NumberDly1 != NumberDly2)
    {
      displayfptwarning ("Different delayed signals number in '%s' and '%s'\n",
			 BehFigure1->NAME, BehFigure2->NAME);
      Error = 1;

    }

  HashTable = createauthtable (NumberDly1 << 1);

  for (BehDly = BehFigure1->BEDLY;
       BehDly != (beaux_list *) 0;
       BehDly = BehDly->NEXT)
    {
      addauthelem (HashTable, BehDly->NAME, 0);
    }

  for (BehDly = BehFigure2->BEDLY;
       BehDly != (beaux_list *) 0;
       BehDly = BehDly->NEXT)
    {
      Element = searchauthelem (HashTable, BehDly->NAME);

      if (Element == (authelem *) 0)
	{
	  displayfptwarning ("Unknown delayed signal '%s' in '%s'\n",
			     BehDly->NAME, BehFigure1->NAME);

	  Error = 1;
	}
    }

  destroyauthtable (HashTable);

  if (Error)
    displayfpterror ("The two figures have different delayed signals.\n");

  return (!Error);
}






/*================================================================####
 * verifyfptcondition
 *     
 */

int
verifyfptcondition (BehFigure1, BehFigure2)

     befig_list *BehFigure1;
     befig_list *BehFigure2;
{
  bereg_list *BehReg;
  binode_list *BiNode;
  bddnode *BddNode;
  int Error = 0;

  BddNode = (bddnode *) 0;

  for (BehReg = BehFigure1->BEREG;
       BehReg != (bereg_list *) 0;
       BehReg = BehReg->NEXT)
    {
      if (getfptdebugflag ())
	{
	  printf ("%s [fig 1] <= ", BehReg->NAME);
	  viewablexpr (BehReg->BIABL->CNDABL, ABL_VIEW_VHDL);
	  printf ("\n");
	}
      BiNode = BehReg->BINODE;

      while (BiNode != (binode_list *) 0)
	{
	  if (BddNode != (bddnode *) 0)
	    {
	      if (BddNode != BiNode->CNDNODE)
		{
		  displayfptwarning ("%s [fig 1] <= ", BehReg->NAME);
		  viewablexpr (BehReg->BIABL->CNDABL, ABL_VIEW_VHDL);
		  printf ("\n");
		  Error = 1;
		}
	    }
	  else
	    {
	      BddNode = BiNode->CNDNODE;
            /* if (getfptdebugflag ())
		{
		  addbddcircuitout ((bddcircuit *) 0, "CND", BddNode);
		  displayfptdebug ("addcircuitout CND \n");
		} */
	    }

	  BiNode = BiNode->NEXT;
	}
    }

  for (BehReg = BehFigure2->BEREG;
       BehReg != (bereg_list *) 0;
       BehReg = BehReg->NEXT)
    {
      if (getfptdebugflag ())
	{
	  printf ("%s [fig 2] <= ", BehReg->NAME);
	  viewablexpr (BehReg->BIABL->CNDABL, ABL_VIEW_VHDL);
	  printf ("\n");
	}
      BiNode = BehReg->BINODE;

      while (BiNode != (binode_list *) 0)
	{
	  if (BddNode != BiNode->CNDNODE)
	    {
	      displayfptwarning ("%s [fig 2] <= ", BehReg->NAME);
	      viewablexpr (BehReg->BIABL->CNDABL, ABL_VIEW_VHDL);
	      printf ("\n");
	      Error = 1;

	    }
	  BiNode = BiNode->NEXT;
	}
    }

  if (Error)
    displayfpterror ("The figures have different register conditions.\n");

  return (!Error);
}




/*================================================================####
 * FptVerifyInOut
 *     
 */

void FptVerifyInOut( BehFigure, BehPort )

  befig_list *BehFigure;
  bepor_list *BehPort;
{
  if ( ( BehPort->TYPE      == 'B' ) &&
       ( BehPort->DIRECTION == 'B' ) )
  {
    BehFigure->BERIN = beh_rmvberin( BehFigure->BERIN, BehPort->NAME );
    BehPort->DIRECTION = 'O';
    displayfptdebug("Considering type of '%s' (from figure '%s') as output type.\n", BehPort->NAME, BehFigure->NAME);
  }
}




/*================================================================####
 * verifyfptport
 *     
 */

int
verifyfptport (BehFigure1, BehFigure2)

     befig_list *BehFigure1;
     befig_list *BehFigure2;

{
  bepor_list *BehPort;
  authtable *HashTable;
  authelem *Element;
  long NumberPort1;
  long NumberPort2;
  long TypeAndDir;
  int Error = 0;


  if (getfptdebugflag ())
    FptAff (BehFigure1);

  NumberPort1 = 0;

  for (BehPort = BehFigure1->BEPOR;
       BehPort != (bepor_list *) 0;
       BehPort = BehPort->NEXT)
    {
      FptVerifyInOut( BehFigure1, BehPort );
      NumberPort1++;
    }

  NumberPort2 = 0;

  for (BehPort = BehFigure2->BEPOR;
       BehPort != (bepor_list *) 0;
       BehPort = BehPort->NEXT)
    {
      FptVerifyInOut( BehFigure2, BehPort );
      NumberPort2++;
    }

  if (NumberPort1 != NumberPort2)
    {
      displayfptwarning ("Different port number in '%s' and '%s'\n",
			 BehFigure1->NAME, BehFigure2->NAME);
      Error = 1;

    }

  HashTable = createauthtable (NumberPort1 << 1);

  for (BehPort = BehFigure1->BEPOR;
       BehPort != (bepor_list *) 0;
       BehPort = BehPort->NEXT)
    {
      TypeAndDir = ((long) (BehPort->TYPE) << 8) |
	((long) (BehPort->DIRECTION));

      addauthelem (HashTable, BehPort->NAME, TypeAndDir);
    }

  for (BehPort = BehFigure2->BEPOR;
       BehPort != (bepor_list *) 0;
       BehPort = BehPort->NEXT)
    {
      TypeAndDir = ((long) (BehPort->TYPE) << 8) |
	((long) (BehPort->DIRECTION));

      Element = searchauthelem (HashTable, BehPort->NAME);

      if (Element == (authelem *) 0)
	{
	  displayfptwarning ("Port '%s' not present in '%s'\n",
			     BehPort->NAME, BehFigure1->NAME);

	  Error = 1;
	}
      else if (Element->VALUE != TypeAndDir)
	{
	  displayfptwarning ("Port '%s' has different type in '%s' and '%s'\n",
			 BehPort->NAME, BehFigure1->NAME, BehFigure2->NAME);

	  Error = 1;
	}
    }

  destroyauthtable (HashTable);

  if (Error)
    {
      HashTable = createauthtable (NumberPort2 << 1);

      for (BehPort = BehFigure2->BEPOR;
	   BehPort != (bepor_list *) 0;
	   BehPort = BehPort->NEXT)
	{
	  addauthelem (HashTable, BehPort->NAME, 0);
	}

      for (BehPort = BehFigure1->BEPOR;
	   BehPort != (bepor_list *) 0;
	   BehPort = BehPort->NEXT)
	{

	  Element = searchauthelem (HashTable, BehPort->NAME);

	  if (Element == (authelem *) 0)
	    {
	      displayfptwarning ("Port '%s' not present in '%s'\n",
				 BehPort->NAME, BehFigure2->NAME);

	      Error = 1;
	    }
	}

      destroyauthtable (HashTable);
    }


  if (Error)
    displayfpterror ("The two figures have different interface.\n");

  /* check for bus, exit if found */

  if ((BehFigure1->BEBUS != NULL) || (BehFigure1->BEBUX != NULL))
    displayfpterror ("figure '%s' contains BEBUS or BEBUX, this is not handled yet.\n", BehFigure1->NAME);

  if ((BehFigure2->BEBUS != NULL) || (BehFigure2->BEBUX != NULL))
    displayfpterror ("figure '%s' contains BEBUS or BEBUX, this is not handled yet.\n", BehFigure2->NAME);



  return (Error);

}






/*================================================================####
 * FptVerifyRegister
 *     
 */

int
FptVerifyRegister (BehFigure1, BehFigure2)

     befig_list *BehFigure1;
     befig_list *BehFigure2;
{
  bereg_list *BehReg;
  authtable *HashTable;
  authelem *Element;
  long NumberReg1;
  long NumberReg2;
  int Error;

  NumberReg1 = 0;

  for (BehReg = BehFigure1->BEREG;
       BehReg != (bereg_list *) 0;
       BehReg = BehReg->NEXT)
    {
      NumberReg1++;
    }

  NumberReg2 = 0;

  for (BehReg = BehFigure2->BEREG;
       BehReg != (bereg_list *) 0;
       BehReg = BehReg->NEXT)
    {
      NumberReg2++;
    }

  if (NumberReg1 != NumberReg2)
    {
      displayfptwarning ("Different register number in '%s' and '%s'\n",
			 BehFigure1->NAME, BehFigure2->NAME);

    }

  HashTable = createauthtable (NumberReg1 << 1);
  Error = 0;

  for (BehReg = BehFigure1->BEREG;
       BehReg != (bereg_list *) 0;
       BehReg = BehReg->NEXT)
    {
      addauthelem (HashTable, BehReg->NAME, 0);
    }

  for (BehReg = BehFigure2->BEREG;
       BehReg != (bereg_list *) 0;
       BehReg = BehReg->NEXT)
    {
      Element = searchauthelem (HashTable, BehReg->NAME);

      if (Element == (authelem *) 0)
	{
	  displayfptwarning ("Unknown register '%s' in '%s'\n",
			     BehReg->NAME, BehFigure1->NAME);

	  Error = 1;
	}
    }

  destroyauthtable (HashTable);

  return (!Error);
}



/* OLD STUFF */

/*================================================================####
 * Compare noms entrees, reg et sorties
 *     
 */

void
FptCompareIO (ptbefig1, ptbefig2)
     struct befig *ptbefig1;
     struct befig *ptbefig2;
{

  berin_list *ptberin;
  bereg_list *ptbereg;
  beout_list *ptbeout;
  long count_in1 = 1;
  long count_in2 = 1;
  long count_reg1 = 1;
  long count_reg2 = 1;
  long count_out1 = 1;
  long count_out2 = 1;
  authtable *HashTable;
  static FindErrorFlag = 0;

/* entrees  ******************************************************* */

  for (ptberin = ptbefig1->BERIN;
       ptberin;
       ptberin = ptberin->NEXT)
    {
      count_in1++;
    }

  for (ptberin = ptbefig2->BERIN;
       ptberin;
       ptberin = ptberin->NEXT)
    {
      count_in2++;
    }

  if (count_in1 != count_in2)
    {
      if (!FindErrorFlag)
	displayfptwarning ("Not same number of Inputs : \"%s\" : %d; \"%s\" : %d\n", ptbefig1->NAME, count_in1, ptbefig2->NAME, count_in2);
      FindErrorFlag = 1;
    }

  /* init la hashtab avec 2* plus de place que nb d'entree */
  HashTable = createauthtable (count_in1 << 1);

  for (ptberin = ptbefig1->BERIN;
       ptberin;
       ptberin = ptberin->NEXT)
    {
      addauthelem (HashTable, namealloc (ptberin->NAME), 0);
    }

  for (ptberin = ptbefig2->BERIN;
       ptberin;
       ptberin = ptberin->NEXT)
    {

      if (!searchauthelem (HashTable, namealloc (ptberin->NAME)))
	displayfpterror ("Input \"%s\" is not in \"%s\"\n", ptberin->NAME, ptbefig1->NAME);
    }

/* resetauthtable( HashTable ); */
  destroyauthtable (HashTable);

/* regs  ******************************************************* */


  for (ptbereg = ptbefig1->BEREG;
       ptbereg;
       ptbereg = ptbereg->NEXT)
    {
      count_reg1++;
    }

  for (ptbereg = ptbefig2->BEREG;
       ptbereg;
       ptbereg = ptbereg->NEXT)
    {
      count_reg2++;
    }

  if (count_reg1 != count_reg2)
    {
      if (!FindErrorFlag)
	displayfptwarning ("Not same number of REGS : \"%s\" : %d; \"%s\" : %d\n", ptbefig1->NAME, count_reg1, ptbefig2->NAME, count_reg2);
      FindErrorFlag = 1;
    }

  /* init la hashtab avec 2* plus de place que nb d'entree */
  HashTable = createauthtable (count_reg1 << 1);

  for (ptbereg = ptbefig1->BEREG;
       ptbereg;
       ptbereg = ptbereg->NEXT)
    {
      addauthelem (HashTable, namealloc (ptbereg->NAME), 0);
    }

  for (ptbereg = ptbefig2->BEREG;
       ptbereg;
       ptbereg = ptbereg->NEXT)
    {
      if (!searchauthelem (HashTable, namealloc (ptbereg->NAME)))
	displayfpterror ("Register \"%s\" is not in \"%s\"\n", ptbereg->NAME, ptbefig1->NAME);
    }

  destroyauthtable (HashTable);

/* sorties  ******************************************************* */


  for (ptbeout = ptbefig1->BEOUT;
       ptbeout;
       ptbeout = ptbeout->NEXT)
    {
      count_out1++;
    }

  for (ptbeout = ptbefig2->BEOUT;
       ptbeout;
       ptbeout = ptbeout->NEXT)
    {
      count_out2++;
    }

  if (count_out1 != count_out2)
    {
      if (!FindErrorFlag)
	displayfptwarning ("Not same number of Outputs : \"%s\" : %d; \"%s\" : %d\n", ptbefig1->NAME, count_out1, ptbefig2->NAME, count_out2);
      FindErrorFlag = 1;
    }

  /* init la hashtab avec 2* plus de place que nb d'entree */

  HashTable = createauthtable (count_out1 << 1);

  for (ptbeout = ptbefig1->BEOUT;
       ptbeout;
       ptbeout = ptbeout->NEXT)
    {
      addauthelem (HashTable, namealloc (ptbeout->NAME), 0);
    }

  for (ptbeout = ptbefig2->BEOUT;
       ptbeout;
       ptbeout = ptbeout->NEXT)
    {
      if (!searchauthelem (HashTable, namealloc (ptbeout->NAME)))
	displayfpterror ("Output \"%s\" is not in \"%s\"\n", ptbeout->NAME, ptbefig1->NAME);
    }

  destroyauthtable (HashTable);

  if (FindErrorFlag)
    FptCompareIO (ptbefig2, ptbefig1);
}



/*================================================================####
 * Compare bdd
 *     
 */

int
FptCompareBdd (ptbefig1, ptbefig2)
     struct befig *ptbefig1;
     struct befig *ptbefig2;
{

  beout_list *ptbeout1;
  beout_list *ptbeout2;


#ifdef DEBUG_MFP
  testbddcircuit ((bddcircuit *) 0);
#endif

  for (ptbeout1 = ptbefig1->BEOUT;
       ptbeout1;
       ptbeout1 = ptbeout1->NEXT)
    {

      for (ptbeout2 = ptbefig2->BEOUT;
	   (ptbeout2->NAME) != (ptbeout1->NAME);
	   ptbeout2 = ptbeout2->NEXT);

#ifdef DEBUG_MFP
      printf ("debug info --> %s <= ", ptbeout1->NAME);
      viewablexpr (ptbeout1->ABL, ABL_VIEW_VHDL);
      printf ("\n");

      printf ("debug info --> %s <= ", ptbeout1->NAME);
      viewbddnode ((bddsystem *) 0, ptbeout1->NODE);
      printf ("\n");

      printf ("debug info --> %s <= ", ptbeout2->NAME);
      viewablexpr (ptbeout2->ABL, ABL_VIEW_VHDL);
      printf ("\n");

      printf ("debug info --> %s <= ", ptbeout2->NAME);
      viewbddnode ((bddsystem *) 0, ptbeout2->NODE);
      printf ("\n");
#endif


      if ((ptbeout1->NODE) != (ptbeout2->NODE))
	{
	  displayfptwarning ("Not same Output \"%s\" \n", ptbeout1->NAME);
	  displayfptwarning ("%s [fig 1] <= ", ptbefig1->NAME);
	  viewablexpr (ptbeout1->ABL, ABL_VIEW_VHDL);
	  displayfptwarning ("\n");
	  displayfptwarning ("%s [fig 2] <= ", ptbefig2->NAME);
	  viewablexpr (ptbeout2->ABL, ABL_VIEW_VHDL);
	  displayfptwarning ("\n");
	  return (1);
	}
    }
  return (0);
}
