/*
 *               ,,,
 *              (o o)
 * ####=====oOO--(_)--OOO=========================================####
 * ##                                                               ##
 * ##   This file is part of FPT : A Formal Proof Toolkit           ##
 * ##   (C) 1996, MASI, CAO-VLSI Team                               ##
 * ##                                                               ##
 * ##   File            :  fpt_main.c                               ##
 * ##   Author          :  Olivier SIROL                            ##
 * ##   Date            :  Jun 1996                                 ##
 * ##                                                               ##
 * ##   E-mail support  :  cao-vlsi@masi.ibp.fr                     ##
 * ##                                                               ##
 * ####===========================================================####
 *
 *
 */

#ident "$Id: yeap_main.c,v 1.26 1997/06/02 12:11:57 sirol Exp $"


/*
 * ####===========================================================####
 * ##                                                               ##
 * ##                         Include Files                         ##
 * ##                                                               ##
 * ####===========================================================####
 */


#include <sys/time.h>

#include <unistd.h>		/* for getopt */
extern char *optarg;
extern int optind, opterr, optopt;

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>

#include MUT_H
#include AUT_H
#include ABL_H
#include BDD_H
#include LOG_H
#include BEH_H
#include BVL_H

#include FPT_H


/*
 * ####===========================================================####
 * ##                                                               ##
 * ##                           Constants                           ##
 * ##                                                               ##
 * ####===========================================================####
 */


/*
 * ####===========================================================####
 * ##                                                               ##
 * ##                             Types                             ##
 * ##                                                               ##
 * ####===========================================================####
 */
/*
 * ####===========================================================####
 * ##                                                               ##
 * ##                           Variables                           ##
 * ##                                                               ##
 * ####===========================================================####
 */

char *YEAP_InputFileName1;
char *YEAP_InputFileName2;
char *YEAP_FigureName1;
char *YEAP_FigureName2;
char *YEAP_InputPatFileName1;
char *YEAP_InputPatFileName2;
char *YEAP_OutputPatFileName;
char *YEAP_Program_name;

int YEAP_BddReorder = 100000;	/*  100 Ko noeuds */
int YEAP_BddMaxNode = 500000;	/*  500 Ko noeuds */
int YEAP_BddStrategy = 1;	/*  reordersimple */

char *YEAP_ResetName;
int YEAP_Reset_Active_HI = 1;	/* Reset actif a letat haut */
char *YEAP_ClockName;
char *YEAP_VddName;


/*
 * ####===========================================================####
 * ##                                                               ##
 * ##                           Functions                           ##
 * ##                                                               ##
 * ####===========================================================####
 */



/*================================================================####
 * Yeap_FastUsage 
 *     
 */

void
Yeap_FastUsage ()
{
  printf ("Type '%s -h' for help on command line options.\n", YEAP_Program_name);

}



/*================================================================####
 * Fpt Usage   
 *     
 */

void
Yeap_Usage ()
{
  printf ("\
Usage : \n\
%s [Options] Input_filename1(.vbe) Input_filename2(.vbe) \n\
\n\
Where options are : \n\
         -h       : this help screen \n\
         -v       : set verbose mode \n\
         -d       : set debug mode (for develloppers) \n\
\n\
Setting the initial state of the input figures :\n\
 Use quick mode : \n\
         -r name  : set RESET name to 'name' (default : 'reset')\n\
         -l       : set RESET active LOW (default : active HI)\n\
 Or use pattern file : \n\
         -i       : use pattern file to compute initial state. \n\
         -1 name  : set configuration input filename 1 to 'name.pat' \n\
                    (default : 'Input_filename1_ini.pat') \n\
         -2 name  : set configuration input filename 2 to 'name.pat' \n\
                    (default : 'Input_filename2_ini.pat') \n\
\n\
Generating output pattern file :\n\
         -p       : generates output pattern file if an error occurs. \n\
         -o name  : set output pattern file filename to 'name.pat' \n\
                    (default : 'Input_filename1_vs_Input_filename2.pat') \n\
         -c name  : force CLOCK name of pattern file to 'name' (default : computed) \n\
         -V name  : set VDD name of pattern file to 'name' (default : 'vdd') \n\
\n\
Modifying the Bdds behaviour :\n\
         -m num   : set max number nodes of bdd to 'num' * 10Ko nodes \n\
                    (default : '50' which means 500000 nodes) \n\
         -s num   : set strategy of dynamic bdd reorder \n\
                         0 : no reorder, garbage only.\n\
                         1 : simple reorder (default).\n\
                         2 : window reorder.\n\
                         3 : top reorder.\n\
         -n num   : set auto reorder of bdd to 'num' * 10Ko nodes \n\
                    (default : '10' which means 100000 nodes) \n\
", YEAP_Program_name);

}




/*================================================================####
 * print_elapsed_time
 *     
 */

void
print_elapsed_time (clock_start, clock_end)

  struct timeval clock_start;
  struct timeval clock_end;

{
  int elapsed_sec;
  int elapsed_usec;

  if (clock_end.tv_sec &&  clock_start.tv_sec)
  {
  elapsed_sec = clock_end.tv_sec - clock_start.tv_sec;

  if (clock_end.tv_usec < clock_start.tv_usec)
    {
      --elapsed_sec;
      elapsed_usec = 1000000 - clock_start.tv_usec + clock_end.tv_usec;
    }
  else
  {
    elapsed_usec = clock_end.tv_usec - clock_start.tv_usec;
  }

  printf ("%d min %02d.%02d sec", 
	  (elapsed_sec / 60),
	  (elapsed_sec % 60),
	  elapsed_usec / 10000); 
  
  
/* this for me for profiling (giving time in minutes) */  
/* 
 min = (double)((double)elapsed_sec + ((double)elapsed_usec / (double)1000000));
  printf ("%.2f sec", min);
*/    
  }
  else
  printf ("?");
   
}



/*================================================================####
 *  Main                        
 *                                                            
 */

int
main (argc, argv)
     int argc;
     char *argv[];
{
  int FPError = 0;
  int c;

  struct befig *ptbefig1 = NULL;
  struct befig *ptbefig2 = NULL;
  bddsystem *BddSystem;
  bddcircuit *BddCircuit;


  int fig1type;			/* 0 sequential, 1 combinatory */
  int fig2type;

  bddnode *BddReset;
  chain_list *ListOfReached;
  fpttransfunc *MP_Transfunc;
  bddnode *BddFautif;
  beout_list *BeoutFautif;
  long NumReg = 0;

  int flag_ResetName = 1;
  int flag_VddName = 1;
  int flag_ClockName = 1;
  int flag_Out_Pat = 1;
  int flag_In_Pat1 = 1;
  int flag_In_Pat2 = 1;

  int flag_Use_In_Pat = 0;
  int flag_Create_Pat = 0;
  int flag_Use_Active_LOW = 0;

  struct timeval tv_clock_start;
  struct timeval tv_clock_start_abl;
  struct timeval tv_clock_start_bdd;
  struct timeval tv_clock_start_fp;
  struct timeval tv_clock_start_pat;
  struct timeval tv_clock_end;

  time_t clock_start;

  int i;
  char *file2load1;
  char *file2load2;
  char *tmpfile;
  char *tmpstr;


  tv_clock_start.tv_sec = 0;
  tv_clock_start_abl.tv_sec = 0;
  tv_clock_start_bdd.tv_sec = 0;
  tv_clock_start_fp.tv_sec = 0;
  tv_clock_start_pat.tv_sec = 0;
  tv_clock_end.tv_sec = 0;
  
  time (&clock_start);
  gettimeofday (&tv_clock_start, NULL);

  YEAP_Program_name = mbkstrdup (argv[0]);

  mbkenv ();
  autenv ();
  ablenv ();
  bddenv ();

  alliancebanner ("Yeap", YEAP_BANNER,
		  "YEt Another formal Proof",
		  "1995-97",
		  ALLIANCE_VERSION);

  clearfptverboseflag ();
  clearfptdebugflag ();

#ifdef DEBUG_YEAP
  setfptverboseflag ();
  setfptdebugflag ();
#endif


  while ((c = getopt (argc, argv, "hdviplr:c:V:m:s:n:1:2:o:")) != -1)
    {
      /* printf("Val renv : '%c' pour '%c' \n",c, optopt); */

      switch (c)
	{

	case 'i':
	  flag_Use_In_Pat = 1;
	  break;

	case '1':
	  flag_In_Pat1 = 0;
	  flag_Use_In_Pat = 1;
	  YEAP_InputPatFileName1 = mbkstrdup (optarg);
	  break;

	case '2':
	  flag_In_Pat2 = 0;
	  flag_Use_In_Pat = 1;
	  YEAP_InputPatFileName2 = mbkstrdup (optarg);
	  break;

	case 'p':
	  flag_Create_Pat = 1;
	  break;

	case 'o':
	  flag_Create_Pat = 1;
	  flag_Out_Pat = 0;
	  YEAP_OutputPatFileName = mbkstrdup (optarg);
	  break;

	case 'c':
	  flag_ClockName = 0;
	  YEAP_ClockName = namealloc (optarg);
	  break;

	case 'l':
	  flag_Use_Active_LOW = 1;
	  YEAP_Reset_Active_HI = 0;
	  break;

	case 'r':
	  flag_ResetName = 0;
	  YEAP_ResetName = namealloc (optarg);
	  break;

	case 'V':
	  flag_VddName = 0;
	  YEAP_VddName = namealloc (optarg);
	  break;

	case 's':
	  if ((sscanf (optarg, "%d", &YEAP_BddStrategy) != 1)
	      || (YEAP_BddStrategy < 0)
	      || (YEAP_BddStrategy > 3))

	    {
	      fprintf (stderr, "%s: argument of -- s must be 0 <= num <= 3\n", YEAP_Program_name);
	      Yeap_FastUsage ();
	      displayfpterror ("Bad command line.\n");
	    }
	  break;


	case 'm':
	  if ((sscanf (optarg, "%d", &YEAP_BddMaxNode) != 1)
	      || (YEAP_BddMaxNode < 1)
	      || (YEAP_BddMaxNode > 1000))

	    {
	      fprintf (stderr, "%s: argument of -- m must be 1 <= num <= 1000\n", YEAP_Program_name);
	      Yeap_FastUsage ();
	      displayfpterror ("Bad command line.\n");
	    }
	  else
	    YEAP_BddMaxNode = YEAP_BddMaxNode * 10000;
	  break;


	case 'n':
	  if ((sscanf (optarg, "%d", &YEAP_BddReorder) != 1)
	      || (YEAP_BddReorder < 1)
	      || (YEAP_BddReorder > 1000))

	    {
	      fprintf (stderr, "%s: argument of -- n must be 1 <= num <= 1000\n", YEAP_Program_name);
	      Yeap_FastUsage ();
	      displayfpterror ("Bad command line.\n");
	    }
	  else
	    YEAP_BddReorder = YEAP_BddReorder * 10000;
	  break;


	case 'v':
	  setfptverboseflag ();
	  break;

	case 'd':
	  setfptverboseflag ();
	  setfptdebugflag ();
	  break;

	case 'h':
	  Yeap_Usage ();
	  EXIT (1);
	  break;

	default:
	  Yeap_FastUsage ();
	  displayfpterror ("Bad command line.\n");
	  break;

	}
    }

  if ((argc - optind) != 2)
    {
      Yeap_FastUsage ();
      displayfpterror ("%s: input or output file not specified correctly\n",
		       YEAP_Program_name);
    }
  else
    {
      YEAP_InputFileName1 = mbkstrdup (argv[optind]);
      YEAP_InputFileName2 = mbkstrdup (argv[optind + 1]);
    }



  displayfptverbose ("\n============================ Command line and files =========================\n\n");

  printf ("---> Command line : ");
  for (i = 0; i < argc; i++)
    printf ("%s ", argv[i]);
  printf ("\n");


  /* existance des vbe a tester (bug dans mbkstrdup(NULL) */
  
  if ((tmpfile = filepath (YEAP_InputFileName1, "vbe")))
  {
    file2load1 = mbkstrdup(tmpfile);
    printf ("---> Using '%s' as first VHDL input file (fig1)\n", file2load1);
  }
  else
    displayfpterror ("can't find file '%s.vbe'\n", YEAP_InputFileName1);

  if ((tmpfile = filepath (YEAP_InputFileName2, "vbe")))
  {
    file2load2 = mbkstrdup(tmpfile);
     printf ("---> Using '%s' as second VHDL input file (fig2)\n", file2load2);
  }
  else
    displayfpterror ("can't find file '%s.vbe'\n", YEAP_InputFileName2);


  /* other options settings */

  if (flag_ResetName)
    {
      YEAP_ResetName = namealloc ("reset");
    }
  if (flag_VddName)
    {
      YEAP_VddName = namealloc ("vdd");
    }
  if (flag_ClockName)
    {
      YEAP_ClockName = NULL ;
    }

  if (flag_Use_In_Pat && flag_In_Pat1)
    {
      tmpstr = mbkalloc (strlen (YEAP_InputFileName1) + 4 + 1);
      strcpy (tmpstr, "");
      strcat (tmpstr, YEAP_InputFileName1);
      strcat (tmpstr, "_ini");
      YEAP_InputPatFileName1 = mbkstrdup (tmpstr);
      mbkfree (tmpstr);
    }

  if (flag_Use_In_Pat && flag_In_Pat2)
    {
      tmpstr = mbkalloc (strlen (YEAP_InputFileName2) + 4 + 1);
      strcpy (tmpstr, "");
      strcat (tmpstr, YEAP_InputFileName2);
      strcat (tmpstr, "_ini");
      YEAP_InputPatFileName2 = mbkstrdup (tmpstr);
      mbkfree (tmpstr);
    }

  if (flag_Out_Pat)
    {
      tmpstr = mbkalloc (strlen (YEAP_InputFileName1) + strlen (YEAP_InputFileName2) + 4 + 1);
      strcpy (tmpstr, "");
      strcat (tmpstr, YEAP_InputFileName1);
      strcat (tmpstr, "_vs_");
      strcat (tmpstr, YEAP_InputFileName2);
      YEAP_OutputPatFileName = mbkstrdup (tmpstr);
      mbkfree (tmpstr);
    }



  /* quick mode -R -L incompatible si on a donne -1 -2 -i */

  if ((!(flag_ResetName) && (flag_Use_In_Pat))
      || ((flag_Use_Active_LOW) && (flag_Use_In_Pat)))
    displayfpterror ("You can't specify quick mode options (-R -L) to set the initial state if you use -i option. Initial state is computed from pat files (options -1, -2)\n");


  if (flag_Use_In_Pat)
    {
      /* existance des pat de init */

      if ((tmpfile = filepath (YEAP_InputPatFileName1, "pat")))
	printf ("---> Using '%s' as initial pat filename 1.\n", tmpfile);
      else
	displayfpterror ("can't find file '%s.pat'\n", YEAP_InputPatFileName1);

      if ((tmpfile = filepath (YEAP_InputPatFileName2, "pat")))
	printf ("---> Using '%s' as initial pat filename 2.\n", tmpfile);
      else
	displayfpterror ("can't find file '%s.pat'\n", YEAP_InputPatFileName2);

    }
  else
    {
      printf ("---> Using '%s' as RESET signal. %s\n", YEAP_ResetName, (YEAP_Reset_Active_HI ? "(Reset active HI)" : "(Reset active LOW)"));
    }


  if (flag_Create_Pat)
    {
  if (!flag_ClockName)
      printf ("---> Using '%s' as CLOCK signal\n", YEAP_ClockName);
      printf ("---> Using '%s' as VDD signal\n", YEAP_VddName);
      printf ("---> Using '%s.pat' as pattern file output filename\n", YEAP_OutputPatFileName);
    }

  printf ("\n");
  displayfptverbose ("\n============================ Loading Beh Figures ============================\n\n");

  gettimeofday (&tv_clock_start_abl, NULL);

  fig1type = loadfptbefig (&ptbefig1, YEAP_InputFileName1, &YEAP_FigureName1);

  fig2type = loadfptbefig (&ptbefig2, YEAP_InputFileName2, &YEAP_FigureName2);

  if (fig1type != fig2type)
    {
      FPError = 1;
      displayfptwarning ("Incompatible figures : '%s' is %s and '%s' is %s.\n",
			 ptbefig1->NAME,
	      ((fig1type == COMB_TYPE) ? "combinatory only" : "sequential"),
			 ptbefig2->NAME,
	     ((fig2type == COMB_TYPE) ? "combinatory only" : "sequential"));
    }

  else
    {
      if (fig1type == COMB_TYPE)
	printf ("===> Using combinatory formal proof.\n");
      else
	printf ("===> Using sequential formal proof.\n");


      displayfptverbose ("\n============================ Verifying Ports ================================\n\n");
      
      printf ("===> Verifying Port declaration.\n");
      verifyfptport (ptbefig1, ptbefig2);

      printf ("===> Verifying Delayed signals.\n");
      verifyfptdelayed (ptbefig1, ptbefig2);

      if (fig1type != COMB_TYPE)
	{
	  displayfptverbose ("\n============================ Renaming fig2 Rgisters =========================\n\n");
	  
	  NumReg = renamefptregister (ptbefig1, ptbefig2);
	}

      
      
      displayfptverbose ("\n============================ Making Bdds ====================================\n\n");
      
      gettimeofday (&tv_clock_start_bdd, NULL);
  
      BddSystem = createbddsystem (100, 1000, 100, YEAP_BddMaxNode);
      
switch (YEAP_BddStrategy)
  {
  case 0:
    reorderbddsystemdynamic (BddSystem, garbagebddsystem, YEAP_BddReorder, 100);
    break;

  case 1:
    reorderbddsystemdynamic (BddSystem, reorderbddsystemsimple, YEAP_BddReorder, 100);
    break;

  case 2:
    reorderbddsystemdynamic (BddSystem, reorderbddsystemwindow, YEAP_BddReorder, 100);
    break;

  case 3:
    reorderbddsystemdynamic (BddSystem, reorderbddsystemtop, YEAP_BddReorder, 100);
    break;
  }
      
     
      BddCircuit = createfptcircuit (ptbefig1, ptbefig2, BddSystem);


      
      displayfptverbose ("\n============================ Verifying Conditions ============================\n\n");

      printf ("===> Verifying Condition on registers.\n");
      verifyfptcondition (ptbefig1, ptbefig2);
      
      if (fig1type == COMB_TYPE)
	{
	  displayfptverbose ("\n============================ Running Formal Proof ============================\n\n");
	  
	  printf ("===> Running Formal Proof.\n");
	  MP_Transfunc = createfptproductmachinetransfunc (ptbefig1, ptbefig2, 0);
	  BeoutFautif = prooffptcombinatory (ptbefig1, ptbefig2);
	  FPError = (BeoutFautif != NULL);
	}
      else
	{
	  displayfptverbose ("\n============================ Searching Initial State ========================\n\n");

	  if (flag_Use_In_Pat)
	    {
	      printf ("===> Using pattern file to compute initial state.\n");
	      BddReset = getfptinitstatebypat (BddSystem, BddCircuit,
					       ptbefig1, ptbefig2,
					       YEAP_InputPatFileName1,
					       YEAP_InputPatFileName2);
	    }
	  else
	    {
	      printf ("===> Using signal '%s' to compute initial state. %s\n", YEAP_ResetName, (YEAP_Reset_Active_HI ? "(Reset active HI)" : "(Reset active LOW)"));
	      BddReset = getfptinitstatebyname (BddSystem, BddCircuit,
						ptbefig1, ptbefig2,
						YEAP_ResetName,
						YEAP_Reset_Active_HI);

	    }

	  
	  
	  displayfptverbose ("\n============================ Running Formal Proof ============================\n\n");
	  
          gettimeofday (&tv_clock_start_fp, NULL);
	  printf ("===> Running Formal Proof.\n");
	  MP_Transfunc = createfptproductmachinetransfunc (ptbefig1, ptbefig2, NumReg);

	  BeoutFautif = prooffptsequential (ptbefig1, ptbefig2,
					    BddReset,
					    &ListOfReached,
					    &BddFautif,
					    MP_Transfunc,
					    flag_Create_Pat);

	  FPError = (BeoutFautif != NULL);

              gettimeofday (&tv_clock_start_pat, NULL);

	  if (flag_Create_Pat && FPError)
	    {
	      displayfptverbose ("\n============================ Formal Proof ERROR : creating pat file ==========\n\n");

	      
	      createfptpattern (ptbefig1, ptbefig2,
				BddFautif,
				BeoutFautif,
				ListOfReached,
				MP_Transfunc,
				BddReset,
				YEAP_ClockName,
				YEAP_VddName,
				YEAP_InputFileName1,
				YEAP_InputFileName2,
				YEAP_OutputPatFileName,
				file2load1,
				file2load2,
				YEAP_FigureName1,
				YEAP_FigureName2);
	    }
	}
      destroybddcircuit (BddCircuit);
      destroybddsystem (BddSystem);
    }


  printf ("\n============================ Result Summary =================================\n\n");

  gettimeofday (&tv_clock_end, NULL);
  
  printf ("===> Formal proof between figures\n");
  printf ("     %s : %s [%s]\n", ptbefig1->NAME, file2load1,
	  ((fig1type == 1) ? "Combinatory" : "Sequential"));
  printf ("     %s : %s [%s]\n\n", ptbefig2->NAME, file2load2,
	  ((fig2type == 1) ? "Combinatory" : "Sequential"));

  if (FPError)
    {
      printf ("===> Formal Proof ERROR\n");
      if (fig1type != fig2type)
	{
	  printf ("     Incompatible figures types,\n");
	  printf ("     one is sequential and the other is combinatory only.\n");
	}
      else
	{
	  printf ("     Error on output : '%s'\n", BeoutFautif->NAME);
	  if ((fig1type == COMB_TYPE) || (!flag_Create_Pat))
	    printf ("     No pattern file generated.\n");
	  else
	    printf ("     See pattern file '%s.pat' for details...\n", YEAP_OutputPatFileName);
	}
    }
  else
    printf ("===> Formal Proof OK\n");

  printf ("\n");

  printf ("===> Started on %s", ctime (&clock_start));
  
  printf ("     Total elapsed time : ");
  print_elapsed_time (tv_clock_start , tv_clock_end); printf ("\n");

  /* profiling purpose (need proof on seq and -p) */
 /* 
  printf ("DEBUG VERSION : THIS IS FOR PROFILING PURPOSE (need proof on seq and -p)\n");
  printf ("        Time Making Abl : ");
  print_elapsed_time (tv_clock_start_abl , tv_clock_start_bdd); printf ("\n");

  printf ("       Time Making Bdds : ");
  print_elapsed_time (tv_clock_start_bdd , tv_clock_start_fp); printf ("\n");

  printf ("     Time Running Proof : ");
  print_elapsed_time (tv_clock_start_fp , tv_clock_start_pat); printf ("\n");

  printf ("   Time Making Patterns : ");
  print_elapsed_time (tv_clock_start_pat , tv_clock_end); printf ("\n");

  printf ("      Time Max Bdd Node : %ld nodes\n", BddSystem->TOP_NODE );
  */
  
  printf ("\n");
  
  printf ("===> Total number of warning : %d\n\n", getfptwarnings ());

  if (FPError)
    {
      printf ("\
                   @@@@@@@@@        @      @@@@@@ @@@@@@     \n\
                     @@    @        @        @@     @@       \n\
                     @@     @      @@@       @@     @@       \n\
                     @@            @@@       @@     @@       \n\
                     @@   @       @  @@      @@     @@       \n\
                     @@@@@@       @  @@      @@     @@       \n\
                     @@   @      @    @@     @@     @@       \n\
                     @@          @@@@@@@     @@     @@       \n\
                     @@         @      @@    @@     @@      @\n\
                     @@         @      @@    @@     @@     @ \n\
                   @@@@@@     @@@@    @@@@ @@@@@@ @@@@@@@@@@ \n\n");
    }
  else
    {
      printf ("\
                   @@@@@@@          @        @@@@ @    @@@@ @ \n\
                     @@   @@        @       @    @@   @    @@ \n\
                     @@    @@      @@@     @@     @  @@     @ \n\
                     @@    @@      @@@     @@@       @@@      \n\
                     @@   @@      @  @@     @@@@      @@@@    \n\
                     @@@@@        @  @@       @@@@      @@@@  \n\
                     @@          @    @@        @@@       @@@ \n\
                     @@          @@@@@@@   @      @@ @      @@\n\
                     @@         @      @@  @@     @@ @@     @@\n\
                     @@         @      @@  @@@    @  @@@    @ \n\
                   @@@@@@     @@@@    @@@@ @  @@@@   @  @@@@  \n\n");
    }

  return (FPError);

}





